@@ -28,6 +28,7 @@
  */
 
 #include <algorithm>
+#include <string_view>
 
 #include "attr_line.hh"
 
@@ -36,9 +37,145 @@
 #include "ansi_scrubber.hh"
 #include "auto_mem.hh"
 #include "config.h"
+#include "fts_fuzzy_match.hh"
+#include "intervaltree/IntervalTree.h"
 #include "lnav_log.hh"
 #include "pcrepp/pcre2pp.hh"
 
+using namespace std::string_view_literals;
+
+attr_line_t
+attr_line_t::from_table_cell_content(const string_fragment& content,
+                                     size_t max_char_width)
+{
+    static constexpr auto TAB_SYMBOL = "\u21e5"sv;
+    static constexpr auto LF_SYMBOL = "\u240a"sv;
+    static constexpr auto CR_SYMBOL = "\u240d"sv;
+    static constexpr auto REP_SYMBOL = "\ufffd"sv;
+    static constexpr auto ELLIPSIS = "\u22ef"_frag;
+
+    auto has_ansi = false;
+    size_t char_width = 0;
+    attr_line_t retval;
+    std::string_view replacement;
+    int copy_start = 0;
+
+    retval.al_string.reserve(max_char_width);
+    for (int index = 0; index < content.length(); ++index) {
+        const auto ch = content.udata()[index];
+
+        switch (ch) {
+            case '\t':
+                replacement = TAB_SYMBOL;
+                char_width += 1;
+                break;
+            case '\n':
+                replacement = LF_SYMBOL;
+                char_width += 1;
+                break;
+            case '\r':
+                replacement = CR_SYMBOL;
+                char_width += 1;
+                break;
+            case '\b':
+            case '\x1b':
+                has_ansi = true;
+                break;
+            default:
+                if (ch < 0x80) {
+                    char_width += 1;
+                } else if (ch < 0xc0) {
+                    replacement = REP_SYMBOL;
+                    char_width += 1;
+                } else if (ch < 0xe0) {
+                    auto next_ch = content[index + 1];
+                    if (next_ch != 0) {
+                        index += 1;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xf0) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0) {
+                        index += 2;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xf8) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0
+                        && content[index + 3] != 0)
+                    {
+                        index += 3;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xfc) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0
+                        && content[index + 3] != 0 && content[index + 4] != 0)
+                    {
+                        index += 4;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else if (ch < 0xfe) {
+                    if (content[index + 1] != 0 && content[index + 2] != 0
+                        && content[index + 3] != 0 && content[index + 4] != 0
+                        && content[index + 5] != 0)
+                    {
+                        index += 5;
+                    } else {
+                        replacement = REP_SYMBOL;
+                    }
+                    char_width += 1;
+                } else {
+                    replacement = REP_SYMBOL;
+                    char_width += 1;
+                }
+                break;
+        }
+
+        if (!replacement.empty()) {
+            auto copy_len = index - copy_start;
+            retval.al_string.append(&content[copy_start], copy_len);
+            copy_start = index + 1;
+            auto lr = line_range{(int) retval.al_string.size(), -1};
+            retval.al_string.append(replacement);
+            lr.lr_end = retval.al_string.size();
+            retval.al_attrs.emplace_back(lr, VC_ROLE.value(role_t::VCR_HIDDEN));
+            replacement = ""sv;
+        }
+    }
+    if (copy_start < content.length()) {
+        auto copy_len = content.length() - copy_start;
+        retval.al_string.append(&content[copy_start], copy_len);
+    }
+
+    if (has_ansi) {
+        scrub_ansi_string(retval.al_string, &retval.al_attrs);
+    }
+
+    if (char_width > max_char_width) {
+        auto chars_to_remove = (char_width - max_char_width) + 1;
+        auto midpoint = char_width / 2;
+        auto chars_to_keep_at_front = midpoint - (chars_to_remove / 2);
+        auto bytes_to_keep_at_front
+            = utf8_char_to_byte_index(retval.al_string, chars_to_keep_at_front);
+        auto remove_up_to_bytes = utf8_char_to_byte_index(
+            retval.al_string, chars_to_keep_at_front + chars_to_remove);
+        auto bytes_to_remove = remove_up_to_bytes - bytes_to_keep_at_front;
+        retval.erase(bytes_to_keep_at_front, bytes_to_remove);
+        retval.insert(bytes_to_keep_at_front, ELLIPSIS);
+        auto lr = line_range{(int) bytes_to_keep_at_front,
+                             (int) bytes_to_keep_at_front + ELLIPSIS.length()};
+        retval.al_attrs.emplace_back(lr, VC_ROLE.value(role_t::VCR_HIDDEN));
+    }
+
+    return retval;
+}
+
 attr_line_t&
 attr_line_t::with_ansi_string(const char* str, ...)
 {
@@ -61,6 +198,17 @@ attr_line_t&
 attr_line_t::with_ansi_string(const std::string& str)
 {
     this->al_string = str;
+    this->al_attrs.clear();
+    scrub_ansi_string(this->al_string, &this->al_attrs);
+
+    return *this;
+}
+
+attr_line_t&
+attr_line_t::with_ansi_string(const string_fragment& str)
+{
+    this->al_string = str.to_string();
+    this->al_attrs.clear();
     scrub_ansi_string(this->al_string, &this->al_attrs);
 
     return *this;
@@ -96,6 +244,8 @@ consume(const string_fragment text)
     static const auto SPACE_RE
         = lnav::pcre2pp::code::from_const(R"((*UTF)^\s)");
 
+    require(text.is_valid());
+
     if (text.empty()) {
         return eof{text};
     }
@@ -122,16 +272,8 @@ consume(const string_fragment text)
         return space{split_res.first, split_res.second};
     }
 
-    auto csize_res = ww898::utf::utf8::char_size(
-        [&text]() { return std::make_pair(text.front(), text.length()); });
-
-    if (csize_res.isErr()) {
-        auto split_res = text.split_n(1);
-
-        return corrupt{split_res->first, split_res->second};
-    }
-
-    auto split_res = text.split_n(csize_res.unwrap());
+    auto next_char_byte_index = text.column_to_byte_index(1);
+    auto split_res = text.split_n(next_char_byte_index);
 
     return word{split_res->first, split_res->second};
 }
@@ -166,6 +308,8 @@ attr_line_t::insert(size_t index,
                     const attr_line_t& al,
                     text_wrap_settings* tws)
 {
+    require(!tws || tws->tws_width > 0);
+
     if (index < this->al_string.length()) {
         shift_string_attrs(this->al_attrs, index, al.al_string.length());
     }
@@ -175,7 +319,7 @@ attr_line_t::insert(size_t index,
     for (const auto& sa : al.al_attrs) {
         this->al_attrs.emplace_back(sa);
 
-        line_range& lr = this->al_attrs.back().sa_range;
+        auto& lr = this->al_attrs.back().sa_range;
 
         lr.shift(0, index);
         if (lr.lr_end == -1) {
@@ -187,7 +331,8 @@ attr_line_t::insert(size_t index,
         return *this;
     }
 
-    auto starting_line_index = this->al_string.rfind('\n', index);
+    auto starting_line_index = index == 0 ? std::string::npos
+                                          : this->al_string.rfind('\n', index - 1);
     if (starting_line_index == std::string::npos) {
         starting_line_index = 0;
     } else {
@@ -230,12 +375,20 @@ attr_line_t::insert(size_t index,
         auto pre_iter = find_string_attr_containing(
             this->al_attrs, &SA_PREFORMATTED, text_to_wrap.sf_begin);
         if (pre_iter != this->al_attrs.end()) {
+            require_ge(pre_iter->sa_range.lr_start, text_to_wrap.sf_begin);
             auto pre_len = pre_iter->sa_range.lr_end - text_to_wrap.sf_begin;
             auto pre_lf = text_to_wrap.find('\n');
-            if (pre_lf && pre_lf.value() < pre_len) {
+            if (pre_lf && pre_lf.value() + 1 < pre_len) {
                 pre_len = pre_lf.value() + 1;
+                auto lr_copy = pre_iter->sa_range;
+                const_cast<int&>(pre_iter->sa_range.lr_end)
+                    = pre_iter->sa_range.lr_start + pre_len;
+                this->al_attrs.emplace_back(lr_copy, SA_PREFORMATTED.value());
+                this->al_attrs.back().sa_range.lr_start
+                    = lr_copy.lr_start + pre_len;
             }
 
+            require_ge(text_to_wrap.length(), pre_len);
             auto pre_pair = text_to_wrap.split_n(pre_len);
             next_chunk = text_stream::word{
                 pre_pair->first,
@@ -247,9 +400,8 @@ attr_line_t::insert(size_t index,
         }
 
         text_to_wrap = next_chunk.match(
-            [&](text_stream::word word) {
-                auto ch_count
-                    = word.w_word.utf8_length().unwrapOr(word.w_word.length());
+            [&](const text_stream::word& word) {
+                ssize_t ch_count = word.w_word.column_width();
 
                 if (line_ch_count > line_indent_count && !last_was_pre
                     && (line_ch_count + ch_count) > usable_width)
@@ -273,8 +425,8 @@ attr_line_t::insert(size_t index,
                     auto trailing_space_count = 0;
                     if (!last_word.empty()) {
                         trailing_space_count
-                            = word.w_word.sf_begin - last_word.sf_begin;
-                        this->erase(last_word.sf_begin, trailing_space_count);
+                            = word.w_word.sf_begin - last_word.sf_end;
+                        this->erase(last_word.sf_end, trailing_space_count);
                     }
                     return word.w_remaining
                         .erase_before(this->al_string.data(),
@@ -291,7 +443,7 @@ attr_line_t::insert(size_t index,
 
                 return word.w_remaining;
             },
-            [&](text_stream::space space) {
+            [&](const text_stream::space& space) {
                 if (space.s_value == "\n") {
                     line_ch_count = 0;
                     line_indent_count = 0;
@@ -300,8 +452,7 @@ attr_line_t::insert(size_t index,
                 }
 
                 if (line_ch_count > 0) {
-                    auto ch_count = space.s_value.utf8_length().unwrapOr(
-                        space.s_value.length());
+                    ssize_t ch_count = space.s_value.column_width();
 
                     if ((line_ch_count + ch_count) > usable_width
                         && find_string_attr_containing(this->al_attrs,
@@ -319,15 +470,18 @@ attr_line_t::insert(size_t index,
                         auto trailing_space_count = 0;
                         if (!last_word.empty()) {
                             trailing_space_count
-                                = space.s_value.sf_begin - last_word.sf_begin;
+                                = space.s_value.sf_begin - last_word.sf_end;
                             this->erase(last_word.sf_end, trailing_space_count);
                         }
+                        last_word.clear();
 
-                        return space.s_remaining
-                            .erase_before(
-                                this->al_string.data(),
-                                space.s_value.length() + trailing_space_count)
-                            .prepend(this->al_string.data(), 1);
+                        auto retval
+                            = space.s_remaining
+                                  .erase_before(this->al_string.data(),
+                                                space.s_value.length()
+                                                    + trailing_space_count)
+                                  .prepend(this->al_string.data(), 1);
+                        return retval;
                     }
                     line_ch_count += ch_count;
                 } else if (find_string_attr_containing(this->al_attrs,
@@ -346,7 +500,7 @@ attr_line_t::insert(size_t index,
             [](text_stream::eof eof) { return eof.e_remaining; });
 
         if (next_chunk.is<text_stream::word>()) {
-            last_word = text_to_wrap;
+            last_word = next_chunk.get<text_stream::word>().w_word;
         }
         last_was_pre = (pre_iter != this->al_attrs.end());
 
@@ -356,6 +510,19 @@ attr_line_t::insert(size_t index,
     return *this;
 }
 
+attr_line_t&
+attr_line_t::wrap_with(text_wrap_settings* tws)
+{
+    attr_line_t tmp;
+
+    tmp.al_string = std::move(this->al_string);
+    tmp.al_attrs = std::move(this->al_attrs);
+
+    this->append(tmp, tws);
+
+    return *this;
+}
+
 attr_line_t
 attr_line_t::subline(size_t start, size_t len) const
 {
@@ -386,13 +553,43 @@ attr_line_t::subline(size_t start, size_t len) const
 void
 attr_line_t::split_lines(std::vector<attr_line_t>& lines) const
 {
+    using line_type_t = interval_tree::Interval<size_t, attr_line_t>;
+    using lines_tree_t = interval_tree::IntervalTree<size_t, attr_line_t>;
+
+    auto eols = std::vector<line_type_t>{};
     size_t pos = 0, next_line;
 
     while ((next_line = this->al_string.find('\n', pos)) != std::string::npos) {
-        lines.emplace_back(this->subline(pos, next_line - pos));
+        eols.emplace_back(
+            pos,
+            next_line > pos ? next_line - 1 : next_line,
+            attr_line_t{this->al_string.substr(pos, next_line - pos)});
         pos = next_line + 1;
     }
-    lines.emplace_back(this->subline(pos));
+    if (pos < this->al_string.length()) {
+        eols.emplace_back(pos,
+                          this->al_string.length() - 1,
+                          attr_line_t{this->al_string.substr(pos)});
+    }
+    auto lines_tree = lines_tree_t{std::move(eols)};
+    for (const auto& sa : this->al_attrs) {
+        if (sa.sa_range.empty()) {
+            continue;
+        }
+        auto range_end = sa.sa_range.end_for_string(this->al_string) - 1;
+        lines_tree.visit_overlapping(
+            sa.sa_range.lr_start, range_end, [&sa](const auto& cinterval) {
+                auto& interval = const_cast<line_type_t&>(cinterval);
+                auto lr = line_range(interval.start, interval.stop + 1);
+                auto ilr = lr.intersection(sa.sa_range).shift(0, -lr.lr_start);
+                interval.value.al_attrs.emplace_back(
+                    ilr, std::make_pair(sa.sa_type, sa.sa_value));
+            });
+    }
+    lines_tree.visit_all([&lines](auto& cinterval) {
+        auto& interval = const_cast<line_type_t&>(cinterval);
+        lines.emplace_back(std::move(interval.value));
+    });
 }
 
 attr_line_t&
@@ -438,7 +635,8 @@ attr_line_t::apply_hide()
     auto& sa = this->al_attrs;
 
     for (auto& sattr : sa) {
-        if (sattr.sa_type == &SA_HIDDEN && sattr.sa_range.length() > 3) {
+        if (sattr.sa_type == &SA_HIDDEN) {
+            auto icon = sattr.sa_value.get<ui_icon_t>();
             auto& lr = sattr.sa_range;
 
             std::for_each(sa.begin(), sa.end(), [&](string_attr& attr) {
@@ -449,15 +647,28 @@ attr_line_t::apply_hide()
 
             this->al_string.replace(lr.lr_start, lr.length(), "\xE2\x8B\xAE");
             shift_string_attrs(sa, lr.lr_start + 1, -(lr.length() - 3));
-            sattr.sa_type = &VC_ROLE;
-            sattr.sa_value = role_t::VCR_HIDDEN;
+            sattr.sa_type = &VC_ICON;
+            sattr.sa_value = icon;
             lr.lr_end = lr.lr_start + 3;
+        } else if (sattr.sa_type == &SA_REPLACED) {
+            auto& lr = sattr.sa_range;
+
+            std::for_each(sa.begin(), sa.end(), [&](string_attr& attr) {
+                if (attr.sa_type == &VC_STYLE && lr.contains(attr.sa_range)) {
+                    attr.sa_type = &SA_REMOVED;
+                }
+            });
+
+            this->al_string.erase(lr.lr_start, lr.length());
+            shift_string_attrs(sa, lr.lr_start, -lr.length());
+            sattr.sa_type = &SA_REMOVED;
+            lr.lr_end = lr.lr_start;
         }
     }
 }
 
 attr_line_t&
-attr_line_t::rtrim(nonstd::optional<const char*> chars)
+attr_line_t::rtrim(std::optional<const char*> chars)
 {
     auto index = this->al_string.length();
 
@@ -510,7 +721,7 @@ attr_line_t::erase(size_t pos, size_t len)
 attr_line_t&
 attr_line_t::pad_to(ssize_t size)
 {
-    const auto curr_len = this->utf8_length_or_length();
+    const ssize_t curr_len = this->column_width();
 
     if (curr_len < size) {
         this->append((size - curr_len), ' ');
@@ -525,6 +736,44 @@ attr_line_t::pad_to(ssize_t size)
     return *this;
 }
 
+size_t
+attr_line_t::column_to_byte_index(size_t column) const
+{
+    return string_fragment::from_str(this->al_string)
+        .column_to_byte_index(column);
+}
+
+attr_line_t&
+attr_line_t::highlight_fuzzy_matches(const std::string& pattern)
+{
+    if (pattern.empty()) {
+        return *this;
+    }
+
+    constexpr size_t MATCH_COUNT = 256;
+    uint8_t matches[MATCH_COUNT];
+    int score;
+
+    memset(matches, 0, sizeof(matches));
+    if (!fts::fuzzy_match(pattern.c_str(),
+                          this->al_string.c_str(),
+                          score,
+                          matches,
+                          MATCH_COUNT))
+    {
+        return *this;
+    }
+
+    for (auto lpc = size_t{0}; (lpc == 0 || matches[lpc]) && lpc < MATCH_COUNT;
+         lpc++)
+    {
+        auto lr = line_range{matches[lpc], matches[lpc] + 1};
+        this->al_attrs.emplace_back(lr, VC_ROLE.value(role_t::VCR_FUZZY_MATCH));
+    }
+
+    return *this;
+}
+
 line_range
 line_range::intersection(const line_range& other) const
 {
@@ -638,7 +887,8 @@ shift_string_attrs(string_attrs_t& sa, const line_range& cover, int32_t amount)
 }
 
 struct line_range
-find_string_attr_range(const string_attrs_t& sa, string_attr_type_base* type)
+find_string_attr_range(const string_attrs_t& sa,
+                       const string_attr_type_base* type)
 {
     auto iter = find_string_attr(sa, type);
 
@@ -660,7 +910,7 @@ remove_string_attr(string_attrs_t& sa, const line_range& lr)
 }
 
 void
-remove_string_attr(string_attrs_t& sa, string_attr_type_base* type)
+remove_string_attr(string_attrs_t& sa, const string_attr_type_base* type)
 {
     for (auto iter = sa.begin(); iter != sa.end();) {
         if (iter->sa_type == type) {
@@ -690,9 +940,8 @@ find_string_attr(const string_attrs_t& sa,
                  const string_attr_type_base* type,
                  int start)
 {
-    string_attrs_t::const_iterator iter;
-
-    for (iter = sa.begin(); iter != sa.end(); ++iter) {
+    auto iter = sa.begin();
+    for (; iter != sa.end(); ++iter) {
         if (iter->sa_type == type && iter->sa_range.lr_start >= start) {
             break;
         }
@@ -701,7 +950,7 @@ find_string_attr(const string_attrs_t& sa,
     return iter;
 }
 
-nonstd::optional<const string_attr*>
+std::optional<const string_attr*>
 get_string_attr(const string_attrs_t& sa,
                 const string_attr_type_base* type,
                 int start)
@@ -709,8 +958,8 @@ get_string_attr(const string_attrs_t& sa,
     auto iter = find_string_attr(sa, type, start);
 
     if (iter == sa.end()) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
-    return nonstd::make_optional(&(*iter));
+    return std::make_optional(&(*iter));
 }
