@@ -36,8 +36,9 @@
 #include "config.h"
 #include "date_time_scanner.cfg.hh"
 #include "injector.hh"
+#include "math_util.hh"
 #include "ptimec.hh"
-#include "scn/scn.h"
+#include "scn/scan.h"
 
 size_t
 date_time_scanner::ftime(char* dst,
@@ -119,13 +120,14 @@ date_time_scanner::scan(const char* time_dest,
     this->dts_zoned_to_local = cfg.c_zoned_to_local;
     while (next_format(time_fmt, curr_time_fmt, this->dts_fmt_lock)) {
         *tm_out = this->dts_base_tm;
+        tm_out->et_tm.tm_yday = -1;
         tm_out->et_flags = 0;
         if (time_len > 1 && time_dest[0] == '+' && isdigit(time_dest[1])) {
             retval = nullptr;
-            auto sv = scn::string_view{time_dest, time_len};
+            auto sv = std::string_view{time_dest, time_len};
             auto epoch_scan_res = scn::scan_value<int64_t>(sv);
             if (epoch_scan_res) {
-                time_t gmt = epoch_scan_res.value();
+                time_t gmt = epoch_scan_res->value();
 
                 if (convert_local
                     && (this->dts_local_time || this->dts_zoned_to_local))
@@ -143,7 +145,7 @@ date_time_scanner::scan(const char* time_dest,
                     | ETF_MACHINE_ORIENTED | ETF_EPOCH_TIME | ETF_ZONE_SET;
 
                 this->dts_fmt_lock = curr_time_fmt;
-                this->dts_fmt_len = sv.length() - epoch_scan_res.range().size();
+                this->dts_fmt_len = sv.length() - epoch_scan_res->range().size();
                 retval = time_dest + this->dts_fmt_len;
                 found = true;
                 break;
@@ -176,13 +178,22 @@ date_time_scanner::scan(const char* time_dest,
                         && !(tm_out->et_flags & ETF_EPOCH_TIME)
                         && this->dts_default_zone != nullptr)
                     {
-                        date::local_seconds stime;
-                        stime += std::chrono::seconds{gmt};
-                        auto ztime
-                            = date::make_zoned(this->dts_default_zone, stime);
-                        gmt = std::chrono::duration_cast<std::chrono::seconds>(
-                                  ztime.get_sys_time().time_since_epoch())
-                                  .count();
+                        try {
+                            date::local_seconds stime;
+                            stime += std::chrono::seconds{gmt};
+                            auto ztime
+                                = date::make_zoned(this->dts_default_zone,
+                                                   stime,
+                                                   date::choose::earliest);
+                            gmt = std::chrono::duration_cast<
+                                      std::chrono::seconds>(
+                                      ztime.get_sys_time().time_since_epoch())
+                                      .count();
+                        } catch (const std::exception& e) {
+                            log_error("failed to convert time %d -- %s",
+                                      gmt,
+                                      e.what());
+                        }
                     }
                     this->to_localtime(gmt, *tm_out);
                 }
@@ -195,7 +206,6 @@ date_time_scanner::scan(const char* time_dest,
                 {
                     const auto sec_diff = tm_out->et_tm.tm_sec - last_tm.tm_sec;
 
-                    // log_debug("diff %d", sec_diff);
                     tv_out = this->dts_last_tv;
                     tv_out.tv_sec += sec_diff;
                     tm_out->et_tm.tm_wday = last_tm.tm_wday;
@@ -231,20 +241,43 @@ date_time_scanner::scan(const char* time_dest,
                 if (convert_local
                     && (this->dts_local_time
                         || tm_out->et_flags & ETF_EPOCH_TIME
-                        || (tm_out->et_flags & ETF_ZONE_SET
+                        || ((tm_out->et_flags & ETF_ZONE_SET
+                             || this->dts_default_zone != nullptr)
                             && this->dts_zoned_to_local)))
                 {
                     time_t gmt = tm_out->to_timeval().tv_sec;
 
+                    if (!(tm_out->et_flags & ETF_ZONE_SET)
+                        && !(tm_out->et_flags & ETF_EPOCH_TIME)
+                        && this->dts_default_zone != nullptr)
+                    {
+                        date::local_seconds stime;
+                        stime += std::chrono::seconds{gmt};
+                        auto ztime
+                            = date::make_zoned(this->dts_default_zone, stime);
+                        gmt = std::chrono::duration_cast<std::chrono::seconds>(
+                                  ztime.get_sys_time().time_since_epoch())
+                                  .count();
+                    }
                     this->to_localtime(gmt, *tm_out);
-#ifdef HAVE_STRUCT_TM_TM_ZONE
-                    tm_out->et_tm.tm_zone = nullptr;
-#endif
-                    tm_out->et_tm.tm_isdst = 0;
                 }
+                const auto& last_tm = this->dts_last_tm.et_tm;
+                if (last_tm.tm_year == tm_out->et_tm.tm_year
+                    && last_tm.tm_mon == tm_out->et_tm.tm_mon
+                    && last_tm.tm_mday == tm_out->et_tm.tm_mday
+                    && last_tm.tm_hour == tm_out->et_tm.tm_hour
+                    && last_tm.tm_min == tm_out->et_tm.tm_min)
+                {
+                    const auto sec_diff = tm_out->et_tm.tm_sec - last_tm.tm_sec;
 
-                tv_out = tm_out->to_timeval();
-                secs2wday(tv_out, &tm_out->et_tm);
+                    tv_out = this->dts_last_tv;
+                    tv_out.tv_sec += sec_diff;
+                    tm_out->et_tm.tm_wday = last_tm.tm_wday;
+                } else {
+                    tv_out = tm_out->to_timeval();
+                    secs2wday(tv_out, &tm_out->et_tm);
+                }
+                tv_out.tv_usec = tm_out->et_nsec / 1000;
 
                 this->dts_fmt_lock = curr_time_fmt;
                 this->dts_fmt_len = retval - time_dest;
@@ -286,17 +319,9 @@ date_time_scanner::scan(const char* time_dest,
                     = std::chrono::duration_cast<std::chrono::microseconds>(
                           std::chrono::nanoseconds{tm_out->et_nsec})
                           .count();
-                this->dts_fmt_len += 7;
-                tm_out->et_flags |= ETF_MICROS_SET | ETF_SUB_NOT_IN_FORMAT;
-                retval += 7;
-            } else if (ptime_L(tm_out, time_dest, off, time_len)) {
-                tv_out.tv_usec
-                    = std::chrono::duration_cast<std::chrono::microseconds>(
-                          std::chrono::nanoseconds{tm_out->et_nsec})
-                          .count();
-                this->dts_fmt_len += 4;
-                tm_out->et_flags |= ETF_MILLIS_SET | ETF_SUB_NOT_IN_FORMAT;
-                retval += 4;
+                this->dts_fmt_len = off;
+                tm_out->et_flags |= ETF_SUB_NOT_IN_FORMAT;
+                retval = time_dest + this->dts_fmt_len;
             }
         }
     }
@@ -304,6 +329,19 @@ date_time_scanner::scan(const char* time_dest,
     return retval;
 }
 
+void
+date_time_scanner::clear()
+{
+    this->dts_base_time = 0;
+    this->dts_base_tm = exttm{};
+    this->dts_fmt_lock = -1;
+    this->dts_fmt_len = -1;
+    this->dts_last_tv = timeval{};
+    this->dts_last_tm = exttm{};
+    this->dts_localtime_cached_gmt = 0;
+    this->dts_localtime_cached_tm = tm{};
+}
+
 void
 date_time_scanner::set_base_time(time_t base_time, const tm& local_tm)
 {
@@ -339,7 +377,29 @@ date_time_scanner::to_localtime(time_t t, exttm& tm_out)
             -= this->dts_local_offset_expiry % EXPIRE_TIME;
     } else {
         time_t adjust_gmt = t + this->dts_local_offset_cache;
-        secs2tm(adjust_gmt, &tm_out.et_tm);
+        auto adjust_gmt_min = adjust_gmt / 60;
+        if (this->dts_localtime_cached_gmt == adjust_gmt_min)
+        {
+            tm_out.et_tm = this->dts_localtime_cached_tm;
+            tm_out.et_tm.tm_sec = adjust_gmt % 60;
+        } else {
+            secs2tm(adjust_gmt, &tm_out.et_tm);
+            this->dts_localtime_cached_gmt = adjust_gmt_min;
+            this->dts_localtime_cached_tm = tm_out.et_tm;
+            this->dts_localtime_cached_tm.tm_sec = 0;
+        }
+#if 0
+        {
+            tm verify_tm;
+            secs2tm(adjust_gmt, &verify_tm);
+            require(tm_out.et_tm.tm_year == verify_tm.tm_year);
+            require(tm_out.et_tm.tm_mon == verify_tm.tm_mon);
+            require(tm_out.et_tm.tm_mday == verify_tm.tm_mday);
+            require(tm_out.et_tm.tm_hour == verify_tm.tm_hour);
+            require(tm_out.et_tm.tm_min == verify_tm.tm_min);
+            require(tm_out.et_tm.tm_sec == verify_tm.tm_sec);
+        }
+#endif
     }
     tm_out.et_gmtoff = 0;
 #ifdef HAVE_STRUCT_TM_TM_ZONE
