@@ -35,15 +35,14 @@
 #include "fmt/color.h"
 #include "itertools.hh"
 #include "lnav.console.into.hh"
+#include "lnav_log.hh"
 #include "log_level_enum.hh"
 #include "pcrepp/pcre2pp.hh"
 #include "snippet_highlighters.hh"
-#include "view_curses.hh"
 
 using namespace lnav::roles::literals;
 
-namespace lnav {
-namespace console {
+namespace lnav::console {
 
 snippet
 snippet::from_content_with_offset(intern_string_t src,
@@ -51,13 +50,13 @@ snippet::from_content_with_offset(intern_string_t src,
                                   size_t offset,
                                   const std::string& errmsg)
 {
-    auto content_sf = string_fragment::from_str(content.get_string());
-    auto line_with_error = content_sf.find_boundaries_around(
+    const auto content_sf = string_fragment::from_str(content.get_string());
+    const auto line_with_error = content_sf.find_boundaries_around(
         offset, string_fragment::tag1{'\n'});
-    auto line_with_context = content_sf.find_boundaries_around(
+    const auto line_with_context = content_sf.find_boundaries_around(
         offset, string_fragment::tag1{'\n'}, 3);
-    auto line_number = content_sf.sub_range(0, offset).count('\n');
-    auto erroff_in_line = offset - line_with_error.sf_begin;
+    const auto line_number = content_sf.sub_range(0, offset).count('\n');
+    const auto erroff_in_line = offset - line_with_error.sf_begin;
 
     attr_line_t pointer;
 
@@ -70,7 +69,7 @@ snippet::from_content_with_offset(intern_string_t src,
     retval.s_content
         = content.subline(line_with_context.sf_begin,
                           line_with_error.sf_end - line_with_context.sf_begin);
-    if (line_with_error.sf_end >= retval.s_content.get_string().size()) {
+    if (line_with_error.sf_end >= (int) retval.s_content.get_string().size()) {
         retval.s_content.append("\n");
     }
     retval.s_content.append(pointer).append(
@@ -134,6 +133,21 @@ user_message::warning(const attr_line_t& al)
     return retval;
 }
 
+user_message&
+user_message::remove_internal_snippets()
+{
+    auto new_end = std::remove_if(
+        this->um_snippets.begin(),
+        this->um_snippets.end(),
+        [](const snippet& snip) {
+            return snip.s_location.sl_source.to_string_fragment().startswith(
+                "__");
+        });
+    this->um_snippets.erase(new_end, this->um_snippets.end());
+
+    return *this;
+}
+
 attr_line_t
 user_message::to_attr_line(std::set<render_flags> flags) const
 {
@@ -149,17 +163,28 @@ user_message::to_attr_line(std::set<render_flags> flags) const
             case level::raw:
                 break;
             case level::ok:
-                retval.append(lnav::roles::ok("\u2714 "));
+                retval.append("  ");
+                retval.al_attrs.emplace_back(line_range{0, 1},
+                                             VC_ICON.value(ui_icon_t::ok));
                 break;
             case level::info:
-                retval.append("\u24d8 info"_info).append(": ");
+                retval.append("  ").append("info"_info).append(": ");
+                retval.al_attrs.emplace_back(line_range{0, 1},
+                                             VC_ICON.value(ui_icon_t::info));
                 break;
             case level::warning:
-                retval.append(lnav::roles::warning("\u26a0 warning"))
+                retval.append("  ")
+                    .append(lnav::roles::warning("warning"))
                     .append(": ");
+                retval.al_attrs.emplace_back(line_range{0, 1},
+                                             VC_ICON.value(ui_icon_t::warning));
                 break;
             case level::error:
-                retval.append(lnav::roles::error("\u2718 error")).append(": ");
+                retval.append("  ")
+                    .append(lnav::roles::error("error"))
+                    .append(": ");
+                retval.al_attrs.emplace_back(line_range{0, 1},
+                                             VC_ICON.value(ui_icon_t::error));
                 break;
         }
     }
@@ -252,29 +277,59 @@ user_message::to_attr_line(std::set<render_flags> flags) const
     return retval;
 }
 
-static nonstd::optional<fmt::terminal_color>
-curses_color_to_terminal_color(int curses_color)
+static std::optional<fmt::terminal_color>
+color_to_terminal_color(const styling::color_unit& curses_color)
 {
-    switch (curses_color) {
-        case COLOR_BLACK:
-            return fmt::terminal_color::black;
-        case COLOR_CYAN:
-            return fmt::terminal_color::cyan;
-        case COLOR_WHITE:
-            return fmt::terminal_color::white;
-        case COLOR_MAGENTA:
-            return fmt::terminal_color::magenta;
-        case COLOR_BLUE:
-            return fmt::terminal_color::blue;
-        case COLOR_YELLOW:
-            return fmt::terminal_color::yellow;
-        case COLOR_GREEN:
-            return fmt::terminal_color::green;
-        case COLOR_RED:
-            return fmt::terminal_color::red;
-        default:
-            return nonstd::nullopt;
-    }
+    return curses_color.cu_value.match(
+        [](const styling::semantic& s) -> std::optional<fmt::terminal_color> {
+            return std::nullopt;
+        },
+        [](const styling::transparent& s)
+            -> std::optional<fmt::terminal_color> { return std::nullopt; },
+        [](const palette_color& pc) -> std::optional<fmt::terminal_color> {
+            switch (pc) {
+                case COLOR_BLACK:
+                    return fmt::terminal_color::black;
+                case COLOR_RED:
+                    return fmt::terminal_color::red;
+                case COLOR_GREEN:
+                    return fmt::terminal_color::green;
+                case COLOR_YELLOW:
+                    return fmt::terminal_color::yellow;
+                case COLOR_BLUE:
+                    return fmt::terminal_color::blue;
+                case COLOR_MAGENTA:
+                    return fmt::terminal_color::magenta;
+                case COLOR_CYAN:
+                    return fmt::terminal_color::cyan;
+                case COLOR_WHITE:
+                    return fmt::terminal_color::white;
+                default:
+                    return std::nullopt;
+            }
+        },
+        [](const rgb_color& rgb) -> std::optional<fmt::terminal_color> {
+            switch (to_ansi_color(rgb)) {
+                case ansi_color::black:
+                    return fmt::terminal_color::black;
+                case ansi_color::cyan:
+                    return fmt::terminal_color::cyan;
+                case ansi_color::white:
+                    return fmt::terminal_color::white;
+                case ansi_color::magenta:
+                    return fmt::terminal_color::magenta;
+                case ansi_color::blue:
+                    return fmt::terminal_color::blue;
+                case ansi_color::yellow:
+                    return fmt::terminal_color::yellow;
+                case ansi_color::green:
+                    return fmt::terminal_color::green;
+                case ansi_color::red:
+                    return fmt::terminal_color::red;
+                default:
+                    return std::nullopt;
+            }
+        });
 }
 
 static bool
@@ -295,6 +350,170 @@ get_fd_tty(int fd)
     return isatty(fd);
 }
 
+static void
+set_rev(fmt::text_style& line_style)
+{
+    if (line_style.has_emphasis()
+        && lnav::enums::to_underlying(line_style.get_emphasis())
+            & lnav::enums::to_underlying(fmt::emphasis::reverse))
+    {
+        auto old_style = line_style;
+        auto old_emph = fmt::emphasis(
+            lnav::enums::to_underlying(old_style.get_emphasis())
+            & ~lnav::enums::to_underlying(fmt::emphasis::reverse));
+        line_style = fmt::text_style{};
+        if (old_style.has_foreground()) {
+            line_style |= fmt::fg(old_style.get_foreground());
+        }
+        if (old_style.has_background()) {
+            line_style |= fmt::bg(old_style.get_background());
+        }
+        line_style |= old_emph;
+    } else {
+        line_style |= fmt::emphasis::reverse;
+    }
+}
+
+static void
+role_to_style(const role_t role,
+              fmt::text_style& default_bg_style,
+              fmt::text_style& default_fg_style,
+              fmt::text_style& line_style)
+{
+    switch (role) {
+        case role_t::VCR_TEXT:
+        case role_t::VCR_IDENTIFIER:
+            break;
+        case role_t::VCR_ALT_ROW:
+            line_style |= fmt::emphasis::bold;
+            break;
+        case role_t::VCR_SEARCH:
+            set_rev(line_style);
+            break;
+        case role_t::VCR_ERROR:
+        case role_t::VCR_DIFF_DELETE:
+            line_style
+                |= fmt::fg(fmt::terminal_color::red) | fmt::emphasis::bold;
+            break;
+        case role_t::VCR_HIDDEN:
+        case role_t::VCR_WARNING:
+        case role_t::VCR_RE_REPEAT:
+            line_style |= fmt::fg(fmt::terminal_color::yellow);
+            break;
+        case role_t::VCR_COMMENT:
+        case role_t::VCR_DIFF_ADD:
+            line_style |= fmt::fg(fmt::terminal_color::green);
+            break;
+        case role_t::VCR_SNIPPET_BORDER:
+            line_style |= fmt::fg(fmt::terminal_color::cyan);
+            break;
+        case role_t::VCR_OK:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::green);
+            break;
+        case role_t::VCR_FOOTNOTE_BORDER:
+            line_style |= fmt::fg(fmt::terminal_color::blue);
+            break;
+        case role_t::VCR_INFO:
+        case role_t::VCR_STATUS:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::magenta);
+            break;
+        case role_t::VCR_KEYWORD:
+        case role_t::VCR_RE_SPECIAL:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::cyan);
+            break;
+        case role_t::VCR_STRING:
+            line_style |= fmt::fg(fmt::terminal_color::magenta);
+            break;
+        case role_t::VCR_VARIABLE:
+            line_style |= fmt::emphasis::underline;
+            break;
+        case role_t::VCR_SYMBOL:
+        case role_t::VCR_NUMBER:
+        case role_t::VCR_FILE:
+            line_style |= fmt::emphasis::bold;
+            break;
+        case role_t::VCR_H1:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::magenta);
+            break;
+        case role_t::VCR_H2:
+            line_style |= fmt::emphasis::bold;
+            break;
+        case role_t::VCR_H3:
+        case role_t::VCR_H4:
+        case role_t::VCR_H5:
+        case role_t::VCR_H6:
+            line_style |= fmt::emphasis::underline;
+            break;
+        case role_t::VCR_LIST_GLYPH:
+            line_style |= fmt::fg(fmt::terminal_color::yellow);
+            break;
+        case role_t::VCR_INLINE_CODE:
+        case role_t::VCR_QUOTED_CODE:
+            default_fg_style = fmt::fg(fmt::terminal_color::white);
+            default_bg_style = fmt::bg(fmt::terminal_color::black);
+            break;
+        case role_t::VCR_LOW_THRESHOLD:
+            line_style |= fmt::bg(fmt::terminal_color::green);
+            break;
+        case role_t::VCR_MED_THRESHOLD:
+            line_style |= fmt::bg(fmt::terminal_color::yellow);
+            break;
+        case role_t::VCR_HIGH_THRESHOLD:
+            line_style |= fmt::bg(fmt::terminal_color::red);
+            break;
+        default:
+            // log_debug("missing role handler %d", (int) role);
+            break;
+    }
+}
+
+static block_elem_t
+wchar_for_icon(ui_icon_t ic)
+{
+    switch (ic) {
+        case ui_icon_t::hidden:
+            return {L'\u22ee', role_t::VCR_HIDDEN};
+        case ui_icon_t::ok:
+            return {L'\u2714', role_t::VCR_OK};
+        case ui_icon_t::info:
+            return {L'\u24d8', role_t::VCR_INFO};
+        case ui_icon_t::warning:
+            return {L'\u26a0', role_t::VCR_WARNING};
+        case ui_icon_t::error:
+            return {L'\u2718', role_t::VCR_ERROR};
+
+        case ui_icon_t::log_level_trace:
+            return {L'\U0001F143', role_t::VCR_TEXT};
+        case ui_icon_t::log_level_debug:
+            return {L'\U0001F133', role_t::VCR_TEXT};
+        case ui_icon_t::log_level_info:
+            return {L'\U0001F138', role_t::VCR_TEXT};
+        case ui_icon_t::log_level_stats:
+            return {L'\U0001F142', role_t::VCR_TEXT};
+        case ui_icon_t::log_level_notice:
+            return {L'\U0001F13d', role_t::VCR_TEXT};
+        case ui_icon_t::log_level_warning:
+            return {L'\U0001F146', role_t::VCR_WARNING};
+        case ui_icon_t::log_level_error:
+            return {L'\U0001F134', role_t::VCR_ERROR};
+        case ui_icon_t::log_level_critical:
+            return {L'\U0001F132', role_t::VCR_ERROR};
+        case ui_icon_t::log_level_fatal:
+            return {L'\U0001F135', role_t::VCR_ERROR};
+
+        case ui_icon_t::play:
+            return {L'\u25b6', role_t::VCR_TEXT};
+        case ui_icon_t::edit:
+            return {L'\u270f', role_t::VCR_TEXT};
+    }
+
+    ensure(false);
+}
+
 void
 println(FILE* file, const attr_line_t& al)
 {
@@ -314,7 +533,7 @@ println(FILE* file, const attr_line_t& al)
         return;
     }
 
-    std::set<size_t> points = {0, static_cast<size_t>(al.length())};
+    auto points = std::set<size_t>{0, static_cast<size_t>(al.length())};
 
     for (const auto& attr : al.get_attrs()) {
         if (!attr.sa_range.is_valid()) {
@@ -326,7 +545,7 @@ println(FILE* file, const attr_line_t& al)
         }
     }
 
-    nonstd::optional<size_t> last_point;
+    std::optional<size_t> last_point;
     for (const auto& point : points) {
         if (!last_point) {
             last_point = point;
@@ -337,7 +556,8 @@ println(FILE* file, const attr_line_t& al)
         auto line_style = fmt::text_style{};
         auto fg_style = fmt::text_style{};
         auto start = last_point.value();
-        nonstd::optional<std::string> href;
+        std::optional<std::string> href;
+        auto replaced = false;
 
         for (const auto& attr : al.get_attrs()) {
             if (!attr.sa_range.contains(start)
@@ -347,19 +567,34 @@ println(FILE* file, const attr_line_t& al)
             }
 
             try {
-                if (attr.sa_type == &VC_HYPERLINK) {
+                if (attr.sa_type == &VC_ICON) {
+                    auto ic = attr.sa_value.get<ui_icon_t>();
+                    auto be = wchar_for_icon(ic);
+                    auto icon_fg_style = default_fg_style;
+                    auto icon_bg_style = default_bg_style;
+                    auto icon_style = line_style;
+                    std::string utf8_out;
+
+                    role_to_style(
+                        be.role, icon_bg_style, icon_fg_style, icon_style);
+                    ww898::utf::utf8::write(
+                        be.value,
+                        [&utf8_out](const char ch) { utf8_out.push_back(ch); });
+                    fmt::print(file, icon_style, FMT_STRING("{}"), utf8_out);
+                    replaced = true;
+                } else if (attr.sa_type == &VC_HYPERLINK) {
                     auto saw = string_attr_wrapper<std::string>(&attr);
                     href = saw.get();
                 } else if (attr.sa_type == &VC_BACKGROUND) {
-                    auto saw = string_attr_wrapper<int64_t>(&attr);
-                    auto color_opt = curses_color_to_terminal_color(saw.get());
+                    auto saw = string_attr_wrapper<styling::color_unit>(&attr);
+                    auto color_opt = color_to_terminal_color(saw.get());
 
                     if (color_opt) {
                         line_style |= fmt::bg(color_opt.value());
                     }
                 } else if (attr.sa_type == &VC_FOREGROUND) {
-                    auto saw = string_attr_wrapper<int64_t>(&attr);
-                    auto color_opt = curses_color_to_terminal_color(saw.get());
+                    auto saw = string_attr_wrapper<styling::color_unit>(&attr);
+                    auto color_opt = color_to_terminal_color(saw.get());
 
                     if (color_opt) {
                         fg_style = fmt::fg(color_opt.value());
@@ -368,26 +603,32 @@ println(FILE* file, const attr_line_t& al)
                     auto saw = string_attr_wrapper<text_attrs>(&attr);
                     auto style = saw.get();
 
-                    if (style.ta_attrs & A_REVERSE) {
-                        line_style |= fmt::emphasis::reverse;
+                    if (style.has_style(text_attrs::style::reverse)) {
+                        set_rev(line_style);
                     }
-                    if (style.ta_attrs & A_BOLD) {
+                    if (style.has_style(text_attrs::style::bold)) {
                         line_style |= fmt::emphasis::bold;
                     }
-                    if (style.ta_attrs & A_UNDERLINE) {
+                    if (style.has_style(text_attrs::style::underline)) {
                         line_style |= fmt::emphasis::underline;
                     }
-                    if (style.ta_fg_color) {
-                        auto color_opt = curses_color_to_terminal_color(
-                            style.ta_fg_color.value());
+                    if (style.has_style(text_attrs::style::italic)) {
+                        line_style |= fmt::emphasis::italic;
+                    }
+                    if (style.has_style(text_attrs::style::struck)) {
+                        line_style |= fmt::emphasis::strikethrough;
+                    }
+                    if (!style.ta_fg_color.empty()) {
+                        auto color_opt
+                            = color_to_terminal_color(style.ta_fg_color);
 
                         if (color_opt) {
                             fg_style = fmt::fg(color_opt.value());
                         }
                     }
-                    if (style.ta_bg_color) {
-                        auto color_opt = curses_color_to_terminal_color(
-                            style.ta_bg_color.value());
+                    if (!style.ta_bg_color.empty()) {
+                        auto color_opt
+                            = color_to_terminal_color(style.ta_bg_color);
 
                         if (color_opt) {
                             line_style |= fmt::bg(color_opt.value());
@@ -409,95 +650,14 @@ println(FILE* file, const attr_line_t& al)
                         default:
                             break;
                     }
-                } else if (attr.sa_type == &VC_ROLE) {
+                } else if (attr.sa_type == &VC_ROLE
+                           || attr.sa_type == &VC_ROLE_FG)
+                {
                     auto saw = string_attr_wrapper<role_t>(&attr);
                     auto role = saw.get();
 
-                    switch (role) {
-                        case role_t::VCR_TEXT:
-                        case role_t::VCR_IDENTIFIER:
-                            break;
-                        case role_t::VCR_ALT_ROW:
-                            line_style |= fmt::emphasis::bold;
-                            break;
-                        case role_t::VCR_SEARCH:
-                            line_style |= fmt::emphasis::reverse;
-                            break;
-                        case role_t::VCR_ERROR:
-                            line_style |= fmt::fg(fmt::terminal_color::red)
-                                | fmt::emphasis::bold;
-                            break;
-                        case role_t::VCR_WARNING:
-                        case role_t::VCR_RE_REPEAT:
-                            line_style |= fmt::fg(fmt::terminal_color::yellow);
-                            break;
-                        case role_t::VCR_COMMENT:
-                            line_style |= fmt::fg(fmt::terminal_color::green);
-                            break;
-                        case role_t::VCR_SNIPPET_BORDER:
-                            line_style |= fmt::fg(fmt::terminal_color::cyan);
-                            break;
-                        case role_t::VCR_OK:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::green);
-                            break;
-                        case role_t::VCR_INFO:
-                        case role_t::VCR_STATUS:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::magenta);
-                            break;
-                        case role_t::VCR_KEYWORD:
-                        case role_t::VCR_RE_SPECIAL:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::cyan);
-                            break;
-                        case role_t::VCR_STRING:
-                            line_style |= fmt::fg(fmt::terminal_color::magenta);
-                            break;
-                        case role_t::VCR_VARIABLE:
-                            line_style |= fmt::emphasis::underline;
-                            break;
-                        case role_t::VCR_SYMBOL:
-                        case role_t::VCR_NUMBER:
-                        case role_t::VCR_FILE:
-                            line_style |= fmt::emphasis::bold;
-                            break;
-                        case role_t::VCR_H1:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::magenta);
-                            break;
-                        case role_t::VCR_H2:
-                            line_style |= fmt::emphasis::bold;
-                            break;
-                        case role_t::VCR_H3:
-                        case role_t::VCR_H4:
-                        case role_t::VCR_H5:
-                        case role_t::VCR_H6:
-                            line_style |= fmt::emphasis::underline;
-                            break;
-                        case role_t::VCR_LIST_GLYPH:
-                            line_style |= fmt::fg(fmt::terminal_color::yellow);
-                            break;
-                        case role_t::VCR_INLINE_CODE:
-                        case role_t::VCR_QUOTED_CODE:
-                            default_fg_style
-                                = fmt::fg(fmt::terminal_color::white);
-                            default_bg_style
-                                = fmt::bg(fmt::terminal_color::black);
-                            break;
-                        case role_t::VCR_LOW_THRESHOLD:
-                            line_style |= fmt::bg(fmt::terminal_color::green);
-                            break;
-                        case role_t::VCR_MED_THRESHOLD:
-                            line_style |= fmt::bg(fmt::terminal_color::yellow);
-                            break;
-                        case role_t::VCR_HIGH_THRESHOLD:
-                            line_style |= fmt::bg(fmt::terminal_color::red);
-                            break;
-                        default:
-                            // log_debug("missing role handler %d", (int) role);
-                            break;
-                    }
+                    role_to_style(
+                        role, default_bg_style, default_fg_style, line_style);
                 }
             } catch (const fmt::format_error& e) {
                 log_error("style error: %s", e.what());
@@ -539,7 +699,7 @@ println(FILE* file, const attr_line_t& al)
         if (href) {
             fmt::print(file, FMT_STRING("\x1b]8;;{}\x1b\\"), href.value());
         }
-        if (start < str.size()) {
+        if (!replaced && start < str.size()) {
             auto actual_end = std::min(str.size(), static_cast<size_t>(point));
             auto sub = std::string{};
 
@@ -566,9 +726,19 @@ println(FILE* file, const attr_line_t& al)
                     case '\x07':
                         sub.append("\U0001F514");
                         break;
+                    case '\t':
+                    case '\n':
+                        sub.push_back(ch);
+                        break;
 
                     default:
-                        sub.append(&str[cp_start], lpc - cp_start);
+                        if (ch <= 0x1f) {
+                            sub.push_back(0xe2);
+                            sub.push_back(0x90);
+                            sub.push_back(0x80 + ch);
+                        } else {
+                            sub.append(&str[cp_start], lpc - cp_start);
+                        }
                         break;
                 }
             }
@@ -619,5 +789,4 @@ to_user_message(intern_string_t src, const lnav::pcre2pp::compile_error& ce)
         .with_snippet(lnav::console::snippet::from(src, pcre_error_content));
 }
 
-}  // namespace console
-}  // namespace lnav
+}  // namespace lnav::console
\ No newline at end of file
