@@ -31,12 +31,16 @@
 #include <iterator>
 #include <regex>
 #include <sstream>
+#include <string_view>
 
 #include "string_util.hh"
 
 #include "config.h"
 #include "is_utf8.hh"
 #include "lnav_log.hh"
+#include "scn/scan.h"
+
+using namespace std::string_view_literals;
 
 void
 scrub_to_utf8(char* buffer, size_t length)
@@ -123,7 +127,9 @@ unquote_content(char* dst, const char* str, size_t len, char quote_char)
 size_t
 unquote(char* dst, const char* str, size_t len)
 {
-    if (str[0] == 'r' || str[0] == 'u') {
+    if (str[0] == 'f' || str[0] == 'r' || str[0] == 'u' || str[0] == 'R'
+        || str[0] == 'x' || str[0] == 'X')
+    {
         str += 1;
         len -= 1;
     }
@@ -190,6 +196,23 @@ truncate_to(std::string& str, size_t max_char_len)
     str.insert(bytes_to_keep_at_front, ELLIPSIS);
 }
 
+ssize_t
+utf8_char_to_byte_index(const std::string& str, ssize_t ch_index)
+{
+    ssize_t retval = 0;
+
+    while (ch_index > 0) {
+        auto ch_len
+            = ww898::utf::utf8::char_size([&str, retval]() {
+                  return std::make_pair(str[retval], str.length() - retval - 1);
+              }).unwrapOr(1);
+
+        retval += ch_len;
+        ch_index -= 1;
+    }
+
+    return retval;
+}
 bool
 is_url(const std::string& fn)
 {
@@ -259,11 +282,20 @@ abbreviate_str(char* str, size_t len, size_t max_len)
 void
 split_ws(const std::string& str, std::vector<std::string>& toks_out)
 {
-    std::stringstream ss(str);
-    std::string buf;
+    auto str_sf = string_fragment::from_str(str);
 
-    while (ss >> buf) {
-        toks_out.push_back(buf);
+    while (true) {
+        auto split_pair = str_sf.split_when(isspace);
+        if (split_pair.first.empty()) {
+            if (split_pair.second.empty()) {
+                break;
+            }
+            str_sf = split_pair.second;
+            continue;
+        }
+
+        toks_out.emplace_back(split_pair.first.to_string());
+        str_sf = split_pair.second;
     }
 }
 
@@ -303,13 +335,17 @@ is_blank(const std::string& str)
 std::string
 scrub_ws(const char* in, ssize_t len)
 {
-    static const std::string TAB_SYMBOL = "\u21e5";
-    static const std::string LF_SYMBOL = "\u240a";
-    static const std::string CR_SYMBOL = "\u240d";
+    static constexpr auto TAB_SYMBOL = "\u21e5"sv;
+    static constexpr auto LF_SYMBOL = "\u240a"sv;
+    static constexpr auto CR_SYMBOL = "\u240d"sv;
 
     std::string retval;
 
-    for (size_t lpc = 0; (len == -1 && in[lpc]) || (len >= 0 && lpc < len);
+    if (len > 0) {
+        retval.reserve(len);
+    }
+
+    for (ssize_t lpc = 0; (len == -1 && in[lpc]) || (len >= 0 && lpc < len);
          lpc++)
     {
         auto ch = in[lpc];
@@ -325,7 +361,7 @@ scrub_ws(const char* in, ssize_t len)
                 retval.append(CR_SYMBOL);
                 break;
             default:
-                retval.append(1, ch);
+                retval.push_back(ch);
                 break;
         }
     }
@@ -333,6 +369,36 @@ scrub_ws(const char* in, ssize_t len)
     return retval;
 }
 
+static constexpr const char* const SUPERSCRIPT_NUMS[] = {
+    "⁰",
+    "¹",
+    "²",
+    "³",
+    "⁴",
+    "⁵",
+    "⁶",
+    "⁷",
+    "⁸",
+    "⁹",
+};
+
+std::string
+to_superscript(const std::string& in)
+{
+    std::string retval;
+    for (const auto ch : in) {
+        if (isdigit(ch)) {
+            auto index = ch - '0';
+
+            retval.append(SUPERSCRIPT_NUMS[index]);
+        } else {
+            retval.push_back(ch);
+        }
+    }
+
+    return retval;
+}
+
 namespace fmt {
 auto
 formatter<lnav::tainted_string>::format(const lnav::tainted_string& ts,
@@ -349,8 +415,7 @@ formatter<lnav::tainted_string>::format(const lnav::tainted_string& ts,
 }
 }  // namespace fmt
 
-namespace lnav {
-namespace pcre2pp {
+namespace lnav::pcre2pp {
 
 static bool
 is_meta(char ch)
@@ -375,7 +440,7 @@ is_meta(char ch)
     }
 }
 
-static nonstd::optional<const char*>
+static std::optional<const char*>
 char_escape_seq(char ch)
 {
     switch (ch) {
@@ -385,7 +450,7 @@ char_escape_seq(char ch)
             return "\\n";
     }
 
-    return nonstd::nullopt;
+    return std::nullopt;
 }
 
 std::string
@@ -420,5 +485,4 @@ quote(string_fragment str)
     return retval;
 }
 
-}  // namespace pcre2pp
-}  // namespace lnav
+}  // namespace lnav::pcre2pp
