@@ -34,6 +34,7 @@
 #include "base/itertools.hh"
 #include "bookmarks.json.hh"
 #include "config.h"
+#include "yajlpp/yajlpp_def.hh"
 
 std::unordered_set<std::string> bookmark_metadata::KNOWN_TAGS;
 
@@ -68,26 +69,30 @@ bookmark_metadata::empty(bookmark_metadata::categories props) const
 {
     switch (props) {
         case categories::any:
-            return this->bm_name.empty() && this->bm_comment.empty()
-                && this->bm_tags.empty()
+            return this->bm_name.empty() && this->bm_opid.empty()
+                && this->bm_comment.empty() && this->bm_tags.empty()
                 && this->bm_annotations.la_pairs.empty();
         case categories::partition:
             return this->bm_name.empty();
         case categories::notes:
             return this->bm_comment.empty() && this->bm_tags.empty()
                 && this->bm_annotations.la_pairs.empty();
+        case categories::opid:
+            return this->bm_opid.empty();
     }
+    ensure(false);
 }
 
 void
 bookmark_metadata::clear()
 {
+    this->bm_opid.clear();
     this->bm_comment.clear();
     this->bm_tags.clear();
     this->bm_annotations.la_pairs.clear();
 }
 
-nonstd::optional<bookmark_type_t*>
+std::optional<bookmark_type_t*>
 bookmark_type_t::find_type(const std::string& name)
 {
     return get_all_types()
@@ -103,3 +108,18 @@ bookmark_type_t::get_all_types()
 
     return all_types;
 }
+
+std::vector<const char*>
+bookmark_type_t::get_type_names()
+{
+    std::vector<const char*> retval;
+
+    for (const auto& bt : get_all_types()) {
+        retval.emplace_back(bt->get_name().data());
+    }
+    std::stable_sort(
+        retval.begin(), retval.end(), [](const char* lhs, const char* rhs) {
+            return strcmp(lhs, rhs) < 0;
+        });
+    return retval;
+}
