@@ -61,29 +61,27 @@ bottom_status_source::bottom_status_source()
 void
 bottom_status_source::update_line_number(listview_curses* lc)
 {
-    status_field& sf = this->bss_fields[BSF_LINE_NUMBER];
+    auto& sf = this->bss_fields[BSF_LINE_NUMBER];
+    auto sel = lc->get_selection();
 
     if (lc->get_inner_height() == 0) {
         sf.set_value(" L0");
+    } else if (sel) {
+        sf.set_value(" L%'d", (int) sel.value());
     } else {
-        sf.set_value(" L%'d", (int) lc->get_top());
+        sf.set_value(" L-");
     }
 
-    if (lc->get_inner_height() > 0) {
-        std::vector<attr_line_t> rows(1);
-
-        lc->get_data_source()->listview_value_for_rows(
-            *lc, lc->get_top(), rows);
-        const auto& sa = rows[0].get_attrs();
-        auto error_wrapper = get_string_attr(sa, SA_ERROR);
-        if (error_wrapper) {
-            this->bss_line_error.set_value(error_wrapper.value().get());
-        } else {
-            this->bss_line_error.clear();
-        }
-    } else {
-        this->bss_line_error.clear();
-    }
+    this->bss_line_error.set_value(
+        lc->map_top_row([](const attr_line_t& top_row)
+                            -> std::optional<std::string> {
+              const auto& sa = top_row.get_attrs();
+              auto error_wrapper = get_string_attr(sa, SA_ERROR);
+              if (error_wrapper) {
+                  return error_wrapper.value().get();
+              }
+              return std::nullopt;
+          }).value_or(""));
 }
 
 void
@@ -127,39 +125,44 @@ bottom_status_source::update_percent(listview_curses* lc)
     sf.set_value("%3d%% ", (int) percent);
 }
 
-void
+bool
 bottom_status_source::update_marks(listview_curses* lc)
 {
     auto* tc = static_cast<textview_curses*>(lc);
-    vis_bookmarks& bm = tc->get_bookmarks();
+    auto& bm = tc->get_bookmarks();
     status_field& sf = this->bss_fields[BSF_HITS];
+    auto retval = false;
 
     if (bm.find(&textview_curses::BM_SEARCH) != bm.end()) {
-        bookmark_vector<vis_line_t>& bv = bm[&textview_curses::BM_SEARCH];
+        const auto& bv = bm[&textview_curses::BM_SEARCH];
 
         if (!bv.empty() || !tc->get_current_search().empty()) {
-            bookmark_vector<vis_line_t>::iterator lb;
-
-            lb = std::lower_bound(bv.begin(), bv.end(), tc->get_top());
-            if (lb != bv.end() && *lb == tc->get_top()) {
-                sf.set_value("  Hit %'d of %'d for ",
-                             std::distance(bv.begin(), lb) + 1,
-                             tc->get_match_count());
-            } else {
-                sf.set_value("  %'d hits for ", tc->get_match_count());
+            auto vl = tc->get_selection();
+            if (vl) {
+                auto lb = bv.bv_tree.find(vl.value());
+                if (lb != bv.bv_tree.end()) {
+                    retval = sf.set_value("  Hit %'d of %'d for ",
+                                          (lb - bv.bv_tree.begin()) + 1,
+                                          tc->get_match_count());
+                } else {
+                    retval = sf.set_value("  %'d hits for ",
+                                          tc->get_match_count());
+                }
             }
         } else {
-            sf.clear();
+            retval = sf.clear();
         }
     } else {
-        sf.clear();
+        retval = sf.clear();
     }
+    return retval;
 }
 
-void
+bool
 bottom_status_source::update_hits(textview_curses* tc)
 {
-    status_field& sf = this->bss_fields[BSF_HITS];
+    auto& sf = this->bss_fields[BSF_HITS];
+    bool retval = false;
     role_t new_role;
 
     if (tc->is_searching()) {
@@ -169,23 +172,33 @@ bottom_status_source::update_hits(textview_curses* tc)
         } else {
             new_role = role_t::VCR_ACTIVE_STATUS2;
         }
-        sf.set_cylon(true);
+        if (!sf.is_cylon()) {
+            sf.set_cylon(true);
+            retval = true;
+        }
     } else {
         new_role = role_t::VCR_STATUS;
-        sf.set_cylon(false);
+        if (sf.is_cylon()) {
+            sf.set_cylon(false);
+            sf.clear();  // clear cylon style attribute
+            retval = true;
+        }
     }
     // this->bss_error.clear();
     sf.set_role(new_role);
-    this->update_marks(tc);
+    retval = this->update_marks(tc) || retval;
+    return retval;
 }
 
 void
-bottom_status_source::update_loading(file_off_t off, file_ssize_t total)
+bottom_status_source::update_loading(file_off_t off,
+                                     file_ssize_t total,
+                                     const char* term)
 {
     auto& sf = this->bss_fields[BSF_LOADING];
 
     require(off >= 0);
-    require(off <= total);
+    require_ge(total, off);
 
     if (total == 0) {
         sf.set_cylon(false);
@@ -218,7 +231,7 @@ bottom_status_source::update_loading(file_off_t off, file_ssize_t total)
 
             sf.set_cylon(true);
             sf.set_role(role_t::VCR_ACTIVE_STATUS2);
-            sf.set_value(" Loading %2d%% ", pct);
+            sf.set_value(" %s %2d%% ", term, pct);
         }
     }
 }
