@@ -29,7 +29,9 @@
 
 #include "breadcrumb_curses.hh"
 
+#include "base/itertools.enumerate.hh"
 #include "base/itertools.hh"
+#include "base/keycodes.hh"
 #include "itertools.similar.hh"
 
 using namespace lnav::roles::literals;
@@ -50,6 +52,7 @@ breadcrumb_curses::breadcrumb_curses()
     this->bc_match_view.set_height(0_vl);
     this->bc_match_view.set_show_scrollbar(true);
     this->bc_match_view.set_default_role(role_t::VCR_POPUP);
+    this->bc_match_view.set_head_space(0_vl);
     this->add_child_view(&this->bc_match_view);
 }
 
@@ -60,9 +63,12 @@ breadcrumb_curses::do_update()
         return false;
     }
 
-    size_t crumb_index = 0;
+    if (!this->vc_needs_update) {
+        return view_curses::do_update();
+    }
+
     size_t sel_crumb_offset = 0;
-    auto width = static_cast<size_t>(getmaxx(this->bc_window));
+    auto width = ncplane_dim_x(this->bc_window);
     auto crumbs = this->bc_focused_crumbs.empty() ? this->bc_line_source()
                                                   : this->bc_focused_crumbs;
     if (this->bc_last_selected_crumb
@@ -72,7 +78,8 @@ breadcrumb_curses::do_update()
     }
     this->bc_displayed_crumbs.clear();
     attr_line_t crumbs_line;
-    for (const auto& crumb : crumbs) {
+    for (const auto& [crumb_index, crumb] : lnav::itertools::enumerate(crumbs))
+    {
         auto accum_width = crumbs_line.column_width();
         auto elem_width = crumb.c_display_value.column_width();
         auto is_selected = this->bc_selected_crumb
@@ -91,21 +98,36 @@ breadcrumb_curses::do_update()
         if (is_selected) {
             sel_crumb_offset = accum_width;
             crumbs_line.get_attrs().emplace_back(
-                crumb_range, VC_STYLE.template value(text_attrs{A_REVERSE}));
+                crumb_range, VC_STYLE.value(text_attrs::with_reverse()));
         }
 
         this->bc_displayed_crumbs.emplace_back(
-            line_range{(int) accum_width,
-                       (int) (accum_width + elem_width),
-                       line_range::unit::codepoint},
+            line_range{
+                (int) accum_width,
+                (int) (accum_width + elem_width),
+                line_range::unit::codepoint,
+            },
             crumb_index);
-        crumb_index += 1;
         crumbs_line.append(" \uff1a"_breadcrumb);
     }
 
+    if (!this->vc_enabled) {
+        for (auto& attr : crumbs_line.al_attrs) {
+            if (attr.sa_type != &VC_ROLE) {
+                continue;
+            }
+
+            auto role = attr.sa_value.get<role_t>();
+            if (role == role_t::VCR_STATUS_TITLE) {
+                attr.sa_value = role_t::VCR_STATUS_DISABLED_TITLE;
+            }
+        }
+    }
+
     line_range lr{0, static_cast<int>(width)};
-    view_curses::mvwattrline(
-        this->bc_window, this->vc_y, 0, crumbs_line, lr, role_t::VCR_STATUS);
+    auto default_role = this->vc_enabled ? role_t::VCR_STATUS
+                                         : role_t::VCR_INACTIVE_STATUS;
+    mvwattrline(this->bc_window, this->vc_y, 0, crumbs_line, lr, default_role);
 
     if (this->bc_selected_crumb) {
         this->bc_match_view.set_x(sel_crumb_offset);
@@ -126,13 +148,16 @@ breadcrumb_curses::reload_data()
         = this->bc_focused_crumbs[this->bc_selected_crumb.value()];
     this->bc_possible_values = selected_crumb_ref.c_possibility_provider();
 
-    nonstd::optional<size_t> selected_value;
+    std::optional<size_t> selected_value;
     this->bc_similar_values = this->bc_possible_values
         | lnav::itertools::similar_to(
                                   [](const auto& elem) { return elem.p_key; },
                                   this->bc_current_search,
                                   128)
         | lnav::itertools::sort_with(breadcrumb::possibility::sort_cmp);
+    for (auto& al : this->bc_similar_values) {
+        al.p_display_value.highlight_fuzzy_matches(this->bc_current_search);
+    }
     if (selected_crumb_ref.c_key.is<std::string>()
         && selected_crumb_ref.c_expected_input
             != breadcrumb::crumb::expected_input_t::anything)
@@ -151,10 +176,12 @@ breadcrumb_curses::reload_data()
         }
     }
 
-    auto matches = attr_line_t().join(
-        this->bc_similar_values
-            | lnav::itertools::map(&breadcrumb::possibility::p_display_value),
-        "\n");
+    auto matches = attr_line_t()
+                       .join(this->bc_similar_values
+                                 | lnav::itertools::map(
+                                     &breadcrumb::possibility::p_display_value),
+                             "\n")
+                       .move();
     this->bc_match_source.replace_with(matches);
     auto width = this->bc_possible_values
         | lnav::itertools::fold(
@@ -210,7 +237,7 @@ breadcrumb_curses::blur()
 {
     this->bc_last_selected_crumb = this->bc_selected_crumb;
     this->bc_focused_crumbs.clear();
-    this->bc_selected_crumb = nonstd::nullopt;
+    this->bc_selected_crumb = std::nullopt;
     this->bc_current_search.clear();
     this->bc_match_view.set_height(0_vl);
     this->bc_match_view.set_selection(-1_vl);
@@ -218,12 +245,40 @@ breadcrumb_curses::blur()
     this->set_needs_update();
 }
 
+void
+breadcrumb_curses::focus_next()
+{
+    this->blur();
+    this->focus();
+    this->reload_data();
+    if (this->bc_selected_crumb.value() < this->bc_focused_crumbs.size() - 1) {
+        this->bc_selected_crumb = this->bc_selected_crumb.value() + 1;
+    }
+    this->bc_current_search.clear();
+    this->reload_data();
+}
+
 bool
-breadcrumb_curses::handle_key(int ch)
+breadcrumb_curses::handle_key(const ncinput& ch)
 {
     bool retval = false;
+    auto mapped_id = ch.id;
 
-    switch (ch) {
+    if (mapped_id == NCKEY_TAB && ncinput_shift_p(&ch)) {
+        mapped_id = NCKEY_LEFT;
+    } else if (ncinput_ctrl_p(&ch)) {
+        switch (mapped_id) {
+            case 'a':
+            case 'A':
+                mapped_id = KEY_CTRL('a');
+                break;
+            case 'e':
+            case 'E':
+                mapped_id = KEY_CTRL('e');
+                break;
+        }
+    }
+    switch (mapped_id) {
         case KEY_CTRL('a'):
             if (this->bc_selected_crumb) {
                 this->bc_selected_crumb = 0;
@@ -240,8 +295,7 @@ breadcrumb_curses::handle_key(int ch)
             }
             retval = true;
             break;
-        case KEY_BTAB:
-        case KEY_LEFT:
+        case NCKEY_LEFT:
             if (this->bc_selected_crumb) {
                 if (this->bc_selected_crumb.value() > 0) {
                     this->bc_selected_crumb
@@ -255,72 +309,62 @@ breadcrumb_curses::handle_key(int ch)
             }
             retval = true;
             break;
-        case '\t':
-        case KEY_RIGHT:
+        case NCKEY_TAB:
+        case NCKEY_RIGHT:
             if (this->bc_selected_crumb) {
-                this->perform_selection(perform_behavior_t::if_different);
-                this->blur();
-                this->focus();
-                this->reload_data();
-                if (this->bc_selected_crumb.value()
-                    < this->bc_focused_crumbs.size() - 1)
+                if (!this->perform_selection(perform_behavior_t::if_different))
                 {
-                    this->bc_selected_crumb
-                        = this->bc_selected_crumb.value() + 1;
-                    retval = true;
+                    this->focus_next();
                 }
-                this->bc_current_search.clear();
-                this->reload_data();
-            } else {
-                retval = true;
             }
+            retval = true;
             break;
-        case KEY_HOME:
+        case NCKEY_HOME:
             this->bc_match_view.set_selection(0_vl);
             retval = true;
             break;
-        case KEY_END:
+        case NCKEY_END:
             this->bc_match_view.set_selection(
                 this->bc_match_view.get_inner_height() - 1_vl);
             retval = true;
             break;
-        case KEY_NPAGE:
+        case NCKEY_PGDOWN:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::down_page);
             retval = true;
             break;
-        case KEY_PPAGE:
+        case NCKEY_PGUP:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::up_page);
             retval = true;
             break;
-        case KEY_UP:
+        case NCKEY_UP:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::up_line);
             retval = true;
             break;
-        case KEY_DOWN:
+        case NCKEY_DOWN:
             this->bc_match_view.shift_selection(
                 listview_curses::shift_amount_t::down_line);
             retval = true;
             break;
         case KEY_DELETE:
-        case KEY_BACKSPACE:
+        case NCKEY_BACKSPACE:
             if (!this->bc_current_search.empty()) {
                 this->bc_current_search.pop_back();
                 this->reload_data();
             }
             retval = true;
             break;
-        case KEY_ENTER:
+        case NCKEY_ENTER:
         case '\r':
             this->perform_selection(perform_behavior_t::always);
             break;
         case KEY_ESCAPE:
             break;
         default:
-            if (isprint(ch)) {
-                this->bc_current_search.push_back(ch);
+            if (ch.id < 0x7f && isprint(ch.id)) {
+                this->bc_current_search.push_back(ch.id);
                 this->reload_data();
                 retval = true;
             }
@@ -334,34 +378,41 @@ breadcrumb_curses::handle_key(int ch)
     return retval;
 }
 
-void
-breadcrumb_curses::perform_selection(
-    breadcrumb_curses::perform_behavior_t behavior)
+bool
+breadcrumb_curses::perform_selection(perform_behavior_t behavior)
 {
+    auto retval = false;
+
     if (!this->bc_selected_crumb) {
-        return;
+        return retval;
     }
 
     auto& selected_crumb_ref
         = this->bc_focused_crumbs[this->bc_selected_crumb.value()];
     auto match_sel = this->bc_match_view.get_selection();
-    if (match_sel >= 0
-        && match_sel < vis_line_t(this->bc_similar_values.size()))
+    if (match_sel.has_value()
+        && match_sel.value() < vis_line_t(this->bc_similar_values.size()))
     {
-        const auto& new_value = this->bc_similar_values[match_sel].p_key;
+        const auto& new_value
+            = this->bc_similar_values[match_sel.value()].p_key;
 
         switch (behavior) {
             case perform_behavior_t::if_different:
                 if (breadcrumb::crumb::key_t{new_value}
                     == selected_crumb_ref.c_key)
                 {
-                    return;
+                    return retval;
                 }
                 break;
             case perform_behavior_t::always:
                 break;
         }
-        selected_crumb_ref.c_performer(new_value);
+        if (this->bc_perform_handler) {
+            this->bc_perform_handler(
+                *this, selected_crumb_ref.c_performer, new_value);
+        }
+
+        retval = true;
     } else if (!this->bc_current_search.empty()) {
         switch (selected_crumb_ref.c_expected_input) {
             case breadcrumb::crumb::expected_input_t::exact:
@@ -377,10 +428,17 @@ breadcrumb_curses::perform_selection(
                 break;
             }
             case breadcrumb::crumb::expected_input_t::anything:
-                selected_crumb_ref.c_performer(this->bc_current_search);
+                if (this->bc_perform_handler) {
+                    this->bc_perform_handler(*this,
+                                             selected_crumb_ref.c_performer,
+                                             this->bc_current_search);
+                }
                 break;
         }
+        retval = true;
     }
+
+    return retval;
 }
 
 bool
@@ -398,7 +456,7 @@ breadcrumb_curses::search_overlay_source::list_static_overlay(
         | lnav::itertools::unwrap_or(
                          breadcrumb::crumb::expected_input_t::exact);
 
-    value_out.with_attr_for_all(VC_STYLE.value(text_attrs{A_UNDERLINE}));
+    value_out.with_attr_for_all(VC_STYLE.value(text_attrs::with_underline()));
 
     if (!parent->bc_current_search.empty()) {
         value_out = parent->bc_current_search;
@@ -477,6 +535,7 @@ breadcrumb_curses::handle_mouse(mouse_event& me)
         this->focus();
         this->on_focus(*this);
         this->do_update();
+        this->bc_initial_mouse_event = true;
     }
 
     auto find_res = this->bc_displayed_crumbs
@@ -500,7 +559,6 @@ breadcrumb_curses::handle_mouse(mouse_event& me)
                     this->blur();
                     this->reload_data();
                     this->on_blur(*this);
-                    this->bc_initial_mouse_event = true;
                 }
                 return true;
             }
@@ -513,7 +571,6 @@ breadcrumb_curses::handle_mouse(mouse_event& me)
             this->blur();
             this->reload_data();
             this->on_blur(*this);
-            this->bc_initial_mouse_event = true;
             return true;
         }
     }
