@@ -39,10 +39,9 @@
 #include "base/opt_util.hh"
 #include "data_scanner.hh"
 
-namespace lnav {
-namespace document {
+namespace lnav::document {
 
-nonstd::optional<hier_node*>
+std::optional<hier_node*>
 hier_node::lookup_child(section_key_t key) const
 {
     return make_optional_from_nullable(key.match(
@@ -62,7 +61,7 @@ hier_node::lookup_child(section_key_t key) const
         }));
 }
 
-nonstd::optional<size_t>
+std::optional<size_t>
 hier_node::child_index(const hier_node* hn) const
 {
     size_t retval = 0;
@@ -74,19 +73,35 @@ hier_node::child_index(const hier_node* hn) const
         retval += 1;
     }
 
-    return nonstd::nullopt;
+    return std::nullopt;
 }
 
-nonstd::optional<hier_node::child_neighbors_result>
-hier_node::child_neighbors(const lnav::document::hier_node* hn,
-                           file_off_t offset) const
+std::optional<section_key_t>
+hier_node::child_key(const hier_node* hn) const
+{
+    for (const auto& named_pair : this->hn_named_children) {
+        if (named_pair.second == hn) {
+            return named_pair.first;
+        }
+    }
+
+    auto index_opt = this->child_index(hn);
+    if (index_opt) {
+        return section_key_t{index_opt.value()};
+    }
+
+    return std::nullopt;
+}
+
+std::optional<hier_node::child_neighbors_result>
+hier_node::child_neighbors(const hier_node* hn, file_off_t offset) const
 {
     auto index_opt = this->child_index(hn);
     if (!index_opt) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
-    hier_node::child_neighbors_result retval;
+    child_neighbors_result retval;
 
     if (index_opt.value() == 0) {
         if (this->hn_parent != nullptr) {
@@ -153,14 +168,14 @@ hier_node::child_neighbors(const lnav::document::hier_node* hn,
     return retval;
 }
 
-nonstd::optional<hier_node::child_neighbors_result>
+std::optional<hier_node::child_neighbors_result>
 hier_node::line_neighbors(size_t ln) const
 {
     if (this->hn_children.empty()) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
-    hier_node::child_neighbors_result retval;
+    child_neighbors_result retval;
 
     for (const auto& child : this->hn_children) {
         if (child->hn_line_number > ln) {
@@ -173,7 +188,7 @@ hier_node::line_neighbors(size_t ln) const
     return retval;
 }
 
-nonstd::optional<const hier_node*>
+std::optional<const hier_node*>
 hier_node::lookup_path(const hier_node* root,
                        const std::vector<section_key_t>& path)
 {
@@ -188,7 +203,7 @@ hier_node::lookup_path(const hier_node* root,
     }
 
     if (!retval) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     return retval;
@@ -212,6 +227,7 @@ struct metadata_builder {
     std::unique_ptr<hier_node> mb_root_node;
     std::set<size_t> mb_indents;
     text_format_t mb_text_format{text_format_t::TF_UNKNOWN};
+    std::set<std::string> mb_words;
 
     metadata to_metadata() &&
     {
@@ -221,6 +237,7 @@ struct metadata_builder {
             std::move(this->mb_type_intervals),
             std::move(this->mb_indents),
             this->mb_text_format,
+            this->mb_words,
         };
     }
 };
@@ -229,25 +246,25 @@ static void
 discover_metadata_int(const attr_line_t& al, metadata_builder& mb)
 {
     const auto& orig_attrs = al.get_attrs();
-    auto headers = orig_attrs
-        | lnav::itertools::filter_in([](const string_attr& attr) {
-                       if (attr.sa_type != &VC_ROLE) {
-                           return false;
-                       }
+    auto headers
+        = orig_attrs | lnav::itertools::filter_in([](const string_attr& attr) {
+              if (attr.sa_type != &VC_ROLE || !attr.sa_range.is_valid()) {
+                  return false;
+              }
 
-                       auto role = attr.sa_value.get<role_t>();
-                       switch (role) {
-                           case role_t::VCR_H1:
-                           case role_t::VCR_H2:
-                           case role_t::VCR_H3:
-                           case role_t::VCR_H4:
-                           case role_t::VCR_H5:
-                           case role_t::VCR_H6:
-                               return true;
-                           default:
-                               return false;
-                       }
-                   })
+              const auto role = attr.sa_value.get<role_t>();
+              switch (role) {
+                  case role_t::VCR_H1:
+                  case role_t::VCR_H2:
+                  case role_t::VCR_H3:
+                  case role_t::VCR_H4:
+                  case role_t::VCR_H5:
+                  case role_t::VCR_H6:
+                      return true;
+                  default:
+                      return false;
+              }
+          })
         | lnav::itertools::sort_by(&string_attr::sa_range);
 
     // Remove headers from quoted text
@@ -274,9 +291,10 @@ discover_metadata_int(const attr_line_t& al, metadata_builder& mb)
     };
     std::vector<open_interval_t> open_intervals;
     auto root_node = std::make_unique<hier_node>();
+    const auto sf = string_fragment::from_str(al.get_string());
 
     for (const auto& hdr_attr : headers) {
-        auto role = hdr_attr.sa_value.get<role_t>();
+        const auto role = hdr_attr.sa_value.get<role_t>();
         auto role_num = lnav::enums::to_underlying(role)
             - lnav::enums::to_underlying(role_t::VCR_H1);
         std::vector<open_interval_t> new_open_intervals;
@@ -284,8 +302,12 @@ discover_metadata_int(const attr_line_t& al, metadata_builder& mb)
         for (auto& oi : open_intervals) {
             if (oi.oi_level >= role_num) {
                 // close out this section
-                intervals.emplace_back(
-                    oi.oi_start, hdr_attr.sa_range.lr_start - 1, oi.oi_id);
+                auto left_sf = sf.find_left_boundary(
+                    hdr_attr.sa_range.lr_start, string_fragment::tag1{'\n'});
+                if (left_sf.sf_begin > 0) {
+                    left_sf.sf_begin -= 1;
+                }
+                intervals.emplace_back(oi.oi_start, left_sf.sf_begin, oi.oi_id);
                 auto* node_ptr = oi.oi_node.get();
                 auto* parent_node = oi.oi_node->hn_parent;
                 if (parent_node != nullptr) {
@@ -304,13 +326,14 @@ discover_metadata_int(const attr_line_t& al, metadata_builder& mb)
             auto* parent_node = new_open_intervals.empty()
                 ? root_node.get()
                 : new_open_intervals.back().oi_node.get();
+            auto left_sf = sf.find_left_boundary(hdr_attr.sa_range.lr_start,
+                                                 string_fragment::tag1{'\n'});
             new_open_intervals.emplace_back(
                 role_num,
-                hdr_attr.sa_range.lr_start,
+                left_sf.sf_begin,
                 al.get_substring(hdr_attr.sa_range));
             new_open_intervals.back().oi_node->hn_parent = parent_node;
-            new_open_intervals.back().oi_node->hn_start
-                = hdr_attr.sa_range.lr_start;
+            new_open_intervals.back().oi_node->hn_start = left_sf.sf_begin;
         }
         open_intervals = std::move(new_open_intervals);
     }
@@ -394,10 +417,11 @@ discover_metadata(const attr_line_t& al)
 
 class structure_walker {
 public:
-    explicit structure_walker(attr_line_t& al, line_range lr, text_format_t tf)
-        : sw_line(al), sw_range(lr), sw_text_format(tf),
-          sw_scanner(string_fragment::from_str_range(
-              al.get_string(), lr.lr_start, lr.lr_end))
+    explicit structure_walker(discover_builder& db)
+        : sw_discover_builder(db), sw_line(db.db_line),
+          sw_scanner(string_fragment::from_str_range(db.db_line.get_string(),
+                                                     db.db_range.lr_start,
+                                                     db.db_range.lr_end))
     {
         this->sw_interval_state.resize(1);
         this->sw_hier_nodes.push_back(std::make_unique<hier_node>());
@@ -405,7 +429,7 @@ public:
 
     bool is_structured_text() const
     {
-        switch (this->sw_text_format) {
+        switch (this->sw_discover_builder.db_text_format) {
             case text_format_t::TF_JSON:
             case text_format_t::TF_YAML:
             case text_format_t::TF_TOML:
@@ -421,10 +445,12 @@ public:
     {
         metadata_builder mb;
 
-        mb.mb_text_format = this->sw_text_format;
+        mb.mb_text_format = this->sw_discover_builder.db_text_format;
         while (true) {
-            auto tokenize_res
-                = this->sw_scanner.tokenize2(this->sw_text_format);
+            require(this->sw_depth == this->sw_container_tokens.size());
+
+            auto tokenize_res = this->sw_scanner.tokenize2(
+                this->sw_discover_builder.db_text_format);
             if (!tokenize_res) {
                 break;
             }
@@ -454,8 +480,8 @@ public:
                         section_types_t::comment);
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start + el.e_capture.c_begin,
-                            this->sw_range.lr_start + el.e_capture.c_end,
+                            el.e_capture.c_begin,
+                            el.e_capture.c_end,
                         },
                         VC_ROLE.value(role_t::VCR_COMMENT));
                     break;
@@ -476,7 +502,9 @@ public:
                     break;
                 case DT_XML_CLOSE_TAG: {
                     auto term = this->flush_values();
-                    if (this->sw_depth > 0) {
+                    if (this->sw_depth > 0
+                        && !this->sw_container_tokens.empty())
+                    {
                         auto found = false;
                         do {
                             if (this->sw_container_tokens.back() == dt) {
@@ -484,14 +512,17 @@ public:
                             }
                             if (term) {
                                 this->append_child_node(term);
-                                term = nonstd::nullopt;
+                                term = std::nullopt;
                             }
                             this->sw_interval_state.pop_back();
+                            if (!found && this->sw_depth > 0) {
+                                this->sw_depth -= 1;
+                            }
                             this->sw_hier_stage
                                 = std::move(this->sw_hier_nodes.back());
                             this->sw_hier_nodes.pop_back();
                             this->sw_container_tokens.pop_back();
-                        } while (!found);
+                        } while (!found && !this->sw_container_tokens.empty());
                     }
                     this->append_child_node(el.e_capture);
                     if (this->sw_depth > 0) {
@@ -503,14 +534,16 @@ public:
                 case DT_H1: {
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start + inner_cap.c_begin,
-                            this->sw_range.lr_start + inner_cap.c_end,
+                            inner_cap.c_begin,
+                            inner_cap.c_end,
                         },
                         VC_ROLE.value(role_t::VCR_H1));
                     this->sw_line_number += 1;
                     break;
                 }
                 case DT_DIFF_FILE_HEADER: {
+                    this->drop_open_children();
+
                     auto sf = this->sw_scanner.to_string_fragment(inner_cap);
                     auto split_res = sf.split_pair(string_fragment::tag1{'\n'});
                     auto file1 = split_res->first.consume_n(4).value();
@@ -525,24 +558,22 @@ public:
                     }
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
                         },
                         VC_ROLE.value(role_t::VCR_H1));
                     if (file1 == "/dev/null" || file1 == file2) {
                         this->sw_line.get_attrs().emplace_back(
                             line_range{
-                                this->sw_range.lr_start + file2.sf_begin,
-                                this->sw_range.lr_start + file2.sf_end,
+                                file2.sf_begin,
+                                file2.sf_end,
                             },
                             VC_ROLE.value(role_t::VCR_H1));
                     } else {
                         this->sw_line.get_attrs().emplace_back(
                             line_range{
-                                this->sw_range.lr_start + inner_cap.c_begin,
-                                this->sw_range.lr_start + inner_cap.c_end,
+                                inner_cap.c_begin,
+                                inner_cap.c_end,
                             },
                             VC_ROLE.value(role_t::VCR_H1));
                     }
@@ -550,18 +581,17 @@ public:
                     break;
                 }
                 case DT_DIFF_HUNK_HEADING: {
+                    this->drop_open_children();
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
-                            this->sw_range.lr_start
-                                + tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
+                            tokenize_res->tr_capture.c_begin,
                         },
                         VC_ROLE.value(role_t::VCR_H2));
                     this->sw_line.get_attrs().emplace_back(
                         line_range{
-                            this->sw_range.lr_start + inner_cap.c_begin,
-                            this->sw_range.lr_start + inner_cap.c_end,
+                            inner_cap.c_begin,
+                            inner_cap.c_end,
                         },
                         VC_ROLE.value(role_t::VCR_H2));
                     this->sw_line_number += 1;
@@ -605,7 +635,7 @@ public:
                                     found = true;
                                 }
                                 this->append_child_node(term);
-                                term = nonstd::nullopt;
+                                term = std::nullopt;
                                 this->sw_depth -= 1;
                                 this->sw_interval_state.pop_back();
                                 this->sw_hier_stage
@@ -675,8 +705,8 @@ public:
                     break;
                 case DT_ZERO_WIDTH_SPACE:
                     break;
-                default:
-                    if (dt == DT_QUOTED_STRING) {
+                default: {
+                    if (dt == DT_QUOTED_STRING || dt == DT_CODE_BLOCK) {
                         auto quoted_sf = tokenize_res->to_string_fragment();
 
                         if (quoted_sf.find('\n')) {
@@ -686,17 +716,25 @@ public:
                                 section_types_t::multiline_string);
                             this->sw_line.get_attrs().emplace_back(
                                 line_range{
-                                    this->sw_range.lr_start
-                                        + el.e_capture.c_begin,
-                                    this->sw_range.lr_start
-                                        + el.e_capture.c_end,
+                                    el.e_capture.c_begin,
+                                    el.e_capture.c_end,
                                 },
                                 VC_ROLE.value(role_t::VCR_STRING));
                         }
                     }
+                    if (this->sw_discover_builder.db_save_words
+                        && (dt == DT_CONSTANT || dt == DT_ID || dt == DT_WORD
+                            || dt == DT_SYMBOL))
+                    {
+                        mb.mb_words.insert(
+                            tokenize_res->to_string_fragment().to_string());
+                    }
                     this->sw_values.emplace_back(el);
                     break;
+                }
             }
+
+            ensure(this->sw_depth == this->sw_container_tokens.size());
         }
         this->flush_values();
 
@@ -758,15 +796,25 @@ private:
     };
 
     struct interval_state {
-        nonstd::optional<file_off_t> is_start;
+        std::optional<file_off_t> is_start;
         size_t is_line_number{0};
         std::string is_name;
     };
 
-    nonstd::optional<data_scanner::capture_t> flush_values()
+    void drop_open_children()
     {
-        nonstd::optional<data_scanner::capture_t> last_key;
-        nonstd::optional<data_scanner::capture_t> retval;
+        while (this->sw_depth > 0) {
+            this->sw_depth -= 1;
+            this->sw_interval_state.pop_back();
+            this->sw_hier_nodes.pop_back();
+            this->sw_container_tokens.pop_back();
+        }
+    }
+
+    std::optional<data_scanner::capture_t> flush_values()
+    {
+        std::optional<data_scanner::capture_t> last_key;
+        std::optional<data_scanner::capture_t> retval;
 
         if (!this->sw_values.empty()) {
             if (!this->sw_interval_state.back().is_start) {
@@ -799,7 +847,7 @@ private:
                             this->sw_interval_state.back().is_line_number
                                 = this->sw_line_number;
                         }
-                        last_key = nonstd::nullopt;
+                        last_key = std::nullopt;
                     }
                     break;
                 default:
@@ -812,11 +860,11 @@ private:
         return retval;
     }
 
-    void append_child_node(nonstd::optional<data_scanner::capture_t> terminator)
+    void append_child_node(std::optional<data_scanner::capture_t> terminator)
     {
         auto& ivstate = this->sw_interval_state.back();
         if (!ivstate.is_start || !terminator || this->sw_depth == 0) {
-            ivstate.is_start = nonstd::nullopt;
+            ivstate.is_start = std::nullopt;
             ivstate.is_line_number = 0;
             ivstate.is_name.clear();
             return;
@@ -847,14 +895,13 @@ private:
             }
             top_node->hn_children.emplace_back(std::move(new_node));
         }
-        ivstate.is_start = nonstd::nullopt;
+        ivstate.is_start = std::nullopt;
         ivstate.is_line_number = 0;
         ivstate.is_name.clear();
     }
 
+    discover_builder& sw_discover_builder;
     attr_line_t& sw_line;
-    line_range sw_range;
-    text_format_t sw_text_format;
     data_scanner sw_scanner;
     int sw_depth{0};
     size_t sw_line_number{0};
@@ -863,44 +910,42 @@ private:
     std::vector<element> sw_values{};
     std::vector<data_token_t> sw_container_tokens;
     std::vector<interval_state> sw_interval_state;
-    std::vector<lnav::document::section_interval_t> sw_intervals;
-    std::vector<lnav::document::section_type_interval_t> sw_type_intervals;
-    std::vector<std::unique_ptr<lnav::document::hier_node>> sw_hier_nodes;
-    std::unique_ptr<lnav::document::hier_node> sw_hier_stage;
+    std::vector<section_interval_t> sw_intervals;
+    std::vector<section_type_interval_t> sw_type_intervals;
+    std::vector<std::unique_ptr<hier_node>> sw_hier_nodes;
+    std::unique_ptr<hier_node> sw_hier_stage;
 };
 
 metadata
-discover_structure(attr_line_t& al, struct line_range lr, text_format_t tf)
+discover_builder::perform()
 {
-    return structure_walker(al, lr, tf).walk();
+    return structure_walker(*this).walk();
 }
 
 std::vector<breadcrumb::possibility>
 metadata::possibility_provider(const std::vector<section_key_t>& path)
 {
     std::vector<breadcrumb::possibility> retval;
-    auto curr_node = lnav::document::hier_node::lookup_path(
-        this->m_sections_root.get(), path);
+    auto curr_node = hier_node::lookup_path(this->m_sections_root.get(), path);
     if (curr_node) {
         auto* parent_node = curr_node.value()->hn_parent;
 
         if (parent_node != nullptr) {
             for (const auto& sibling : parent_node->hn_named_children) {
-                retval.template emplace_back(sibling.first);
+                retval.emplace_back(sibling.first);
             }
         }
     }
     return retval;
 }
 
-}  // namespace document
-}  // namespace lnav
+}  // namespace lnav::document
 
 namespace fmt {
 auto
 formatter<lnav::document::section_key_t>::format(
-    const lnav::document::section_key_t& key,
-    fmt::format_context& ctx) -> decltype(ctx.out()) const
+    const lnav::document::section_key_t& key, fmt::format_context& ctx)
+    -> decltype(ctx.out()) const
 {
     return key.match(
         [this, &ctx](const std::string& str) {
