@@ -37,8 +37,10 @@
 #include "log_format_ext.hh"
 #include "log_vtab_impl.hh"
 #include "md2attr_line.hh"
+#include "msg.text.hh"
 #include "ptimec.hh"
 #include "readline_highlighters.hh"
+#include "sql_util.hh"
 #include "vtab_module.hh"
 #include "vtab_module_json.hh"
 
@@ -63,8 +65,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         return;
     }
 
-    content_line_t cl = lss.at(row);
-    std::shared_ptr<logfile> file = lss.find(cl);
+    auto cl = lss.at(row);
+    auto file = lss.find(cl);
     auto ll = file->begin() + cl;
     auto format = file->get_format();
     bool display = false;
@@ -95,33 +97,42 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             auto emsg = fmt::format(
                 FMT_STRING("   Invalid log message: {}"),
                 sattr.sa_value.get<decltype(SA_INVALID)::value_type>());
-            auto al = attr_line_t(emsg)
-                          .with_attr(string_attr(
-                              line_range{1, 2}, VC_GRAPHIC.value(ACS_LLCORNER)))
-                          .with_attr(string_attr(
-                              line_range{0, 22},
-                              VC_ROLE.value(role_t::VCR_INVALID_MSG)));
+            auto al
+                = attr_line_t(emsg)
+                      .with_attr(string_attr(line_range{1, 2},
+                                             VC_GRAPHIC.value(NCACS_LLCORNER)))
+                      .with_attr(
+                          string_attr(line_range{0, 22},
+                                      VC_ROLE.value(role_t::VCR_INVALID_MSG)))
+                      .move();
             this->fos_lines.emplace_back(al);
         }
     }
 
     char old_timestamp[64], curr_timestamp[64], orig_timestamp[64];
-    struct timeval curr_tv, offset_tv, orig_tv, diff_tv = {0, 0};
+    timeval curr_tv, offset_tv, orig_tv, diff_tv = {0, 0};
     attr_line_t time_line;
     auto& time_str = time_line.get_string();
-    struct line_range time_lr;
+    line_range time_lr;
+    off_t ts_len = sql_strftime(
+        curr_timestamp, sizeof(curr_timestamp), ll->get_timeval(), 'T');
+    {
+        exttm tmptm;
 
-    sql_strftime(curr_timestamp,
-                 sizeof(curr_timestamp),
-                 ll->get_time(),
-                 ll->get_millis(),
-                 'T');
+        tmptm.et_flags |= ETF_ZONE_SET;
+        tmptm.et_gmtoff
+            = lnav::local_time_to_info(
+                  date::local_seconds{ll->get_time<std::chrono::seconds>()})
+                  .first.offset.count();
+        ftime_z(curr_timestamp, ts_len, sizeof(curr_timestamp), tmptm);
+        curr_timestamp[ts_len] = '\0';
+    }
 
     if (ll->is_time_skewed()) {
         time_lr.lr_start = 1;
         time_lr.lr_end = 2;
         time_line.with_attr(
-            string_attr(time_lr, VC_GRAPHIC.value(ACS_LLCORNER)));
+            string_attr(time_lr, VC_GRAPHIC.value(NCACS_LLCORNER)));
         time_str.append("   Out-Of-Time-Order Message");
         time_lr.lr_start = 3;
         time_lr.lr_end = time_str.length();
@@ -135,7 +146,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
     time_str.append(curr_timestamp);
     time_lr.lr_end = time_str.length();
     time_line.with_attr(
-        string_attr(time_lr, VC_STYLE.value(text_attrs{A_BOLD})));
+        string_attr(time_lr, VC_STYLE.value(text_attrs::with_bold())));
     time_str.append(" \u2014 ");
     time_lr.lr_start = time_str.length();
     time_str.append(humanize::time::point::from_tv(ll->get_timeval())
@@ -143,10 +154,10 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
                         .as_precise_time_ago());
     time_lr.lr_end = time_str.length();
     time_line.with_attr(
-        string_attr(time_lr, VC_STYLE.value(text_attrs{A_BOLD})));
+        string_attr(time_lr, VC_STYLE.value(text_attrs::with_bold())));
 
-    struct line_range time_range = find_string_attr_range(
-        this->fos_log_helper.ldh_line_attrs, &logline::L_TIMESTAMP);
+    auto time_range = find_string_attr_range(
+        this->fos_log_helper.ldh_line_attrs, &L_TIMESTAMP);
 
     curr_tv = this->fos_log_helper.ldh_line->get_timeval();
     if (ll->is_time_skewed() && time_range.lr_end != -1) {
@@ -183,7 +194,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
                 humanize::time::duration::from_tv(diff_tv).to_string());
             time_lr.lr_end = time_str.length();
             time_line.with_attr(
-                string_attr(time_lr, VC_STYLE.value(text_attrs{A_BOLD})));
+                string_attr(time_lr, VC_STYLE.value(text_attrs::with_bold())));
         }
     }
 
@@ -212,11 +223,19 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         }
         time_line.append("  Format: ")
             .append(lnav::roles::symbol(
-                ts_formats[format->lf_date_time.dts_fmt_lock]));
+                ts_formats[format->lf_date_time.dts_fmt_lock]))
+            .append("  Default Zone: ");
         if (format->lf_date_time.dts_default_zone != nullptr) {
-            time_line.append("  Default Zone: ")
-                .append(lnav::roles::symbol(
-                    format->lf_date_time.dts_default_zone->name()));
+            time_line.append(lnav::roles::symbol(
+                format->lf_date_time.dts_default_zone->name()));
+        } else {
+            time_line.append("none"_comment);
+        }
+
+        auto file_opts = file->get_file_options();
+        if (file_opts) {
+            time_line.append("  File Options: ")
+                .append(lnav::roles::file(file_opts->first));
         }
     }
 
@@ -230,6 +249,16 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         return;
     }
 
+    auto anchor_opt = this->fos_lss.anchor_for_row(row);
+    if (anchor_opt) {
+        auto permalink
+            = attr_line_t(" Permalink: ")
+                  .append(lnav::roles::hyperlink(anchor_opt.value()));
+        this->fos_row_to_field_meta.emplace(
+            this->fos_lines.size(), row_info{std::nullopt, anchor_opt.value()});
+        this->fos_lines.emplace_back(permalink);
+    }
+
     this->fos_known_key_size = LOG_BODY.length();
     if (!this->fos_contexts.empty()) {
         this->fos_known_key_size += this->fos_contexts.top().c_prefix.length();
@@ -307,7 +336,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         auto* curr_elf = dynamic_cast<external_log_format*>(curr_format);
         const auto format_name = curr_format->get_name().to_string();
         attr_line_t al;
-        std::string value_str = lv.to_string();
+        auto value_str = lv.to_string();
 
         if (curr_format != last_format) {
             this->fos_lines.emplace_back(" Known message fields for table "
@@ -315,13 +344,13 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             this->fos_lines.back().with_attr(
                 string_attr(line_range(32, 32 + format_name.length()),
                             VC_STYLE.value(vc.attrs_for_ident(format_name)
-                                           | text_attrs{A_BOLD})));
+                                           | text_attrs::style::bold)));
             last_format = curr_format;
         }
 
         std::string field_name, orig_field_name;
         line_range hl_range;
-        al.append(" ").append("|", VC_GRAPHIC.value(ACS_LTEE)).append(" ");
+        al.append(" ").append("|", VC_GRAPHIC.value(NCACS_LTEE)).append(" ");
         if (meta.lvm_struct_name.empty()) {
             if (curr_elf && curr_elf->elf_body_field == meta.lvm_name) {
                 field_name = LOG_BODY;
@@ -352,13 +381,14 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
                     al.append(":bar_chart:"_emoji).append(" ");
                     break;
             }
-            auto prefix_len = al.utf8_length_or_length();
+            auto prefix_len = al.column_width();
             hl_range.lr_start = al.get_string().length();
             al.append(field_name);
             hl_range.lr_end = al.get_string().length();
             al.pad_to(prefix_len + this->fos_known_key_size);
 
-            this->fos_row_to_field_meta.emplace(this->fos_lines.size(), meta);
+            this->fos_row_to_field_meta.emplace(this->fos_lines.size(),
+                                                row_info{meta, value_str});
         } else {
             auto jget_str = lnav::sql::mprintf("jget(%s, '/%q')",
                                                meta.lvm_struct_name.get(),
@@ -366,8 +396,11 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             hl_range.lr_start = al.get_string().length();
             al.append(jget_str.in());
             hl_range.lr_end = al.get_string().length();
+
+            this->fos_row_to_field_meta.emplace(
+                this->fos_lines.size(), row_info{std::nullopt, value_str});
         }
-        readline_sqlite_highlighter_int(al, -1, hl_range);
+        readline_sqlite_highlighter_int(al, std::nullopt, hl_range);
 
         al.append(" = ").append(scrub_ws(value_str.c_str()));
 
@@ -401,10 +434,16 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         auto qname = lnav::sql::mprintf("%Q", extra_pair.first.c_str());
         auto key_line = attr_line_t("   jget(log_raw_text, ")
                             .append(qname.in())
-                            .append(")");
-        readline_sqlite_highlighter(key_line, 0);
+                            .append(")")
+                            .move();
+        readline_sqlite_highlighter(key_line, std::nullopt);
         auto key_size = key_line.length();
         key_line.append(" = ").append(scrub_ws(extra_pair.second));
+        this->fos_row_to_field_meta.emplace(this->fos_lines.size(),
+                                            row_info{
+                                                std::nullopt,
+                                                extra_pair.second,
+                                            });
         this->fos_lines.emplace_back(key_line);
         this->add_key_line_attrs(key_size - 3);
     }
@@ -413,11 +452,16 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         const auto& jpairs = jpairs_map.second;
 
         for (size_t lpc = 0; lpc < jpairs.size(); lpc++) {
-            auto key_line = attr_line_t("   ").append(
-                this->fos_log_helper.format_json_getter(jpairs_map.first, lpc));
-            readline_sqlite_highlighter(key_line, 0);
+            auto key_line = attr_line_t("   ")
+                                .append(this->fos_log_helper.format_json_getter(
+                                    jpairs_map.first, lpc))
+                                .move();
+            readline_sqlite_highlighter(key_line, std::nullopt);
             auto key_size = key_line.length();
             key_line.append(" = ").append(scrub_ws(jpairs[lpc].wt_value));
+            this->fos_row_to_field_meta.emplace(
+                this->fos_lines.size(),
+                row_info{std::nullopt, jpairs[lpc].wt_value});
             this->fos_lines.emplace_back(key_line);
             this->add_key_line_attrs(key_size - 3);
         }
@@ -434,10 +478,12 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             xml_pair.first.second.c_str(),
             this->fos_log_helper.ldh_file->get_format()->get_name().c_str(),
             qname.in());
-        auto key_line = attr_line_t("   ").append(xp_call.in());
-        readline_sqlite_highlighter(key_line, 0);
+        auto key_line = attr_line_t("   ").append(xp_call.in()).move();
+        readline_sqlite_highlighter(key_line, std::nullopt);
         auto key_size = key_line.length();
         key_line.append(" = ").append(scrub_ws(xml_pair.second));
+        this->fos_row_to_field_meta.emplace(
+            this->fos_lines.size(), row_info{std::nullopt, xml_pair.second});
         this->fos_lines.emplace_back(key_line);
         this->add_key_line_attrs(key_size - 3);
     }
@@ -455,7 +501,7 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
         auto& disc_str = al.get_string();
 
         al.with_attr(string_attr(line_range(disc_str.length(), -1),
-                                 VC_STYLE.value(text_attrs{A_BOLD})));
+                                 VC_STYLE.value(text_attrs::with_bold())));
         disc_str.append(this->fos_log_helper.ldh_msg_format);
     }
 
@@ -472,6 +518,8 @@ field_overlay_source::build_field_lines(const listview_curses& lv,
             string_attr(line_range(3, 3 + name.length()),
                         VC_STYLE.value(vc.attrs_for_ident(name.to_string()))));
 
+        this->fos_row_to_field_meta.emplace(this->fos_lines.size(),
+                                            row_info{std::nullopt, val});
         this->fos_lines.emplace_back(al);
         this->add_key_line_attrs(
             this->fos_unknown_key_size,
@@ -489,29 +537,48 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
     if (!this->fos_contexts.empty()
         && this->fos_contexts.top().c_show_applicable_annotations)
     {
+        if (this->fos_index_generation != this->fos_lss.lss_index_generation) {
+            this->fos_anno_cache.clear();
+            this->fos_index_generation = this->fos_lss.lss_index_generation;
+        }
+
         auto file_and_line = this->fos_lss.find_line_with_file(row);
 
         if (file_and_line && !file_and_line->second->is_continued()) {
-            auto applicable_anno = lnav::log::annotate::applicable(row);
-            if (!applicable_anno.empty()
-                && (!line_meta_opt
-                    || line_meta_opt.value()->bm_annotations.la_pairs.empty()))
-            {
-                auto anno_msg
-                    = attr_line_t(" ")
-                          .append(":memo:"_emoji)
-                          .append(" Annotations available, ")
-                          .append(lv.get_selection() == row
-                                      ? "use "
-                                      : "focus on this line and use ")
-                          .append(":annotate"_quoted_code)
-                          .append(" to apply them")
-                          .append(lv.get_selection() == row ? " to this line"
-                                                            : "")
-                          .with_attr_for_all(
-                              VC_ROLE.value(role_t::VCR_COMMENT));
+            auto get_res = this->fos_anno_cache.get(row);
+            if (get_res) {
+                auto anno_val = get_res.value();
+                if (anno_val) {
+                    dst.emplace_back(anno_val.value());
+                }
+            } else {
+                auto applicable_anno = lnav::log::annotate::applicable(row);
+                if (!applicable_anno.empty()
+                    && (!line_meta_opt
+                        || line_meta_opt.value()
+                               ->bm_annotations.la_pairs.empty()))
+                {
+                    auto anno_msg
+                        = attr_line_t(" ")
+                              .append(":memo:"_emoji)
+                              .append(" Annotations available, ")
+                              .append(lv.get_selection() == row
+                                          ? "use "
+                                          : "focus on this line and use ")
+                              .append(":annotate"_quoted_code)
+                              .append(" to apply them")
+                              .append(lv.get_selection() == row
+                                          ? " to this line"
+                                          : "")
+                              .with_attr_for_all(
+                                  VC_ROLE.value(role_t::VCR_COMMENT))
+                              .move();
 
-                dst.emplace_back(anno_msg);
+                    this->fos_anno_cache.put(row, anno_msg);
+                    dst.emplace_back(anno_msg);
+                } else {
+                    this->fos_anno_cache.put(row, std::nullopt);
+                }
             }
         }
     }
@@ -524,11 +591,42 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
     const auto& line_meta = *(line_meta_opt.value());
     size_t filename_width = this->fos_lss.get_filename_offset();
 
+    auto file_and_line = this->fos_lss.find_line_with_file(row);
+    auto* format = file_and_line->first->get_format_ptr();
+    auto field_states = format->get_field_states();
+    auto show_opid = false;
+    auto field_iter = field_states.find(log_format::LOG_OPID_STR);
+    if (field_iter != field_states.end() && !field_iter->second.is_hidden()) {
+        show_opid = true;
+    }
+    if (row == tc->get_selection() && !this->fos_contexts.empty()
+        && this->fos_contexts.top().c_show)
+    {
+        show_opid = true;
+    }
+    if (show_opid && !line_meta.bm_opid.empty()) {
+        auto al = attr_line_t()
+                      .append(" Op ID: "_table_header)
+                      .append(lnav::roles::identifier(line_meta.bm_opid))
+                      .move();
+
+        dst.emplace_back(al);
+    }
+
     if (!line_meta.bm_comment.empty()) {
         const auto* lead = line_meta.bm_tags.empty() ? " \u2514 " : " \u251c ";
         md2attr_line mdal;
         attr_line_t al;
 
+        auto comment_id = intern_string::lookup(fmt::format(
+            FMT_STRING("{}-line{}-comment"),
+            file_and_line->first->get_filename().filename().string(),
+            std::distance(file_and_line->first->begin(),
+                          file_and_line->second)));
+        mdal.with_source_id(comment_id);
+        if (tc->tc_interactive) {
+            mdal.add_lnav_script_icons();
+        }
         auto parse_res = md4cpp::parse(line_meta.bm_comment, mdal);
         if (parse_res.isOk()) {
             al = parse_res.unwrap();
@@ -574,14 +672,6 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
                                      VC_STYLE.value(vc.attrs_for_ident(str)));
         }
 
-        if (tc != nullptr) {
-            const auto& hm = tc->get_highlights();
-            auto hl_iter = hm.find({highlight_source_t::PREVIEW, "search"});
-
-            if (hl_iter != hm.end()) {
-                hl_iter->second.annotate(al, 2);
-            }
-        }
         al.insert(0, filename_width, ' ');
         if (tc != nullptr) {
             auto hl = tc->get_highlights();
@@ -598,6 +688,7 @@ field_overlay_source::build_meta_line(const listview_curses& lv,
             attr_line_t al;
             md2attr_line mdal;
 
+            mdal.add_lnav_script_icons();
             dst.push_back(
                 attr_line_t()
                     .append(filename_width, ' ')
@@ -651,12 +742,13 @@ field_overlay_source::add_key_line_attrs(int key_size, bool last_line)
 {
     auto& sa = this->fos_lines.back().get_attrs();
     struct line_range lr(1, 2);
-    int64_t graphic = (int64_t) (last_line ? ACS_LLCORNER : ACS_LTEE);
+
+    auto graphic = (last_line ? NCACS_LLCORNER : NCACS_LTEE);
     sa.emplace_back(lr, VC_GRAPHIC.value(graphic));
 
     lr.lr_start = 3 + key_size + 3;
     lr.lr_end = -1;
-    sa.emplace_back(lr, VC_STYLE.value(text_attrs{A_BOLD}));
+    sa.emplace_back(lr, VC_STYLE.value(text_attrs::with_bold()));
 }
 
 void
@@ -665,6 +757,7 @@ field_overlay_source::list_value_for_overlay(
     vis_line_t row,
     std::vector<attr_line_t>& value_out)
 {
+    // log_debug("value for overlay %d", row);
     if (row == lv.get_selection()) {
         this->build_field_lines(lv, row);
         value_out = this->fos_lines;
@@ -672,7 +765,35 @@ field_overlay_source::list_value_for_overlay(
     this->build_meta_line(lv, value_out, row);
 }
 
-nonstd::optional<attr_line_t>
+bool
+field_overlay_source::list_static_overlay(const listview_curses& lv,
+                                          int y,
+                                          int bottom,
+                                          attr_line_t& value_out)
+{
+    const std::vector<attr_line_t>* lines = nullptr;
+    if (this->fos_lss.text_line_count() == 0) {
+        if (this->fos_tss.empty()) {
+            lines = lnav::messages::view::no_files();
+        } else {
+            lines = lnav::messages::view::only_text_files();
+        }
+    }
+
+    if (lines != nullptr && y < (ssize_t) lines->size()) {
+        value_out = lines->at(y);
+        value_out.with_attr_for_all(VC_ROLE.value(role_t::VCR_STATUS));
+        if (y == (ssize_t) lines->size() - 1) {
+            value_out.with_attr_for_all(
+                VC_STYLE.value(text_attrs::with_underline()));
+        }
+        return true;
+    }
+
+    return false;
+}
+
+std::optional<attr_line_t>
 field_overlay_source::list_header_for_overlay(const listview_curses& lv,
                                               vis_line_t vl)
 {
@@ -697,6 +818,8 @@ field_overlay_source::list_header_for_overlay(const listview_curses& lv,
         retval.append("  ")
             .append("SPC"_hotkey)
             .append(": hide/show field  ")
+            .append("c"_hotkey)
+            .append(": copy field value  ")
             .append("Esc"_hotkey)
             .append(": exit this panel");
     } else {
