@@ -66,7 +66,7 @@ child_poller::poll(file_collection& fc)
     }
 
     auto poll_res = std::move(this->cp_child.value()).poll();
-    this->cp_child = nonstd::nullopt;
+    this->cp_child = std::nullopt;
     return poll_res.match(
         [this](auto_pid<process_state::running>& alive) {
             this->cp_child = std::move(alive);
@@ -134,7 +134,8 @@ file_collection::close_files(const std::vector<std::shared_ptr<logfile>>& files)
         } else {
             this->fc_file_names.erase(lf->get_filename());
         }
-        auto file_iter = find(this->fc_files.begin(), this->fc_files.end(), lf);
+        auto file_iter
+            = std::find(this->fc_files.begin(), this->fc_files.end(), lf);
         if (file_iter != this->fc_files.end()) {
             this->fc_files.erase(file_iter);
         }
@@ -160,7 +161,7 @@ file_collection::regenerate_unique_file_names()
         safe::ReadAccess<safe_name_to_errors> errs(*this->fc_name_to_errors);
 
         for (const auto& pair : *errs) {
-            auto path = ghc::filesystem::path(pair.first).filename().string();
+            auto path = std::filesystem::path(pair.first).filename().string();
 
             if (path.length() > this->fc_largest_path_length) {
                 this->fc_largest_path_length = path.length();
@@ -178,8 +179,9 @@ file_collection::regenerate_unique_file_names()
         switch (pair.second.ofd_format) {
             case file_format_t::UNKNOWN:
             case file_format_t::ARCHIVE:
+            case file_format_t::MULTIPLEXED:
             case file_format_t::SQLITE_DB: {
-                auto bn = ghc::filesystem::path(pair.first).filename().string();
+                auto bn = std::filesystem::path(pair.first).filename().string();
                 if (bn.length() > this->fc_largest_path_length) {
                     this->fc_largest_path_length = bn.length();
                 }
@@ -218,8 +220,12 @@ file_collection::merge(file_collection& other)
 
         errs->insert(new_errors.begin(), new_errors.end());
     }
-    this->fc_file_names.insert(other.fc_file_names.begin(),
-                               other.fc_file_names.end());
+    if (!other.fc_file_names.empty()) {
+        this->fc_files_generation += 1;
+    }
+    for (const auto& fn_pair : other.fc_file_names) {
+        this->fc_file_names[fn_pair.first] = fn_pair.second;
+    }
     if (!other.fc_files.empty()) {
         for (const auto& lf : other.fc_files) {
             this->fc_name_to_errors->writeAccess()->erase(lf->get_filename());
@@ -252,7 +258,11 @@ file_collection::merge(file_collection& other)
  * Functor used to compare files based on their device and inode number.
  */
 struct same_file {
-    explicit same_file(const struct stat& stat) : sf_stat(stat) {}
+    explicit same_file(const std::filesystem::path& filename,
+                       const struct stat& stat)
+        : sf_filename(filename), sf_stat(stat)
+    {
+    }
 
     /**
      * Compare the given log file against the 'stat' given in the constructor.
@@ -266,6 +276,12 @@ struct same_file {
             return false;
         }
 
+        if (lf->get_actual_path()
+            && lf->get_actual_path().value() == this->sf_filename)
+        {
+            return true;
+        }
+
         const auto& lf_loo = lf->get_open_options();
 
         if (lf_loo.loo_temp_dev != 0
@@ -279,6 +295,7 @@ struct same_file {
             && this->sf_stat.st_ino == lf->get_stat().st_ino;
     }
 
+    const std::filesystem::path& sf_filename;
     const struct stat& sf_stat;
 };
 
@@ -291,7 +308,7 @@ struct same_file {
  * @param fd       An already-opened descriptor for 'filename'.
  * @param required Specifies whether or not the file must exist and be valid.
  */
-nonstd::optional<std::future<file_collection>>
+std::optional<std::future<file_collection>>
 file_collection::watch_logfile(const std::string& filename,
                                logfile_open_options& loo,
                                bool required)
@@ -300,8 +317,11 @@ file_collection::watch_logfile(const std::string& filename,
     int rc;
 
     auto filename_key = loo.loo_filename.empty() ? filename : loo.loo_filename;
-    if (this->fc_closed_files.count(filename)) {
-        return nonstd::nullopt;
+    if (this->fc_closed_files.count(filename)
+        || this->fc_closed_files.count(filename_key))
+    {
+        log_trace("%s: file is closed, ignore", filename.c_str());
+        return std::nullopt;
     }
 
     rc = stat(filename.c_str(), &st);
@@ -321,14 +341,14 @@ file_collection::watch_logfile(const std::string& filename,
                         .with_visible_size_limit(256 * 1024));
                 return lnav::futures::make_ready_future(std::move(retval));
             }
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         if (!S_ISREG(st.st_mode)) {
             if (required) {
                 rc = -1;
                 errno = EINVAL;
             } else {
-                return nonstd::nullopt;
+                return std::nullopt;
             }
         }
         {
@@ -338,6 +358,8 @@ file_collection::watch_logfile(const std::string& filename,
             auto err_iter = errs->find(filename_key);
             if (err_iter != errs->end()) {
                 if (err_iter->second.fei_mtime != st.st_mtime) {
+                    log_debug("clearing error info for file: %s",
+                              filename_key.c_str());
                     errs->erase(err_iter);
                 }
             }
@@ -357,26 +379,28 @@ file_collection::watch_logfile(const std::string& filename,
                 });
             return lnav::futures::make_ready_future(std::move(retval));
         }
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     if (this->fc_new_stats | lnav::itertools::find_if([&st](const auto& elem) {
             return st.st_ino == elem.st_ino && st.st_dev == elem.st_dev;
         }))
     {
+        log_trace("same stat: %s", filename.c_str());
         // this file is probably a link that we have already scanned in this
         // pass.
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     this->fc_new_stats.emplace_back(st);
 
-    auto file_iter = std::find_if(
-        this->fc_files.begin(), this->fc_files.end(), same_file(st));
+    const auto fn_path = std::filesystem::path(filename);
+    const auto file_iter = std::find_if(
+        this->fc_files.begin(), this->fc_files.end(), same_file(fn_path, st));
 
     if (file_iter == this->fc_files.end()) {
         if (this->fc_other_files.find(filename) != this->fc_other_files.end()) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
 
         require(this->fc_progress.get() != nullptr);
@@ -397,16 +421,47 @@ file_collection::watch_logfile(const std::string& filename,
                 }
             }
 
-            auto ff = detect_file_format(filename);
+            auto ff_res = detect_file_format(filename);
 
-            loo.loo_file_format = ff;
-            switch (ff) {
+            loo.loo_file_format = ff_res.dffr_file_format;
+            switch (ff_res.dffr_file_format) {
                 case file_format_t::SQLITE_DB:
-                    retval.fc_other_files[filename].ofd_format = ff;
+                    retval.fc_other_files[filename].ofd_format
+                        = ff_res.dffr_file_format;
+                    retval.fc_other_files[filename].ofd_details
+                        = ff_res.dffr_details;
                     break;
 
+                case file_format_t::MULTIPLEXED: {
+                    log_info("%s: file is multiplexed, creating piper",
+                             filename.c_str());
+
+                    auto open_res
+                        = lnav::filesystem::open_file(filename, O_RDONLY);
+                    if (open_res.isOk()) {
+                        auto looper_options = lnav::piper::options{};
+                        looper_options.with_follow(loo.loo_follow);
+                        auto create_res
+                            = lnav::piper::create_looper(filename,
+                                                         open_res.unwrap(),
+                                                         auto_fd{-1},
+                                                         looper_options);
+
+                        if (create_res.isOk()) {
+                            auto& ofd = retval.fc_other_files[filename];
+
+                            ofd.ofd_format = ff_res.dffr_file_format;
+                            ofd.ofd_details = ff_res.dffr_details;
+                            retval.fc_file_names[filename] = loo;
+                            retval.fc_file_names[filename].with_piper(
+                                create_res.unwrap());
+                        }
+                    }
+                    break;
+                }
+
                 case file_format_t::ARCHIVE: {
-                    nonstd::optional<
+                    std::optional<
                         std::list<archive_manager::extract_progress>::iterator>
                         prog_iter_opt;
 
@@ -432,7 +487,7 @@ file_collection::watch_logfile(const std::string& filename,
                         },
                         [&filename, &retval](const auto& tmp_path,
                                              const auto& entry) {
-                            auto arc_path = ghc::filesystem::relative(
+                            auto arc_path = std::filesystem::relative(
                                 entry.path(), tmp_path);
                             auto custom_name = filename / arc_path;
                             bool is_visible = true;
@@ -464,7 +519,9 @@ file_collection::watch_logfile(const std::string& filename,
                                 res.unwrapErr(),
                             });
                     } else {
-                        retval.fc_other_files[filename] = ff;
+                        auto& ofd = retval.fc_other_files[filename];
+                        ofd.ofd_format = ff_res.dffr_file_format;
+                        ofd.ofd_details = ff_res.dffr_details;
                     }
                     {
                         prog_iter_opt | [&prog](auto prog_iter) {
@@ -478,6 +535,7 @@ file_collection::watch_logfile(const std::string& filename,
                 default: {
                     auto filename_to_open = filename;
 
+                    loo.loo_match_details = ff_res.dffr_details;
                     auto eff = detect_mime_type(filename);
 
                     if (eff) {
@@ -559,11 +617,14 @@ file_collection::watch_logfile(const std::string& filename,
          */
         file_collection retval;
 
+        log_info("renamed file: %s -> %s",
+                 lf->get_filename().c_str(),
+                 filename.c_str());
         retval.fc_renamed_files.emplace_back(lf, filename);
         return lnav::futures::make_ready_future(std::move(retval));
     }
 
-    return nonstd::nullopt;
+    return std::nullopt;
 }
 
 /**
@@ -582,7 +643,7 @@ file_collection::expand_filename(
     static_root_mem<glob_t, globfree> gl;
 
     {
-        std::lock_guard<std::mutex> lg(REALPATH_CACHE_MUTEX);
+        std::lock_guard lg(REALPATH_CACHE_MUTEX);
 
         if (REALPATH_CACHE.find(path) != REALPATH_CACHE.end()) {
             return;
@@ -594,10 +655,8 @@ file_collection::expand_filename(
     }
 
     auto filename_key = loo.loo_filename.empty() ? path : loo.loo_filename;
-    auto glob_flags = lnav::filesystem::is_glob(path) ? 0 : GLOB_NOCHECK;
-    if (glob(path.c_str(), glob_flags, nullptr, gl.inout()) == 0) {
-        int lpc;
-
+    auto glob_rc = glob(path.c_str(), GLOB_NOCHECK, nullptr, gl.inout());
+    if (glob_rc == 0) {
         if (gl->gl_pathc == 1 /*&& gl.gl_matchc == 0*/) {
             /* It's a pattern that doesn't match any files
              * yet, allow it through since we'll load it in
@@ -640,8 +699,8 @@ file_collection::expand_filename(
             required = false;
         }
 
-        std::lock_guard<std::mutex> lg(REALPATH_CACHE_MUTEX);
-        for (lpc = 0; lpc < (int) gl->gl_pathc; lpc++) {
+        std::lock_guard lg(REALPATH_CACHE_MUTEX);
+        for (size_t lpc = 0; lpc < gl->gl_pathc; lpc++) {
             auto path_str = std::string(gl->gl_pathv[lpc]);
             auto iter = REALPATH_CACHE.find(path_str);
 
@@ -657,10 +716,7 @@ file_collection::expand_filename(
                                 "Cannot find file: %s -- %s",
                                 gl->gl_pathv[lpc],
                                 errmsg);
-                    } else if (loo.loo_source != logfile_name_source::REMOTE) {
-                        // XXX The remote code path adds the file name before
-                        // the file exists...  not sure checking for that here
-                        // is a good idea (prolly not)
+                    } else if (loo.loo_filename.empty()) {
                         file_collection retval;
 
                         if (gl->gl_pathc == 1) {
@@ -668,8 +724,9 @@ file_collection::expand_filename(
                                     filename_key)
                                 == 0)
                             {
-                                log_error("failed to find path: %s -- %s",
+                                log_error("failed to find path: %s (%s) -- %s",
                                           filename_key.c_str(),
+                                          gl->gl_pathv[lpc],
                                           errmsg);
                                 retval.fc_name_to_errors->writeAccess()
                                     ->emplace(filename_key,
@@ -705,14 +762,15 @@ file_collection::expand_filename(
                 if (future_opt) {
                     auto fut = std::move(future_opt.value());
                     if (fq.push_back(std::move(fut))
-                        == lnav::futures::future_queue<
-                            file_collection>::processor_result_t::interrupt)
+                        == lnav::progress_result_t::interrupt)
                     {
                         break;
                     }
                 }
             }
         }
+    } else if (glob_rc != GLOB_NOMATCH) {
+        log_error("glob(%s) failed -- %s", path.c_str(), strerror(errno));
     }
 }
 
@@ -735,13 +793,12 @@ file_collection::rescan_files(bool required)
                 && this->fc_files.size() + retval.fc_files.size()
                     < get_limits().l_open_files)
             {
-                return lnav::futures::future_queue<
-                    file_collection>::processor_result_t::ok;
+                return lnav::progress_result_t::ok;
             }
-            return lnav::futures::future_queue<
-                file_collection>::processor_result_t::interrupt;
+            return lnav::progress_result_t::interrupt;
         });
 
+    this->fc_new_stats.clear();
     for (auto& pair : this->fc_file_names) {
         if (this->fc_files.size() + retval.fc_files.size()
             >= get_limits().l_open_files)
@@ -756,6 +813,14 @@ file_collection::rescan_files(bool required)
                 pair.second.loo_piper->get_out_pattern().string(),
                 pair.second,
                 required);
+            if (!pair.second.loo_piper.value().get_demux_id().empty()
+                && this->fc_other_files.count(pair.first) == 0)
+            {
+                auto& ofd = retval.fc_other_files[pair.first];
+                ofd.ofd_format = file_format_t::MULTIPLEXED;
+                ofd.ofd_details
+                    = pair.second.loo_piper.value().get_demux_details();
+            }
         } else {
             this->expand_filename(fq, pair.first, pair.second, required);
             if (this->fc_rotated) {
@@ -773,8 +838,6 @@ file_collection::rescan_files(bool required)
 
     fq.pop_to();
 
-    this->fc_new_stats.clear();
-
     return retval;
 }
 
@@ -785,6 +848,22 @@ file_collection::request_close(const std::shared_ptr<logfile>& lf)
     this->fc_files_generation += 1;
 }
 
+size_t
+file_collection::initial_indexing_pipers() const
+{
+    size_t retval = 0;
+
+    for (const auto& pair : this->fc_file_names) {
+        if (pair.second.loo_piper
+            && pair.second.loo_piper->get_loop_count() == 0)
+        {
+            retval += 1;
+        }
+    }
+
+    return retval;
+}
+
 size_t
 file_collection::active_pipers() const
 {
@@ -822,6 +901,18 @@ file_collection::copy()
     return retval;
 }
 
+void
+file_collection::clear()
+{
+    this->fc_name_to_errors->writeAccess()->clear();
+    this->fc_file_names.clear();
+    this->fc_files.clear();
+    this->fc_renamed_files.clear();
+    this->fc_closed_files.clear();
+    this->fc_other_files.clear();
+    this->fc_new_stats.clear();
+}
+
 size_t
 file_collection::other_file_format_count(file_format_t ff) const
 {
