@@ -27,48 +27,53 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <memory>
+#include <utility>
+
 #include "filter_sub_source.hh"
 
 #include "base/attr_line.builder.hh"
-#include "base/enum_util.hh"
+#include "base/auto_mem.hh"
 #include "base/func_util.hh"
+#include "base/itertools.hh"
 #include "base/opt_util.hh"
-#include "bound_tags.hh"
+#include "cmd.parser.hh"
 #include "config.h"
+#include "data_scanner.hh"
 #include "lnav.hh"
+#include "lnav.prompt.hh"
 #include "readline_highlighters.hh"
 #include "readline_possibilities.hh"
 #include "sql_util.hh"
+#include "textinput_curses.hh"
 
 using namespace lnav::roles::literals;
 
-filter_sub_source::filter_sub_source(std::shared_ptr<readline_curses> editor)
-    : fss_editor(editor)
+filter_sub_source::filter_sub_source(std::shared_ptr<textinput_curses> editor)
+    : fss_editor(std::move(editor)),
+      fss_regexp_history(
+          lnav::textinput::history::for_context("regexp-filter"_frag)),
+      fss_sql_history(lnav::textinput::history::for_context("sql-filter"_frag))
 {
+    this->fss_editor->set_visible(false);
     this->fss_editor->set_x(25);
-    this->fss_editor->set_width(-1);
-    this->fss_editor->set_save_history(!(lnav_data.ld_flags & LNF_SECURE_MODE));
-    this->fss_regex_context.set_highlighter(readline_regex_highlighter)
-        .set_append_character(0);
-    this->fss_editor->add_context(filter_lang_t::REGEX,
-                                  this->fss_regex_context);
-    this->fss_sql_context.set_highlighter(readline_sqlite_highlighter)
-        .set_append_character(0);
-    this->fss_editor->add_context(filter_lang_t::SQL, this->fss_sql_context);
-    this->fss_editor->set_change_action(
-        bind_mem(&filter_sub_source::rl_change, this));
-    this->fss_editor->set_perform_action(
-        bind_mem(&filter_sub_source::rl_perform, this));
-    this->fss_editor->set_abort_action(
-        bind_mem(&filter_sub_source::rl_abort, this));
-    this->fss_editor->set_display_match_action(
-        bind_mem(&filter_sub_source::rl_display_matches, this));
-    this->fss_editor->set_display_next_action(
-        bind_mem(&filter_sub_source::rl_display_next, this));
-    this->fss_match_view.set_sub_source(&this->fss_match_source);
-    this->fss_match_view.set_height(0_vl);
-    this->fss_match_view.set_show_scrollbar(true);
-    this->fss_match_view.set_default_role(role_t::VCR_POPUP);
+    this->fss_editor->tc_popup.set_title("Pattern");
+    this->fss_editor->tc_height = 1;
+    this->fss_editor->tc_on_change
+        = bind_mem(&filter_sub_source::rl_change, this);
+    this->fss_editor->tc_on_history_search
+        = bind_mem(&filter_sub_source::rl_history, this);
+    this->fss_editor->tc_on_history_list
+        = bind_mem(&filter_sub_source::rl_history, this);
+    this->fss_editor->tc_on_completion_request
+        = bind_mem(&filter_sub_source::rl_completion_request, this);
+    this->fss_editor->tc_on_completion
+        = bind_mem(&filter_sub_source::rl_completion, this);
+    this->fss_editor->tc_on_perform
+        = bind_mem(&filter_sub_source::rl_perform, this);
+    this->fss_editor->tc_on_blur = bind_mem(&filter_sub_source::rl_blur, this);
+    this->fss_editor->tc_on_abort
+        = bind_mem(&filter_sub_source::rl_abort, this);
 }
 
 void
@@ -76,25 +81,18 @@ filter_sub_source::register_view(textview_curses* tc)
 {
     text_sub_source::register_view(tc);
     tc->add_child_view(this->fss_editor.get());
-    tc->add_child_view(&this->fss_match_view);
 }
 
 bool
-filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
+filter_sub_source::list_input_handle_key(listview_curses& lv, const ncinput& ch)
 {
+    static auto& prompt = lnav::prompt::get();
+
     if (this->fss_editing) {
-        switch (ch) {
-            case KEY_ESCAPE:
-            case KEY_CTRL(']'):
-                this->fss_editor->abort();
-                return true;
-            default:
-                this->fss_editor->handle_key(ch);
-                return true;
-        }
+        return this->fss_editor->handle_key(ch);
     }
 
-    switch (ch) {
+    switch (ch.eff_text[0]) {
         case 'f': {
             auto* top_view = *lnav_data.ld_view_stack.top();
             auto* tss = top_view->get_sub_source();
@@ -104,15 +102,15 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             break;
         }
         case ' ': {
-            textview_curses* top_view = *lnav_data.ld_view_stack.top();
-            text_sub_source* tss = top_view->get_sub_source();
-            filter_stack& fs = tss->get_filters();
+            auto* top_view = *lnav_data.ld_view_stack.top();
+            auto* tss = top_view->get_sub_source();
+            auto& fs = tss->get_filters();
 
             if (fs.empty()) {
                 return true;
             }
 
-            auto tf = *(fs.begin() + lv.get_selection());
+            auto tf = *(fs.begin() + lv.get_selection().value());
 
             fs.set_filter_enabled(tf, !tf->is_enabled());
             tss->text_filters_changed();
@@ -121,15 +119,15 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             return true;
         }
         case 't': {
-            textview_curses* top_view = *lnav_data.ld_view_stack.top();
-            text_sub_source* tss = top_view->get_sub_source();
-            filter_stack& fs = tss->get_filters();
+            auto* top_view = *lnav_data.ld_view_stack.top();
+            auto* tss = top_view->get_sub_source();
+            auto& fs = tss->get_filters();
 
             if (fs.empty()) {
                 return true;
             }
 
-            auto tf = *(fs.begin() + lv.get_selection());
+            auto tf = *(fs.begin() + lv.get_selection().value());
 
             if (tf->get_type() == text_filter::INCLUDE) {
                 tf->set_type(text_filter::EXCLUDE);
@@ -143,15 +141,15 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             return true;
         }
         case 'D': {
-            textview_curses* top_view = *lnav_data.ld_view_stack.top();
-            text_sub_source* tss = top_view->get_sub_source();
-            filter_stack& fs = tss->get_filters();
+            auto* top_view = *lnav_data.ld_view_stack.top();
+            auto* tss = top_view->get_sub_source();
+            auto& fs = tss->get_filters();
 
             if (fs.empty()) {
                 return true;
             }
 
-            auto tf = *(fs.begin() + lv.get_selection());
+            auto tf = *(fs.begin() + lv.get_selection().value());
 
             fs.delete_filter(tf->get_id());
             lv.reload_data();
@@ -178,19 +176,15 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             lv.reload_data();
 
             this->fss_editing = true;
-            this->tss_view->vc_enabled = false;
-
-            add_view_text_possibilities(this->fss_editor.get(),
-                                        filter_lang_t::REGEX,
-                                        "*",
-                                        top_view,
-                                        text_quoting::regex);
-            this->fss_editor->set_window(lv.get_window());
+            this->tss_view->set_enabled(false);
+            this->fss_view_text_possibilities
+                = view_text_possibilities(*top_view);
+            this->fss_editor->tc_text_format = text_format_t::TF_PCRE;
+            this->fss_editor->set_y(lv.get_y_for_selection());
+            this->fss_editor->set_content("");
+            this->fss_editor->tc_suggestion = top_view->get_input_suggestion();
             this->fss_editor->set_visible(true);
-            this->fss_editor->set_y(
-                lv.get_y() + (int) (lv.get_selection() - lv.get_top()));
-            this->fss_editor->window_change();
-            this->fss_editor->focus(filter_lang_t::REGEX, "", "");
+            this->fss_editor->focus();
             this->fss_filter_state = true;
             ef->disable();
             return true;
@@ -214,68 +208,67 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             lv.reload_data();
 
             this->fss_editing = true;
-            this->tss_view->vc_enabled = false;
+            this->tss_view->set_enabled(false);
 
-            add_view_text_possibilities(this->fss_editor.get(),
-                                        filter_lang_t::REGEX,
-                                        "*",
-                                        top_view,
-                                        text_quoting::regex);
-            this->fss_editor->set_window(lv.get_window());
+            this->fss_editor->tc_text_format = text_format_t::TF_PCRE;
+            this->fss_editor->set_y(lv.get_y_for_selection());
             this->fss_editor->set_visible(true);
-            this->fss_editor->set_y(
-                lv.get_y() + (int) (lv.get_selection() - lv.get_top()));
-            this->fss_editor->window_change();
-            this->fss_editor->focus(filter_lang_t::REGEX, "", "");
+            this->fss_editor->set_content("");
+            this->fss_view_text_possibilities
+                = view_text_possibilities(*top_view);
+            this->fss_editor->tc_suggestion = top_view->get_input_suggestion();
+            this->fss_editor->focus();
             this->fss_filter_state = true;
             ef->disable();
             return true;
         }
         case '\r':
-        case KEY_ENTER: {
-            textview_curses* top_view = *lnav_data.ld_view_stack.top();
-            text_sub_source* tss = top_view->get_sub_source();
-            filter_stack& fs = tss->get_filters();
+        case NCKEY_ENTER: {
+            auto* top_view = *lnav_data.ld_view_stack.top();
+            auto* tss = top_view->get_sub_source();
+            auto& fs = tss->get_filters();
 
             if (fs.empty()) {
                 return true;
             }
 
-            auto tf = *(fs.begin() + lv.get_selection());
+            auto tf = *(fs.begin() + lv.get_selection().value());
 
             this->fss_editing = true;
-            this->tss_view->vc_enabled = false;
+            this->tss_view->set_enabled(false);
 
-            auto tq = tf->get_lang() == filter_lang_t::SQL
-                ? text_quoting::sql
-                : text_quoting::regex;
-            add_view_text_possibilities(
-                this->fss_editor.get(), tf->get_lang(), "*", top_view, tq);
-            if (top_view == &lnav_data.ld_views[LNV_LOG]) {
-                add_filter_expr_possibilities(
-                    this->fss_editor.get(), filter_lang_t::SQL, "*");
+            this->fss_editor->tc_text_format
+                = tf->get_lang() == filter_lang_t::SQL ? text_format_t::TF_SQL
+                                                       : text_format_t::TF_PCRE;
+            if (tf->get_lang() == filter_lang_t::SQL) {
+                prompt.refresh_sql_completions(*top_view);
+                prompt.refresh_sql_expr_completions(*top_view);
             }
-            this->fss_editor->set_window(lv.get_window());
+            this->fss_editor->set_y(lv.get_y_for_selection());
             this->fss_editor->set_visible(true);
-            this->fss_editor->set_y(
-                lv.get_y() + (int) (lv.get_selection() - lv.get_top()));
-            this->fss_editor->focus(tf->get_lang(), "");
-            this->fss_editor->rewrite_line(0, tf->get_id().c_str());
+            this->fss_editor->tc_suggestion.clear();
+            this->fss_editor->set_content(tf->get_id());
+            this->fss_view_text_possibilities
+                = view_text_possibilities(*top_view);
+            this->fss_editor->focus();
             this->fss_filter_state = tf->is_enabled();
             tf->disable();
             tss->text_filters_changed();
             return true;
         }
         case 'n': {
-            execute_command(lnav_data.ld_exec_context, "next-mark search");
+            lnav_data.ld_exec_context.execute(INTERNAL_SRC_LOC,
+                                              ":next-mark search");
             return true;
         }
         case 'N': {
-            execute_command(lnav_data.ld_exec_context, "prev-mark search");
+            lnav_data.ld_exec_context.execute(INTERNAL_SRC_LOC,
+                                              ":prev-mark search");
             return true;
         }
         case '/': {
-            execute_command(lnav_data.ld_exec_context, "prompt search-filters");
+            lnav_data.ld_exec_context.execute(INTERNAL_SRC_LOC,
+                                              ":prompt search-filters");
             return true;
         }
         default:
@@ -290,14 +283,14 @@ size_t
 filter_sub_source::text_line_count()
 {
     return (lnav_data.ld_view_stack.top() |
-                [](auto tc) -> nonstd::optional<size_t> {
+                [](auto tc) -> std::optional<size_t> {
                text_sub_source* tss = tc->get_sub_source();
 
                if (tss == nullptr) {
-                   return nonstd::nullopt;
+                   return std::nullopt;
                }
                auto& fs = tss->get_filters();
-               return nonstd::make_optional(fs.size());
+               return std::make_optional(fs.size());
            })
         .value_or(0);
 }
@@ -317,11 +310,11 @@ filter_sub_source::text_line_width(textview_curses& curses)
     return retval;
 }
 
-void
+line_info
 filter_sub_source::text_value_for_line(textview_curses& tc,
                                        int line,
                                        std::string& value_out,
-                                       text_sub_source::line_flags_t flags)
+                                       line_flags_t flags)
 {
     auto* top_view = *lnav_data.ld_view_stack.top();
     auto* tss = top_view->get_sub_source();
@@ -335,7 +328,7 @@ filter_sub_source::text_value_for_line(textview_curses& tc,
     attr_line_builder alb(al);
 
     if (selected) {
-        al.append(" ", VC_GRAPHIC.value(ACS_RARROW));
+        al.append(" ", VC_GRAPHIC.value(NCACS_RARROW));
     } else {
         al.append(" ");
     }
@@ -367,20 +360,20 @@ filter_sub_source::text_value_for_line(textview_curses& tc,
         if (this->fss_editing && line == tc.get_selection()) {
             alb.appendf(FMT_STRING("{:>9}"), "-");
         } else {
-            alb.appendf(FMT_STRING("{:>9}"),
+            alb.appendf(FMT_STRING("{:>9L}"),
                         tss->get_filtered_count_for(tf->get_index()));
         }
     }
 
-    al.append(" hits ").append("|", VC_GRAPHIC.value(ACS_VLINE)).append(" ");
+    al.append(" hits ").append("|", VC_GRAPHIC.value(NCACS_VLINE)).append(" ");
 
     attr_line_t content{tf->get_id()};
     switch (tf->get_lang()) {
         case filter_lang_t::REGEX:
-            readline_regex_highlighter(content, content.length());
+            readline_regex_highlighter(content, std::nullopt);
             break;
         case filter_lang_t::SQL:
-            readline_sqlite_highlighter(content, content.length());
+            readline_sqlite_highlighter(content, std::nullopt);
             break;
         case filter_lang_t::NONE:
             break;
@@ -392,6 +385,8 @@ filter_sub_source::text_value_for_line(textview_curses& tc,
     }
 
     value_out = al.get_string();
+
+    return {};
 }
 
 void
@@ -407,28 +402,30 @@ filter_sub_source::text_size_for_line(textview_curses& tc,
                                       int line,
                                       text_sub_source::line_flags_t raw)
 {
-    textview_curses* top_view = *lnav_data.ld_view_stack.top();
-    text_sub_source* tss = top_view->get_sub_source();
-    filter_stack& fs = tss->get_filters();
+    auto* top_view = *lnav_data.ld_view_stack.top();
+    auto* tss = top_view->get_sub_source();
+    auto& fs = tss->get_filters();
     auto tf = *(fs.begin() + line);
 
     return 8 + tf->get_id().size();
 }
 
 void
-filter_sub_source::rl_change(readline_curses* rc)
+filter_sub_source::rl_change(textinput_curses& rc)
 {
-    textview_curses* top_view = *lnav_data.ld_view_stack.top();
-    text_sub_source* tss = top_view->get_sub_source();
-    filter_stack& fs = tss->get_filters();
+    auto* top_view = *lnav_data.ld_view_stack.top();
+    auto* tss = top_view->get_sub_source();
+    auto& fs = tss->get_filters();
     if (fs.empty()) {
         return;
     }
 
-    auto iter = fs.begin() + this->tss_view->get_selection();
+    auto iter = fs.begin() + this->tss_view->get_selection().value();
     auto tf = *iter;
-    auto new_value = rc->get_line_buffer();
+    auto new_value = rc.get_content();
 
+    top_view->get_highlights().erase({highlight_source_t::PREVIEW, "preview"});
+    top_view->set_needs_update();
     switch (tf->get_lang()) {
         case filter_lang_t::NONE:
             break;
@@ -439,12 +436,16 @@ filter_sub_source::rl_change(readline_curses* rc)
                     sugg = top_view->tc_selected_text->sti_value;
                 }
                 if (fs.get_filter(sugg) == nullptr) {
-                    this->fss_editor->set_suggestion(sugg);
+                    this->fss_editor->tc_suggestion = sugg;
+                } else {
+                    this->fss_editor->tc_suggestion.clear();
                 }
             } else {
                 auto regex_res
                     = lnav::pcre2pp::code::from(new_value, PCRE2_CASELESS);
 
+                this->rl_completion_request_int(
+                    rc, completion_request_type_t::partial);
                 if (regex_res.isErr()) {
                     auto pe = regex_res.unwrapErr();
                     lnav_data.ld_filter_help_status_source.fss_error.set_value(
@@ -456,8 +457,8 @@ filter_sub_source::rl_change(readline_curses* rc)
                         ? role_t::VCR_DIFF_DELETE
                         : role_t::VCR_DIFF_ADD;
                     hl.with_role(role);
-                    hl.with_attrs(text_attrs{A_BLINK | A_REVERSE});
-
+                    hl.with_attrs(text_attrs::with_styles(
+                        text_attrs::style::blink, text_attrs::style::reverse));
                     hm[{highlight_source_t::PREVIEW, "preview"}] = hl;
                     top_view->set_needs_update();
                     lnav_data.ld_filter_help_status_source.fss_error.clear();
@@ -466,6 +467,9 @@ filter_sub_source::rl_change(readline_curses* rc)
             break;
         }
         case filter_lang_t::SQL: {
+            this->rl_completion_request_int(rc,
+                                            completion_request_type_t::partial);
+
             auto full_sql
                 = fmt::format(FMT_STRING("SELECT 1 WHERE {}"), new_value);
             auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
@@ -508,22 +512,128 @@ filter_sub_source::rl_change(readline_curses* rc)
 }
 
 void
-filter_sub_source::rl_perform(readline_curses* rc)
+filter_sub_source::rl_history(textinput_curses& tc)
+{
+    switch (tc.tc_text_format) {
+        case text_format_t::TF_PCRE: {
+            std::vector<attr_line_t> poss;
+            this->fss_regexp_history.query_entries(
+                tc.get_content(),
+                [&poss](const auto& e) { poss.emplace_back(e.e_content); });
+            tc.open_popup_for_history(poss);
+            break;
+        }
+        case text_format_t::TF_SQL: {
+            break;
+        }
+        default:
+            ensure(false);
+            break;
+    }
+}
+
+void
+filter_sub_source::rl_completion_request_int(textinput_curses& tc,
+                                             completion_request_type_t crt)
+{
+    static const auto FILTER_HELP
+        = help_text("filter", "filter the view by a pattern")
+              .with_parameter(
+                  help_text("pattern", "The pattern to filter by")
+                      .with_format(help_parameter_format_t::HPF_REGEX));
+    static const auto FILTER_EXPR_HELP
+        = help_text("filter-expr", "filter the view by a SQL expression")
+              .with_parameter(
+                  help_text("expr", "The expression to evaluate")
+                      .with_format(help_parameter_format_t::HPF_SQL_EXPR));
+    static auto& prompt = lnav::prompt::get();
+
+    auto* top_view = *lnav_data.ld_view_stack.top();
+    auto& al = tc.tc_lines[tc.tc_cursor.y];
+    auto al_sf = al.to_string_fragment().sub_cell_range(0, tc.tc_cursor.x);
+    std::string prefix;
+    auto is_regex = tc.tc_text_format == text_format_t::TF_PCRE;
+    auto parse_res = lnav::command::parse_for_prompt(
+        lnav_data.ld_exec_context,
+        al_sf,
+        is_regex ? FILTER_HELP : FILTER_EXPR_HELP);
+
+    switch (crt) {
+        case completion_request_type_t::partial: {
+            if (al_sf.endswith(" ")) {
+                if (tc.is_cursor_at_end_of_line()) {
+                    tc.tc_suggestion
+                        = prompt.get_regex_suggestion(*top_view, al.al_string);
+                }
+                return;
+            }
+            break;
+        }
+        case completion_request_type_t::full:
+            break;
+    }
+
+    auto byte_x = al_sf.column_to_byte_index(tc.tc_cursor.x);
+    auto arg_res_opt = parse_res.arg_at(byte_x);
+    if (arg_res_opt) {
+        auto arg_pair = arg_res_opt.value();
+        if (crt == completion_request_type_t::full
+            || tc.tc_popup_type != textinput_curses::popup_type_t::none)
+        {
+            auto poss = prompt.get_cmd_parameter_completion(
+                *top_view,
+                &FILTER_HELP,
+                arg_pair.aar_help,
+                arg_pair.aar_element.se_value);
+            auto left = arg_pair.aar_element.se_value.empty()
+                ? tc.tc_cursor.x
+                : al_sf.byte_to_column_index(
+                      arg_pair.aar_element.se_origin.sf_begin);
+            tc.open_popup_for_completion(left, poss);
+            tc.tc_popup.set_title(arg_pair.aar_help->ht_name);
+        } else if (arg_pair.aar_element.se_value.empty()
+                   && tc.is_cursor_at_end_of_line())
+        {
+            tc.tc_suggestion
+                = prompt.get_regex_suggestion(*top_view, al.al_string);
+        } else {
+            log_debug("not at end of line");
+            tc.tc_suggestion.clear();
+        }
+    } else {
+        log_debug("no arg");
+    }
+}
+
+void
+filter_sub_source::rl_completion_request(textinput_curses& tc)
+{
+    this->rl_completion_request_int(tc, completion_request_type_t::full);
+}
+
+void
+filter_sub_source::rl_completion(textinput_curses& tc)
+{
+    static auto& prompt = lnav::prompt::get();
+
+    prompt.rl_completion(tc);
+}
+
+void
+filter_sub_source::rl_perform(textinput_curses& rc)
 {
     static const intern_string_t INPUT_SRC = intern_string::lookup("input");
 
-    textview_curses* top_view = *lnav_data.ld_view_stack.top();
-    text_sub_source* tss = top_view->get_sub_source();
-    filter_stack& fs = tss->get_filters();
-    auto iter = fs.begin() + this->tss_view->get_selection();
+    auto* top_view = *lnav_data.ld_view_stack.top();
+    auto* tss = top_view->get_sub_source();
+    auto& fs = tss->get_filters();
+    auto iter = fs.begin() + this->tss_view->get_selection().value();
     auto tf = *iter;
-    auto new_value = rc->get_value().get_string();
+    auto new_value = rc.get_content();
 
     if (new_value.empty()) {
         this->rl_abort(rc);
     } else {
-        top_view->get_highlights().erase(
-            {highlight_source_t::PREVIEW, "preview"});
         switch (tf->get_lang()) {
             case filter_lang_t::NONE:
             case filter_lang_t::REGEX: {
@@ -533,18 +643,17 @@ filter_sub_source::rl_perform(readline_curses* rc)
                 if (compile_res.isErr()) {
                     auto ce = compile_res.unwrapErr();
                     auto um = lnav::console::to_user_message(INPUT_SRC, ce);
-                    lnav_data.ld_exec_context.ec_error_callback_stack.back()(
-                        um);
+                    lnav_data.ld_exec_context.ec_msg_callback_stack.back()(um);
                     this->rl_abort(rc);
                 } else {
+                    auto code_ptr = compile_res.unwrap().to_shared();
                     tf->lf_deleted = true;
                     tss->text_filters_changed();
 
                     auto pf = std::make_shared<pcre_filter>(
-                        tf->get_type(),
-                        new_value,
-                        tf->get_index(),
-                        compile_res.unwrap().to_shared());
+                        tf->get_type(), new_value, tf->get_index(), code_ptr);
+
+                    this->fss_regexp_history.insert_plain_content(new_value);
 
                     *iter = pf;
                     tss->text_filters_changed();
@@ -578,8 +687,7 @@ filter_sub_source::rl_perform(readline_curses* rc)
                               .with_reason(sqlite3_errmsg(lnav_data.ld_db.in()))
                               .with_snippet(lnav::console::snippet::from(
                                   INPUT_SRC, sqlerr));
-                    lnav_data.ld_exec_context.ec_error_callback_stack.back()(
-                        um);
+                    lnav_data.ld_exec_context.ec_msg_callback_stack.back()(um);
                     this->rl_abort(rc);
                 } else {
                     lnav_data.ld_log_source.set_sql_filter(new_value,
@@ -591,97 +699,45 @@ filter_sub_source::rl_perform(readline_curses* rc)
         }
     }
 
-    lnav_data.ld_log_source.set_preview_sql_filter(nullptr);
-    lnav_data.ld_filter_help_status_source.fss_prompt.clear();
-    this->fss_editing = false;
-    this->tss_view->vc_enabled = true;
-    this->fss_editor->set_visible(false);
     top_view->reload_data();
     this->tss_view->reload_data();
 }
 
 void
-filter_sub_source::rl_abort(readline_curses* rc)
+filter_sub_source::rl_blur(textinput_curses& tc)
 {
-    textview_curses* top_view = *lnav_data.ld_view_stack.top();
-    text_sub_source* tss = top_view->get_sub_source();
-    filter_stack& fs = tss->get_filters();
-    auto iter = fs.begin() + this->tss_view->get_selection();
-    auto tf = *iter;
-
+    auto* top_view = *lnav_data.ld_view_stack.top();
+    top_view->get_highlights().erase({highlight_source_t::PREVIEW, "preview"});
     lnav_data.ld_log_source.set_preview_sql_filter(nullptr);
     lnav_data.ld_filter_help_status_source.fss_prompt.clear();
     lnav_data.ld_filter_help_status_source.fss_error.clear();
-    top_view->get_highlights().erase({highlight_source_t::PREVIEW, "preview"});
+    this->fss_editing = false;
+    tc.set_visible(false);
+    this->tss_view->set_enabled(true);
+}
+
+void
+filter_sub_source::rl_abort(textinput_curses& rc)
+{
+    auto* top_view = *lnav_data.ld_view_stack.top();
+    auto* tss = top_view->get_sub_source();
+    auto& fs = tss->get_filters();
+    auto iter = fs.begin() + this->tss_view->get_selection().value();
+    auto tf = *iter;
+
     top_view->reload_data();
     fs.delete_filter("");
     this->tss_view->reload_data();
-    this->fss_editor->set_visible(false);
-    this->fss_editing = false;
-    this->tss_view->vc_enabled = true;
     this->tss_view->set_needs_update();
     tf->set_enabled(this->fss_filter_state);
     tss->text_filters_changed();
     this->tss_view->reload_data();
 }
 
-void
-filter_sub_source::rl_display_matches(readline_curses* rc)
-{
-    const std::vector<std::string>& matches = rc->get_matches();
-    unsigned long width = 0;
-
-    if (matches.empty()) {
-        this->fss_match_source.clear();
-        this->fss_match_view.set_height(0_vl);
-        this->tss_view->set_needs_update();
-    } else {
-        auto current_match = rc->get_match_string();
-        attr_line_t al;
-        vis_line_t line, selected_line;
-
-        for (const auto& match : matches) {
-            if (match == current_match) {
-                al.append(match, VC_STYLE.value(text_attrs{A_REVERSE}));
-                selected_line = line;
-            } else {
-                al.append(match);
-            }
-            al.append(1, '\n');
-            width = std::max(width, (unsigned long) match.size());
-            line += 1_vl;
-        }
-
-        this->fss_match_view.set_selection(selected_line);
-        this->fss_match_source.replace_with(al);
-        this->fss_match_view.set_height(
-            std::min(vis_line_t(matches.size()), 3_vl));
-    }
-
-    this->fss_match_view.set_window(this->tss_view->get_window());
-    this->fss_match_view.set_y(rc->get_y() + 1);
-    this->fss_match_view.set_x(rc->get_x() + rc->get_match_start());
-    this->fss_match_view.set_width(width + 3);
-    this->fss_match_view.set_needs_update();
-    this->fss_match_view.reload_data();
-}
-
-void
-filter_sub_source::rl_display_next(readline_curses* rc)
-{
-    textview_curses& tc = this->fss_match_view;
-
-    if (tc.get_top() >= (tc.get_top_for_last_row() - 1)) {
-        tc.set_top(0_vl);
-    } else {
-        tc.shift_top(tc.get_height());
-    }
-}
-
 void
 filter_sub_source::list_input_handle_scroll_out(listview_curses& lv)
 {
-    lnav_data.ld_mode = ln_mode_t::PAGING;
+    set_view_mode(ln_mode_t::PAGING);
     lnav_data.ld_filter_view.reload_data();
 }
 
@@ -694,15 +750,22 @@ filter_sub_source::text_handle_mouse(
     if (this->fss_editing) {
         return true;
     }
+    auto nci = ncinput{};
     if (me.is_click_in(mouse_button_t::BUTTON_LEFT, 1, 3)) {
-        this->list_input_handle_key(tc, ' ');
+        nci.id = ' ';
+        nci.eff_text[0] = ' ';
+        this->list_input_handle_key(tc, nci);
     }
     if (me.is_click_in(mouse_button_t::BUTTON_LEFT, 4, 7)) {
-        this->list_input_handle_key(tc, 't');
+        nci.id = 't';
+        nci.eff_text[0] = 't';
+        this->list_input_handle_key(tc, nci);
     }
     if (me.is_double_click_in(mouse_button_t::BUTTON_LEFT, line_range{25, -1}))
     {
-        this->list_input_handle_key(tc, '\r');
+        nci.id = '\r';
+        nci.eff_text[0] = '\r';
+        this->list_input_handle_key(tc, nci);
     }
     return true;
 }
