@@ -44,10 +44,7 @@
 #include <algorithm>
 #include <set>
 
-#ifdef HAVE_X86INTRIN_H
-#    include "simdutf8check.h"
-#endif
-
+#include "base/auto_mem.hh"
 #include "base/auto_pid.hh"
 #include "base/fs_util.hh"
 #include "base/injector.bind.hh"
@@ -59,8 +56,9 @@
 #include "fmtlib/fmt/format.h"
 #include "hasher.hh"
 #include "line_buffer.hh"
-#include "piper.looper.hh"
-#include "scn/scn.h"
+#include "piper.header.hh"
+#include "scn/scan.h"
+#include "yajlpp/yajlpp_def.hh"
 
 using namespace std::chrono_literals;
 
@@ -99,7 +97,7 @@ class lock_hack {
 public:
     class guard {
     public:
-        guard() : g_lock(lock_hack::singleton()) { this->g_lock.lock(); }
+        guard() : g_lock(singleton()) { this->g_lock.lock(); }
 
         ~guard() { this->g_lock.unlock(); }
 
@@ -137,7 +135,7 @@ private:
 #define SYNCPOINT_SIZE (1024 * 1024)
 line_buffer::gz_indexed::gz_indexed()
 {
-    if ((this->inbuf = auto_mem<Bytef>::malloc(Z_BUFSIZE)) == NULL) {
+    if ((this->inbuf = auto_mem<Bytef>::malloc(Z_BUFSIZE)) == nullptr) {
         throw std::bad_alloc();
     }
 }
@@ -162,13 +160,8 @@ line_buffer::gz_indexed::init_stream()
     }
 
     // initialize inflate struct
-    this->strm.zalloc = Z_NULL;
-    this->strm.zfree = Z_NULL;
-    this->strm.opaque = Z_NULL;
+    int rc = inflateInit2(&this->strm, GZ_HEADER_MODE);
     this->strm.avail_in = 0;
-    this->strm.next_in = Z_NULL;
-    this->strm.avail_out = 0;
-    int rc = inflateInit2(&strm, GZ_HEADER_MODE);
     if (rc != Z_OK) {
         throw(rc);  // FIXME: exception wrapper
     }
@@ -227,7 +220,6 @@ line_buffer::gz_indexed::open(int fd, lnav::gzip::header& hd)
 
             inflate(&this->strm, Z_BLOCK);
             inflateEnd(&this->strm);
-
             this->strm.next_out = Z_NULL;
             this->strm.next_in = Z_NULL;
             this->strm.next_in = Z_NULL;
@@ -241,8 +233,18 @@ line_buffer::gz_indexed::open(int fd, lnav::gzip::header& hd)
                     break;
                 case 1:
                     hd.h_mtime.tv_sec = gz_hd.time;
+                    name[sizeof(name) - 1] = '\0';
+                    comment[sizeof(comment) - 1] = '\0';
                     hd.h_name = std::string((char*) name);
                     hd.h_comment = std::string((char*) comment);
+                    log_info(
+                        "%d: read gzip header (mtime=%d; name='%s'; "
+                        "comment='%s'; crc=%x)",
+                        fd,
+                        hd.h_mtime.tv_sec,
+                        hd.h_name.c_str(),
+                        hd.h_comment.c_str(),
+                        gz_hd.hcrc);
                     break;
                 default:
                     log_error("%d: failed to read gzip header data", fd);
@@ -283,9 +285,15 @@ line_buffer::gz_indexed::stream_data(void* buf, size_t size)
                 // stream
                 continue_stream();
             } else if (err != Z_OK) {
-                log_error(" inflate-error: %d  %s",
+                log_error(" inflate-error at offset %d: %d  %s",
+                          this->strm.total_in,
                           (int) err,
                           this->strm.msg ? this->strm.msg : "");
+                this->parent->lb_decompress_error = fmt::format(
+                    FMT_STRING("inflate-error at offset {}: {}  {}"),
+                    this->strm.total_in,
+                    err,
+                    this->strm.msg ? this->strm.msg : "");
                 break;
             }
 
@@ -366,11 +374,17 @@ line_buffer::gz_indexed::read(void* buf, size_t offset, size_t size)
 
 line_buffer::line_buffer()
 {
+    this->lb_gz_file.writeAccess()->parent = this;
+
     ensure(this->invariant());
 }
 
 line_buffer::~line_buffer()
 {
+    if (this->lb_loader_future.valid()) {
+        this->lb_loader_future.wait();
+    }
+
     auto empty_fd = auto_fd();
 
     // Make sure any shared refs take ownership of the data.
@@ -491,8 +505,9 @@ line_buffer::set_fd(auto_fd& fd)
 void
 line_buffer::resize_buffer(size_t new_max)
 {
-    if (new_max <= MAX_LINE_BUFFER_SIZE
-        && new_max > (size_t) this->lb_buffer.capacity())
+    if (((this->lb_compressed && new_max <= MAX_COMPRESSED_BUFFER_SIZE)
+         || (!this->lb_compressed && new_max <= MAX_LINE_BUFFER_SIZE))
+        && !this->lb_buffer.has_capacity_for(new_max))
     {
         /* Still need more space, try a realloc. */
         this->lb_share_manager.invalidate_refs();
@@ -501,9 +516,11 @@ line_buffer::resize_buffer(size_t new_max)
 }
 
 void
-line_buffer::ensure_available(file_off_t start, ssize_t max_length)
+line_buffer::ensure_available(file_off_t start,
+                              ssize_t max_length,
+                              scan_direction dir)
 {
-    ssize_t prefill, available;
+    ssize_t prefill;
 
     require(this->lb_compressed || max_length <= MAX_LINE_BUFFER_SIZE);
 
@@ -529,19 +546,40 @@ line_buffer::ensure_available(file_off_t start, ssize_t max_length)
         this->lb_share_manager.invalidate_refs();
         prefill = 0;
         this->lb_buffer.clear();
-        if ((this->lb_file_size != (ssize_t) -1)
-            && (start + this->lb_buffer.capacity() > this->lb_file_size))
-        {
+
+        switch (dir) {
+            case scan_direction::forward:
+                break;
+            case scan_direction::backward: {
+                auto padded_max_length = max_length * 4;
+                if (this->lb_buffer.has_capacity_for(padded_max_length)) {
+                    start = std::max(
+                        file_off_t{0},
+                        static_cast<file_off_t>(start
+                                                - (this->lb_buffer.capacity()
+                                                   - padded_max_length)));
+                }
+                break;
+            }
+        }
+
+        if (this->lb_file_size == (ssize_t) -1) {
+            this->lb_file_offset = start;
+        } else {
             require(start <= this->lb_file_size);
             /*
              * If the start is near the end of the file, move the offset back a
              * bit so we can get more of the file in the cache.
              */
-            this->lb_file_offset = this->lb_file_size
-                - std::min(this->lb_file_size,
-                           (file_ssize_t) this->lb_buffer.capacity());
-        } else {
-            this->lb_file_offset = start;
+            if (start + (ssize_t) this->lb_buffer.capacity()
+                > this->lb_file_size)
+            {
+                this->lb_file_offset = this->lb_file_size
+                    - std::min(this->lb_file_size,
+                               (file_ssize_t) this->lb_buffer.capacity());
+            } else {
+                this->lb_file_offset = start;
+            }
         }
     } else {
         /* The request is in the cached range.  Record how much extra data is in
@@ -550,11 +588,10 @@ line_buffer::ensure_available(file_off_t start, ssize_t max_length)
         prefill = start - this->lb_file_offset;
     }
     require(this->lb_file_offset <= start);
-    require(prefill <= this->lb_buffer.size());
-
-    available = this->lb_buffer.capacity() - (start - this->lb_file_offset);
-    require(available <= this->lb_buffer.capacity());
+    require(prefill <= (ssize_t) this->lb_buffer.size());
 
+    ssize_t available
+        = this->lb_buffer.capacity() - (start - this->lb_file_offset);
     if (max_length > available) {
         // log_debug("need more space!");
         /*
@@ -601,12 +638,16 @@ line_buffer::load_next_buffer()
                           start + this->lb_alt_buffer.value().size(),
                           this->lb_alt_buffer.value().available());
             this->lb_compressed_offset = gi->get_source_offset();
-            if (rc != -1 && (rc < this->lb_alt_buffer.value().available())
-                && (start + this->lb_alt_buffer.value().size() + rc
+            if (rc != -1
+                && rc < (ssize_t) this->lb_alt_buffer.value().available()
+                && (start + (ssize_t) this->lb_alt_buffer.value().size() + rc
                     > this->lb_file_size))
             {
                 this->lb_file_size
                     = (start + this->lb_alt_buffer.value().size() + rc);
+                log_info("fd(%d): set file size to %llu",
+                         this->lb_fd.get(),
+                         this->lb_file_size);
             }
 #if 0
             log_debug("async decomp end  %d+%d:%d",
@@ -665,12 +706,16 @@ line_buffer::load_next_buffer()
             this->lb_compressed_offset = lseek(bzfd, 0, SEEK_SET);
             BZ2_bzclose(bz_file);
 
-            if (rc != -1 && (rc < (this->lb_alt_buffer.value().available()))
-                && (start + this->lb_alt_buffer.value().size() + rc
+            if (rc != -1
+                && (rc < (ssize_t) (this->lb_alt_buffer.value().available()))
+                && (start + (ssize_t) this->lb_alt_buffer.value().size() + rc
                     > this->lb_file_size))
             {
                 this->lb_file_size
                     = (start + this->lb_alt_buffer.value().size() + rc);
+                log_info("fd(%d): set file size to %llu",
+                         this->lb_fd.get(),
+                         this->lb_file_size);
             }
         }
     }
@@ -685,7 +730,7 @@ line_buffer::load_next_buffer()
     }
     // XXX For some reason, cygwin is giving us a bogus return value when
     // up to the end of the file.
-    if (rc > (this->lb_alt_buffer.value().available())) {
+    if (rc > (ssize_t) this->lb_alt_buffer.value().available()) {
         rc = -1;
 #ifdef ENODATA
         errno = ENODATA;
@@ -728,10 +773,11 @@ line_buffer::load_next_buffer()
 
         do {
             auto before = line_start - this->lb_alt_buffer->begin();
-            auto remaining = this->lb_alt_buffer.value().size() - before;
-            auto frag = string_fragment::from_bytes(line_start, remaining);
+            const auto remaining = this->lb_alt_buffer.value().size() - before;
+            const auto frag
+                = string_fragment::from_bytes(line_start, remaining);
             auto utf_scan_res = is_utf8(frag, '\n');
-            auto lf = utf_scan_res.remaining_ptr(frag);
+            const auto* lf = utf_scan_res.remaining_ptr();
             this->lb_alt_line_starts.emplace_back(before);
             this->lb_alt_line_is_utf.emplace_back(utf_scan_res.is_valid());
             this->lb_alt_line_has_ansi.emplace_back(utf_scan_res.usr_has_ansi);
@@ -745,13 +791,14 @@ line_buffer::load_next_buffer()
 }
 
 bool
-line_buffer::fill_range(file_off_t start, ssize_t max_length)
+line_buffer::fill_range(file_off_t start,
+                        ssize_t max_length,
+                        scan_direction dir)
 {
     bool retval = false;
 
     require(start >= 0);
 
-    // log_debug("fill range %d %d", start, max_length);
 #if 0
     log_debug("(%p) fill range %d %d (%d) %d",
               this,
@@ -768,13 +815,12 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
                   start,
                   this->lb_loader_file_offset.value());
 #endif
-        nonstd::optional<std::chrono::system_clock::time_point> wait_start;
+        std::optional<std::chrono::system_clock::time_point> wait_start;
 
         if (this->lb_loader_future.wait_for(std::chrono::seconds(0))
             != std::future_status::ready)
         {
-            wait_start
-                = nonstd::make_optional(std::chrono::system_clock::now());
+            wait_start = std::make_optional(std::chrono::system_clock::now());
         }
         retval = this->lb_loader_future.get();
         if (false && wait_start) {
@@ -785,7 +831,7 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
         this->lb_loader_future = {};
         this->lb_share_manager.invalidate_refs();
         this->lb_file_offset = this->lb_loader_file_offset.value();
-        this->lb_loader_file_offset = nonstd::nullopt;
+        this->lb_loader_file_offset = std::nullopt;
         this->lb_buffer.swap(this->lb_alt_buffer.value());
         this->lb_alt_buffer.value().clear();
         this->lb_line_starts = std::move(this->lb_alt_line_starts);
@@ -795,6 +841,8 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
         this->lb_line_has_ansi = std::move(this->lb_alt_line_has_ansi);
         this->lb_alt_line_has_ansi.clear();
         this->lb_stats.s_used_preloads += 1;
+        this->lb_next_line_start_index = 0;
+        this->lb_next_buffer_offset = 0;
     }
     if (this->in_range(start)
         && (max_length == 0 || this->in_range(start + max_length - 1)))
@@ -841,7 +889,7 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
         ssize_t rc;
 
         /* Make sure there is enough space, then */
-        this->ensure_available(start, max_length);
+        this->ensure_available(start, max_length, dir);
 
         safe::WriteAccess<safe_gz_indexed> gi(this->lb_gz_file);
 
@@ -863,9 +911,13 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
                               this->lb_file_offset + this->lb_buffer.size(),
                               this->lb_buffer.available());
                 this->lb_compressed_offset = gi->get_source_offset();
-                if (rc != -1 && (rc < this->lb_buffer.available())) {
+                if (rc != -1 && (rc < (ssize_t) this->lb_buffer.available())) {
                     this->lb_file_size
                         = (this->lb_file_offset + this->lb_buffer.size() + rc);
+                    log_info("fd(%d): rc (%d) -- set file size to %llu",
+                             this->lb_fd.get(),
+                             rc,
+                             this->lb_file_size);
                 }
             }
 #if 0
@@ -925,9 +977,12 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
                 this->lb_compressed_offset = lseek(bzfd, 0, SEEK_SET);
                 BZ2_bzclose(bz_file);
 
-                if (rc != -1 && (rc < (this->lb_buffer.available()))) {
+                if (rc != -1 && (rc < (ssize_t) this->lb_buffer.available())) {
                     this->lb_file_size
                         = (this->lb_file_offset + this->lb_buffer.size() + rc);
+                    log_info("fd(%d): set file size to %llu",
+                             this->lb_fd.get(),
+                             this->lb_file_size);
                 }
             }
         }
@@ -958,7 +1013,7 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
         }
         // XXX For some reason, cygwin is giving us a bogus return value when
         // up to the end of the file.
-        if (rc > (this->lb_buffer.available())) {
+        if (rc > (ssize_t) this->lb_buffer.available()) {
             rc = -1;
 #ifdef ENODATA
             errno = ENODATA;
@@ -1072,7 +1127,9 @@ line_buffer::load_next_line(file_range prev_line)
     retval.li_file_range.fr_offset = offset;
     if (this->lb_buffer.empty() || !this->in_range(offset)) {
         this->fill_range(offset, this->lb_buffer.capacity());
-    } else if (offset == this->lb_file_offset + this->lb_buffer.size()) {
+    } else if (offset
+               == this->lb_file_offset + (ssize_t) this->lb_buffer.size())
+    {
         if (!this->fill_range(offset, INITIAL_REQUEST_SIZE)) {
             retval.li_file_range.fr_offset = offset;
             retval.li_file_range.fr_size = 0;
@@ -1101,38 +1158,80 @@ line_buffer::load_next_line(file_range prev_line)
         /* ... look for the end-of-line or end-of-file. */
         ssize_t utf8_end = -1;
 
-        bool found_in_cache = false;
+        if (!retval.li_utf8_scan_result.is_valid()) {
+            retval.li_utf8_scan_result = {};
+        }
+        auto found_in_cache = false;
+        auto has_ansi = false;
+        auto valid_utf8 = true;
         if (!this->lb_line_starts.empty()) {
             auto buffer_offset = offset - this->lb_file_offset;
 
-            auto start_iter = std::lower_bound(this->lb_line_starts.begin(),
-                                               this->lb_line_starts.end(),
-                                               buffer_offset);
-            if (start_iter != this->lb_line_starts.end()) {
+            if (this->lb_next_buffer_offset == buffer_offset) {
+                require(this->lb_next_line_start_index
+                        < this->lb_line_starts.size());
+                auto start_iter = this->lb_line_starts.begin()
+                    + this->lb_next_line_start_index;
                 auto next_line_iter = start_iter + 1;
-
-                // log_debug("found offset %d %d", buffer_offset, *start_iter);
                 if (next_line_iter != this->lb_line_starts.end()) {
                     utf8_end = *next_line_iter - 1 - *start_iter;
                     found_in_cache = true;
                     lf = line_start + utf8_end;
+                    has_ansi = this->lb_line_has_ansi
+                                   [this->lb_next_line_start_index];
+                    valid_utf8
+                        = this->lb_line_is_utf[this->lb_next_line_start_index];
+
+                    // log_debug("hit cache");
+                    this->lb_next_buffer_offset = *next_line_iter;
+                    this->lb_next_line_start_index += 1;
                 } else {
                     // log_debug("no next iter");
                 }
             } else {
-                // log_debug("no buffer_offset found");
+                auto start_iter = std::lower_bound(this->lb_line_starts.begin(),
+                                                   this->lb_line_starts.end(),
+                                                   buffer_offset);
+                if (start_iter != this->lb_line_starts.end()) {
+                    auto next_line_iter = start_iter + 1;
+
+                    // log_debug("found offset %d %d", buffer_offset,
+                    // *start_iter);
+                    if (next_line_iter != this->lb_line_starts.end()) {
+                        auto start_index = std::distance(
+                            this->lb_line_starts.begin(), start_iter);
+                        utf8_end = *next_line_iter - 1 - *start_iter;
+                        found_in_cache = true;
+                        lf = line_start + utf8_end;
+                        has_ansi = this->lb_line_has_ansi[start_index];
+                        valid_utf8 = this->lb_line_is_utf[start_index];
+
+                        this->lb_next_line_start_index = start_index + 1;
+                        this->lb_next_buffer_offset = *next_line_iter;
+                    } else {
+                        // log_debug("no next iter");
+                    }
+                } else {
+                    // log_debug("no buffer_offset found");
+                }
             }
         }
 
-        if (!found_in_cache) {
+        if (found_in_cache && valid_utf8) {
+            retval.li_utf8_scan_result.usr_has_ansi = has_ansi;
+        } else {
             auto frag = string_fragment::from_bytes(
                 line_start, retval.li_file_range.fr_size);
             auto scan_res = is_utf8(frag, '\n');
-            lf = scan_res.remaining_ptr(frag);
+            lf = scan_res.remaining_ptr();
             if (lf != nullptr) {
                 lf -= 1;
             }
             retval.li_utf8_scan_result = scan_res;
+            if (!scan_res.is_valid()) {
+                log_warning("line is not utf8 -- %lld",
+                            retval.li_file_range.fr_offset);
+            }
         }
 
         auto got_new_data = old_retval_size != retval.li_file_range.fr_size;
@@ -1213,7 +1312,7 @@ line_buffer::load_next_line(file_range prev_line)
         }
     }
 
-    ensure(retval.li_file_range.fr_size <= this->lb_buffer.size());
+    ensure(retval.li_file_range.fr_size <= (ssize_t) this->lb_buffer.size());
     ensure(this->invariant());
 #if 0
     log_debug("got line part %d %d",
@@ -1227,18 +1326,16 @@ line_buffer::load_next_line(file_range prev_line)
         = retval.li_utf8_scan_result.is_valid();
 
     if (this->lb_line_metadata) {
-        auto sv = scn::string_view{
+        auto sv = std::string_view{
             line_start,
             (size_t) retval.li_file_range.fr_size,
         };
 
-        char level;
-        auto scan_res = scn::scan(sv,
-                                  "{}.{}:{};",
-                                  retval.li_timestamp.tv_sec,
-                                  retval.li_timestamp.tv_usec,
-                                  level);
+        auto scan_res = scn::scan<int64_t, int64_t, char>(sv, "{}.{}:{};");
         if (scan_res) {
+            auto& [tv_sec, tv_usec, level] = scan_res->values();
+            retval.li_timestamp.tv_sec = tv_sec;
+            retval.li_timestamp.tv_usec = tv_usec;
             retval.li_timestamp.tv_sec
                 = lnav::to_local_time(date::sys_seconds{std::chrono::seconds{
                                           retval.li_timestamp.tv_sec}})
@@ -1252,7 +1349,7 @@ line_buffer::load_next_line(file_range prev_line)
 }
 
 Result<shared_buffer_ref, std::string>
-line_buffer::read_range(file_range fr)
+line_buffer::read_range(file_range fr, scan_direction dir)
 {
     shared_buffer_ref retval;
     const char* line_start;
@@ -1277,7 +1374,7 @@ line_buffer::read_range(file_range fr)
     if (!(this->in_range(fr.fr_offset)
           && this->in_range(fr.fr_offset + fr.fr_size - 1)))
     {
-        if (!this->fill_range(fr.fr_offset, fr.fr_size)) {
+        if (!this->fill_range(fr.fr_offset, fr.fr_size, dir)) {
             return Err(std::string("unable to read file"));
         }
     }
@@ -1371,7 +1468,7 @@ line_buffer::quiesce()
     }
 }
 
-static ghc::filesystem::path
+static std::filesystem::path
 line_buffer_cache_path()
 {
     return lnav::paths::workdir() / "buffer-cache";
@@ -1402,7 +1499,7 @@ line_buffer::enable_cache()
                                 .to_string();
     auto cache_dir = line_buffer_cache_path() / cached_base_name.substr(0, 2);
 
-    ghc::filesystem::create_directories(cache_dir);
+    std::filesystem::create_directories(cache_dir);
 
     auto cached_file_name = fmt::format(FMT_STRING("{}.bin"), cached_base_name);
     auto cached_file_path = cache_dir / cached_file_name;
@@ -1415,14 +1512,14 @@ line_buffer::enable_cache()
     auto fl = lnav::filesystem::file_lock(cached_file_path);
     auto guard = lnav::filesystem::file_lock::guard(&fl);
 
-    if (ghc::filesystem::exists(cached_done_path)) {
-        log_info("%d:using existing cache file");
+    if (std::filesystem::exists(cached_done_path)) {
+        log_info("%d:using existing cache file", this->lb_fd.get());
         auto open_res = lnav::filesystem::open_file(cached_file_path, O_RDWR);
         if (open_res.isOk()) {
             this->lb_cached_fd = open_res.unwrap();
             return;
         }
-        ghc::filesystem::remove(cached_done_path);
+        std::filesystem::remove(cached_done_path);
     }
 
     auto create_res = lnav::filesystem::create_file(
@@ -1437,7 +1534,7 @@ line_buffer::enable_cache()
     auto write_fd = create_res.unwrap();
     auto done = false;
 
-    static const ssize_t FILL_LENGTH = 1024 * 1024;
+    static constexpr ssize_t FILL_LENGTH = 1024 * 1024;
     auto off = file_off_t{0};
     while (!done) {
         log_debug("%d: caching file content at %d", this->lb_fd.get(), off);
@@ -1467,18 +1564,18 @@ void
 line_buffer::cleanup_cache()
 {
     (void) std::async(std::launch::async, []() {
-        auto now = std::chrono::system_clock::now();
+        auto now = std::filesystem::file_time_type::clock::now();
         auto cache_path = line_buffer_cache_path();
-        std::vector<ghc::filesystem::path> to_remove;
+        std::vector<std::filesystem::path> to_remove;
         std::error_code ec;
 
         for (const auto& cache_subdir :
-             ghc::filesystem::directory_iterator(cache_path, ec))
+             std::filesystem::directory_iterator(cache_path, ec))
         {
             for (const auto& entry :
-                 ghc::filesystem::directory_iterator(cache_subdir, ec))
+                 std::filesystem::directory_iterator(cache_subdir, ec))
             {
-                auto mtime = ghc::filesystem::last_write_time(entry.path());
+                auto mtime = std::filesystem::last_write_time(entry.path());
                 auto exp_time = mtime + 1h;
                 if (now < exp_time) {
                     continue;
@@ -1490,7 +1587,7 @@ line_buffer::cleanup_cache()
 
         for (auto& entry : to_remove) {
             log_debug("removing compressed file cache: %s", entry.c_str());
-            ghc::filesystem::remove_all(entry, ec);
+            std::filesystem::remove_all(entry, ec);
         }
     });
 }
