@@ -27,6 +27,8 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <unordered_map>
+
 #include "lnav.indexing.hh"
 
 #include "bound_tags.hh"
@@ -35,6 +37,7 @@
 #include "service_tags.hh"
 #include "session_data.hh"
 #include "sql_util.hh"
+#include "yajlpp/yajlpp_def.hh"
 
 using namespace std::chrono_literals;
 using namespace lnav::roles::literals;
@@ -44,16 +47,20 @@ using namespace lnav::roles::literals;
  */
 class loading_observer : public logfile_observer {
 public:
-    loading_observer() : lo_last_offset(0){};
+    loading_observer() : lo_last_offset(0) {}
 
-    indexing_result logfile_indexing(const std::shared_ptr<logfile>& lf,
-                                     file_off_t off,
-                                     file_size_t total) override
+    lnav::progress_result_t logfile_indexing(const logfile* lf,
+                                             file_off_t off,
+                                             file_ssize_t total) override
     {
         static sig_atomic_t index_counter = 0;
 
         if (lnav_data.ld_window == nullptr) {
-            return indexing_result::CONTINUE;
+            return lnav::progress_result_t::ok;
+        }
+
+        if (lnav_data.ld_sigint_count.load() > 0) {
+            return lnav::progress_result_t::interrupt;
         }
 
         /* XXX require(off <= total); */
@@ -61,7 +68,8 @@ public:
             off = total;
         }
 
-        if ((((size_t) off == total) && (this->lo_last_offset != off))
+        auto retval = lnav::progress_result_t::ok;
+        if (((off == total) && (this->lo_last_offset != off))
             || ui_periodic_timer::singleton().time_to_update(index_counter))
         {
             if (off == total) {
@@ -69,45 +77,43 @@ public:
             } else {
                 lnav_data.ld_bottom_source.update_loading(off, total);
             }
-            do_observer_update(lf);
+            lnav_data.ld_status[LNS_BOTTOM].set_needs_update();
+            if (do_observer_update(lf) == lnav::progress_result_t::interrupt) {
+                retval = lnav::progress_result_t::interrupt;
+            }
             this->lo_last_offset = off;
         }
 
         if (!lnav_data.ld_looping) {
-            return indexing_result::BREAK;
+            retval = lnav::progress_result_t::interrupt;
         }
-        return indexing_result::CONTINUE;
+        return retval;
     }
 
     off_t lo_last_offset;
 };
 
-void
-do_observer_update(const std::shared_ptr<logfile>& lf)
+lnav::progress_result_t
+do_observer_update(const logfile* lf)
 {
-    if (isendwin()) {
-        return;
-    }
-    lnav_data.ld_status_refresher();
-    if (lf && lnav_data.ld_mode == ln_mode_t::FILES
+    if (lf != nullptr && lnav_data.ld_mode == ln_mode_t::FILES
         && lnav_data.ld_exec_phase < lnav_exec_phase::INTERACTIVE)
     {
-        auto& fc = lnav_data.ld_active_files;
-        auto iter = std::find(fc.fc_files.begin(), fc.fc_files.end(), lf);
+        const auto& fc = lnav_data.ld_active_files;
+        size_t index = 0;
 
-        if (iter != fc.fc_files.end()) {
-            auto index = std::distance(fc.fc_files.begin(), iter);
+        for (const auto& curr_file : fc.fc_files) {
+            if (curr_file.get() != lf) {
+                index++;
+                continue;
+            }
             lnav_data.ld_files_view.set_selection(
                 vis_line_t(fc.fc_other_files.size() + index));
             lnav_data.ld_files_view.reload_data();
             lnav_data.ld_files_view.do_update();
         }
     }
-    if (handle_winch()) {
-        layout_views();
-        lnav_data.ld_view_stack.do_update();
-    }
-    refresh();
+    return lnav_data.ld_status_refresher(lnav::func::op_type::interactive);
 }
 
 void
@@ -137,8 +143,8 @@ public:
         auto& ftf = lnav_data.ld_files_to_front;
 
         ftf.remove_if([&lf](const auto& elem) {
-            return elem.first == lf->get_filename()
-                || elem.first == lf->get_open_options().loo_filename;
+            return elem == lf->get_filename()
+                || elem == lf->get_open_options().loo_filename;
         });
         if (lnav_data.ld_log_source.insert_file(lf)) {
             this->did_promotion = true;
@@ -153,6 +159,10 @@ public:
                     lnav_data.ld_vtab_manager->register_vtab(vt);
                 }
             }
+            if (lf->get_open_options().loo_source == logfile_name_source::USER)
+            {
+                lf->set_include_in_session(true);
+            }
 
             auto iter = session_data.sd_file_states.find(lf->get_filename());
             if (iter != session_data.sd_file_states.end()) {
@@ -179,11 +189,11 @@ public:
         const auto& ftf = lnav_data.ld_files_to_front;
 
         if (!ftf.empty()
-            && (ftf.front().first == lf->get_filename()
-                || ftf.front().first == lf->get_open_options().loo_filename))
+            && (ftf.front() == lf->get_filename()
+                || ftf.front() == lf->get_open_options().loo_filename))
         {
             this->front_file = lf;
-            this->front_top = lnav_data.ld_files_to_front.front().second;
+            this->front_top = lf->get_open_options().loo_init_location;
 
             lnav_data.ld_files_to_front.pop_front();
         }
@@ -200,11 +210,11 @@ public:
 };
 
 rebuild_indexes_result_t
-rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
+rebuild_indexes(std::optional<ui_clock::time_point> deadline)
 {
-    logfile_sub_source& lss = lnav_data.ld_log_source;
-    textview_curses& log_view = lnav_data.ld_views[LNV_LOG];
-    textview_curses& text_view = lnav_data.ld_views[LNV_TEXT];
+    auto& lss = lnav_data.ld_log_source;
+    auto& log_view = lnav_data.ld_views[LNV_LOG];
+    auto& text_view = lnav_data.ld_views[LNV_TEXT];
     bool scroll_downs[LNV__MAX];
     rebuild_indexes_result_t retval;
 
@@ -227,6 +237,7 @@ rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
         }
     }
 
+    // log_trace("rescanning text files");
     {
         auto* tss = &lnav_data.ld_text_source;
         textfile_callback cb;
@@ -247,37 +258,56 @@ rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
                 tss->to_front(cb.front_file);
             }
 
-            nonstd::optional<vis_line_t> new_top_opt;
-            cb.front_top.match(
-                [&new_top_opt](vis_line_t vl) {
-                    log_info("file open request to jump to line: %d", (int) vl);
-                    if (vl < 0_vl) {
-                        vl += lnav_data.ld_views[LNV_TEXT].get_inner_height();
-                    }
-                    if (vl < lnav_data.ld_views[LNV_TEXT].get_inner_height()) {
-                        new_top_opt = vl;
-                    }
-                },
-                [&new_top_opt](const std::string& loc) {
-                    log_info("file open request to jump to anchor: %s",
-                             loc.c_str());
-                    auto* ta = dynamic_cast<text_anchors*>(
-                        lnav_data.ld_views[LNV_TEXT].get_sub_source());
+            std::optional<vis_line_t> new_top_opt;
+            if (cb.front_top.valid()) {
+                cb.front_top.match(
+                    [&new_top_opt, &cb](file_location_tail tail) {
+                        switch (cb.front_file->get_text_format()) {
+                            case text_format_t::TF_UNKNOWN:
+                            case text_format_t::TF_LOG:
+                                log_info("file open request to tail");
+                                break;
+                            default:
+                                log_info("file open is %s, moving to top",
+                                         fmt::to_string(
+                                             cb.front_file->get_text_format())
+                                             .c_str());
+                                new_top_opt = 0_vl;
+                                break;
+                        }
+                    },
+                    [&new_top_opt](int vl) {
+                        log_info("file open request to jump to line: %d", vl);
+                        if (vl < 0) {
+                            vl += lnav_data.ld_views[LNV_TEXT]
+                                      .get_inner_height();
+                        }
+                        if (vl
+                            < lnav_data.ld_views[LNV_TEXT].get_inner_height()) {
+                            new_top_opt = vis_line_t(vl);
+                        }
+                    },
+                    [&new_top_opt](const std::string& loc) {
+                        log_info("file open request to jump to anchor: %s",
+                                 loc.c_str());
+                        auto* ta = dynamic_cast<text_anchors*>(
+                            lnav_data.ld_views[LNV_TEXT].get_sub_source());
 
-                    if (ta != nullptr) {
-                        new_top_opt = ta->row_for_anchor(loc);
-                    }
-                });
+                        if (ta != nullptr) {
+                            new_top_opt = ta->row_for_anchor(loc);
+                        }
+                    });
+            }
             if (new_top_opt) {
                 log_info("  setting requested top line: %d",
                          (int) new_top_opt.value());
                 text_view.set_selection(new_top_opt.value());
                 log_info("  actual top is now: %d", (int) text_view.get_top());
                 log_info("  actual selection is now: %d",
-                         (int) text_view.get_selection());
+                         (int) text_view.get_selection().value_or(-1_vl));
                 scroll_downs[LNV_TEXT] = false;
             } else {
-                log_warning("could not jump to requested line");
+                log_info("no line requested");
             }
         }
         if (cb.did_promotion && deadline) {
@@ -287,19 +317,24 @@ rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
         }
     }
 
+    // log_trace("closing files");
     std::vector<std::shared_ptr<logfile>> closed_files;
     for (auto& lf : lnav_data.ld_active_files.fc_files) {
-        if ((!lf->exists() || lf->is_closed())) {
-            log_info("closed log file: %s", lf->get_filename().c_str());
+        if (!lf->exists() || lf->is_closed()) {
+            log_info("%s file: %s",
+                     !lf->exists() ? "deleted" : "closed",
+                     lf->get_filename().c_str());
             lnav_data.ld_text_source.remove(lf);
             lnav_data.ld_log_source.remove_file(lf);
             closed_files.emplace_back(lf);
+            retval.rir_rescan_needed = true;
         }
     }
     if (!closed_files.empty()) {
         lnav_data.ld_active_files.close_files(closed_files);
     }
 
+    // log_trace("rebuilding logs indexes");
     auto result = lss.rebuild_index(deadline);
     if (result != logfile_sub_source::rebuild_result::rr_no_change) {
         size_t new_count = lss.text_line_count();
@@ -347,6 +382,7 @@ rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
             }
 
             if (reload) {
+                log_trace("text filters changed");
                 lss.text_filters_changed();
             }
         }
@@ -354,6 +390,7 @@ rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
         retval.rir_changes += 1;
     }
 
+    // log_trace("updating top/selections");
     for (auto lpc : {LNV_LOG, LNV_TEXT}) {
         auto& scroll_view = lnav_data.ld_views[lpc];
 
@@ -373,10 +410,11 @@ rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
     }
 
     lnav_data.ld_view_stack.top() | [&closed_files, &retval](auto tc) {
-        if (!closed_files.empty() && tc == &lnav_data.ld_views[LNV_GANTT]) {
-            auto* gantt_source = lnav_data.ld_views[LNV_GANTT].get_sub_source();
-            if (gantt_source != nullptr) {
-                gantt_source->text_filters_changed();
+        if (!closed_files.empty() && tc == &lnav_data.ld_views[LNV_TIMELINE]) {
+            auto* timeline_source
+                = lnav_data.ld_views[LNV_TIMELINE].get_sub_source();
+            if (timeline_source != nullptr) {
+                timeline_source->text_filters_changed();
             }
         }
 
@@ -386,6 +424,7 @@ rebuild_indexes(nonstd::optional<ui_clock::time_point> deadline)
             lnav_data.ld_scroll_broadcaster(tc);
         }
     };
+    // log_trace("done");
 
     return retval;
 }
@@ -411,7 +450,7 @@ update_active_files(file_collection& new_files)
         return true;
     }
 
-    bool was_below_open_file_limit
+    const auto was_below_open_file_limit
         = lnav_data.ld_active_files.is_below_open_file_limit();
 
     for (const auto& lf : new_files.fc_files) {
@@ -444,7 +483,8 @@ update_active_files(file_collection& new_files)
         });
 
     if (was_below_open_file_limit
-        && !lnav_data.ld_active_files.is_below_open_file_limit())
+        && !lnav_data.ld_active_files.is_below_open_file_limit()
+        && !lnav_data.ld_exec_context.ec_msg_callback_stack.empty())
     {
         auto um
             = lnav::console::user_message::error("Unable to open more files")
@@ -456,10 +496,10 @@ update_active_files(file_collection& new_files)
                   .with_help(
                       attr_line_t("Use ")
                           .append("ulimit -n"_quoted_code)
-                          .append(
-                              " to increase the limit before running lnav"));
+                          .append(" to increase the limit before running lnav"))
+                  .move();
 
-        lnav_data.ld_exec_context.ec_error_callback_stack.back()(um);
+        lnav_data.ld_exec_context.ec_msg_callback_stack.back()(um);
     }
 
     return true;
@@ -506,7 +546,7 @@ rescan_files(bool req)
         if (!done && !(lnav_data.ld_flags & LNF_HEADLESS)) {
             lnav_data.ld_files_view.set_needs_update();
             lnav_data.ld_files_view.do_update();
-            lnav_data.ld_status_refresher();
+            lnav_data.ld_status_refresher(lnav::func::op_type::interactive);
         }
     } while (!done && lnav_data.ld_looping);
     return true;
