@@ -86,19 +86,22 @@ is_dev_null(int fd)
     return is_dev_null(fd_stat);
 }
 
-void
+size_t
 write_line_to(FILE* outfile, const attr_line_t& al)
 {
     const auto& al_attrs = al.get_attrs();
-    auto lr = find_string_attr_range(al_attrs, &SA_ORIGINAL_LINE);
+    const auto lr = find_string_attr_range(al_attrs, &SA_ORIGINAL_LINE);
 
-    if (!lr.is_valid() || lr.lr_start > 1) {
+    if (lr.empty() || !lr.is_valid() || lr.lr_start > 1) {
         // If the line is prefixed with some extra information, include that
         // in the output.  For example, the log file name or time offset.
         lnav::console::println(outfile, al);
-    } else {
-        lnav::console::println(outfile, al.subline(lr.lr_start, lr.length()));
+        return al.column_width();
     }
+    const auto sub_al = al.subline(lr.lr_start, lr.length());
+    lnav::console::println(outfile, sub_al);
+
+    return sub_al.column_width();
 }
 
 namespace lnav {
@@ -151,7 +154,11 @@ to_json(yajlpp_gen& gen, const attr_line_t& al)
                     [&](const bookmark_metadata* bm) { elem_map.gen(""); },
                     [&](const timespec& ts) { elem_map.gen(""); },
                     [&](const string_fragment& sf) { elem_map.gen(sf); },
-                    [&](const block_elem_t& be) { elem_map.gen(""); });
+                    [&](const block_elem_t& be) { elem_map.gen(""); },
+                    [&](const styling::color_unit& rgb) { elem_map.gen(""); },
+                    [&](const ui_icon_t& ic) { elem_map.gen(""); },
+                    [&](const ui_command& uc) { elem_map.gen(""); },
+                    [&](const text_format_t tf) { elem_map.gen(""); });
             }
         }
     }
@@ -234,7 +241,8 @@ to_json(const lnav::console::user_message& um)
 static int
 read_string_attr_type(yajlpp_parse_context* ypc,
                       const unsigned char* str,
-                      size_t len)
+                      size_t len,
+                      yajl_string_props_t*)
 {
     auto* sa = (string_attr*) ypc->ypc_obj_stack.top();
     auto type = std::string((const char*) str, len);
@@ -261,7 +269,7 @@ read_string_attr_int_value(yajlpp_parse_context* ypc, long long in)
         sa->sa_value = static_cast<role_t>(in);
     } else if (sa->sa_type == &VC_STYLE) {
         sa->sa_value = text_attrs{
-            static_cast<int32_t>(in),
+            static_cast<uint32_t>(in),
         };
     }
     return 1;
