@@ -30,6 +30,8 @@
  */
 
 #include <algorithm>
+#include <chrono>
+#include <memory>
 #include <utility>
 
 #include "log_format.hh"
@@ -41,7 +43,8 @@
 #include "config.h"
 #include "formats/logfmt/logfmt.parser.hh"
 #include "log_vtab_impl.hh"
-#include "scn/scn.h"
+#include "ptimec.hh"
+#include "scn/scan.h"
 #include "sql_util.hh"
 #include "yajlpp/yajlpp.hh"
 
@@ -66,30 +69,50 @@ public:
         {
             dst.emplace_back(
                 li.li_file_range.fr_offset, li.li_timestamp, li.li_level);
-            return scan_match{100};
+            return scan_match{1};
         }
 
-        return scan_no_match{""};
+        return scan_no_match{"not a piper capture"};
     }
 
-    void annotate(uint64_t line_number,
+    static constexpr int TIMESTAMP_SIZE = 28;
+
+    void annotate(logfile* lf,
+                  uint64_t line_number,
                   string_attrs_t& sa,
                   logline_value_vector& values,
                   bool annotate_module) const override
     {
-        auto lr = line_range{0, 0};
-        sa.emplace_back(lr, logline::L_TIMESTAMP.value());
+        auto lr = line_range{0, TIMESTAMP_SIZE};
+        sa.emplace_back(lr, L_TIMESTAMP.value());
+        log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
     void get_subline(const logline& ll,
                      shared_buffer_ref& sbr,
                      bool full_message) override
     {
-        this->plf_cached_line.resize(23);
-        sql_strftime(this->plf_cached_line.data(),
-                     this->plf_cached_line.size(),
-                     ll.get_timeval(),
-                     'T');
+        this->plf_cached_line.resize(TIMESTAMP_SIZE);
+        auto tlen = sql_strftime(this->plf_cached_line.data(),
+                                 this->plf_cached_line.size(),
+                                 ll.get_timeval(),
+                                 'T');
+        this->plf_cached_line.resize(tlen);
+        {
+            char zone_str[16];
+            exttm tmptm;
+
+            tmptm.et_flags |= ETF_ZONE_SET;
+            tmptm.et_gmtoff
+                = lnav::local_time_to_info(
+                      date::local_seconds{ll.get_time<std::chrono::seconds>()})
+                      .first.offset.count();
+            off_t zone_len = 0;
+            ftime_z(zone_str, zone_len, sizeof(zone_str), tmptm);
+            for (off_t lpc = 0; lpc < zone_len; lpc++) {
+                this->plf_cached_line.push_back(zone_str[lpc]);
+            }
+        }
         this->plf_cached_line.push_back(' ');
         const auto prefix_len = this->plf_cached_line.size();
         this->plf_cached_line.resize(this->plf_cached_line.size()
@@ -107,6 +130,7 @@ public:
         auto retval = std::make_shared<piper_log_format>(*this);
 
         retval->lf_specialized = true;
+        retval->lf_timestamp_flags |= ETF_ZONE_SET | ETF_MICROS_SET;
         return retval;
     }
 
@@ -123,7 +147,7 @@ public:
             pcre_format(
                 "^(?:\\*\\*\\*\\s+)?(?<timestamp>@[0-9a-zA-Z]{16,24})(.*)"),
             pcre_format(
-                R"(^(?:\*\*\*\s+)?(?<timestamp>(?:\s|\d{4}[\-\/]\d{2}[\-\/]\d{2}|T|\d{1,2}:\d{2}(?::\d{2}(?:[\.,]\d{1,6})?)?|Z|[+\-]\d{2}:?\d{2}|(?!ERR|INFO|WARN)[A-Z]{3,4})+)(?:\s+|[:|])([^:]+))"),
+                R"(^(?:\*\*\*\s+)?(?<timestamp>(?:\s|\d{4}[\-\/]\d{2}[\-\/]\d{2}|T|\d{1,2}:\d{2}(?::\d{2}(?:[\.,]\d{1,6})?)?|Z|[+\-]\d{2}:?\d{2}|(?!DBG|ERR|INFO|WARN|NONE)[A-Z]{3,4})+)(?:\s+|[:|])([^:]+))"),
             pcre_format(
                 "^(?:\\*\\*\\*\\s+)?(?<timestamp>[\\w:+/\\.-]+) \\[\\w (.*)"),
             pcre_format("^(?:\\*\\*\\*\\s+)?(?<timestamp>[\\w:,/\\.-]+) (.*)"),
@@ -172,10 +196,10 @@ public:
                        shared_buffer_ref& sbr,
                        scan_batch_context& sbc) override
     {
-        struct exttm log_time;
-        struct timeval log_tv;
+        exttm log_time;
+        timeval log_tv;
         string_fragment ts;
-        nonstd::optional<string_fragment> level;
+        std::optional<string_fragment> level;
         const char* last_pos;
 
         if (dst.empty()) {
@@ -200,7 +224,7 @@ public:
                                         &level))
             != nullptr)
         {
-            log_level_t level_val = log_level_t::LEVEL_UNKNOWN;
+            auto level_val = log_level_t::LEVEL_UNKNOWN;
             if (level) {
                 level_val = string2level(level->data(), level->length());
             }
@@ -214,11 +238,17 @@ public:
 
             if (!(this->lf_timestamp_flags
                   & (ETF_MILLIS_SET | ETF_MICROS_SET | ETF_NANOS_SET))
-                && !dst.empty() && dst.back().get_time() == log_tv.tv_sec
-                && dst.back().get_millis() != 0)
+                && !dst.empty()
+                && dst.back().get_time<std::chrono::seconds>().count()
+                    == log_tv.tv_sec
+                && dst.back()
+                        .get_subsecond_time<std::chrono::microseconds>()
+                        .count()
+                    != 0)
             {
                 auto log_ms
-                    = std::chrono::milliseconds(dst.back().get_millis());
+                    = dst.back()
+                          .get_subsecond_time<std::chrono::microseconds>();
 
                 log_time.et_nsec
                     = std::chrono::duration_cast<std::chrono::nanoseconds>(
@@ -231,13 +261,14 @@ public:
             }
 
             dst.emplace_back(li.li_file_range.fr_offset, log_tv, level_val);
-            return scan_match{0};
+            return scan_match{5};
         }
 
         return scan_no_match{"no patterns matched"};
     }
 
-    void annotate(uint64_t line_number,
+    void annotate(logfile* lf,
+                  uint64_t line_number,
                   string_attrs_t& sa,
                   logline_value_vector& values,
                   bool annotate_module) const override
@@ -257,7 +288,7 @@ public:
 
         auto ts_cap = md[fmt.pf_timestamp_index].value();
         auto lr = to_line_range(ts_cap.trim());
-        sa.emplace_back(lr, logline::L_TIMESTAMP.value());
+        sa.emplace_back(lr, L_TIMESTAMP.value());
 
         values.lvv_values.emplace_back(TS_META, line, lr);
         values.lvv_values.back().lv_meta.lvm_format = (log_format*) this;
@@ -274,16 +305,23 @@ public:
                     LEVEL_META, line, to_line_range(level_cap->trim()));
                 values.lvv_values.back().lv_meta.lvm_format
                     = (log_format*) this;
+
+                lr = to_line_range(level_cap->trim());
+                if (lr.lr_end != (ssize_t) line.length()) {
+                    sa.emplace_back(lr, L_LEVEL.value());
+                }
             }
         }
 
         lr.lr_start = 0;
         lr.lr_end = prefix_len;
-        sa.emplace_back(lr, logline::L_PREFIX.value());
+        sa.emplace_back(lr, L_PREFIX.value());
 
         lr.lr_start = prefix_len;
         lr.lr_end = line.length();
         sa.emplace_back(lr, SA_BODY.value());
+
+        log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
     std::shared_ptr<log_format> specialized(int fmt_lock) override
@@ -299,10 +337,15 @@ public:
         if (field_name == TS_META.lvm_name) {
             TS_META.lvm_user_hidden = val;
             return true;
-        } else if (field_name == LEVEL_META.lvm_name) {
+        }
+        if (field_name == LEVEL_META.lvm_name) {
             LEVEL_META.lvm_user_hidden = val;
             return true;
         }
+        if (field_name == OPID_META.lvm_name) {
+            OPID_META.lvm_user_hidden = val;
+            return true;
+        }
         return false;
     }
 
@@ -311,12 +354,14 @@ public:
         return {
             {TS_META.lvm_name, TS_META},
             {LEVEL_META.lvm_name, LEVEL_META},
+            {OPID_META.lvm_name, OPID_META},
         };
     }
 
 private:
     static logline_value_meta TS_META;
     static logline_value_meta LEVEL_META;
+    static logline_value_meta OPID_META;
 };
 
 logline_value_meta generic_log_format::TS_META{
@@ -324,10 +369,17 @@ logline_value_meta generic_log_format::TS_META{
     value_kind_t::VALUE_TEXT,
     logline_value_meta::table_column{2},
 };
+
 logline_value_meta generic_log_format::LEVEL_META{
     intern_string::lookup("log_level"),
     value_kind_t::VALUE_TEXT,
-    logline_value_meta::table_column{4},
+    logline_value_meta::table_column{3},
+};
+
+logline_value_meta generic_log_format::OPID_META{
+    intern_string::lookup("log_opid"),
+    value_kind_t::VALUE_TEXT,
+    logline_value_meta::internal_column{},
 };
 
 std::string
@@ -356,7 +408,7 @@ from_escaped_string(const char* str, size_t len)
     return retval;
 }
 
-nonstd::optional<const char*>
+std::optional<const char*>
 lnav_strnstr(const char* s, const char* find, size_t slen)
 {
     char c, sc;
@@ -367,13 +419,13 @@ lnav_strnstr(const char* s, const char* find, size_t slen)
         do {
             do {
                 if (slen < 1 || (sc = *s) == '\0') {
-                    return nonstd::nullopt;
+                    return std::nullopt;
                 }
                 --slen;
                 ++s;
             } while (sc != c);
             if (len > slen) {
-                return nonstd::nullopt;
+                return std::nullopt;
             }
         } while (strncmp(s, find, len) != 0);
         s--;
@@ -470,11 +522,12 @@ struct separated_string {
 
 class bro_log_format : public log_format {
 public:
+    static const intern_string_t TS;
     struct field_def {
         logline_value_meta fd_meta;
         logline_value_meta* fd_root_meta;
         std::string fd_collator;
-        nonstd::optional<size_t> fd_numeric_index;
+        std::optional<size_t> fd_numeric_index;
 
         explicit field_def(const intern_string_t name,
                            size_t col,
@@ -549,7 +602,6 @@ public:
     {
         static const intern_string_t STATUS_CODE
             = intern_string::lookup("bro_status_code");
-        static const intern_string_t TS = intern_string::lookup("bro_ts");
         static const intern_string_t UID = intern_string::lookup("bro_uid");
         static const intern_string_t ID_ORIG_H
             = intern_string::lookup("bro_id_orig_h");
@@ -567,7 +619,7 @@ public:
 
         for (auto iter = ss.begin(); iter != ss.end(); ++iter) {
             if (iter.index() == 0 && *iter == "#close") {
-                return scan_match{0};
+                return scan_match{2000};
             }
 
             if (iter.index() >= this->blf_field_defs.size()) {
@@ -577,10 +629,11 @@ public:
             const auto& fd = this->blf_field_defs[iter.index()];
 
             if (TS == fd.fd_meta.lvm_name) {
-                string_fragment sf = *iter;
+                static const char* const TIME_FMT[] = {"%s.%f"};
+                const auto sf = *iter;
 
                 if (this->lf_date_time.scan(
-                        sf.data(), sf.length(), nullptr, &tm, tv))
+                        sf.data(), sf.length(), TIME_FMT, &tm, tv))
                 {
                     this->lf_timestamp_flags = tm.et_flags;
                     found_ts = true;
@@ -607,7 +660,7 @@ public:
                         auto scan_float_res = scn::scan_value<double>(sv);
                         if (scan_float_res) {
                             this->lf_value_stats[fd.fd_numeric_index.value()]
-                                .add_value(scan_float_res.value());
+                                .add_value(scan_float_res->value());
                         }
                         break;
                     }
@@ -625,18 +678,8 @@ public:
             }
 
             if (opid_cap.is_valid()) {
-                auto opid_iter = sbc.sbc_opids.los_opid_ranges.find(opid_cap);
-
-                if (opid_iter == sbc.sbc_opids.los_opid_ranges.end()) {
-                    auto opid_copy = opid_cap.to_owned(sbc.sbc_allocator);
-                    auto otr = opid_time_range{time_range{tv, tv}};
-                    auto emplace_res
-                        = sbc.sbc_opids.los_opid_ranges.emplace(opid_copy, otr);
-                    opid_iter = emplace_res.first;
-                } else {
-                    opid_iter->second.otr_range.extend_to(tv);
-                }
-
+                auto opid_iter
+                    = sbc.sbc_opids.insert_op(sbc.sbc_allocator, opid_cap, tv);
                 opid_iter->second.otr_level_stats.update_msg_count(level);
 
                 auto& otr = opid_iter->second;
@@ -649,9 +692,9 @@ public:
                 }
             }
             dst.emplace_back(li.li_file_range.fr_offset, tv, level, 0, opid);
-            return scan_match{0};
+            return scan_match{2000};
         }
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
     scan_result_t scan(logfile& lf,
@@ -681,7 +724,7 @@ public:
         if (dst.empty() || dst.size() > 20 || sbr.empty()
             || sbr.get_data()[0] == '#')
         {
-            return scan_no_match{};
+            return scan_no_match{"no header found"};
         }
 
         auto line_iter = dst.begin();
@@ -808,7 +851,7 @@ public:
                         fd.with_kind(value_kind_t::VALUE_BOOLEAN);
                     } else if (field_type == "addr") {
                         fd.with_kind(
-                            value_kind_t::VALUE_TEXT, true, "ipaddress");
+                            value_kind_t::VALUE_TEXT, true, false, "ipaddress");
                     } else if (field_type == "port") {
                         fd.with_kind(value_kind_t::VALUE_INTEGER, true);
                     } else if (field_type == "interval") {
@@ -833,15 +876,15 @@ public:
         this->blf_format_name.clear();
         this->lf_value_stats.clear();
 
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
-    void annotate(uint64_t line_number,
+    void annotate(logfile* lf,
+                  uint64_t line_number,
                   string_attrs_t& sa,
                   logline_value_vector& values,
                   bool annotate_module) const override
     {
-        static const intern_string_t TS = intern_string::lookup("bro_ts");
         static const intern_string_t UID = intern_string::lookup("bro_uid");
 
         auto& sbr = values.lvv_sbr;
@@ -866,9 +909,9 @@ public:
             auto lr = line_range(sf.sf_begin, sf.sf_end);
 
             if (fd.fd_meta.lvm_name == TS) {
-                sa.emplace_back(lr, logline::L_TIMESTAMP.value());
+                sa.emplace_back(lr, L_TIMESTAMP.value());
             } else if (fd.fd_meta.lvm_name == UID) {
-                sa.emplace_back(lr, logline::L_OPID.value());
+                sa.emplace_back(lr, L_OPID.value());
             }
 
             if (lr.is_valid()) {
@@ -879,6 +922,8 @@ public:
             values.lvv_values.back().lv_meta.lvm_user_hidden
                 = fd.fd_root_meta->lvm_user_hidden;
         }
+
+        log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
     const logline_value_stats* stats_for_value(
@@ -900,8 +945,12 @@ public:
         return retval;
     }
 
-    bool hide_field(const intern_string_t field_name, bool val) override
+    bool hide_field(intern_string_t field_name, bool val) override
     {
+        if (field_name == LOG_TIME_STR) {
+            field_name = TS;
+        }
+
         auto fd_iter = FIELD_META.find(field_name);
         if (fd_iter == FIELD_META.end()) {
             return false;
@@ -954,13 +1003,13 @@ public:
         }
 
         void get_foreign_keys(
-            std::vector<std::string>& keys_inout) const override
+            std::unordered_set<std::string>& keys_inout) const override
         {
             this->log_vtab_impl::get_foreign_keys(keys_inout);
 
             for (const auto& fd : this->blt_format.blf_field_defs) {
                 if (fd.fd_meta.lvm_identifier || fd.fd_meta.lvm_foreign_key) {
-                    keys_inout.push_back(fd.fd_meta.lvm_name.to_string());
+                    keys_inout.emplace(fd.fd_meta.lvm_name.to_string());
                 }
             }
         }
@@ -985,7 +1034,7 @@ public:
         std::shared_ptr<bro_log_table> retval = nullptr;
 
         auto& tables = get_tables();
-        auto iter = tables.find(this->blf_format_name);
+        const auto iter = tables.find(this->blf_format_name);
         if (iter == tables.end()) {
             retval = std::make_shared<bro_log_table>(*this);
             tables[this->blf_format_name] = retval;
@@ -1011,6 +1060,8 @@ public:
 std::unordered_map<const intern_string_t, logline_value_meta>
     bro_log_format::FIELD_META;
 
+const intern_string_t bro_log_format::TS = intern_string::lookup("bro_ts");
+
 struct ws_separated_string {
     const char* ss_str;
     size_t ss_len;
@@ -1110,12 +1161,15 @@ struct ws_separated_string {
 
 class w3c_log_format : public log_format {
 public:
+    static const intern_string_t F_DATE;
+    static const intern_string_t F_TIME;
+
     struct field_def {
         const intern_string_t fd_name;
         logline_value_meta fd_meta;
         logline_value_meta* fd_root_meta{nullptr};
         std::string fd_collator;
-        nonstd::optional<size_t> fd_numeric_index;
+        std::optional<size_t> fd_numeric_index;
 
         explicit field_def(const intern_string_t name)
             : fd_name(name), fd_meta(intern_string::lookup(sql_safe_ident(
@@ -1206,12 +1260,10 @@ public:
                            const line_info& li,
                            shared_buffer_ref& sbr)
     {
-        static const intern_string_t F_DATE = intern_string::lookup("date");
         static const intern_string_t F_DATE_LOCAL
             = intern_string::lookup("date-local");
         static const intern_string_t F_DATE_UTC
             = intern_string::lookup("date-UTC");
-        static const intern_string_t F_TIME = intern_string::lookup("time");
         static const intern_string_t F_TIME_LOCAL
             = intern_string::lookup("time-local");
         static const intern_string_t F_TIME_UTC
@@ -1220,10 +1272,8 @@ public:
             = intern_string::lookup("sc-status");
 
         ws_separated_string ss(sbr.get_data(), sbr.length());
-        struct timeval date_tv {
-            0, 0
-        }, time_tv{0, 0};
-        struct exttm date_tm, time_tm;
+        timeval date_tv{0, 0}, time_tv{0, 0};
+        exttm date_tm, time_tm;
         bool found_date = false, found_time = false;
         log_level_t level = LEVEL_INFO;
 
@@ -1244,8 +1294,8 @@ public:
                     if (sbr_sf_opt) {
                         auto sbr_sf = sbr_sf_opt.value().trim();
                         date_time_scanner dts;
-                        struct exttm tm;
-                        struct timeval tv;
+                        exttm tm;
+                        timeval tv;
 
                         if (dts.scan(sbr_sf.data(),
                                      sbr_sf.length(),
@@ -1260,9 +1310,11 @@ public:
                         }
                     }
                 }
-                dst.emplace_back(
-                    li.li_file_range.fr_offset, 0, 0, LEVEL_IGNORE, 0);
-                return scan_match{0};
+                dst.emplace_back(li.li_file_range.fr_offset,
+                                 std::chrono::microseconds{0},
+                                 LEVEL_IGNORE,
+                                 0);
+                return scan_match{2000};
             }
 
             sf = sf.trim("\" \t");
@@ -1299,7 +1351,7 @@ public:
 
                         if (scan_float_res) {
                             this->lf_value_stats[fd.fd_numeric_index.value()]
-                                .add_value(scan_float_res.value());
+                                .add_value(scan_float_res->value());
                         }
                         break;
                     }
@@ -1310,8 +1362,7 @@ public:
         }
 
         if (found_time) {
-            struct exttm tm = time_tm;
-            struct timeval tv;
+            auto tm = time_tm;
 
             if (found_date) {
                 tm.et_tm.tm_year = date_tm.et_tm.tm_year;
@@ -1321,17 +1372,17 @@ public:
                 tm.et_tm.tm_yday = date_tm.et_tm.tm_yday;
             }
 
-            tv = tm.to_timeval();
+            auto tv = tm.to_timeval();
             if (!this->lf_specialized) {
                 for (auto& ll : dst) {
                     ll.set_ignore(true);
                 }
             }
             dst.emplace_back(li.li_file_range.fr_offset, tv, level, 0);
-            return scan_match{0};
+            return scan_match{2000};
         }
 
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
     scan_result_t scan(logfile& lf,
@@ -1366,7 +1417,7 @@ public:
         if (dst.empty() || dst.size() > 20 || sbr.empty()
             || sbr.get_data()[0] == '#')
         {
-            return scan_no_match{};
+            return scan_no_match{"no header found"};
         }
 
         this->clear();
@@ -1428,7 +1479,7 @@ public:
                             common_iter = emp_res.first;
                         }
                         fd.fd_root_meta = &common_iter->second;
-                    } else if (sf == "date" || sf == "time") {
+                    } else if (sf.is_one_of("date", "time")) {
                         this->wlf_field_defs.emplace_back(
                             intern_string::lookup(sf));
                         auto& fd = this->wlf_field_defs.back();
@@ -1476,7 +1527,7 @@ public:
                         }
                     }
                     auto& fd = this->wlf_field_defs.back();
-                    fd.fd_meta.lvm_format = nonstd::make_optional(this);
+                    fd.fd_meta.lvm_format = std::make_optional(this);
                     switch (fd.fd_meta.lvm_kind) {
                         case value_kind_t::VALUE_FLOAT:
                         case value_kind_t::VALUE_INTEGER:
@@ -1502,19 +1553,22 @@ public:
         this->wlf_format_name.clear();
         this->lf_value_stats.clear();
 
-        return scan_no_match{};
+        return scan_no_match{"no header found"};
     }
 
-    void annotate(uint64_t line_number,
+    void annotate(logfile* lf,
+                  uint64_t line_number,
                   string_attrs_t& sa,
                   logline_value_vector& values,
                   bool annotate_module) const override
     {
         auto& sbr = values.lvv_sbr;
         ws_separated_string ss(sbr.get_data(), sbr.length());
+        std::optional<line_range> date_lr;
+        std::optional<line_range> time_lr;
 
         for (auto iter = ss.begin(); iter != ss.end(); ++iter) {
-            string_fragment sf = *iter;
+            auto sf = *iter;
 
             if (iter.index() >= this->wlf_field_defs.size()) {
                 sa.emplace_back(line_range{sf.sf_begin, -1},
@@ -1531,6 +1585,11 @@ public:
             auto lr = line_range(sf.sf_begin, sf.sf_end);
 
             if (lr.is_valid()) {
+                if (fd.fd_meta.lvm_name == F_DATE) {
+                    date_lr = lr;
+                } else if (fd.fd_meta.lvm_name == F_TIME) {
+                    time_lr = lr;
+                }
                 values.lvv_values.emplace_back(fd.fd_meta, sbr, lr);
                 if (sf.startswith("\"")) {
                     auto& meta = values.lvv_values.back().lv_meta;
@@ -1549,6 +1608,18 @@ public:
                     = fd.fd_root_meta->lvm_user_hidden;
             }
         }
+        if (time_lr) {
+            auto ts_lr = time_lr.value();
+            if (date_lr) {
+                if (date_lr->lr_end + 1 == time_lr->lr_start) {
+                    ts_lr.lr_start = date_lr->lr_start;
+                    ts_lr.lr_end = time_lr->lr_end;
+                }
+            }
+
+            sa.emplace_back(ts_lr, L_TIMESTAMP.value());
+        }
+        log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
     const logline_value_stats* stats_for_value(
@@ -1572,6 +1643,18 @@ public:
 
     bool hide_field(const intern_string_t field_name, bool val) override
     {
+        if (field_name == LOG_TIME_STR) {
+            auto date_iter = FIELD_META.find(F_DATE);
+            auto time_iter = FIELD_META.find(F_TIME);
+            if (date_iter == FIELD_META.end() || time_iter == FIELD_META.end())
+            {
+                return false;
+            }
+            date_iter->second.lvm_user_hidden = val;
+            time_iter->second.lvm_user_hidden = val;
+            return true;
+        }
+
         auto fd_iter = FIELD_META.find(field_name);
         if (fd_iter == FIELD_META.end()) {
             return false;
@@ -1631,13 +1714,13 @@ public:
         };
 
         void get_foreign_keys(
-            std::vector<std::string>& keys_inout) const override
+            std::unordered_set<std::string>& keys_inout) const override
         {
             this->log_vtab_impl::get_foreign_keys(keys_inout);
 
             for (const auto& fd : KNOWN_FIELDS) {
                 if (fd.fd_meta.lvm_identifier || fd.fd_meta.lvm_foreign_key) {
-                    keys_inout.push_back(fd.fd_meta.lvm_name.to_string());
+                    keys_inout.emplace(fd.fd_meta.lvm_name.to_string());
                 }
             }
         }
@@ -1662,7 +1745,7 @@ public:
         std::shared_ptr<w3c_log_table> retval = nullptr;
 
         auto& tables = get_tables();
-        auto iter = tables.find(this->wlf_format_name);
+        const auto iter = tables.find(this->wlf_format_name);
         if (iter == tables.end()) {
             retval = std::make_shared<w3c_log_table>(*this);
             tables[this->wlf_format_name] = retval;
@@ -1685,6 +1768,9 @@ public:
 std::unordered_map<const intern_string_t, logline_value_meta>
     w3c_log_format::FIELD_META;
 
+const intern_string_t w3c_log_format::F_DATE = intern_string::lookup("date");
+const intern_string_t w3c_log_format::F_TIME = intern_string::lookup("time");
+
 static size_t KNOWN_FIELD_INDEX = 0;
 const std::vector<w3c_log_format::field_def> w3c_log_format::KNOWN_FIELDS = {
     {
@@ -1805,31 +1891,33 @@ struct logfmt_pair_handler {
     {
     }
 
-    bool process_value(const string_fragment& value_frag)
+    log_format::scan_result_t process_value(const string_fragment& value_frag)
     {
-        if (this->lph_key_frag == "time" || this->lph_key_frag == "ts") {
+        if (this->lph_key_frag.is_one_of("timestamp", "time", "ts", "t")) {
             if (!this->lph_dt_scanner.scan(value_frag.data(),
                                            value_frag.length(),
                                            nullptr,
                                            &this->lph_time_tm,
                                            this->lph_tv))
             {
-                return false;
+                return log_format::scan_no_match{
+                    "timestamp value did not parse correctly"};
             }
+            char buf[1024];
+            this->lph_dt_scanner.ftime(
+                buf, sizeof(buf), nullptr, this->lph_time_tm);
             this->lph_found_time = true;
-        } else if (this->lph_key_frag == "level") {
+        } else if (this->lph_key_frag == "level"_frag) {
             this->lph_level
                 = string2level(value_frag.data(), value_frag.length());
         }
-        return true;
+        return log_format::scan_match{};
     }
 
     date_time_scanner& lph_dt_scanner;
     bool lph_found_time{false};
-    struct exttm lph_time_tm {};
-    struct timeval lph_tv {
-        0, 0
-    };
+    exttm lph_time_tm;
+    timeval lph_tv{0, 0};
     log_level_t lph_level{log_level_t::LEVEL_INFO};
     string_fragment lph_key_frag{""};
 };
@@ -1890,34 +1978,42 @@ public:
         while (!done) {
             auto parse_result = p.step();
 
-            done = parse_result.match(
-                [](const logfmt::parser::end_of_input&) { return true; },
-                [&lph](const logfmt::parser::kvpair& kvp) {
+            auto value_res = parse_result.match(
+                [&done](const logfmt::parser::end_of_input&) -> scan_result_t {
+                    done = true;
+                    return scan_match{};
+                },
+                [&lph](const logfmt::parser::kvpair& kvp) -> scan_result_t {
                     lph.lph_key_frag = kvp.first;
 
                     return kvp.second.match(
-                        [](const logfmt::parser::bool_value& bv) {
-                            return false;
-                        },
-                        [&lph](const logfmt::parser::float_value& fv) {
+                        [](const logfmt::parser::bool_value& bv)
+                            -> scan_result_t { return scan_match{}; },
+                        [&lph](const logfmt::parser::float_value& fv)
+                            -> scan_result_t {
                             return lph.process_value(fv.fv_str_value);
                         },
-                        [&lph](const logfmt::parser::int_value& iv) {
+                        [&lph](const logfmt::parser::int_value& iv)
+                            -> scan_result_t {
                             return lph.process_value(iv.iv_str_value);
                         },
-                        [&lph](const logfmt::parser::quoted_value& qv) {
+                        [&lph](const logfmt::parser::quoted_value& qv)
+                            -> scan_result_t {
                             auto_mem<yajl_handle_t> handle(yajl_free);
                             yajl_callbacks cb;
+                            scan_result_t retval;
 
                             memset(&cb, 0, sizeof(cb));
                             handle = yajl_alloc(&cb, nullptr, &lph);
                             cb.yajl_string = +[](void* ctx,
                                                  const unsigned char* str,
-                                                 size_t len) -> int {
+                                                 size_t len,
+                                                 yajl_string_props_t*) -> int {
                                 auto& lph = *((logfmt_pair_handler*) ctx);
                                 string_fragment value_frag{str, 0, (int) len};
 
-                                return lph.process_value(value_frag);
+                                auto value_res = lph.process_value(value_frag);
+                                return value_res.is<scan_match>();
                             };
 
                             if (yajl_parse(
@@ -1938,28 +2034,35 @@ public:
                                 return lph.process_value(unq_frag);
                             }
 
-                            return false;
+                            return scan_match{};
                         },
-                        [&lph](const logfmt::parser::unquoted_value& uv) {
+                        [&lph](const logfmt::parser::unquoted_value& uv)
+                            -> scan_result_t {
                             return lph.process_value(uv.uv_value);
                         });
                 },
-                [](const logfmt::parser::error& err) {
+                [](const logfmt::parser::error& err) -> scan_result_t {
                     // log_error("logfmt parse error: %s", err.e_msg.c_str());
-                    return true;
+                    return scan_no_match{};
                 });
+            if (value_res.is<scan_no_match>()) {
+                retval = value_res;
+                done = true;
+            }
         }
 
         if (lph.lph_found_time) {
+            this->lf_timestamp_flags = lph.lph_time_tm.et_flags;
             dst.emplace_back(
                 li.li_file_range.fr_offset, lph.lph_tv, lph.lph_level);
-            retval = scan_match{0};
+            retval = scan_match{2000};
         }
 
         return retval;
     }
 
-    void annotate(uint64_t line_number,
+    void annotate(logfile* lf,
+                  uint64_t line_number,
                   string_attrs_t& sa,
                   logline_value_vector& values,
                   bool annotate_module) const override
@@ -2028,10 +2131,11 @@ public:
                     auto value_lr
                         = line_range{value_frag.sf_begin, value_frag.sf_end};
 
-                    if (kvp.first == "time" || kvp.first == "ts") {
-                        sa.emplace_back(value_lr, logline::L_TIMESTAMP.value());
-                    } else if (kvp.first == "level") {
-                    } else if (kvp.first == "msg") {
+                    if (kvp.first.is_one_of("timestamp", "time", "ts", "t")) {
+                        sa.emplace_back(value_lr, L_TIMESTAMP.value());
+                    } else if (kvp.first == "level"_frag) {
+                        sa.emplace_back(value_lr, L_LEVEL.value());
+                    } else if (kvp.first == "msg"_frag) {
                         sa.emplace_back(value_lr, SA_BODY.value());
                     } else if (kvp.second.is<logfmt::parser::quoted_value>()
                                || kvp.second
@@ -2060,6 +2164,8 @@ public:
                     return true;
                 });
         }
+
+        log_format::annotate(lf, line_number, sa, values, annotate_module);
     }
 
     std::shared_ptr<log_format> specialized(int fmt_lock) override
