@@ -34,9 +34,12 @@
 #include "config.h"
 #include "sql_util.hh"
 
-const static std::string MATCH_INDEX = "match_index";
+static const std::string MATCH_INDEX = "match_index";
 static auto match_index_name = intern_string::lookup("match_index");
 
+static const std::string MATCH_ROWID = "match_rowid";
+static auto match_rowid_name = intern_string::lookup("match_rowid");
+
 log_search_table::log_search_table(std::shared_ptr<lnav::pcre2pp::code> code,
                                    intern_string_t table_name)
     : log_vtab_impl(table_name), lst_regex(code),
@@ -47,6 +50,10 @@ log_search_table::log_search_table(std::shared_ptr<lnav::pcre2pp::code> code,
 void
 log_search_table::get_columns_int(std::vector<vtab_column>& cols) const
 {
+    if (!this->lst_cols.empty()) {
+        return;
+    }
+
     column_namer cn{column_namer::language::SQL};
 
     if (this->lst_format != nullptr) {
@@ -55,24 +62,33 @@ log_search_table::get_columns_int(std::vector<vtab_column>& cols) const
         cols.resize(this->lst_column_metas.size());
         for (const auto& meta : this->lst_column_metas) {
             if (!meta.lvm_column.is<logline_value_meta::table_column>()) {
+                cols.pop_back();
                 continue;
             }
             auto col
                 = meta.lvm_column.get<logline_value_meta::table_column>().value;
-            auto type_pair
-                = log_vtab_impl::logline_value_to_sqlite_type(meta.lvm_kind);
+            auto type_pair = logline_value_to_sqlite_type(meta.lvm_kind);
             cols[col].vc_name = meta.lvm_name.to_string();
             cols[col].vc_type = type_pair.first;
             cols[col].vc_subtype = type_pair.second;
+
+            ensure(!cols[col].vc_name.empty());
         }
     }
 
+    this->lst_column_metas.emplace_back(
+        match_rowid_name,
+        value_kind_t::VALUE_INTEGER,
+        logline_value_meta::table_column{cols.size()});
+    cols.emplace_back(MATCH_ROWID, SQLITE_INTEGER);
+    cols.back().vc_comment = "The row number of the log message that matched";
     this->lst_column_metas.emplace_back(
         match_index_name,
         value_kind_t::VALUE_INTEGER,
         logline_value_meta::table_column{cols.size()});
     cols.emplace_back(MATCH_INDEX, SQLITE_INTEGER);
-    cn.add_column(string_fragment::from_const("__all__"));
+    cols.back().vc_comment = "The index of the match within a log message";
+    cn.add_column("__all__"_frag);
     auto captures = this->lst_regex->get_captures();
     for (size_t lpc = 0; lpc < this->lst_regex->get_capture_count(); lpc++) {
         std::string collator;
@@ -111,10 +127,12 @@ log_search_table::get_columns_int(std::vector<vtab_column>& cols) const
 }
 
 void
-log_search_table::get_foreign_keys(std::vector<std::string>& keys_inout) const
+log_search_table::get_foreign_keys(
+    std::unordered_set<std::string>& keys_inout) const
 {
     log_vtab_impl::get_foreign_keys(keys_inout);
-    keys_inout.emplace_back(MATCH_INDEX);
+    keys_inout.emplace(MATCH_ROWID);
+    keys_inout.emplace(MATCH_INDEX);
 }
 
 bool
@@ -175,7 +193,7 @@ log_search_table::next(log_cursor& lc, logfile_sub_source& lss)
     lf->read_full_message(lf_iter, sbr);
     sbr.erase_ansi();
     lf->get_format()->annotate(
-        cl, this->vi_attrs, this->lst_line_values_cache, false);
+        lf, cl, this->vi_attrs, this->lst_line_values_cache, false);
     this->lst_content
         = this->lst_line_values_cache.lvv_sbr.to_string_fragment();
 
@@ -189,6 +207,7 @@ log_search_table::next(log_cursor& lc, logfile_sub_source& lss)
         return false;
     }
 
+    this->lst_rowid += 1;
     this->lst_remaining = match_res->f_remaining;
     this->lst_match_index = 0;
 
@@ -204,20 +223,23 @@ log_search_table::extract(logfile* lf,
     if (this->lst_format != nullptr) {
         values = this->lst_line_values_cache;
     }
-    values.lvv_values.emplace_back(
-        this->lst_column_metas[this->lst_format_column_count],
-        this->lst_match_index);
-    for (size_t lpc = 0; lpc < this->lst_regex->get_capture_count(); lpc++) {
+    auto curr_col = this->lst_format_column_count;
+    values.lvv_values.emplace_back(this->lst_column_metas[curr_col],
+                                   this->lst_rowid);
+    curr_col += 1;
+    values.lvv_values.emplace_back(this->lst_column_metas[curr_col],
+                                   this->lst_match_index);
+    curr_col += 1;
+    for (size_t lpc = 0; lpc < this->lst_regex->get_capture_count();
+         lpc++, curr_col++)
+    {
         const auto cap = this->lst_match_data[lpc + 1];
         if (cap) {
-            values.lvv_values.emplace_back(
-                this->lst_column_metas[this->lst_format_column_count + 1 + lpc],
-                line,
-                to_line_range(cap.value()));
+            values.lvv_values.emplace_back(this->lst_column_metas[curr_col],
+                                           line,
+                                           to_line_range(cap.value()));
         } else {
-            values.lvv_values.emplace_back(
-                this->lst_column_metas[this->lst_format_column_count + 1
-                                       + lpc]);
+            values.lvv_values.emplace_back(this->lst_column_metas[curr_col]);
         }
     }
 }
@@ -245,6 +267,7 @@ log_search_table::filter(log_cursor& lc, logfile_sub_source& lss)
             this->lst_log_level.value(),
         };
     }
+    this->lst_rowid = -1;
     this->lst_match_index = -1;
 
     if (lss.lss_index_generation != this->lst_index_generation) {
@@ -267,7 +290,9 @@ log_search_table::filter(log_cursor& lc, logfile_sub_source& lss)
                   this->lst_mismatch_bitmap.capacity());
 #endif
     }
-    if (!lc.lc_indexed_lines.empty()) {
+    if (!lc.lc_indexed_lines.empty()
+        && lc.lc_indexed_lines_range.contains(lc.lc_curr_line))
+    {
         lc.lc_curr_line = lc.lc_indexed_lines.back();
         lc.lc_indexed_lines.pop_back();
     }
