@@ -27,13 +27,20 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <map>
+
 #include "md2attr_line.hh"
 
 #include "base/attr_line.builder.hh"
+#include "base/intern_string.hh"
+#include "base/itertools.enumerate.hh"
 #include "base/itertools.hh"
 #include "base/lnav_log.hh"
 #include "base/map_util.hh"
+#include "base/text_format_enum.hh"
+#include "base/types.hh"
 #include "document.sections.hh"
+#include "lnav.script.parser.hh"
 #include "pcrepp/pcre2pp.hh"
 #include "pugixml/pugixml.hpp"
 #include "readline_highlighters.hh"
@@ -56,6 +63,20 @@ static const std::map<string_fragment, text_format_t> CODE_NAME_TO_TEXT_FORMAT
         {"xml"_frag, text_format_t::TF_XML},
 };
 
+struct md_script_annotator : lnav::script::parser {
+    md_script_annotator(std::string src) : parser(src) {}
+
+    std::map<unsigned long, std::string> mda_line_to_command;
+
+    Result<void, lnav::console::user_message> handle_command(
+        const std::string& cmd) override
+    {
+        this->mda_line_to_command[this->p_starting_line_number] = cmd;
+
+        return Ok();
+    }
+};
+
 static highlight_map_t
 get_highlight_map()
 {
@@ -74,12 +95,13 @@ md2attr_line::flush_footnotes()
 
     auto& block_text = this->ml_blocks.back();
     auto longest_foot = this->ml_footnotes
-        | lnav::itertools::map(&attr_line_t::utf8_length_or_length)
+        | lnav::itertools::map(&attr_line_t::column_width)
         | lnav::itertools::max(0);
-    size_t index = 1;
 
     block_text.append("\n");
-    for (auto& foot : this->ml_footnotes) {
+    for (const auto& [index, foot] :
+         lnav::itertools::enumerate(this->ml_footnotes, 1))
+    {
         auto footline
             = attr_line_t(" ")
                   .append("\u258c"_footnote_border)
@@ -88,10 +110,10 @@ md2attr_line::flush_footnotes()
                   .append(lnav::roles::footnote_text(
                       fmt::format(FMT_STRING("[{}] - "), index)))
                   .append(foot.pad_to(longest_foot))
-                  .with_attr_for_all(SA_PREFORMATTED.value());
+                  .with_attr_for_all(SA_PREFORMATTED.value())
+                  .move();
 
         block_text.append(footline).append("\n");
-        index += 1;
     }
     this->ml_footnotes.clear();
 }
@@ -99,6 +121,11 @@ md2attr_line::flush_footnotes()
 Result<void, std::string>
 md2attr_line::enter_block(const md4cpp::event_handler::block& bl)
 {
+    if (this->ml_source_path) {
+        log_trace("enter_block %s",
+                  mapbox::util::apply_visitor(type_visitor(), bl));
+    }
+
     if (this->ml_list_stack.empty()
         && (bl.is<MD_BLOCK_H_DETAIL*>() || bl.is<block_hr>()
             || bl.is<block_p>()))
@@ -128,6 +155,11 @@ md2attr_line::enter_block(const md4cpp::event_handler::block& bl)
 Result<void, std::string>
 md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 {
+    if (this->ml_source_path) {
+        log_trace("leave_block %s",
+                  mapbox::util::apply_visitor(type_visitor(), bl));
+    }
+
     auto block_text = std::move(this->ml_blocks.back());
     this->ml_blocks.pop_back();
 
@@ -164,7 +196,8 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
     } else if (bl.is<block_hr>()) {
         block_text = attr_line_t()
                          .append(lnav::roles::hr(repeat("\u2501", 70)))
-                         .with_attr_for_all(SA_PREFORMATTED.value());
+                         .with_attr_for_all(SA_PREFORMATTED.value())
+                         .move();
         last_block.append("\n").append(block_text).append("\n");
     } else if (bl.is<MD_BLOCK_UL_DETAIL*>() || bl.is<MD_BLOCK_OL_DETAIL*>()) {
         this->ml_list_stack.pop_back();
@@ -183,7 +216,11 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
         last_block.append(block_text);
     } else if (bl.is<MD_BLOCK_LI_DETAIL*>()) {
         auto last_list_block = this->ml_list_stack.back();
-        text_wrap_settings tws = {0, 60};
+        const auto* li_detail = bl.get<MD_BLOCK_LI_DETAIL*>();
+        auto tws = text_wrap_settings{
+            0,
+            63 - (int) (this->ml_list_stack.size() * 3),
+        };
 
         attr_line_builder alb(last_block);
         {
@@ -191,11 +228,22 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 
             alb.append(" ")
                 .append(last_list_block.match(
-                    [this, &tws](const MD_BLOCK_UL_DETAIL*) {
+                    [this, li_detail, &tws](const MD_BLOCK_UL_DETAIL*) {
+                        static const std::string glyph1 = "\u2022";
+                        static const std::string glyph2 = "\u2014";
+                        static const std::string unchecked = "[ ]";
+                        static const std::string checked = "[\u2713]";
                         tws.tws_indent = 3;
-                        return this->ml_list_stack.size() % 2 == 1
-                            ? "\u2022"_list_glyph
-                            : "\u2014"_list_glyph;
+
+                        if (li_detail->is_task) {
+                            return lnav::roles::list_glyph(
+                                li_detail->task_mark == ' ' ? unchecked
+                                                            : checked);
+                        }
+
+                        return lnav::roles::list_glyph(
+                            this->ml_list_stack.size() % 2 == 1 ? glyph1
+                                                                : glyph2);
                     },
                     [this, &tws](MD_BLOCK_OL_DETAIL ol_detail) {
                         auto retval = lnav::roles::list_glyph(
@@ -214,6 +262,7 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 
         alb.append(block_text, &tws);
     } else if (bl.is<MD_BLOCK_CODE_DETAIL*>()) {
+        auto mda = md_script_annotator{this->ml_source_path.value_or("")};
         auto* code_detail = bl.get<MD_BLOCK_CODE_DETAIL*>();
 
         this->ml_code_depth -= 1;
@@ -224,28 +273,36 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
         if (tf_opt) {
             static const auto highlighters = get_highlight_map();
 
-            lnav::document::discover_structure(
-                block_text, line_range{0, -1}, tf_opt.value());
+            lnav::document::discover(block_text)
+                .with_text_format(tf_opt.value())
+                .perform();
             for (const auto& hl_pair : highlighters) {
                 const auto& hl = hl_pair.second;
 
-                if (!hl.h_text_formats.empty()
-                    && hl.h_text_formats.count(tf_opt.value()) == 0)
-                {
+                if (!hl.applies_to_format(tf_opt.value())) {
                     continue;
                 }
                 hl.annotate(block_text, 0);
             }
         } else if (lang_sf == "lnav") {
             readline_lnav_highlighter(block_text, block_text.length());
-        } else if (lang_sf == "sql" || lang_sf == "sqlite" || lang_sf == "prql")
-        {
+
+            if (this->ml_add_lnav_script_icons) {
+                for (auto line_sf :
+                     block_text.to_string_fragment().split_lines())
+                {
+                    mda.push_back(line_sf);
+                }
+                mda.final();
+            }
+            tf_opt = std::make_optional(text_format_t::TF_LNAV_SCRIPT);
+        } else if (lang_sf.is_one_of("sql", "sqlite", "prql")) {
             readline_sqlite_highlighter(block_text, block_text.length());
-        } else if (lang_sf == "shell" || lang_sf == "bash") {
+            tf_opt = std::make_optional(text_format_t::TF_SQL);
+        } else if (lang_sf.is_one_of("shell", "bash")) {
             readline_shlex_highlighter(block_text, block_text.length());
-        } else if (lang_sf == "console"
-                   || lang_sf.iequal(
-                       string_fragment::from_const("shellsession")))
+            tf_opt = std::make_optional(text_format_t::TF_SHELL_SCRIPT);
+        } else if (lang_sf == "console" || lang_sf.iequal("shellsession"_frag))
         {
             static const auto SH_PROMPT
                 = lnav::pcre2pp::code::from_const(R"([^\$>#%]*[\$>#%]\s+)");
@@ -286,45 +343,122 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                     new_block_text.append(line).append("\n");
                 }
             }
-            block_text = new_block_text;
+            if (!cmd_block.empty()) {
+                new_block_text.append(cmd_block);
+            }
+            block_text = new_block_text.move();
         }
 
         auto code_lines = block_text.rtrim().split_lines();
         auto max_width = code_lines
-            | lnav::itertools::map(&attr_line_t::utf8_length_or_length)
+            | lnav::itertools::map(&attr_line_t::column_width)
             | lnav::itertools::max(0);
         attr_line_t padded_text;
 
-        for (auto& line : code_lines) {
-            line.pad_to(std::max(max_width + 4, ssize_t{40}))
+        for (const auto& [lineno, line] :
+             lnav::itertools::enumerate(code_lines, 1))
+        {
+            line.pad_to(std::max(max_width + 4, size_t{40}))
                 .with_attr_for_all(VC_ROLE.value(role_t::VCR_QUOTED_CODE));
-            padded_text.append(lnav::string::attrs::preformatted(" "))
-                .append("\u258c"_code_border)
-                .append(line)
-                .append("\n");
+            padded_text.append(" ");
+            if (!mda.mda_line_to_command.empty()) {
+                auto line_iter = mda.mda_line_to_command.find(lineno);
+                auto start = (int) padded_text.al_string.size();
+                padded_text.append("   ");
+                if (line_iter != mda.mda_line_to_command.end()) {
+                    auto icon_lr = line_range{start + 1, start + 2};
+                    padded_text.al_attrs.emplace_back(
+                        icon_lr, VC_ICON.value(ui_icon_t::play));
+                    auto cmd_lr = line_range{start, start + 3};
+                    auto ui_cmd = ui_command{
+                        source_location{
+                            this->ml_source_id,
+                            this->eh_line_number + lineno,
+                        },
+                        line_iter->second,
+                    };
+                    padded_text.al_attrs.emplace_back(cmd_lr,
+                                                      VC_COMMAND.value(ui_cmd));
+                }
+            }
+            padded_text.append("\u258c"_code_border).append(line).append("\n");
         }
         if (!padded_text.empty()) {
-            padded_text.with_attr_for_all(SA_PREFORMATTED.value());
+            auto tf = tf_opt.value_or(text_format_t::TF_UNKNOWN);
+            padded_text.with_attr_for_all(SA_PREFORMATTED.value())
+                .with_attr_for_all(SA_QUOTED_TEXT.value(tf));
             last_block.append("\n").append(padded_text);
         }
     } else if (bl.is<block_quote>()) {
-        text_wrap_settings tws = {0, 60};
-        attr_line_t wrapped_text;
+        const static auto ALERT_TYPE = lnav::pcre2pp::code::from_const(
+            R"(^\s*\[!(NOTE|TIP|IMPORTANT|WARNING|CAUTION)\])");
 
-        wrapped_text.append(block_text.rtrim(), &tws);
+        auto tws = text_wrap_settings{0, 60};
+        attr_line_t wrapped_text;
+        auto md = ALERT_TYPE.create_match_data();
+        std::optional<role_t> border_role;
+
+        block_text.rtrim();
+        if (ALERT_TYPE.capture_from(block_text.al_string)
+                .into(md)
+                .matches()
+                .ignore_error())
+        {
+            attr_line_t replacement;
+
+            if (md[1] == "NOTE") {
+                replacement.append("\u24d8  Note\n"_footnote_border);
+                border_role = role_t::VCR_FOOTNOTE_BORDER;
+            } else if (md[1] == "TIP") {
+                replacement.append(":bulb:"_emoji)
+                    .append(" Tip\n")
+                    .with_attr_for_all(VC_ROLE.value(role_t::VCR_OK));
+                border_role = role_t::VCR_OK;
+            } else if (md[1] == "IMPORTANT") {
+                replacement.append(":star2:"_emoji)
+                    .append(" Important\n")
+                    .with_attr_for_all(VC_ROLE.value(role_t::VCR_INFO));
+                border_role = role_t::VCR_INFO;
+            } else if (md[1] == "WARNING") {
+                replacement.append(":warning:"_emoji)
+                    .append(" Warning\n")
+                    .with_attr_for_all(VC_ROLE.value(role_t::VCR_WARNING));
+                border_role = role_t::VCR_WARNING;
+            } else if (md[1] == "CAUTION") {
+                replacement.append(":small_red_triangle:"_emoji)
+                    .append(" Caution\n")
+                    .with_attr_for_all(VC_ROLE.value(role_t::VCR_ERROR));
+                border_role = role_t::VCR_ERROR;
+            } else {
+                ensure(0);
+            }
+            block_text.erase(md[0]->sf_begin, md[0]->length());
+            block_text.insert(0, replacement);
+        }
+
+        wrapped_text.append(block_text, &tws);
         auto quoted_lines = wrapped_text.split_lines();
         auto max_width = quoted_lines
-            | lnav::itertools::map(&attr_line_t::utf8_length_or_length)
-            | lnav::itertools::max(0);
+            | lnav::itertools::map(&attr_line_t::column_width)
+            | lnav::itertools::max(tws.tws_width);
         attr_line_t padded_text;
 
         for (auto& line : quoted_lines) {
             line.pad_to(max_width + 1)
                 .with_attr_for_all(VC_ROLE.value(role_t::VCR_QUOTED_TEXT));
-            padded_text.append(" ")
-                .append("\u258c"_quote_border)
-                .append(line)
-                .append("\n");
+            padded_text.append(" ");
+            auto start_index = padded_text.length();
+            padded_text.append("\u258c"_quote_border);
+            if (border_role) {
+                padded_text.with_attr(string_attr{
+                    line_range{
+                        (int) start_index,
+                        (int) padded_text.length(),
+                    },
+                    VC_ROLE_FG.value(border_role.value()),
+                });
+            }
+            padded_text.append(line).append("\n");
         }
         if (!padded_text.empty()) {
             padded_text.with_attr_for_all(SA_PREFORMATTED.value());
@@ -340,19 +474,20 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
         block_text.append("\n");
         max_col_sizes.resize(table_detail->col_count);
         for (size_t lpc = 0; lpc < table_detail->col_count; lpc++) {
-            if (lpc < tab.t_headers.size()) {
-                max_col_sizes[lpc] = tab.t_headers[lpc].utf8_length_or_length();
-                tab.t_headers[lpc].with_attr_for_all(
-                    VC_ROLE.value(role_t::VCR_TABLE_HEADER));
+            if (lpc >= tab.t_headers.size()) {
+                continue;
             }
+            max_col_sizes[lpc] = tab.t_headers[lpc].column_width();
+            tab.t_headers[lpc].with_attr_for_all(
+                VC_ROLE.value(role_t::VCR_TABLE_HEADER));
         }
         for (const auto& row : tab.t_rows) {
             for (size_t lpc = 0; lpc < table_detail->col_count; lpc++) {
                 if (lpc >= row.r_columns.size()) {
                     continue;
                 }
-                auto col_len = row.r_columns[lpc].utf8_length_or_length();
-                if (col_len > max_col_sizes[lpc]) {
+                auto col_len = row.r_columns[lpc].c_contents.column_width();
+                if ((ssize_t) col_len > max_col_sizes[lpc]) {
                     max_col_sizes[lpc] = col_len;
                 }
             }
@@ -370,44 +505,48 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 
             attr_line_t td_block;
             td_block.append(tab.t_headers[lpc], &tws);
-            cells.emplace_back(td_block.rtrim().split_lines());
+            cells.emplace_back(MD_ALIGN_CENTER, td_block.rtrim().split_lines());
             if (cells.back().cl_lines.size() > max_cell_lines) {
                 max_cell_lines = cells.back().cl_lines.size();
             }
         }
         for (size_t line_index = 0; line_index < max_cell_lines; line_index++) {
-            size_t col = 0;
-            for (const auto& cell : cells) {
+            for (const auto& [col, cell] : lnav::itertools::enumerate(cells)) {
                 block_text.append(" ");
                 if (line_index < cell.cl_lines.size()) {
                     block_text.append(cell.cl_lines[line_index]);
                     block_text.append(
                         col_sizes[col]
-                            - cell.cl_lines[line_index].utf8_length_or_length(),
+                            - cell.cl_lines[line_index].column_width(),
                         ' ');
                 } else {
                     block_text.append(col_sizes[col], ' ');
                 }
-                col += 1;
             }
             block_text.append("\n")
                 .append(lnav::roles::table_border(
                     repeat("\u2550", full_width + col_sizes.size())))
                 .append("\n");
         }
+        size_t row_index = 0;
         for (const auto& row : tab.t_rows) {
             cells.clear();
             max_cell_lines = 0;
-            for (size_t lpc = 0; lpc < row.r_columns.size(); lpc++) {
-                tws.with_width(col_sizes[lpc]);
+            for (const auto& [col_index, cell] :
+                 lnav::itertools::enumerate(row.r_columns))
+            {
+                tws.with_width(col_sizes[col_index]);
 
                 attr_line_t td_block;
-                td_block.append(row.r_columns[lpc], &tws);
-                cells.emplace_back(td_block.rtrim().split_lines());
+                td_block.append(cell.c_contents, &tws);
+                cells.emplace_back(cell.c_align,
+                                   td_block.rtrim().split_lines());
                 if (cells.back().cl_lines.size() > max_cell_lines) {
                     max_cell_lines = cells.back().cl_lines.size();
                 }
             }
+            auto alt_row_index = row_index % 4;
+            auto line_lr = line_range{(int) block_text.al_string.size(), 0};
             for (size_t line_index = 0; line_index < max_cell_lines;
                  line_index++)
             {
@@ -415,14 +554,36 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                 for (const auto& cell : cells) {
                     block_text.append(" ");
                     if (line_index < cell.cl_lines.size()) {
-                        block_text.append(cell.cl_lines[line_index]);
-                        if (col < col_sizes.size() - 1) {
-                            block_text.append(
-                                col_sizes[col]
-                                    - cell.cl_lines[line_index]
-                                          .utf8_length_or_length(),
-                                ' ');
+                        auto padding = size_t{0};
+
+                        if (col < col_sizes.size()) {
+                            if (col_sizes[col]
+                                > (ssize_t) cell.cl_lines[line_index]
+                                      .column_width())
+                            {
+                                padding = col_sizes[col]
+                                    - cell.cl_lines[line_index].column_width();
+                            }
                         }
+
+                        auto lpadding = size_t{0};
+                        auto rpadding = size_t{0};
+                        switch (cell.cl_align) {
+                            case MD_ALIGN_DEFAULT:
+                            case MD_ALIGN_LEFT:
+                                rpadding = padding;
+                                break;
+                            case MD_ALIGN_CENTER:
+                                lpadding = padding / 2;
+                                rpadding = padding - lpadding;
+                                break;
+                            case MD_ALIGN_RIGHT:
+                                lpadding = padding;
+                                break;
+                        }
+                        block_text.append(lpadding, ' ');
+                        block_text.append(cell.cl_lines[line_index]);
+                        block_text.append(rpadding, ' ');
                     } else if (col < col_sizes.size() - 1) {
                         block_text.append(col_sizes[col], ' ');
                     }
@@ -430,6 +591,14 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
                 }
                 block_text.append("\n");
             }
+            if (alt_row_index == 2 || alt_row_index == 3) {
+                line_lr.lr_end = block_text.al_string.size();
+                block_text.al_attrs.emplace_back(
+                    line_lr, VC_ROLE.value(role_t::VCR_ALT_ROW));
+            }
+            if (max_cell_lines > 0) {
+                row_index += 1;
+            }
         }
         if (!block_text.empty()) {
             block_text.with_attr_for_all(SA_PREFORMATTED.value());
@@ -438,7 +607,9 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
     } else if (bl.is<block_th>()) {
         this->ml_tables.back().t_headers.push_back(block_text);
     } else if (bl.is<MD_BLOCK_TD_DETAIL*>()) {
-        this->ml_tables.back().t_rows.back().r_columns.push_back(block_text);
+        auto td_detail = bl.get<MD_BLOCK_TD_DETAIL*>();
+        this->ml_tables.back().t_rows.back().r_columns.emplace_back(
+            td_detail->align, block_text);
     } else {
         if (bl.is<block_html>()) {
             if (startswith(block_text.get_string(), "<!--")) {
@@ -462,6 +633,11 @@ md2attr_line::leave_block(const md4cpp::event_handler::block& bl)
 Result<void, std::string>
 md2attr_line::enter_span(const md4cpp::event_handler::span& sp)
 {
+    if (this->ml_source_path) {
+        log_trace("enter_span %s",
+                  mapbox::util::apply_visitor(type_visitor(), sp));
+    }
+
     auto& last_block = this->ml_blocks.back();
     this->ml_span_starts.push_back(last_block.length());
     if (sp.is<span_code>()) {
@@ -476,6 +652,11 @@ md2attr_line::enter_span(const md4cpp::event_handler::span& sp)
 Result<void, std::string>
 md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
 {
+    if (this->ml_source_path) {
+        log_trace("leave_span %s",
+                  mapbox::util::apply_visitor(type_visitor(), sp));
+    }
+
     auto& last_block = this->ml_blocks.back();
     if (sp.is<span_code>()) {
         this->ml_code_depth -= 1;
@@ -497,12 +678,10 @@ md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
             static_cast<int>(this->ml_span_starts.back()),
             static_cast<int>(last_block.length()),
         };
-#if defined(A_ITALIC)
         last_block.with_attr({
             lr,
-            VC_STYLE.value(text_attrs{(int32_t) A_ITALIC}),
+            VC_STYLE.value(text_attrs{text_attrs::with_italic()}),
         });
-#endif
     } else if (sp.is<span_strong>()) {
         line_range lr{
             static_cast<int>(this->ml_span_starts.back()),
@@ -510,7 +689,7 @@ md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
         };
         last_block.with_attr({
             lr,
-            VC_STYLE.value(text_attrs{A_BOLD}),
+            VC_STYLE.value(text_attrs::with_bold()),
         });
     } else if (sp.is<span_u>()) {
         line_range lr{
@@ -519,23 +698,33 @@ md2attr_line::leave_span(const md4cpp::event_handler::span& sp)
         };
         last_block.with_attr({
             lr,
-            VC_STYLE.value(text_attrs{A_UNDERLINE}),
+            VC_STYLE.value(text_attrs::with_underline()),
         });
-    } else if (sp.is<MD_SPAN_A_DETAIL*>()) {
-        auto* a_detail = sp.get<MD_SPAN_A_DETAIL*>();
-        auto href_str = std::string(a_detail->href.text, a_detail->href.size);
-        line_range lr{
+    } else if (sp.is<span_del>()) {
+        auto lr = line_range{
             static_cast<int>(this->ml_span_starts.back()),
             static_cast<int>(last_block.length()),
         };
         last_block.with_attr({
             lr,
-            VC_HYPERLINK.value(href_str),
+            VC_STYLE.value(text_attrs::with_struck()),
+        });
+    } else if (sp.is<MD_SPAN_A_DETAIL*>()) {
+        const auto* a_detail = sp.get<MD_SPAN_A_DETAIL*>();
+        auto href_str = std::string(a_detail->href.text, a_detail->href.size);
+        line_range lr{
+            static_cast<int>(this->ml_span_starts.back()),
+            static_cast<int>(last_block.length()),
+        };
+        auto abs_href = this->append_url_footnote(href_str);
+        last_block.with_attr({
+            lr,
+            VC_HYPERLINK.value(abs_href),
         });
-        this->append_url_footnote(href_str);
     } else if (sp.is<MD_SPAN_IMG_DETAIL*>()) {
-        auto* img_detail = sp.get<MD_SPAN_IMG_DETAIL*>();
-        auto src_str = std::string(img_detail->src.text, img_detail->src.size);
+        const auto* img_detail = sp.get<MD_SPAN_IMG_DETAIL*>();
+        const auto src_str
+            = std::string(img_detail->src.text, img_detail->src.size);
 
         this->append_url_footnote(src_str);
     }
@@ -565,6 +754,7 @@ left_border_string(border_line_width width)
         case border_line_width::thick:
             return "\u258C";
     }
+    ensure(false);
 }
 
 static const char*
@@ -578,17 +768,18 @@ right_border_string(border_line_width width)
         case border_line_width::thick:
             return "\u2590";
     }
+    ensure(false);
 }
 
 static attr_line_t
 span_style_border(border_side side, const string_fragment& value)
 {
-    static const auto NAME_THIN = string_fragment::from_const("thin");
-    static const auto NAME_MEDIUM = string_fragment::from_const("medium");
-    static const auto NAME_THICK = string_fragment::from_const("thick");
-    static const auto NAME_SOLID = string_fragment::from_const("solid");
-    static const auto NAME_DASHED = string_fragment::from_const("dashed");
-    static const auto NAME_DOTTED = string_fragment::from_const("dotted");
+    static constexpr auto NAME_THIN = "thin"_frag;
+    static constexpr auto NAME_MEDIUM = "medium"_frag;
+    static constexpr auto NAME_THICK = "thick"_frag;
+    static constexpr auto NAME_SOLID = "solid"_frag;
+    static constexpr auto NAME_DASHED = "dashed"_frag;
+    static constexpr auto NAME_DOTTED = "dotted"_frag;
     static const auto& vc = view_colors::singleton();
 
     text_attrs border_attrs;
@@ -634,35 +825,35 @@ span_style_border(border_side side, const string_fragment& value)
 attr_line_t
 md2attr_line::to_attr_line(const pugi::xml_node& doc)
 {
-    static const auto NAME_IMG = string_fragment::from_const("img");
-    static const auto NAME_SPAN = string_fragment::from_const("span");
-    static const auto NAME_PRE = string_fragment::from_const("pre");
-    static const auto NAME_FG = string_fragment::from_const("color");
-    static const auto NAME_BG = string_fragment::from_const("background-color");
-    static const auto NAME_FONT_WEIGHT
-        = string_fragment::from_const("font-weight");
-    static const auto NAME_TEXT_DECO
-        = string_fragment::from_const("text-decoration");
-    static const auto NAME_BORDER_LEFT
-        = string_fragment::from_const("border-left");
-    static const auto NAME_BORDER_RIGHT
-        = string_fragment::from_const("border-right");
+    static const auto NAME_IMG = "img"_frag;
+    static const auto NAME_SPAN = "span"_frag;
+    static const auto NAME_PRE = "pre"_frag;
+    static const auto NAME_FG = "color"_frag;
+    static const auto NAME_BG = "background-color"_frag;
+    static const auto NAME_FONT_WEIGHT = "font-weight"_frag;
+    static const auto NAME_TEXT_DECO = "text-decoration"_frag;
+    static const auto NAME_BORDER_LEFT = "border-left"_frag;
+    static const auto NAME_BORDER_RIGHT = "border-right"_frag;
     static const auto& vc = view_colors::singleton();
 
+    if (this->ml_source_path) {
+        log_trace("converting HTML to attr_line");
+    }
+
     attr_line_t retval;
     if (doc.children().empty()) {
         retval.append(doc.text().get());
     }
     for (const auto& child : doc.children()) {
         if (child.name() == NAME_IMG) {
-            nonstd::optional<std::string> src_href;
+            std::optional<std::string> src_href;
             std::string link_label;
             auto img_src = child.attribute("src");
             auto img_alt = child.attribute("alt");
             if (img_alt) {
                 link_label = img_alt.value();
             } else if (img_src) {
-                link_label = ghc::filesystem::path(img_src.value())
+                link_label = std::filesystem::path(img_src.value())
                                  .filename()
                                  .string();
             } else {
@@ -674,14 +865,14 @@ md2attr_line::to_attr_line(const pugi::xml_node& doc)
                 if (is_url(src_value)) {
                     src_href = src_value;
                 } else {
-                    auto src_path = ghc::filesystem::path(src_value);
+                    auto src_path = std::filesystem::path(src_value);
                     std::error_code ec;
 
                     if (src_path.is_relative() && this->ml_source_path) {
                         src_path = this->ml_source_path.value().parent_path()
                             / src_path;
                     }
-                    auto canon_path = ghc::filesystem::canonical(src_path, ec);
+                    auto canon_path = std::filesystem::canonical(src_path, ec);
                     if (!ec) {
                         src_path = canon_path;
                     }
@@ -696,22 +887,23 @@ md2attr_line::to_attr_line(const pugi::xml_node& doc)
                     .append("  ")
                     .append(
                         lnav::string::attrs::href(link_label, src_href.value()))
-                    .appendf(FMT_STRING("[{}]"), this->ml_footnotes.size() + 1);
+                    .append(to_superscript(this->ml_footnotes.size() + 1));
 
                 auto href
                     = attr_line_t()
                           .append(lnav::roles::hyperlink(src_href.value()))
-                          .append(" ");
-                href.with_attr_for_all(
-                    VC_ROLE.value(role_t::VCR_FOOTNOTE_TEXT));
-                href.with_attr_for_all(SA_PREFORMATTED.value());
+                          .append(" ")
+                          .with_attr_for_all(
+                              VC_ROLE.value(role_t::VCR_FOOTNOTE_TEXT))
+                          .with_attr_for_all(SA_PREFORMATTED.value())
+                          .move();
                 this->ml_footnotes.emplace_back(href);
             } else {
                 retval.append(link_label);
             }
         } else if (child.name() == NAME_SPAN) {
-            nonstd::optional<attr_line_t> left_border;
-            nonstd::optional<attr_line_t> right_border;
+            std::optional<attr_line_t> left_border;
+            std::optional<attr_line_t> right_border;
             auto styled_span = attr_line_t(child.text().get());
 
             auto span_class = child.attribute("class");
@@ -766,8 +958,8 @@ md2attr_line::to_attr_line(const pugi::xml_node& doc)
                                     = vc.match_color(color_res.unwrap());
                             }
                         } else if (key == NAME_FONT_WEIGHT) {
-                            if (value == "bold" || value == "bolder") {
-                                ta.ta_attrs |= A_BOLD;
+                            if (value.is_one_of("bold", "bolder")) {
+                                ta |= text_attrs::style::bold;
                             }
                         } else if (key == NAME_TEXT_DECO) {
                             auto deco_sf = value;
@@ -778,7 +970,7 @@ md2attr_line::to_attr_line(const pugi::xml_node& doc)
 
                                 if (deco_split_res.first.trim() == "underline")
                                 {
-                                    ta.ta_attrs |= A_UNDERLINE;
+                                    ta |= text_attrs::style::underline;
                                 }
 
                                 deco_sf = deco_split_res.second;
@@ -862,12 +1054,18 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
             };
             struct empty_tag {};
 
-            mapbox::util::variant<open_tag, close_tag, empty_tag> tag{
-                mapbox::util::no_init{}};
+            using html_tag_t
+                = mapbox::util::variant<open_tag, close_tag, empty_tag>;
 
-            if (sf.startswith("</")) {
+            html_tag_t tag{mapbox::util::no_init{}};
+
+            auto lbracket = sf.find('<');
+            if (!lbracket) {
+            } else if (lbracket && lbracket.value() + 1 < sf.length()
+                       && sf[lbracket.value() + 1] == '/')
+            {
                 tag = close_tag{
-                    sf.substr(2)
+                    sf.substr(lbracket.value() + 2)
                         .split_when(string_fragment::tag1{'>'})
                         .first.to_string(),
                 };
@@ -895,9 +1093,14 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
                     },
                     [this, &last_block](const close_tag& ct) {
                         if (this->ml_html_starts.empty()) {
+                            log_warning("closing tag %s with no open tag",
+                                        ct.ct_name.c_str());
                             return;
                         }
                         if (this->ml_html_starts.back().first != ct.ct_name) {
+                            log_warning(
+                                "closing tag %s with no matching open tag",
+                                ct.ct_name.c_str());
                             return;
                         }
 
@@ -961,9 +1164,8 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
             }
 
             std::string span_text;
-
             auto loop_res = REPL_RE.capture_from(sf).for_each(
-                [&span_text](lnav::pcre2pp::match_data& md) {
+                [&span_text](const lnav::pcre2pp::match_data& md) {
                     span_text += md.leading();
 
                     auto matched = *md[0];
@@ -993,39 +1195,49 @@ md2attr_line::text(MD_TEXTTYPE tt, const string_fragment& sf)
             text_wrap_settings tws
                 = {0, this->ml_blocks.size() == 1 ? 70 : 10000};
 
-            last_block.append(span_text, &tws);
+            auto span_al = attr_line_t::from_ansi_str(span_text);
+            last_block.append(span_al, &tws);
             break;
         }
     }
     return Ok();
 }
 
-void
+std::string
 md2attr_line::append_url_footnote(std::string href_str)
 {
-    if (startswith(href_str, "#")) {
-        return;
-    }
-
+    auto is_internal = startswith(href_str, "#");
     auto& last_block = this->ml_blocks.back();
-    last_block.appendf(FMT_STRING("[{}]"), this->ml_footnotes.size() + 1);
     last_block.with_attr(string_attr{
         line_range{
             (int) this->ml_span_starts.back(),
             (int) last_block.length(),
         },
-        VC_STYLE.value(text_attrs{A_UNDERLINE}),
+        VC_STYLE.value(text_attrs::with_underline()),
     });
+    if (is_internal) {
+        return href_str;
+    }
+
+    if (this->ml_last_superscript_index == last_block.length()) {
+        last_block.append("\u02d2");
+    }
+    last_block.append(to_superscript(this->ml_footnotes.size() + 1));
+    this->ml_last_superscript_index = last_block.length();
     if (this->ml_source_path && href_str.find(':') == std::string::npos) {
-        auto link_path = ghc::filesystem::absolute(
+        auto link_path = std::filesystem::absolute(
             this->ml_source_path.value().parent_path() / href_str);
 
         href_str = fmt::format(FMT_STRING("file://{}"), link_path.string());
     }
 
-    auto href
-        = attr_line_t().append(lnav::roles::hyperlink(href_str)).append(" ");
-    href.with_attr_for_all(VC_ROLE.value(role_t::VCR_FOOTNOTE_TEXT));
-    href.with_attr_for_all(SA_PREFORMATTED.value());
+    auto href = attr_line_t()
+                    .append(lnav::roles::hyperlink(href_str))
+                    .append(" ")
+                    .with_attr_for_all(VC_ROLE.value(role_t::VCR_FOOTNOTE_TEXT))
+                    .with_attr_for_all(SA_PREFORMATTED.value())
+                    .move();
     this->ml_footnotes.emplace_back(href);
+
+    return href_str;
 }
