@@ -33,13 +33,15 @@
 
 #include <string.h>
 
+#include "base/short_alloc.h"
 #include "config.h"
 
 bool
 ptime_b_slow(struct exttm* dst, const char* str, off_t& off_inout, ssize_t len)
 {
     size_t zone_len = len - off_inout;
-    char zone[zone_len + 1];
+    stack_buf allocator;
+    auto* zone = allocator.allocate(zone_len + 1);
     const char* end_of_date;
 
     memcpy(zone, &str[off_inout], zone_len);
@@ -71,7 +73,8 @@ ptime_fmt(const char* fmt,
             switch (fmt[lpc + 1]) {
                 case 'B': {
                     size_t b_len = len - off;
-                    char full_month[b_len + 1];
+                    stack_buf allocator;
+                    auto* full_month = allocator.allocate(b_len + 1);
                     const char* end_of_date;
 
                     memcpy(full_month, &str[off], b_len);
@@ -108,9 +111,11 @@ ptime_fmt(const char* fmt,
                     FMT_CASE('H', H);
                     FMT_CASE('i', i);
                     FMT_CASE('6', 6);
+                    FMT_CASE('9', 9);
                     FMT_CASE('I', I);
                     FMT_CASE('d', d);
                     FMT_CASE('e', e);
+                    FMT_CASE('j', j);
                     FMT_CASE('f', f);
                     FMT_CASE('k', k);
                     FMT_CASE('l', l);
@@ -159,9 +164,11 @@ ftime_fmt(char* dst, size_t len, const char* fmt, const struct exttm& tm)
                     FTIME_FMT_CASE('H', H);
                     FTIME_FMT_CASE('i', i);
                     FTIME_FMT_CASE('6', 6);
+                    FTIME_FMT_CASE('9', 9);
                     FTIME_FMT_CASE('I', I);
                     FTIME_FMT_CASE('d', d);
                     FTIME_FMT_CASE('e', e);
+                    FTIME_FMT_CASE('j', j);
                     FTIME_FMT_CASE('f', f);
                     FTIME_FMT_CASE('k', k);
                     FTIME_FMT_CASE('l', l);
