@@ -39,6 +39,7 @@
 #include "shlex.hh"
 #include "sql_help.hh"
 #include "sql_util.hh"
+#include "textfile_highlighters.hh"
 #include "view_curses.hh"
 
 static bool
@@ -57,7 +58,7 @@ static void
 find_matching_bracket(
     attr_line_t& al, int x, line_range sub, char left, char right)
 {
-    bool is_lit = (left == 'Q');
+    const auto is_lit = left == 'Q';
     attr_line_builder alb(al);
     const auto& line = al.get_string();
     int depth = 0;
@@ -73,7 +74,10 @@ find_matching_bracket(
             } else if (line[lpc] == left && is_bracket(line, lpc, is_lit)) {
                 if (depth == 0) {
                     alb.overlay_attr_for_char(
-                        lpc, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+                        lpc,
+                        VC_STYLE.value(text_attrs ::with_styles(
+                            text_attrs::style::bold,
+                            text_attrs::style::reverse)));
                     alb.overlay_attr_for_char(lpc,
                                               VC_ROLE.value(role_t::VCR_OK));
                     break;
@@ -84,13 +88,16 @@ find_matching_bracket(
     }
 
     if (line[x] == left && is_bracket(line, x, is_lit)) {
-        for (size_t lpc = x + 1; lpc < sub.lr_end; lpc++) {
+        for (auto lpc = x + 1; lpc < sub.lr_end; lpc++) {
             if (line[lpc] == left && is_bracket(line, lpc, is_lit)) {
                 depth += 1;
             } else if (line[lpc] == right && is_bracket(line, lpc, is_lit)) {
                 if (depth == 0) {
                     alb.overlay_attr_for_char(
-                        lpc, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+                        lpc,
+                        VC_STYLE.value(text_attrs ::with_styles(
+                            text_attrs::style::bold,
+                            text_attrs::style::reverse)));
                     alb.overlay_attr_for_char(lpc,
                                               VC_ROLE.value(role_t::VCR_OK));
                     break;
@@ -100,11 +107,11 @@ find_matching_bracket(
         }
     }
 
-    nonstd::optional<int> first_left;
+    std::optional<int> first_left;
 
     depth = 0;
 
-    for (size_t lpc = sub.lr_start; lpc < sub.lr_end; lpc++) {
+    for (auto lpc = sub.lr_start; lpc < sub.lr_end; lpc++) {
         if (line[lpc] == left && is_bracket(line, lpc, is_lit)) {
             depth += 1;
             if (!first_left) {
@@ -116,7 +123,9 @@ find_matching_bracket(
             } else {
                 auto lr = line_range(is_lit ? lpc - 1 : lpc, lpc + 1);
                 alb.overlay_attr(
-                    lr, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+                    lr,
+                    VC_STYLE.value(text_attrs ::with_styles(
+                        text_attrs::style::bold, text_attrs::style::reverse)));
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_ERROR));
             }
         }
@@ -126,27 +135,44 @@ find_matching_bracket(
         auto lr
             = line_range(is_lit ? first_left.value() - 1 : first_left.value(),
                          first_left.value() + 1);
-        alb.overlay_attr(lr, VC_STYLE.value(text_attrs{A_BOLD | A_REVERSE}));
+        alb.overlay_attr(
+            lr,
+            VC_STYLE.value(text_attrs ::with_styles(
+                text_attrs::style::bold, text_attrs::style::reverse)));
         alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_ERROR));
     }
 }
 
 void
-readline_regex_highlighter(attr_line_t& al, int x)
+readline_regex_highlighter(attr_line_t& al, std::optional<int> x)
 {
     lnav::snippets::regex_highlighter(
         al, x, line_range{1, (int) al.get_string().size()});
 }
 
-void
-readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
+static highlight_map_t
+get_hl_map()
 {
+    highlight_map_t retval;
+
+    setup_highlights(retval);
+    return retval;
+}
+
+void
+readline_command_highlighter_int(attr_line_t& al,
+                                 std::optional<int> x,
+                                 line_range sub)
+{
+    static const auto TEXT_HIGHLIGHTERS = get_hl_map();
     static const auto RE_PREFIXES = lnav::pcre2pp::code::from_const(
         R"(^:(filter-in|filter-out|delete-filter|enable-filter|disable-filter|highlight|clear-highlight|create-search-table\s+[^\s]+\s+))");
     static const auto SH_PREFIXES = lnav::pcre2pp::code::from_const(
         "^:(eval|open|append-to|write-to|write-csv-to|write-json-to)");
     static const auto SQL_PREFIXES
         = lnav::pcre2pp::code::from_const("^:(filter-expr|mark-expr)");
+    static const auto MD_PREFIXES
+        = lnav::pcre2pp::code::from_const("^:comment");
     static const auto IDENT_PREFIXES
         = lnav::pcre2pp::code::from_const("^:(tag|untag|delete-tags)");
     static const auto COLOR_PREFIXES
@@ -163,8 +189,10 @@ readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
     ws_index = line.find(' ', sub.lr_start);
     auto command = line.substr(sub.lr_start, ws_index);
     if (ws_index != std::string::npos) {
-        alb.overlay_attr(line_range(sub.lr_start + 1, ws_index),
-                         VC_ROLE.value(role_t::VCR_KEYWORD));
+        auto has_prefix = al.al_string[sub.lr_start] == ':';
+        alb.overlay_attr(
+            line_range(sub.lr_start + (has_prefix ? 1 : 0), ws_index),
+            VC_ROLE.value(role_t::VCR_KEYWORD));
 
         if (RE_PREFIXES.find_in(in_frag).ignore_error()) {
             lnav::snippets::regex_highlighter(
@@ -186,18 +214,17 @@ readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
                     .then([&](const auto& rgb_fg) {
                         auto color
                             = view_colors::singleton().match_color(rgb_fg);
-                        alb.template overlay_attr(to_line_range(md[0].value()),
-                                                  VC_STYLE.value(text_attrs{
-                                                      A_BOLD,
-                                                      color,
-                                                  }));
+                        auto ta = text_attrs::with_bold();
+                        ta.ta_fg_color = color;
+                        alb.overlay_attr(to_line_range(md[0].value()),
+                                         VC_STYLE.value(ta));
                     });
             });
     }
     if (IDENT_PREFIXES.find_in(in_frag).ignore_error()
         && ws_index != std::string::npos)
     {
-        size_t start = ws_index, last;
+        ssize_t start = ws_index, last;
 
         do {
             for (; start < sub.length() && isspace(line[start]); start++)
@@ -205,11 +232,11 @@ readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
             for (last = start; last < sub.length() && !isspace(line[last]);
                  last++)
                 ;
-            struct line_range lr {
-                (int) start, (int) last
-            };
+            line_range lr{(int) start, (int) last};
 
-            if (lr.length() > 0 && !lr.contains(x) && !lr.contains(x - 1)) {
+            if (x && lr.length() > 0 && !lr.contains(x.value())
+                && !lr.contains(x.value() - 1))
+            {
                 std::string value(lr.substr(line), lr.sublen(line));
 
                 if ((command == ":tag" || command == ":untag"
@@ -224,24 +251,33 @@ readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
             start = last;
         } while (start < sub.length());
     }
+    if (MD_PREFIXES.find_in(in_frag).ignore_error()) {
+        for (const auto& [src, hl] : TEXT_HIGHLIGHTERS) {
+            if (!hl.applies_to_format(text_format_t::TF_MARKDOWN)) {
+                continue;
+            }
+
+            hl.annotate(al, sub.lr_start + command.length());
+        }
+    }
 }
 
 void
-readline_command_highlighter(attr_line_t& al, int x)
+readline_command_highlighter(attr_line_t& al, std::optional<int> x)
 {
     readline_command_highlighter_int(
         al, x, line_range{0, (int) al.get_string().length()});
 }
 
 void
-readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
+readline_sqlite_highlighter_int(attr_line_t& al,
+                                std::optional<int> x,
+                                line_range sub)
 {
     static const char* brackets[] = {
         "[]",
         "()",
         "{}",
-
-        nullptr,
     };
 
     attr_line_builder alb(al);
@@ -252,7 +288,7 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
     annotate_sql_statement(anno_sql);
 
     for (const auto& attr : anno_sql.al_attrs) {
-        line_range lr{
+        auto lr = line_range{
             sub.lr_start + attr.sa_range.lr_start,
             sub.lr_start + attr.sa_range.lr_end,
         };
@@ -263,9 +299,12 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
         {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_KEYWORD));
         } else if (attr.sa_type == &SQL_IDENTIFIER_ATTR
-                   || attr.sa_type == &lnav::sql::PRQL_IDENTIFIER_ATTR)
+                   || attr.sa_type == &lnav::sql::PRQL_FQID_ATTR)
         {
-            if (!attr.sa_range.contains(x) && attr.sa_range.lr_end != x) {
+            if (!x
+                || (x && !attr.sa_range.contains(x.value())
+                    && attr.sa_range.lr_end != x.value()))
+            {
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_IDENTIFIER));
             }
         } else if (attr.sa_type == &SQL_FUNCTION_ATTR) {
@@ -276,12 +315,21 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
                    || attr.sa_type == &lnav::sql::PRQL_NUMBER_ATTR)
         {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_NUMBER));
+        } else if (attr.sa_type == &SQL_HEX_LIT_ATTR) {
+            if (lr.length() > 1 && al.al_string[lr.lr_end - 1] == '\'') {
+                alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
+            } else {
+                alb.overlay_attr_for_char(
+                    lr.lr_start, VC_STYLE.value(text_attrs::with_reverse()));
+                alb.overlay_attr_for_char(lr.lr_start,
+                                          VC_ROLE.value(role_t::VCR_ERROR));
+            }
         } else if (attr.sa_type == &SQL_STRING_ATTR) {
             if (lr.length() > 1 && al.al_string[lr.lr_end - 1] == '\'') {
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
             } else {
                 alb.overlay_attr_for_char(
-                    lr.lr_start, VC_STYLE.value(text_attrs{A_REVERSE}));
+                    lr.lr_start, VC_STYLE.value(text_attrs::with_reverse()));
                 alb.overlay_attr_for_char(lr.lr_start,
                                           VC_ROLE.value(role_t::VCR_ERROR));
             }
@@ -298,24 +346,27 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
         }
     }
 
-    for (int lpc = 0; brackets[lpc]; lpc++) {
-        find_matching_bracket(al, x, sub, brackets[lpc][0], brackets[lpc][1]);
+    for (const auto& bracket : brackets) {
+        find_matching_bracket(
+            al, x.value_or(al.length()), sub, bracket[0], bracket[1]);
     }
 }
 
 void
-readline_sqlite_highlighter(attr_line_t& al, int x)
+readline_sqlite_highlighter(attr_line_t& al, std::optional<int> x)
 {
     readline_sqlite_highlighter_int(
         al, x, line_range{0, (int) al.get_string().length()});
 }
 
 void
-readline_shlex_highlighter_int(attr_line_t& al, int x, line_range sub)
+readline_shlex_highlighter_int(attr_line_t& al,
+                               std::optional<int> x,
+                               line_range sub)
 {
     attr_line_builder alb(al);
     const auto& str = al.get_string();
-    nonstd::optional<int> quote_start;
+    std::optional<int> quote_start;
     shlex lexer(string_fragment{al.al_string.data(), sub.lr_start, sub.lr_end});
     bool done = false;
 
@@ -326,7 +377,7 @@ readline_shlex_highlighter_int(attr_line_t& al, int x, line_range sub)
 
             alb.overlay_attr(line_range(sub.lr_start + te.te_source.sf_begin,
                                         sub.lr_start + te.te_source.sf_end),
-                             VC_STYLE.value(text_attrs{A_REVERSE}));
+                             VC_STYLE.value(text_attrs::with_reverse()));
             alb.overlay_attr(line_range(sub.lr_start + te.te_source.sf_begin,
                                         sub.lr_start + te.te_source.sf_end),
                              VC_ROLE.value(role_t::VCR_ERROR));
@@ -355,7 +406,7 @@ readline_shlex_highlighter_int(attr_line_t& al, int x, line_range sub)
                     line_range(quote_start.value(),
                                sub.lr_start + token.tr_frag.sf_end),
                     VC_ROLE.value(role_t::VCR_STRING));
-                quote_start = nonstd::nullopt;
+                quote_start = std::nullopt;
                 break;
             case shlex_token_t::variable_ref:
             case shlex_token_t::quoted_variable_ref: {
@@ -397,14 +448,16 @@ readline_shlex_highlighter_int(attr_line_t& al, int x, line_range sub)
 }
 
 void
-readline_shlex_highlighter(attr_line_t& al, int x)
+readline_shlex_highlighter(attr_line_t& al, std::optional<int> x)
 {
     readline_shlex_highlighter_int(
         al, x, line_range{0, (int) al.al_string.length()});
 }
 
 static void
-readline_lnav_highlighter_int(attr_line_t& al, int x, line_range sub)
+readline_lnav_highlighter_int(attr_line_t& al,
+                              std::optional<int> x,
+                              line_range sub)
 {
     switch (al.al_string[sub.lr_start]) {
         case ':':
@@ -432,13 +485,13 @@ readline_lnav_highlighter_int(attr_line_t& al, int x, line_range sub)
 }
 
 void
-readline_lnav_highlighter(attr_line_t& al, int x)
+readline_lnav_highlighter(attr_line_t& al, std::optional<int> x)
 {
     static const auto COMMENT_RE = lnav::pcre2pp::code::from_const(R"(^\s*#)");
 
     attr_line_builder alb(al);
     size_t start = 0, lf_pos;
-    nonstd::optional<size_t> section_start;
+    std::optional<size_t> section_start;
 
     while ((lf_pos = al.get_string().find('\n', start)) != std::string::npos) {
         line_range line{(int) start, (int) lf_pos};
@@ -460,7 +513,7 @@ readline_lnav_highlighter(attr_line_t& al, int x)
                                                   (int) section_start.value(),
                                                   line.lr_start,
                                               });
-                section_start = nonstd::nullopt;
+                section_start = std::nullopt;
             }
             alb.overlay_attr(line_range{find_res->f_all.sf_begin, line.lr_end},
                              VC_ROLE.value(role_t::VCR_COMMENT));
@@ -501,3 +554,28 @@ readline_lnav_highlighter(attr_line_t& al, int x)
                                       });
     }
 }
+
+void
+highlight_syntax(text_format_t tf, attr_line_t& al, std::optional<int> x)
+{
+    switch (tf) {
+        case text_format_t::TF_SQL: {
+            readline_sqlite_highlighter(al, x);
+            break;
+        }
+        case text_format_t::TF_PCRE: {
+            readline_regex_highlighter(al, x);
+            break;
+        }
+        case text_format_t::TF_SHELL_SCRIPT: {
+            readline_shlex_highlighter(al, x);
+            break;
+        }
+        case text_format_t::TF_LNAV_SCRIPT: {
+            readline_lnav_highlighter(al, x);
+            break;
+        }
+        default:
+            break;
+    }
+}
