@@ -35,7 +35,7 @@
 #include "base/time_util.hh"
 #include "config.h"
 #include "pcrepp/pcre2pp.hh"
-#include "scn/scn.h"
+#include "scn/scan.h"
 
 using namespace std::chrono_literals;
 
@@ -45,7 +45,7 @@ static const struct {
 } MATCHERS[relative_time::RTT__MAX] = {
     {
         "ws",
-        lnav::pcre2pp::code::from_const("\\A\\s+\\b"),
+        lnav::pcre2pp::code::from_const("\\A\\s+(?:\\b|$)"),
     },
     {
         "am",
@@ -202,7 +202,7 @@ static const struct {
     },
 };
 
-static int64_t TIME_SCALES[] = {
+static constexpr int64_t TIME_SCALES[] = {
     1000 * 1000,
     60,
     60,
@@ -295,10 +295,9 @@ relative_time::from_str(string_fragment str)
 
         bool found = false;
         for (int lpc = 0; lpc < RTT__MAX && !found; lpc++) {
-            static thread_local auto md
-                = lnav::pcre2pp::match_data::unitialized();
+            thread_local auto md = lnav::pcre2pp::match_data::unitialized();
 
-            token_t token = (token_t) lpc;
+            auto token = (token_t) lpc;
             auto match_res = MATCHERS[lpc]
                                  .pcre.capture_from(remaining)
                                  .into(md)
@@ -506,7 +505,7 @@ relative_time::from_str(string_fragment str)
                             FMT_STRING("Invalid number: {}"), md[0].value());
                         return Err(pe_out);
                     }
-                    number = num_scan_res.value();
+                    number = num_scan_res->value();
                     number_set = true;
                     break;
                 }
@@ -779,7 +778,7 @@ relative_time::from_usecs(std::chrono::microseconds usecs)
 std::string
 relative_time::to_string() const
 {
-    static const char* DAYS[] = {
+    static constexpr const char* DAYS[] = {
         "sun",
         "mon",
         "tue",
@@ -887,15 +886,26 @@ relative_time::to_string() const
     return dst;
 }
 
-struct exttm
+exttm
 relative_time::adjust(const exttm& tm) const
 {
+    size_t high_set = RTF_MICROSECONDS;
     auto retval = tm;
 
+    for (auto lpc = 0; lpc < RTF__MAX; lpc++) {
+        if (this->rt_field[lpc].is_set
+            && this->is_absolute((rt_field_type) lpc))
+        {
+            high_set = lpc;
+        }
+    }
+
     if (this->rt_field[RTF_MICROSECONDS].is_set
         && this->is_absolute(RTF_MICROSECONDS))
     {
         retval.et_nsec = this->rt_field[RTF_MICROSECONDS].value * 1000;
+    } else if (RTF_MICROSECONDS < high_set) {
+        retval.et_nsec = 0;
     } else {
         retval.et_nsec += this->rt_field[RTF_MICROSECONDS].value * 1000;
     }
@@ -911,6 +921,8 @@ relative_time::adjust(const exttm& tm) const
             retval.et_tm.tm_min -= 1;
         }
         retval.et_tm.tm_sec = this->rt_field[RTF_SECONDS].value;
+    } else if (RTF_SECONDS < high_set) {
+        retval.et_tm.tm_sec = 0;
     } else {
         retval.et_tm.tm_sec += this->rt_field[RTF_SECONDS].value;
     }
@@ -931,6 +943,7 @@ relative_time::adjust(const exttm& tm) const
         retval.et_tm.tm_min += this->rt_field[RTF_MINUTES].value;
     }
     if (this->rt_field[RTF_HOURS].is_set && this->is_absolute(RTF_HOURS)) {
+        retval.et_tm.tm_yday = -1;
         if (this->rt_next
             && this->rt_field[RTF_HOURS].value <= tm.et_tm.tm_hour)
         {
@@ -955,8 +968,10 @@ relative_time::adjust(const exttm& tm) const
         {
             retval.et_tm.tm_mon -= 1;
         }
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday = this->rt_field[RTF_DAYS].value;
     } else {
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday += this->rt_field[RTF_DAYS].value;
     }
     if (this->rt_field[RTF_MONTHS].is_set && this->is_absolute(RTF_MONTHS)) {
@@ -983,7 +998,7 @@ relative_time::adjust(const exttm& tm) const
     return retval;
 }
 
-nonstd::optional<exttm>
+std::optional<exttm>
 relative_time::window_start(const struct exttm& tm) const
 {
     auto retval = tm;
@@ -999,17 +1014,14 @@ relative_time::window_start(const struct exttm& tm) const
         tv.tv_sec = us / 1000000ULL;
         tv.tv_usec = us % 1000000ULL;
 
-        retval.et_tm = *gmtime(&tv.tv_sec);
-        retval.et_nsec = tv.tv_usec * 1000ULL;
-
-        return retval;
+        return exttm::from_tv(tv);
     }
 
     bool clear = false;
 
     if (this->rt_field[RTF_YEARS].is_set) {
         if (this->rt_field[RTF_YEARS].value > tm.et_tm.tm_year) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_year = this->rt_field[RTF_YEARS].value;
         clear = true;
@@ -1017,21 +1029,25 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_MONTHS].is_set) {
         if (this->rt_field[RTF_MONTHS].value > tm.et_tm.tm_mon) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mon = this->rt_field[RTF_MONTHS].value;
         clear = true;
     } else if (clear) {
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mon = 0;
     }
 
     if (this->rt_field[RTF_DAYS].is_set) {
         if (this->rt_field[RTF_DAYS].value > tm.et_tm.tm_mday) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday = this->rt_field[RTF_DAYS].value;
         clear = true;
     } else if (clear) {
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday = 1;
     }
 
@@ -1040,14 +1056,14 @@ relative_time::window_start(const struct exttm& tm) const
             (token_t) (RTT_SUNDAY + tm.et_tm.tm_wday));
 
         if (iter == this->rt_included_days.end()) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         clear = true;
     }
 
     if (this->rt_field[RTF_HOURS].is_set) {
         if (this->rt_field[RTF_HOURS].value > tm.et_tm.tm_hour) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_hour = this->rt_field[RTF_HOURS].value;
         clear = true;
@@ -1057,7 +1073,7 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_MINUTES].is_set) {
         if (this->rt_field[RTF_MINUTES].value > tm.et_tm.tm_min) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_min = this->rt_field[RTF_MINUTES].value;
         clear = true;
@@ -1067,7 +1083,7 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_SECONDS].is_set) {
         if (this->rt_field[RTF_SECONDS].value > tm.et_tm.tm_sec) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_sec = this->rt_field[RTF_SECONDS].value;
         clear = true;
@@ -1077,7 +1093,7 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_MICROSECONDS].is_set) {
         if (this->rt_field[RTF_MICROSECONDS].value > tm.et_nsec / 1000) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_nsec = this->rt_field[RTF_MICROSECONDS].value * 1000ULL;
         clear = true;
@@ -1092,7 +1108,7 @@ relative_time::window_start(const struct exttm& tm) const
                         .to_timeval();
 
     if (tv < start_time || end_time < tv) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     return retval;
@@ -1114,6 +1130,7 @@ relative_time::to_microseconds() const
         } else {
             etm.et_tm.tm_mday = 1;
         }
+        etm.et_tm.tm_yday = -1;
         etm.et_tm.tm_min = this->rt_field[RTF_MINUTES].value;
         etm.et_tm.tm_sec = this->rt_field[RTF_SECONDS].value;
 
