@@ -63,9 +63,9 @@
 session_data_t session_data;
 recent_refs_t recent_refs;
 
-static const char* LOG_METADATA_NAME = "log_metadata.db";
+static const char* const LOG_METADATA_NAME = "log_metadata.db";
 
-static const char* META_TABLE_DEF = R"(
+static const char* const META_TABLE_DEF = R"(
 CREATE TABLE IF NOT EXISTS bookmarks (
     log_time datetime,
     log_format varchar(64),
@@ -76,6 +76,7 @@ CREATE TABLE IF NOT EXISTS bookmarks (
     comment text DEFAULT '',
     tags text DEFAULT '',
     annotations text DEFAULT NULL,
+    log_opid text DEFAULT NULL,
 
     PRIMARY KEY (log_time, log_format, log_hash, session_time)
 );
@@ -115,24 +116,25 @@ CREATE TABLE IF NOT EXISTS regex101_entries (
 );
 )";
 
-static const char* BOOKMARK_LRU_STMT
+static const char* const BOOKMARK_LRU_STMT
     = "DELETE FROM bookmarks WHERE access_time <= "
       "  (SELECT access_time FROM bookmarks "
       "   ORDER BY access_time DESC LIMIT 1 OFFSET 50000)";
 
-static const char* NETLOC_LRU_STMT
+static const char* const NETLOC_LRU_STMT
     = "DELETE FROM recent_netlocs WHERE access_time <= "
       "  (SELECT access_time FROM bookmarks "
       "   ORDER BY access_time DESC LIMIT 1 OFFSET 10)";
 
-static const char* UPGRADE_STMTS[] = {
+static const char* const UPGRADE_STMTS[] = {
     R"(ALTER TABLE bookmarks ADD COLUMN comment text DEFAULT '';)",
     R"(ALTER TABLE bookmarks ADD COLUMN tags text DEFAULT '';)",
     R"(ALTER TABLE bookmarks ADD COLUMN annotations text DEFAULT NULL;)",
+    R"(ALTER TABLE bookmarks ADD COLUMN log_opid text DEFAULT NULL;)",
 };
 
-static const size_t MAX_SESSIONS = 8;
-static const size_t MAX_SESSION_FILE_COUNT = 256;
+static constexpr size_t MAX_SESSIONS = 8;
+static constexpr size_t MAX_SESSION_FILE_COUNT = 256;
 
 struct session_line {
     session_line(struct timeval tv,
@@ -302,7 +304,7 @@ init_session()
     session_data.sd_view_states[LNV_LOG].vs_top = -1;
 }
 
-static nonstd::optional<std::string>
+static std::optional<std::string>
 compute_session_id()
 {
     bool has_files = false;
@@ -315,7 +317,7 @@ compute_session_id()
         has_files = true;
         h.update(ld_file_name.first);
     }
-    for (auto& lf : lnav_data.ld_active_files.fc_files) {
+    for (const auto& lf : lnav_data.ld_active_files.fc_files) {
         if (lf->is_valid_filename()) {
             continue;
         }
@@ -324,16 +326,16 @@ compute_session_id()
         }
 
         has_files = true;
-        h.update(lf->get_filename());
+        h.update(lf->get_content_id());
     }
     if (!has_files) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     return h.to_string();
 }
 
-nonstd::optional<session_pair_t>
+std::optional<session_pair_t>
 scan_sessions()
 {
     static_root_mem<glob_t, globfree> view_info_list;
@@ -342,11 +344,9 @@ scan_sessions()
 
     const auto session_id = compute_session_id();
     if (!session_id) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
-    std::list<session_pair_t>& session_file_names
-        = lnav_data.ld_session_id[session_id.value()];
-
+    auto& session_file_names = lnav_data.ld_session_id[session_id.value()];
     session_file_names.clear();
 
     auto view_info_pattern_base
@@ -394,10 +394,10 @@ scan_sessions()
     }
 
     if (session_file_names.empty()) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
-    return nonstd::make_optional(session_file_names.back());
+    return std::make_optional(session_file_names.back());
 }
 
 void
@@ -414,6 +414,7 @@ load_time_bookmarks()
          comment,
          tags,
          annotations,
+         log_opid,
          session_time=? AS same_session
        FROM bookmarks WHERE
          log_time BETWEEN ? AND ? AND
@@ -421,7 +422,7 @@ load_time_bookmarks()
        ORDER BY same_session DESC, session_time DESC
 )";
 
-    logfile_sub_source& lss = lnav_data.ld_log_source;
+    auto& lss = lnav_data.ld_log_source;
     auto_sqlite3 db;
     auto db_path = lnav::paths::dotlnav() / LOG_METADATA_NAME;
     auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
@@ -487,12 +488,14 @@ load_time_bookmarks()
          ++file_iter)
     {
         auto lf = (*file_iter)->get_file();
-        const auto* format = lf->get_format_ptr();
-        content_line_t base_content_line;
-
         if (lf == nullptr) {
             continue;
         }
+        if (lf->size() == 0) {
+            continue;
+        }
+        const auto* format = lf->get_format_ptr();
+        content_line_t base_content_line;
 
         base_content_line = lss.get_file_base_content_line(file_iter);
 
@@ -529,6 +532,7 @@ load_time_bookmarks()
                         = (const char*) sqlite3_column_text(stmt.in(), 0);
                     const char* log_hash
                         = (const char*) sqlite3_column_text(stmt.in(), 2);
+                    int64_t mark_time = sqlite3_column_int64(stmt.in(), 3);
                     const char* part_name
                         = (const char*) sqlite3_column_text(stmt.in(), 4);
                     const char* comment
@@ -536,7 +540,7 @@ load_time_bookmarks()
                     const char* tags
                         = (const char*) sqlite3_column_text(stmt.in(), 7);
                     const auto annotations = sqlite3_column_text(stmt.in(), 8);
-                    int64_t mark_time = sqlite3_column_int64(stmt.in(), 3);
+                    const auto log_opid = sqlite3_column_text(stmt.in(), 9);
                     struct timeval log_tv;
                     struct exttm log_tm;
 
@@ -568,7 +572,11 @@ load_time_bookmarks()
                     while (line_iter != lf->end()) {
                         const auto line_tv = line_iter->get_timeval();
 
-                        if (line_tv != log_tv) {
+                        // NB: only milliseconds were stored in the DB, but the
+                        // internal rep stores micros now.
+                        if (line_tv.tv_sec != log_tv.tv_sec
+                            || line_tv.tv_usec / 1000 != log_tv.tv_usec / 1000)
+                        {
                             if (format->lf_time_ordered) {
                                 break;
                             }
@@ -669,6 +677,12 @@ load_time_bookmarks()
                                 meta = true;
                             }
                         }
+                        if (log_opid != nullptr && log_opid[0] != '\0') {
+                            auto opid_sf
+                                = string_fragment::from_c_str(log_opid);
+                            lf->set_logline_opid(line_number, opid_sf);
+                            meta = true;
+                        }
                         if (!meta) {
                             marked_session_lines.emplace_back(
                                 lf->original_line_time(line_iter),
@@ -722,6 +736,9 @@ load_time_bookmarks()
         if (lf == nullptr) {
             continue;
         }
+        if (lf->size() == 0) {
+            continue;
+        }
 
         lss.find(lf->get_filename().c_str(), base_content_line);
 
@@ -760,8 +777,8 @@ load_time_bookmarks()
                     const char* log_hash
                         = (const char*) sqlite3_column_text(stmt.in(), 2);
                     int64_t mark_time = sqlite3_column_int64(stmt.in(), 3);
-                    struct timeval log_tv;
-                    struct exttm log_tm;
+                    timeval log_tv;
+                    exttm log_tm;
 
                     if (last_mark_time == -1) {
                         last_mark_time = mark_time;
@@ -786,10 +803,11 @@ load_time_bookmarks()
                     auto line_iter
                         = lower_bound(lf->begin(), lf->end(), log_tv);
                     while (line_iter != lf->end()) {
-                        struct timeval line_tv = line_iter->get_timeval();
+                        auto line_tv = line_iter->get_timeval();
 
                         if ((line_tv.tv_sec != log_tv.tv_sec)
-                            || (line_tv.tv_usec != log_tv.tv_usec))
+                            || (line_tv.tv_usec / 1000
+                                != log_tv.tv_usec / 1000))
                         {
                             break;
                         }
@@ -797,7 +815,7 @@ load_time_bookmarks()
                         if (lf->get_content_id() == log_hash) {
                             int file_line
                                 = std::distance(lf->begin(), line_iter);
-                            struct timeval offset;
+                            timeval offset;
 
                             offset_session_lines.emplace_back(
                                 lf->original_line_time(line_iter),
@@ -835,7 +853,10 @@ load_time_bookmarks()
 }
 
 static int
-read_files(yajlpp_parse_context* ypc, const unsigned char* str, size_t len)
+read_files(yajlpp_parse_context* ypc,
+           const unsigned char* str,
+           size_t len,
+           yajl_string_props_t*)
 {
     return 1;
 }
@@ -843,6 +864,7 @@ read_files(yajlpp_parse_context* ypc, const unsigned char* str, size_t len)
 static const struct json_path_container view_def_handlers = {
     json_path_handler("top_line").for_field(&view_state::vs_top),
     json_path_handler("focused_line").for_field(&view_state::vs_selection),
+    json_path_handler("anchor").for_field(&view_state::vs_anchor),
     json_path_handler("search").for_field(&view_state::vs_search),
     json_path_handler("word_wrap").for_field(&view_state::vs_word_wrap),
     json_path_handler("filtering").for_field(&view_state::vs_filtering),
@@ -853,17 +875,16 @@ static const struct json_path_container view_handlers = {
     yajlpp::pattern_property_handler("(?<view_name>[\\w\\-]+)")
         .with_obj_provider<view_state, session_data_t>(
             +[](const yajlpp_provider_context& ypc, session_data_t* root) {
-                const char** view_name;
-                int view_index;
-
-                view_name = find(lnav_view_strings,
-                                 lnav_view_strings + LNV__MAX,
-                                 ypc.get_substr("view_name"));
-                view_index = view_name - lnav_view_strings;
-                if (view_index < LNV__MAX) {
-                    return &root->sd_view_states[view_index];
+                auto view_index_opt
+                    = view_from_string(ypc.get_substr("view_name").c_str());
+                if (view_index_opt) {
+                    return &root->sd_view_states[view_index_opt.value()];
                 }
-                return (view_state*) nullptr;
+
+                log_error("unknown view name: %s",
+                          ypc.get_substr("view_name").c_str());
+                static view_state dummy;
+                return &dummy;
             })
         .with_children(view_def_handlers),
 };
@@ -947,7 +968,7 @@ load_session()
         }
         session_data = complete_res.unwrap();
 
-        bool log_changes = false, text_changes = false;
+        bool log_changes = false;
 
         for (auto& lf : lnav_data.ld_active_files.fc_files) {
             auto iter = session_data.sd_file_states.find(lf->get_filename());
@@ -973,9 +994,6 @@ load_session()
         if (log_changes) {
             lnav_data.ld_log_source.text_filters_changed();
         }
-        if (text_changes) {
-            lnav_data.ld_text_source.text_filters_changed();
-        }
     };
 
     lnav::events::publish(lnav_data.ld_db.in(),
@@ -999,9 +1017,11 @@ save_user_bookmarks(sqlite3* db,
 {
     auto& lss = lnav_data.ld_log_source;
 
-    for (auto iter = user_marks.begin(); iter != user_marks.end(); ++iter) {
+    for (auto iter = user_marks.bv_tree.begin();
+         iter != user_marks.bv_tree.end();
+         ++iter)
+    {
         content_line_t cl = *iter;
-        auto line_meta_opt = lss.find_bookmark_metadata(cl);
         auto lf = lss.find(cl);
         if (lf == nullptr) {
             continue;
@@ -1009,6 +1029,58 @@ save_user_bookmarks(sqlite3* db,
 
         sqlite3_clear_bindings(stmt);
 
+        const auto line_iter = lf->begin() + cl;
+        auto read_result = lf->read_line(line_iter);
+
+        if (read_result.isErr()) {
+            continue;
+        }
+
+        auto line_hash = read_result
+                             .map([cl](auto sbr) {
+                                 return hasher()
+                                     .update(sbr.get_data(), sbr.length())
+                                     .update(cl)
+                                     .to_string();
+                             })
+                             .unwrap();
+
+        if (bind_values(stmt,
+                        lf->original_line_time(line_iter),
+                        lf->get_format()->get_name(),
+                        line_hash,
+                        lnav_data.ld_session_time)
+            != SQLITE_OK)
+        {
+            continue;
+        }
+
+        if (sqlite3_bind_text(stmt, 5, "", 0, SQLITE_TRANSIENT) != SQLITE_OK) {
+            log_error("could not bind log hash -- %s", sqlite3_errmsg(db));
+            return;
+        }
+
+        if (sqlite3_step(stmt) != SQLITE_DONE) {
+            log_error("could not execute bookmark insert statement -- %s",
+                      sqlite3_errmsg(db));
+            return;
+        }
+
+        marked_session_lines.emplace_back(lf->original_line_time(line_iter),
+                                          lf->get_format_ptr()->get_name(),
+                                          line_hash);
+
+        sqlite3_reset(stmt);
+    }
+}
+
+static void
+save_meta_bookmarks(sqlite3* db, sqlite3_stmt* stmt, logfile* lf)
+{
+    for (const auto& bm_pair : lf->get_bookmark_metadata()) {
+        auto cl = content_line_t(bm_pair.first);
+        sqlite3_clear_bindings(stmt);
+
         auto line_iter = lf->begin() + cl;
         auto read_result = lf->read_line(line_iter);
 
@@ -1035,85 +1107,82 @@ save_user_bookmarks(sqlite3* db,
             continue;
         }
 
-        if (!line_meta_opt) {
-            if (sqlite3_bind_text(stmt, 5, "", 0, SQLITE_TRANSIENT)
+        const auto& line_meta = bm_pair.second;
+        if (line_meta.empty(bookmark_metadata::categories::any)) {
+            continue;
+        }
+
+        if (sqlite3_bind_text(stmt,
+                              5,
+                              line_meta.bm_name.c_str(),
+                              line_meta.bm_name.length(),
+                              SQLITE_TRANSIENT)
+            != SQLITE_OK)
+        {
+            log_error("could not bind part name -- %s", sqlite3_errmsg(db));
+            return;
+        }
+
+        if (sqlite3_bind_text(stmt,
+                              6,
+                              line_meta.bm_comment.c_str(),
+                              line_meta.bm_comment.length(),
+                              SQLITE_TRANSIENT)
+            != SQLITE_OK)
+        {
+            log_error("could not bind comment -- %s", sqlite3_errmsg(db));
+            return;
+        }
+
+        std::string tags;
+
+        if (!line_meta.bm_tags.empty()) {
+            yajlpp_gen gen;
+
+            yajl_gen_config(gen, yajl_gen_beautify, false);
+
+            {
+                yajlpp_array arr(gen);
+
+                for (const auto& str : line_meta.bm_tags) {
+                    arr.gen(str);
+                }
+            }
+
+            tags = gen.to_string_fragment().to_string();
+        }
+
+        if (sqlite3_bind_text(
+                stmt, 7, tags.c_str(), tags.length(), SQLITE_TRANSIENT)
+            != SQLITE_OK)
+        {
+            log_error("could not bind tags -- %s", sqlite3_errmsg(db));
+            return;
+        }
+
+        if (!line_meta.bm_annotations.la_pairs.empty()) {
+            auto anno_str = logmsg_annotations_handlers.to_string(
+                line_meta.bm_annotations);
+
+            if (sqlite3_bind_text(stmt,
+                                  8,
+                                  anno_str.c_str(),
+                                  anno_str.length(),
+                                  SQLITE_TRANSIENT)
                 != SQLITE_OK)
             {
-                log_error("could not bind log hash -- %s", sqlite3_errmsg(db));
+                log_error("could not bind annotations -- %s",
+                          sqlite3_errmsg(db));
                 return;
             }
         } else {
-            const auto& line_meta = *(line_meta_opt.value());
-            if (line_meta.empty(bookmark_metadata::categories::any)) {
-                continue;
-            }
+            sqlite3_bind_null(stmt, 8);
+        }
 
-            if (sqlite3_bind_text(stmt,
-                                  5,
-                                  line_meta.bm_name.c_str(),
-                                  line_meta.bm_name.length(),
-                                  SQLITE_TRANSIENT)
-                != SQLITE_OK)
-            {
-                log_error("could not bind part name -- %s", sqlite3_errmsg(db));
-                return;
-            }
-
-            if (sqlite3_bind_text(stmt,
-                                  6,
-                                  line_meta.bm_comment.c_str(),
-                                  line_meta.bm_comment.length(),
-                                  SQLITE_TRANSIENT)
-                != SQLITE_OK)
-            {
-                log_error("could not bind comment -- %s", sqlite3_errmsg(db));
-                return;
-            }
-
-            std::string tags;
-
-            if (!line_meta.bm_tags.empty()) {
-                yajlpp_gen gen;
-
-                yajl_gen_config(gen, yajl_gen_beautify, false);
-
-                {
-                    yajlpp_array arr(gen);
-
-                    for (const auto& str : line_meta.bm_tags) {
-                        arr.gen(str);
-                    }
-                }
-
-                tags = gen.to_string_fragment().to_string();
-            }
-
-            if (sqlite3_bind_text(
-                    stmt, 7, tags.c_str(), tags.length(), SQLITE_TRANSIENT)
-                != SQLITE_OK)
-            {
-                log_error("could not bind tags -- %s", sqlite3_errmsg(db));
-                return;
-            }
-
-            if (!line_meta.bm_annotations.la_pairs.empty()) {
-                auto anno_str = logmsg_annotations_handlers.to_string(
-                    line_meta.bm_annotations);
-
-                if (sqlite3_bind_text(stmt,
-                                      8,
-                                      anno_str.c_str(),
-                                      anno_str.length(),
-                                      SQLITE_TRANSIENT)
-                    != SQLITE_OK)
-                {
-                    log_error("could not bind annotations -- %s",
-                              sqlite3_errmsg(db));
-                    return;
-                }
-            } else {
-                sqlite3_bind_null(stmt, 8);
-            }
+        if (line_meta.bm_opid.empty()) {
+            sqlite3_bind_null(stmt, 9);
+        } else {
+            bind_to_sqlite(stmt, 9, line_meta.bm_opid);
         }
 
         if (sqlite3_step(stmt) != SQLITE_DONE) {
@@ -1190,8 +1259,8 @@ save_time_bookmarks()
         recent_refs.rr_netlocs.insert(netlocs.begin(), netlocs.end());
     }
 
-    logfile_sub_source& lss = lnav_data.ld_log_source;
-    bookmarks<content_line_t>::type& bm = lss.get_user_bookmarks();
+    auto& lss = lnav_data.ld_log_source;
+    auto& bm = lss.get_user_bookmarks();
 
     if (sqlite3_prepare_v2(db.in(),
                            "DELETE FROM bookmarks WHERE "
@@ -1234,8 +1303,8 @@ save_time_bookmarks()
     if (sqlite3_prepare_v2(db.in(),
                            "REPLACE INTO bookmarks"
                            " (log_time, log_format, log_hash, session_time, "
-                           "part_name, comment, tags, annotations)"
-                           " VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
+                           "part_name, comment, tags, annotations, log_opid)"
+                           " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
                            -1,
                            stmt.out(),
                            nullptr)
@@ -1247,9 +1316,7 @@ save_time_bookmarks()
     }
 
     {
-        logfile_sub_source::iterator file_iter;
-
-        for (file_iter = lnav_data.ld_log_source.begin();
+        for (auto file_iter = lnav_data.ld_log_source.begin();
              file_iter != lnav_data.ld_log_source.end();
              ++file_iter)
         {
@@ -1258,6 +1325,9 @@ save_time_bookmarks()
             if (lf == nullptr) {
                 continue;
             }
+            if (lf->size() == 0) {
+                continue;
+            }
 
             content_line_t base_content_line;
             base_content_line = lss.get_file_base_content_line(file_iter);
@@ -1289,11 +1359,14 @@ save_time_bookmarks()
     }
 
     save_user_bookmarks(db.in(), stmt.in(), bm[&textview_curses::BM_USER]);
-    auto all_meta_marks = bm[&textview_curses::BM_META];
-    const auto& bm_parts = bm[&textview_curses::BM_PARTITION];
-    all_meta_marks.insert(
-        all_meta_marks.end(), bm_parts.begin(), bm_parts.end());
-    save_user_bookmarks(db.in(), stmt.in(), all_meta_marks);
+    for (const auto& ldd : lss) {
+        auto* lf = ldd->get_file_ptr();
+        if (lf == nullptr) {
+            continue;
+        }
+
+        save_meta_bookmarks(db.in(), stmt.in(), lf);
+    }
 
     if (sqlite3_prepare_v2(db.in(),
                            "DELETE FROM time_offset WHERE "
@@ -1340,7 +1413,7 @@ save_time_bookmarks()
                            " VALUES (?, ?, ?, ?, ?, ?)",
                            -1,
                            stmt.out(),
-                           NULL)
+                           nullptr)
         != SQLITE_OK)
     {
         log_error("could not prepare time_offset replace statement -- %s",
@@ -1349,26 +1422,24 @@ save_time_bookmarks()
     }
 
     {
-        logfile_sub_source::iterator file_iter;
-
-        for (file_iter = lnav_data.ld_log_source.begin();
+        for (auto file_iter = lnav_data.ld_log_source.begin();
              file_iter != lnav_data.ld_log_source.end();
              ++file_iter)
         {
             auto lf = (*file_iter)->get_file();
-            content_line_t base_content_line;
-
             if (lf == nullptr) {
                 continue;
             }
+            if (lf->size() == 0) {
+                continue;
+            }
 
-            base_content_line = lss.get_file_base_content_line(file_iter);
-
-            if (!bind_values(stmt,
-                             lf->original_line_time(lf->begin()),
-                             lf->get_format()->get_name(),
-                             lf->get_content_id(),
-                             lnav_data.ld_session_time))
+            if (bind_values(stmt,
+                            lf->original_line_time(lf->begin()),
+                            lf->get_format()->get_name(),
+                            lf->get_content_id(),
+                            lnav_data.ld_session_time)
+                != SQLITE_OK)
             {
                 continue;
             }
@@ -1395,20 +1466,18 @@ save_time_bookmarks()
         }
     }
 
-    for (auto& ls : lss) {
+    for (const auto& ls : lss) {
         if (ls->get_file() == nullptr) {
             continue;
         }
 
-        auto lf = ls->get_file();
-
+        const auto lf = ls->get_file();
         if (!lf->is_time_adjusted()) {
             continue;
         }
 
         auto line_iter = lf->begin() + lf->get_time_offset_line();
-        struct timeval offset = lf->get_time_offset();
-
+        auto offset = lf->get_time_offset();
         auto read_result = lf->read_line(line_iter);
 
         if (read_result.isErr()) {
@@ -1527,8 +1596,7 @@ save_session_with_id(const std::string& session_id)
 
                 for (int lpc = 0; lpc < LNV__MAX; lpc++) {
                     auto& tc = lnav_data.ld_views[lpc];
-                    unsigned long width;
-                    vis_line_t height;
+                    auto* ta = dynamic_cast<text_anchors*>(tc.get_sub_source());
 
                     top_view_map.gen(lnav_view_strings[lpc]);
 
@@ -1536,7 +1604,6 @@ save_session_with_id(const std::string& session_id)
 
                     view_map.gen("top_line");
 
-                    tc.get_dimensions(height, width);
                     if (tc.get_top() >= tc.get_top_for_last_row()) {
                         view_map.gen(-1LL);
                     } else {
@@ -1547,8 +1614,20 @@ save_session_with_id(const std::string& session_id)
                         && tc.get_inner_height() > 0_vl
                         && tc.get_selection() != tc.get_inner_height() - 1)
                     {
-                        view_map.gen("focused_line");
-                        view_map.gen((long long) tc.get_selection());
+                        auto sel = tc.get_selection();
+                        if (sel) {
+                            view_map.gen("focused_line");
+                            view_map.gen((long long) sel.value());
+
+                            if (ta != nullptr) {
+                                auto anchor_opt
+                                    = ta->anchor_for_row(sel.value());
+                                if (anchor_opt) {
+                                    view_map.gen("anchor");
+                                    view_map.gen(anchor_opt.value());
+                                }
+                            }
+                        }
                     }
 
                     view_map.gen("search");
@@ -1565,7 +1644,7 @@ save_session_with_id(const std::string& session_id)
                     view_map.gen("filtering");
                     view_map.gen(tss->tss_apply_filters);
 
-                    filter_stack& fs = tss->get_filters();
+                    auto& fs = tss->get_filters();
 
                     view_map.gen("commands");
                     yajlpp_array cmd_array(handle);
@@ -1593,7 +1672,8 @@ save_session_with_id(const std::string& session_id)
                         cmd_array.gen("highlight " + hl.first.second);
                     }
 
-                    if (lpc == LNV_LOG) {
+                    auto* ttt = dynamic_cast<text_time_translator*>(tss);
+                    if (ttt != nullptr) {
                         for (const auto& format :
                              log_format::get_root_formats())
                         {
@@ -1618,10 +1698,8 @@ save_session_with_id(const std::string& session_id)
                             }
                         }
 
-                        auto& lss = lnav_data.ld_log_source;
-
-                        auto min_time_opt = lss.get_min_log_time();
-                        auto max_time_opt = lss.get_max_log_time();
+                        auto min_time_opt = ttt->get_min_row_time();
+                        auto max_time_opt = ttt->get_max_row_time();
                         char min_time_str[32], max_time_str[32];
 
                         if (min_time_opt) {
@@ -1639,9 +1717,12 @@ save_session_with_id(const std::string& session_id)
                                           + std::string(max_time_str));
                         }
 
-                        auto mark_expr = lss.get_sql_marker_text();
-                        if (!mark_expr.empty()) {
-                            cmd_array.gen("mark-expr " + mark_expr);
+                        if (lpc == LNV_LOG) {
+                            auto& lss = lnav_data.ld_log_source;
+                            auto mark_expr = lss.get_sql_marker_text();
+                            if (!mark_expr.empty()) {
+                                cmd_array.gen("mark-expr " + mark_expr);
+                            }
                         }
                     }
                 }
@@ -1692,6 +1773,7 @@ reset_session()
     session_data.sd_file_states.clear();
 
     for (auto& tc : lnav_data.ld_views) {
+        auto* ttt = dynamic_cast<text_time_translator*>(tc.get_sub_source());
         auto& hmap = tc.get_highlights();
         auto hl_iter = hmap.begin();
 
@@ -1702,19 +1784,26 @@ reset_session()
                 hmap.erase(hl_iter++);
             }
         }
+
+        if (ttt != nullptr) {
+            ttt->clear_min_max_row_times();
+        }
     }
 
     for (const auto& lf : lnav_data.ld_active_files.fc_files) {
         lf->reset_state();
     }
 
+    // XXX clean this up
+    lnav_data.ld_log_source.set_force_rebuild();
     lnav_data.ld_log_source.set_marked_only(false);
-    lnav_data.ld_log_source.clear_min_max_log_times();
     lnav_data.ld_log_source.set_min_log_level(LEVEL_UNKNOWN);
     lnav_data.ld_log_source.set_sql_filter("", nullptr);
     lnav_data.ld_log_source.set_sql_marker("", nullptr);
-
     lnav_data.ld_log_source.clear_bookmark_metadata();
+    lnav_data.ld_log_source.rebuild_index();
+
+    lnav_data.ld_db_row_source.reset_user_state();
 
     for (auto& tc : lnav_data.ld_views) {
         text_sub_source* tss = tc.get_sub_source();
@@ -1744,7 +1833,7 @@ reset_session()
         bool changed = false;
         for (const auto& vd : elf->elf_value_defs) {
             if (vd.second->vd_meta.lvm_user_hidden) {
-                vd.second->vd_meta.lvm_user_hidden = nonstd::nullopt;
+                vd.second->vd_meta.lvm_user_hidden = std::nullopt;
                 changed = true;
             }
         }
@@ -1761,12 +1850,14 @@ lnav::session::restore_view_states()
     for (size_t view_index = 0; view_index < LNV__MAX; view_index++) {
         const auto& vs = session_data.sd_view_states[view_index];
         auto& tview = lnav_data.ld_views[view_index];
+        auto* ta = dynamic_cast<text_anchors*>(tview.get_sub_source());
         bool has_loc = false;
 
         if (view_index == LNV_TEXT) {
             auto lf = lnav_data.ld_text_source.current_file();
             if (lf != nullptr) {
-                has_loc = lf->get_open_options().loo_init_location.valid();
+                const auto& init_loc = lf->get_open_options().loo_init_location;
+                has_loc = init_loc.valid() || init_loc.is<file_location_tail>();
                 if (!has_loc) {
                     switch (lf->get_text_format()) {
                         case text_format_t::TF_UNKNOWN:
@@ -1781,24 +1872,30 @@ lnav::session::restore_view_states()
                     }
                 }
             }
-        }
+        } else if (view_index == LNV_LOG) {
+            file_location_t log_loc{mapbox::util::no_init{}};
+            for (const auto& ld : lnav_data.ld_log_source) {
+                const auto* lf = ld->get_file_ptr();
+                if (lf == nullptr) {
+                    continue;
+                }
 
-        if (!has_loc && vs.vs_top >= 0
-            && (view_index == LNV_LOG || tview.get_top() == 0_vl
-                || tview.get_top() == tview.get_top_for_last_row()))
-        {
-            log_info("restoring %s view top: %d",
-                     lnav_view_strings[view_index],
-                     vs.vs_top);
-            lnav_data.ld_views[view_index].set_top(vis_line_t(vs.vs_top), true);
-            lnav_data.ld_views[view_index].set_selection(-1_vl);
-        }
-        if (!has_loc && vs.vs_selection) {
-            log_info("restoring %s view selection: %d",
-                     lnav_view_strings[view_index],
-                     vs.vs_selection.value());
-            lnav_data.ld_views[view_index].set_selection(
-                vis_line_t(vs.vs_selection.value()));
+                const auto& init_loc = lf->get_open_options().loo_init_location;
+                if (init_loc.valid() && init_loc.is<std::string>()) {
+                    has_loc = true;
+                    log_loc = init_loc;
+                }
+            }
+
+            if (log_loc.valid()) {
+                auto anchor = log_loc.get<std::string>();
+                auto row_opt = lnav_data.ld_log_source.row_for_anchor(anchor);
+                if (row_opt) {
+                    log_info("setting LOG view to desired loc: %s",
+                             anchor.c_str());
+                    tview.set_selection(row_opt.value());
+                }
+            }
         }
 
         if (!vs.vs_search.empty()) {
@@ -1826,7 +1923,7 @@ lnav::session::restore_view_states()
                 lnav_data.ld_view_stack.pop_back();
                 lnav_data.ld_view_stack.top() | [](auto* tc) {
                     // XXX
-                    if (tc == &lnav_data.ld_views[LNV_GANTT]) {
+                    if (tc == &lnav_data.ld_views[LNV_TIMELINE]) {
                         auto tss = tc->get_sub_source();
                         tss->text_filters_changed();
                         tc->reload_data();
@@ -1834,6 +1931,43 @@ lnav::session::restore_view_states()
                 };
             }
         }
+
+        if (!has_loc && vs.vs_top >= 0
+            && (view_index == LNV_LOG || tview.get_top() == 0_vl
+                || tview.get_top() == tview.get_top_for_last_row()))
+        {
+            log_info("restoring %s view top: %d",
+                     lnav_view_strings[view_index],
+                     vs.vs_top);
+            tview.set_top(vis_line_t(vs.vs_top), true);
+            tview.set_selection(-1_vl);
+        }
+        if (!has_loc && vs.vs_selection) {
+            log_info("restoring %s view selection: %d",
+                     lnav_view_strings[view_index],
+                     vs.vs_selection.value());
+            tview.set_selection(vis_line_t(vs.vs_selection.value()));
+        }
+        auto sel = tview.get_selection();
+        if (sel && ta != nullptr && vs.vs_anchor && !vs.vs_anchor->empty()) {
+            auto curr_anchor = ta->anchor_for_row(sel.value());
+
+            if (!curr_anchor || curr_anchor.value() != vs.vs_anchor.value()) {
+                log_info("%s view anchor mismatch %s != %s",
+                         lnav_view_strings[view_index],
+                         curr_anchor.value_or("").c_str(),
+                         vs.vs_anchor.value().c_str());
+
+                auto row_opt = ta->row_for_anchor(vs.vs_anchor.value());
+                if (row_opt) {
+                    tview.set_selection(row_opt.value());
+                }
+            }
+        }
+        log_info("%s view actual top/selection: %d/%d",
+                 lnav_view_strings[view_index],
+                 tview.get_top(),
+                 tview.get_selection());
     }
 }
 
