@@ -32,14 +32,15 @@
 #include "spectro_source.hh"
 
 #include "base/ansi_scrubber.hh"
+#include "base/keycodes.hh"
 #include "base/math_util.hh"
 #include "config.h"
 
-nonstd::optional<size_t>
+std::optional<size_t>
 spectrogram_row::nearest_column(size_t current) const
 {
-    nonstd::optional<size_t> retval;
-    nonstd::optional<size_t> nearest_distance;
+    std::optional<size_t> retval;
+    std::optional<size_t> nearest_distance;
 
     for (size_t lpc = 0; lpc < this->sr_width; lpc++) {
         if (this->sr_values[lpc].rb_counter == 0) {
@@ -57,12 +58,14 @@ spectrogram_row::nearest_column(size_t current) const
 }
 
 bool
-spectrogram_source::list_input_handle_key(listview_curses& lv, int ch)
+spectrogram_source::list_input_handle_key(listview_curses& lv,
+                                          const ncinput& ch)
 {
-    switch (ch) {
+    switch (ch.eff_text[0]) {
         case 'm': {
             auto sel = lv.get_selection();
-            if (sel < 0 || (size_t) sel >= this->text_line_count()
+            if (!sel || sel.value() < 0
+                || (size_t) sel.value() >= this->text_line_count()
                 || !this->ss_cursor_column || this->ss_value_source == nullptr)
             {
                 alerter::singleton().chime(
@@ -77,14 +80,14 @@ spectrogram_source::list_input_handle_key(listview_curses& lv, int ch)
             width -= 2;
 
             auto& sb = this->ss_cached_bounds;
-            auto begin_time_opt = this->time_for_row_int(sel);
+            auto begin_time_opt = this->time_for_row_int(sel.value());
             if (!begin_time_opt) {
                 return true;
             }
             auto begin_time = begin_time_opt.value();
-            struct timeval end_time = begin_time.ri_time;
+            auto end_time = to_us(begin_time.ri_time);
 
-            end_time.tv_sec += this->ss_granularity;
+            end_time += this->ss_granularity;
             double range_min, range_max, column_size;
 
             column_size = (sb.sb_max_value_out - sb.sb_min_value_out)
@@ -93,8 +96,8 @@ spectrogram_source::list_input_handle_key(listview_curses& lv, int ch)
                 + this->ss_cursor_column.value_or(0) * column_size;
             range_max = range_min + column_size;
             this->ss_value_source->spectro_mark((textview_curses&) lv,
-                                                begin_time.ri_time.tv_sec,
-                                                end_time.tv_sec,
+                                                to_us(begin_time.ri_time),
+                                                end_time,
                                                 range_min,
                                                 range_max);
             this->invalidate();
@@ -120,8 +123,8 @@ spectrogram_source::list_input_handle_key(listview_curses& lv, int ch)
             return true;
         }
 
-        case KEY_LEFT:
-        case KEY_RIGHT: {
+        case NCKEY_LEFT:
+        case NCKEY_RIGHT: {
             auto sel = lv.get_selection();
             unsigned long width;
             vis_line_t height;
@@ -129,24 +132,25 @@ spectrogram_source::list_input_handle_key(listview_curses& lv, int ch)
 
             lv.get_dimensions(height, width);
 
-            this->text_attrs_for_line((textview_curses&) lv, sel, sa);
+            this->text_attrs_for_line(
+                (textview_curses&) lv, sel.value_or(0_vl), sa);
 
             if (sa.empty()) {
                 this->ss_details_source.reset();
-                this->ss_cursor_column = nonstd::nullopt;
+                this->ss_cursor_column = std::nullopt;
                 return true;
             }
 
             if (!this->ss_cursor_column) {
                 lv.set_selection(0_vl);
             }
-            struct line_range lr(this->ss_cursor_column.value(),
-                                 this->ss_cursor_column.value() + 1);
+            line_range lr(this->ss_cursor_column.value(),
+                          this->ss_cursor_column.value() + 1);
 
             auto current = find_string_attr(sa, lr);
 
             if (current != sa.end()) {
-                if (ch == KEY_LEFT) {
+                if (ch.id == NCKEY_LEFT) {
                     if (current == sa.begin()) {
                         current = sa.end();
                     } else {
@@ -158,7 +162,7 @@ spectrogram_source::list_input_handle_key(listview_curses& lv, int ch)
             }
 
             if (current == sa.end()) {
-                if (ch == KEY_LEFT) {
+                if (ch.id == NCKEY_LEFT) {
                     current = sa.end();
                     --current;
                 } else {
@@ -184,7 +188,10 @@ spectrogram_source::text_handle_mouse(
     mouse_event& me)
 {
     auto sel = tc.get_selection();
-    const auto& s_row = this->load_row(tc, sel);
+    if (!sel) {
+        return false;
+    }
+    const auto& s_row = this->load_row(tc, sel.value());
 
     for (int lpc = 0; lpc <= (int) s_row.sr_width; lpc++) {
         int col_value = s_row.sr_values[lpc].rb_counter;
@@ -211,23 +218,20 @@ spectrogram_source::list_value_for_overlay(const listview_curses& lv,
                                            vis_line_t row,
                                            std::vector<attr_line_t>& value_out)
 {
-    vis_line_t height;
-    unsigned long width;
-
-    lv.get_dimensions(height, width);
+    auto [height, width] = lv.get_dimensions();
     width -= 2;
 
     auto sel = lv.get_selection();
 
-    if (row == sel && this->ss_cursor_column) {
-        const auto& s_row = this->load_row(lv, sel);
+    if (sel && row == sel.value() && this->ss_cursor_column) {
+        const auto& s_row = this->load_row(lv, sel.value());
         const auto& bucket = s_row.sr_values[this->ss_cursor_column.value()];
         auto& sb = this->ss_cached_bounds;
         spectrogram_request sr(sb);
         attr_line_t retval;
 
         auto sel_time = rounddown(sb.sb_begin_time, this->ss_granularity)
-            + sel * this->ss_granularity;
+            + (sel.value() * this->ss_granularity);
         sr.sr_width = width;
         sr.sr_begin_time = sel_time;
         sr.sr_end_time = sel_time + this->ss_granularity;
@@ -254,7 +258,7 @@ spectrogram_source::list_value_for_overlay(const listview_curses& lv,
 
         retval.al_attrs.emplace_back(line_range{0, -1},
                                      VC_ROLE.value(role_t::VCR_STATUS_INFO));
-        if (desc.length() + 8 > width) {
+        if (desc.length() + 8 > (ssize_t) width) {
             desc.clear();
         }
 
@@ -320,7 +324,7 @@ spectrogram_source::text_line_width(textview_curses& tc)
     return width;
 }
 
-nonstd::optional<text_time_translator::row_info>
+std::optional<text_time_translator::row_info>
 spectrogram_source::time_for_row(vis_line_t row)
 {
     if (this->ss_details_source != nullptr) {
@@ -335,45 +339,41 @@ spectrogram_source::time_for_row(vis_line_t row)
     return this->time_for_row_int(row);
 }
 
-nonstd::optional<text_time_translator::row_info>
+std::optional<text_time_translator::row_info>
 spectrogram_source::time_for_row_int(vis_line_t row)
 {
-    struct timeval retval {
-        0, 0
-    };
+    timeval retval{0, 0};
 
     this->cache_bounds();
-    retval.tv_sec
-        = rounddown(this->ss_cached_bounds.sb_begin_time, this->ss_granularity)
-        + row * this->ss_granularity;
+    retval.tv_sec = to_time_t(
+        rounddown(this->ss_cached_bounds.sb_begin_time, this->ss_granularity)
+        + row * this->ss_granularity);
 
     return row_info{retval, row};
 }
 
-nonstd::optional<vis_line_t>
-spectrogram_source::row_for_time(struct timeval time_bucket)
+std::optional<vis_line_t>
+spectrogram_source::row_for_time(timeval time_bucket)
 {
     if (this->ss_value_source == nullptr) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
-    time_t diff;
-    int retval;
-
     this->cache_bounds();
-    auto grain_begin_time
+    const auto tb_us = to_us(time_bucket);
+    const auto grain_begin_time
         = rounddown(this->ss_cached_bounds.sb_begin_time, this->ss_granularity);
-    if (time_bucket.tv_sec < grain_begin_time) {
+    if (tb_us < grain_begin_time) {
         return 0_vl;
     }
 
-    diff = time_bucket.tv_sec - grain_begin_time;
-    retval = diff / this->ss_granularity;
+    const auto diff = tb_us - grain_begin_time;
+    const auto retval = diff / this->ss_granularity;
 
     return vis_line_t(retval);
 }
 
-void
+line_info
 spectrogram_source::text_value_for_line(textview_curses& tc,
                                         int row,
                                         std::string& value_out,
@@ -386,7 +386,7 @@ spectrogram_source::text_value_for_line(textview_curses& tc,
     auto row_time_opt = this->time_for_row_int(vis_line_t(row));
     if (!row_time_opt) {
         value_out.clear();
-        return;
+        return {};
     }
     auto ri = row_time_opt.value();
 
@@ -401,6 +401,8 @@ spectrogram_source::text_value_for_line(textview_curses& tc,
             value_out[lpc] = 'x';
         }
     }
+
+    return {};
 }
 
 void
@@ -456,8 +458,9 @@ spectrogram_source::cache_bounds()
 {
     if (this->ss_value_source == nullptr) {
         this->ss_cached_bounds.sb_count = 0;
-        this->ss_cached_bounds.sb_begin_time = 0;
-        this->ss_cursor_column = nonstd::nullopt;
+        this->ss_cached_bounds.sb_begin_time
+            = std::chrono::microseconds::zero();
+        this->ss_cursor_column = std::nullopt;
         this->reset_details_source();
         return;
     }
@@ -474,17 +477,19 @@ spectrogram_source::cache_bounds()
 
     if (sb.sb_count == 0) {
         this->ss_cached_line_count = 0;
-        this->ss_cursor_column = nonstd::nullopt;
+        this->ss_cursor_column = std::nullopt;
         this->reset_details_source();
         return;
     }
 
-    time_t grain_begin_time = rounddown(sb.sb_begin_time, this->ss_granularity);
-    time_t grain_end_time = roundup_size(sb.sb_end_time, this->ss_granularity);
+    auto grain_begin_time = rounddown(sb.sb_begin_time, this->ss_granularity);
+    auto grain_end_time = roundup_size(sb.sb_end_time, this->ss_granularity);
 
-    time_t diff = std::max((time_t) 1, grain_end_time - grain_begin_time);
+    auto diff = std::max(std::chrono::microseconds{1},
+                         grain_end_time - grain_begin_time);
     this->ss_cached_line_count
-        = (diff + this->ss_granularity - 1) / this->ss_granularity;
+        = (diff + this->ss_granularity - std::chrono::microseconds{1})
+        / this->ss_granularity;
 
     int64_t samples_per_row = sb.sb_count / this->ss_cached_line_count;
     auto& st = this->ss_cached_thresholds;
@@ -556,15 +561,15 @@ void
 spectrogram_source::text_selection_changed(textview_curses& tc)
 {
     if (this->ss_value_source == nullptr || this->text_line_count() == 0) {
-        this->ss_cursor_column = nonstd::nullopt;
+        this->ss_cursor_column = std::nullopt;
         this->ss_details_source.reset();
         return;
     }
 
-    if (tc.get_selection() == -1_vl) {
+    if (!tc.get_selection()) {
         tc.set_selection(0_vl);
     }
-    const auto& s_row = this->load_row(tc, tc.get_selection());
+    const auto& s_row = this->load_row(tc, tc.get_selection().value());
     this->ss_cursor_column
         = s_row.nearest_column(this->ss_cursor_column.value_or(0));
     this->ss_details_source.reset();
@@ -633,8 +638,8 @@ spectrogram_source::list_static_overlay(const listview_curses& lv,
     }
     line.append(buf);
 
-    value_out.with_attr(string_attr(line_range(0, -1),
-                                    VC_STYLE.value(text_attrs{A_UNDERLINE})));
+    value_out.with_attr(string_attr(
+        line_range(0, -1), VC_STYLE.value(text_attrs::with_underline())));
 
     return true;
 }
