@@ -32,9 +32,11 @@
 #include "base/injector.bind.hh"
 #include "base/itertools.hh"
 #include "base/lnav_log.hh"
+#include "base/opt_util.hh"
 #include "bound_tags.hh"
 #include "command_executor.hh"
 #include "config.h"
+#include "fmt/chrono.h"
 #include "lnav.hh"
 #include "readline_context.hh"
 #include "shlex.hh"
@@ -63,6 +65,10 @@ sql_cmd_dump(exec_context& ec,
         return ec.make_error("expecting a file name to write to");
     }
 
+    if (args.size() < 3) {
+        return ec.make_error("expecting a table name to dump");
+    }
+
     if (lnav_flags & LNF_SECURE_MODE) {
         return ec.make_error("{} -- unavailable in secure mode", args[0]);
     }
@@ -113,9 +119,10 @@ sql_cmd_read(exec_context& ec,
         auto split_err = split_args_res.unwrapErr();
         auto um
             = lnav::console::user_message::error("unable to parse file name")
-                  .with_reason(split_err.te_msg)
+                  .with_reason(split_err.se_error.te_msg)
                   .with_snippet(lnav::console::snippet::from(
-                      SRC, lexer.to_attr_line(split_err)));
+                      SRC, lexer.to_attr_line(split_err.se_error)))
+                  .move();
 
         return Err(um);
     }
@@ -171,10 +178,19 @@ sql_cmd_schema(exec_context& ec,
     std::string retval;
 
     if (args.empty()) {
+        args.emplace_back("sql-table");
         return Ok(retval);
     }
 
     ensure_view(LNV_SCHEMA);
+    if (args.size() == 2) {
+        const auto id = text_anchors::to_anchor_string(args[1]);
+        auto* tss = lnav_data.ld_views[LNV_SCHEMA].get_sub_source();
+        if (auto* ta = dynamic_cast<text_anchors*>(tss); ta != nullptr) {
+            ta->row_for_anchor(id) |
+                [](auto row) { lnav_data.ld_views[LNV_SCHEMA].set_top(row); };
+        }
+    }
 
     return Ok(retval);
 }
@@ -248,12 +264,13 @@ prql_cmd_from_prompt(exec_context& ec, const std::string& cmdline)
 
     auto* tc = *lnav_data.ld_view_stack.top();
     auto* lss = dynamic_cast<logfile_sub_source*>(tc->get_sub_source());
+    auto sel = tc->get_selection();
 
-    if (lss == nullptr || lss->text_line_count() == 0) {
+    if (!sel || lss == nullptr || lss->text_line_count() == 0) {
         return {};
     }
 
-    auto line_pair = lss->find_line_with_file(lss->at(tc->get_selection()));
+    auto line_pair = lss->find_line_with_file(lss->at(sel.value()));
     if (!line_pair) {
         return {};
     }
@@ -409,6 +426,8 @@ static readline_context::command_t sql_commands[] = {
         help_text(".dump", "Dump the contents of the database")
             .sql_command()
             .with_parameter({"path", "The path to the file to write"})
+            .with_parameter(help_text{"table", "The name of the table to dump"}
+                                .one_or_more())
             .with_tags({
                 "io",
             }),
@@ -437,7 +456,8 @@ static readline_context::command_t sql_commands[] = {
         help_text(".schema",
                   "Switch to the SCHEMA view that contains a dump of the "
                   "current database schema")
-            .sql_command(),
+            .sql_command()
+            .with_parameter({"name", "The name of a table to jump to"}),
     },
     {
         "ATTACH",
