@@ -57,7 +57,7 @@ using namespace lnav::roles::literals;
 /**
  * Copied from -- http://www.sqlite.org/lang_keywords.html
  */
-const char* sql_keywords[] = {
+const std::array<const char*, 145> sql_keywords = {
     "ABORT",
     "ACTION",
     "ADD",
@@ -290,9 +290,7 @@ std::multimap<std::string, const help_text*> sqlite_function_help;
 static int
 handle_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct sqlite_metadata_callbacks* smc;
-
-    smc = (struct sqlite_metadata_callbacks*) ptr;
+    auto* smc = (struct sqlite_metadata_callbacks*) ptr;
 
     smc->smc_db_list[colvalues[1]] = std::vector<std::string>();
     if (!smc->smc_database_list) {
@@ -303,14 +301,14 @@ handle_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 }
 
 struct table_list_data {
-    struct sqlite_metadata_callbacks* tld_callbacks;
+    sqlite_metadata_callbacks* tld_callbacks;
     db_table_map_t::iterator* tld_iter;
 };
 
 static int
 handle_table_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct table_list_data* tld = (struct table_list_data*) ptr;
+    auto* tld = (struct table_list_data*) ptr;
 
     (*tld->tld_iter)->second.emplace_back(colvalues[0]);
     if (!tld->tld_callbacks->smc_table_list) {
@@ -322,7 +320,7 @@ handle_table_list(void* ptr, int ncols, char** colvalues, char** colnames)
 }
 
 int
-walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
+walk_sqlite_metadata(sqlite3* db, sqlite_metadata_callbacks& smc)
 {
     auto_mem<char, sqlite3_free> errmsg;
     int retval;
@@ -349,7 +347,7 @@ walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
     for (auto iter = smc.smc_db_list.begin(); iter != smc.smc_db_list.end();
          ++iter)
     {
-        struct table_list_data tld = {&smc, &iter};
+        table_list_data tld = {&smc, &iter};
         auto_mem<char, sqlite3_free> query;
 
         query = sqlite3_mprintf(
@@ -368,11 +366,11 @@ walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
              ++table_iter)
         {
             auto_mem<char, sqlite3_free> table_query;
-            std::string& table_name = *table_iter;
+            smc.smc_table_name = *table_iter;
 
             table_query = sqlite3_mprintf("pragma %Q.table_xinfo(%Q)",
                                           iter->first.c_str(),
-                                          table_name.c_str());
+                                          smc.smc_table_name.c_str());
             if (table_query == nullptr) {
                 return SQLITE_NOMEM;
             }
@@ -388,7 +386,7 @@ walk_sqlite_metadata(sqlite3* db, struct sqlite_metadata_callbacks& smc)
 
             table_query = sqlite3_mprintf("pragma %Q.foreign_key_list(%Q)",
                                           iter->first.c_str(),
-                                          table_name.c_str());
+                                          smc.smc_table_name.c_str());
             if (table_query == nullptr) {
                 return SQLITE_NOMEM;
             }
@@ -420,8 +418,8 @@ schema_collation_list(void* ptr, int ncols, char** colvalues, char** colnames)
 static int
 schema_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct sqlite_metadata_callbacks* smc = (sqlite_metadata_callbacks*) ptr;
-    std::string& schema_out = *((std::string*) smc->smc_userdata);
+    auto smc = (sqlite_metadata_callbacks*) ptr;
+    auto& schema_out = *((std::string*) smc->smc_userdata);
     auto_mem<char, sqlite3_free> attach_sql;
 
     attach_sql = sqlite3_mprintf(
@@ -435,8 +433,8 @@ schema_db_list(void* ptr, int ncols, char** colvalues, char** colnames)
 static int
 schema_table_list(void* ptr, int ncols, char** colvalues, char** colnames)
 {
-    struct sqlite_metadata_callbacks* smc = (sqlite_metadata_callbacks*) ptr;
-    std::string& schema_out = *((std::string*) smc->smc_userdata);
+    auto smc = (sqlite_metadata_callbacks*) ptr;
+    auto& schema_out = *((std::string*) smc->smc_userdata);
     auto_mem<char, sqlite3_free> create_sql;
 
     create_sql = sqlite3_mprintf("%s;\n", colvalues[1]);
@@ -461,7 +459,7 @@ schema_foreign_key_list(void* ptr, int ncols, char** colvalues, char** colnames)
 void
 dump_sqlite_schema(sqlite3* db, std::string& schema_out)
 {
-    struct sqlite_metadata_callbacks schema_sql_meta_callbacks = {
+    sqlite_metadata_callbacks schema_sql_meta_callbacks = {
         schema_collation_list,
         schema_db_list,
         schema_table_list,
@@ -574,6 +572,13 @@ sql_ident_needs_quote(const char* ident)
     return false;
 }
 
+std::string
+sql_quote_text(const std::string& str)
+{
+    auto quoted_token = lnav::sql::mprintf("%Q", str.c_str());
+    return {quoted_token};
+}
+
 auto_mem<char, sqlite3_free>
 sql_quote_ident(const char* ident)
 {
@@ -851,22 +856,31 @@ sql_execute_script(sqlite3* db,
     sql_compile_script(db, global_vars, src_name, script, errors);
 }
 
-static struct {
+static const struct {
     int sqlite_type;
     const char* collator;
     const char* sample;
 } TYPE_TEST_VALUE[] = {
     {SQLITE3_TEXT, "", "foobar"},
-    {SQLITE_INTEGER, "", "123"},
     {SQLITE_FLOAT, "", "123.0"},
+    {SQLITE_INTEGER, "", "123"},
     {SQLITE_TEXT, "ipaddress", "127.0.0.1"},
+    {SQLITE_TEXT, "measure_with_units", "123ms"},
+    {SQLITE_TEXT, "measure_with_units", "123 ms"},
+    {SQLITE_TEXT, "measure_with_units", "123KB"},
+    {SQLITE_TEXT, "measure_with_units", "123 KB"},
+    {SQLITE_TEXT, "measure_with_units", "123Kbps"},
+    {SQLITE_TEXT, "measure_with_units", "123.0 Kbps"},
+    {SQLITE_TEXT, "measure_with_units", "123.0KB"},
+    {SQLITE_TEXT, "measure_with_units", "123.0 KB"},
+    {SQLITE_TEXT, "measure_with_units", "123.0Kbps"},
+    {SQLITE_TEXT, "measure_with_units", "123.0 Kbps"},
 };
 
 int
 guess_type_from_pcre(const std::string& pattern, std::string& collator)
 {
-    static const std::vector<int> number_matches = {1, 2};
-
+    log_info("guessing SQL type from pattern: %s", pattern.c_str());
     auto compile_res = lnav::pcre2pp::code::from(pattern);
     if (compile_res.isErr()) {
         return SQLITE3_TEXT;
@@ -879,14 +893,22 @@ guess_type_from_pcre(const std::string& pattern, std::string& collator)
 
     collator.clear();
     for (const auto& test_value : TYPE_TEST_VALUE) {
-        auto find_res
+        log_info("  testing sample: %s", test_value.sample);
+        const auto find_res
             = re.find_in(string_fragment::from_c_str(test_value.sample),
                          PCRE2_ANCHORED)
                   .ignore_error();
         if (find_res && find_res->f_all.sf_begin == 0
             && find_res->f_remaining.empty())
         {
+            log_info("    matched!");
             matches.push_back(index);
+            break;
+        }
+        if (!find_res) {
+            log_info("    mismatch");
+        } else if (!find_res->f_remaining.empty()) {
+            log_info("    incomplete match");
         }
 
         index += 1;
@@ -895,9 +917,6 @@ guess_type_from_pcre(const std::string& pattern, std::string& collator)
     if (matches.size() == 1) {
         retval = TYPE_TEST_VALUE[matches.front()].sqlite_type;
         collator = TYPE_TEST_VALUE[matches.front()].collator;
-    } else if (matches == number_matches) {
-        retval = SQLITE_FLOAT;
-        collator = "";
     }
 
     return retval;
@@ -919,7 +938,7 @@ sqlite3_type_to_string(int type)
             return "BLOB";
     }
 
-    ensure("Invalid sqlite type");
+    ensure(!!!"Invalid sqlite type");
 
     return nullptr;
 }
@@ -984,18 +1003,18 @@ sql_keyword_re()
     return retval;
 }
 
-string_attr_type<void> SQL_COMMAND_ATTR("sql_command");
-string_attr_type<void> SQL_KEYWORD_ATTR("sql_keyword");
-string_attr_type<void> SQL_IDENTIFIER_ATTR("sql_ident");
-string_attr_type<void> SQL_FUNCTION_ATTR("sql_func");
-string_attr_type<void> SQL_STRING_ATTR("sql_string");
-string_attr_type<void> SQL_NUMBER_ATTR("sql_number");
-string_attr_type<void> SQL_UNTERMINATED_STRING_ATTR("sql_unstring");
-string_attr_type<void> SQL_OPERATOR_ATTR("sql_oper");
-string_attr_type<void> SQL_PAREN_ATTR("sql_paren");
-string_attr_type<void> SQL_COMMA_ATTR("sql_comma");
-string_attr_type<void> SQL_GARBAGE_ATTR("sql_garbage");
-string_attr_type<void> SQL_COMMENT_ATTR("sql_comment");
+constexpr string_attr_type<void> SQL_COMMAND_ATTR("sql_command");
+constexpr string_attr_type<void> SQL_KEYWORD_ATTR("sql_keyword");
+constexpr string_attr_type<void> SQL_IDENTIFIER_ATTR("sql_ident");
+constexpr string_attr_type<std::string> SQL_FUNCTION_ATTR("sql_func");
+constexpr string_attr_type<void> SQL_STRING_ATTR("sql_string");
+constexpr string_attr_type<void> SQL_HEX_LIT_ATTR("sql_hex_lit");
+constexpr string_attr_type<void> SQL_NUMBER_ATTR("sql_number");
+constexpr string_attr_type<void> SQL_OPERATOR_ATTR("sql_oper");
+constexpr string_attr_type<void> SQL_PAREN_ATTR("sql_paren");
+constexpr string_attr_type<void> SQL_COMMA_ATTR("sql_comma");
+constexpr string_attr_type<void> SQL_GARBAGE_ATTR("sql_garbage");
+constexpr string_attr_type<void> SQL_COMMENT_ATTR("sql_comment");
 
 void
 annotate_sql_statement(attr_line_t& al)
@@ -1004,7 +1023,7 @@ annotate_sql_statement(attr_line_t& al)
 
     static const struct {
         lnav::pcre2pp::code re;
-        string_attr_type<void>* type;
+        const string_attr_type<void>* type;
     } PATTERNS[] = {
         {
             lnav::pcre2pp::code::from_const(R"(\A,)"),
@@ -1018,6 +1037,11 @@ annotate_sql_statement(attr_line_t& al)
             lnav::pcre2pp::code::from(keyword_re_str, PCRE2_CASELESS).unwrap(),
             &SQL_KEYWORD_ATTR,
         },
+        {
+            lnav::pcre2pp::code::from_const(
+                R"(\A(?:x|X)'(?:(?:[0-9a-fA-F]{2})+'|[0-9a-fA-F]*$))"),
+            &SQL_HEX_LIT_ATTR,
+        },
         {
             lnav::pcre2pp::code::from_const(R"(\A'[^']*('(?:'[^']*')*|$))"),
             &SQL_STRING_ATTR,
@@ -1028,7 +1052,7 @@ annotate_sql_statement(attr_line_t& al)
         },
         {
             lnav::pcre2pp::code::from_const(
-                R"(\A-?\d+(?:\.\d+)?(?:[eE][\-\+]?\d+)?)"),
+                R"(\A-?\d+(?:\.\d+)?(?:[eE][\-\+]?\d+)?\b)"),
             &SQL_NUMBER_ATTR,
         },
         {
@@ -1046,6 +1070,10 @@ annotate_sql_statement(attr_line_t& al)
                 R"(\A(\*|\->{1,2}|<|>|=|!|\-|\+|\|\|))"),
             &SQL_OPERATOR_ATTR,
         },
+        {
+            lnav::pcre2pp::code::from_const(R"(\A[0-9][a-zA-Z0-9\-\._]+)"),
+            &SQL_GARBAGE_ATTR,
+        },
         {
             lnav::pcre2pp::code::from_const(R"(\A.)"),
             &SQL_GARBAGE_ATTR,
@@ -1053,10 +1081,10 @@ annotate_sql_statement(attr_line_t& al)
     };
 
     static const auto cmd_pattern
-        = lnav::pcre2pp::code::from_const(R"(^(\.\w+))");
+        = lnav::pcre2pp::code::from_const(R"(^;?(\.\w+))");
     static const auto ws_pattern = lnav::pcre2pp::code::from_const(R"(\A\s+)");
 
-    auto& line = al.get_string();
+    const auto& line = al.get_string();
     auto& sa = al.get_attrs();
 
     if (lnav::sql::is_prql(line)) {
@@ -1098,22 +1126,10 @@ annotate_sql_statement(attr_line_t& al)
            != sa.end())
     {
         string_attrs_t::const_iterator piter;
-        bool found_open = false;
-        ssize_t lpc;
 
         start = iter->sa_range.lr_end;
-        for (lpc = iter->sa_range.lr_end; lpc < (int) line.length(); lpc++) {
-            if (line[lpc] == '(') {
-                found_open = true;
-                break;
-            }
-            if (!isspace(line[lpc])) {
-                break;
-            }
-        }
-
-        if (found_open) {
-            ssize_t pstart = lpc + 1;
+        if (start < (ssize_t) line.length() && line[start] == '(') {
+            ssize_t pstart = start + 1;
             int depth = 1;
 
             while (depth > 0
@@ -1132,13 +1148,16 @@ annotate_sql_statement(attr_line_t& al)
             if (piter == sa.end()) {
                 func_range.lr_end = line.length();
             } else {
-                func_range.lr_end = piter->sa_range.lr_end - 1;
+                func_range.lr_end = piter->sa_range.lr_end;
             }
-            sa.emplace_back(func_range, SQL_FUNCTION_ATTR.value());
+            auto func_name = al.to_string_fragment(iter);
+            sa.emplace_back(
+                func_range,
+                SQL_FUNCTION_ATTR.value(tolower(func_name.to_string())));
         }
     }
 
-    remove_string_attr(sa, &SQL_PAREN_ATTR);
+    // remove_string_attr(sa, &SQL_PAREN_ATTR);
     stable_sort(sa.begin(), sa.end());
 }
 
@@ -1203,9 +1222,8 @@ find_sql_help_for_line(const attr_line_t& al, size_t x)
             return false;
         }
 
-        const std::string& str = al.get_string();
-        const line_range& lr = sa.sa_range;
-        int lpc;
+        const auto& str = al.get_string();
+        const auto& lr = sa.sa_range;
 
         if (sa.sa_type == &SQL_FUNCTION_ATTR) {
             if (!sa.sa_range.contains(x)) {
@@ -1213,7 +1231,8 @@ find_sql_help_for_line(const attr_line_t& al, size_t x)
             }
         }
 
-        for (lpc = lr.lr_start; lpc < lr.lr_end; lpc++) {
+        auto lpc = lr.lr_start;
+        for (; lpc < lr.lr_end; lpc++) {
             if (!isalnum(str[lpc]) && str[lpc] != '_') {
                 break;
             }
@@ -1282,7 +1301,7 @@ is_prql(const string_fragment& sf)
     return (trimmed.startswith("let ") || trimmed.startswith("from"));
 }
 
-const char* prql_transforms[] = {
+static const char* const prql_transforms[] = {
     "aggregate",
     "append",
     "derive",
@@ -1299,7 +1318,7 @@ const char* prql_transforms[] = {
     nullptr,
 };
 
-const char* prql_keywords[] = {
+const char* const prql_keywords[] = {
     "average", "avg", "case", "count", "count_distinct", "false", "func",
     "into",    "let", "max",  "min",   "module",         "null",  "prql",
     "stddev",  "sum", "true", "type",
@@ -1355,20 +1374,21 @@ prql_transform_re()
     return retval;
 }
 
-string_attr_type<void> PRQL_STAGE_ATTR("prql_stage");
-string_attr_type<void> PRQL_TRANSFORM_ATTR("prql_transform");
-string_attr_type<void> PRQL_KEYWORD_ATTR("prql_keyword");
-string_attr_type<void> PRQL_IDENTIFIER_ATTR("prql_ident");
-string_attr_type<void> PRQL_FQID_ATTR("prql_fqid");
-string_attr_type<void> PRQL_DOT_ATTR("prql_dot");
-string_attr_type<void> PRQL_PIPE_ATTR("prql_pipe");
-string_attr_type<void> PRQL_STRING_ATTR("prql_string");
-string_attr_type<void> PRQL_NUMBER_ATTR("prql_number");
-string_attr_type<void> PRQL_OPERATOR_ATTR("prql_oper");
-string_attr_type<void> PRQL_PAREN_ATTR("prql_paren");
-string_attr_type<void> PRQL_UNTERMINATED_PAREN_ATTR("prql_unterminated_paren");
-string_attr_type<void> PRQL_GARBAGE_ATTR("prql_garbage");
-string_attr_type<void> PRQL_COMMENT_ATTR("prql_comment");
+constexpr string_attr_type<void> PRQL_STAGE_ATTR("prql_stage");
+constexpr string_attr_type<void> PRQL_TRANSFORM_ATTR("prql_transform");
+constexpr string_attr_type<void> PRQL_KEYWORD_ATTR("prql_keyword");
+constexpr string_attr_type<void> PRQL_IDENTIFIER_ATTR("prql_ident");
+constexpr string_attr_type<void> PRQL_FQID_ATTR("prql_fqid");
+constexpr string_attr_type<void> PRQL_DOT_ATTR("prql_dot");
+constexpr string_attr_type<void> PRQL_PIPE_ATTR("prql_pipe");
+constexpr string_attr_type<void> PRQL_STRING_ATTR("prql_string");
+constexpr string_attr_type<void> PRQL_NUMBER_ATTR("prql_number");
+constexpr string_attr_type<void> PRQL_OPERATOR_ATTR("prql_oper");
+constexpr string_attr_type<void> PRQL_PAREN_ATTR("prql_paren");
+constexpr string_attr_type<void> PRQL_UNTERMINATED_PAREN_ATTR(
+    "prql_unterminated_paren");
+constexpr string_attr_type<void> PRQL_GARBAGE_ATTR("prql_garbage");
+constexpr string_attr_type<void> PRQL_COMMENT_ATTR("prql_comment");
 
 void
 annotate_prql_statement(attr_line_t& al)
@@ -1378,7 +1398,7 @@ annotate_prql_statement(attr_line_t& al)
 
     static const struct {
         lnav::pcre2pp::code re;
-        string_attr_type<void>* type;
+        const string_attr_type<void>* type;
     } PATTERNS[] = {
         {
             lnav::pcre2pp::code::from_const(R"(\A(?:\[|\]|\{|\}|\(|\)))"),
@@ -1424,7 +1444,7 @@ annotate_prql_statement(attr_line_t& al)
             &PRQL_OPERATOR_ATTR,
         },
         {
-            lnav::pcre2pp::code::from_const(R"(\A\|)"),
+            lnav::pcre2pp::code::from_const(R"(\A(?:\||\n))"),
             &PRQL_PIPE_ATTR,
         },
         {
@@ -1437,7 +1457,8 @@ annotate_prql_statement(attr_line_t& al)
         },
     };
 
-    static const auto ws_pattern = lnav::pcre2pp::code::from_const(R"(\A\s+)");
+    static const auto ws_pattern
+        = lnav::pcre2pp::code::from_const(R"(\A[ \t\r]+)");
 
     const auto& line = al.get_string();
     auto& sa = al.get_attrs();
@@ -1453,6 +1474,11 @@ annotate_prql_statement(attr_line_t& al)
             if (pat_find_res) {
                 sa.emplace_back(to_line_range(pat_find_res->f_all),
                                 pat.type->value());
+                if (sa.back().sa_type == &PRQL_PIPE_ATTR
+                    && pat_find_res->f_all == "\n"_frag)
+                {
+                    sa.back().sa_range.lr_start += 1;
+                }
                 remaining = pat_find_res->f_remaining;
                 break;
             }
@@ -1462,12 +1488,16 @@ annotate_prql_statement(attr_line_t& al)
     auto stages = std::vector<int>{};
     std::vector<std::pair<char, int>> groups;
     std::vector<line_range> fqids;
-    nonstd::optional<line_range> id_start;
+    std::optional<line_range> id_start;
+    const string_attr_type_base* last_attr_type = nullptr;
     bool saw_id_dot = false;
     for (const auto& attr : sa) {
-        if (groups.empty() && attr.sa_type == &PRQL_PIPE_ATTR) {
+        if (groups.empty() && attr.sa_type == &PRQL_PIPE_ATTR
+            && last_attr_type != &PRQL_PIPE_ATTR)
+        {
             stages.push_back(attr.sa_range.lr_start);
         }
+        last_attr_type = attr.sa_type;
         if (!id_start) {
             if (attr.sa_type == &PRQL_IDENTIFIER_ATTR) {
                 id_start = attr.sa_range;
@@ -1478,7 +1508,7 @@ annotate_prql_statement(attr_line_t& al)
                 saw_id_dot = true;
             } else {
                 fqids.emplace_back(id_start.value());
-                id_start = nonstd::nullopt;
+                id_start = std::nullopt;
                 saw_id_dot = false;
             }
         } else {
@@ -1488,7 +1518,7 @@ annotate_prql_statement(attr_line_t& al)
                     attr.sa_range.lr_end,
                 };
             } else {
-                id_start = nonstd::nullopt;
+                id_start = std::nullopt;
             }
             saw_id_dot = false;
         }
@@ -1539,6 +1569,8 @@ annotate_prql_statement(attr_line_t& al)
     for (const auto& fqid_range : fqids) {
         sa.emplace_back(fqid_range, PRQL_FQID_ATTR.value());
     }
+    remove_string_attr(sa, &PRQL_IDENTIFIER_ATTR);
+    remove_string_attr(sa, &PRQL_DOT_ATTR);
 
     stable_sort(sa.begin(), sa.end());
 }
