@@ -43,15 +43,19 @@ status_field::no_op_action(status_field&)
 {
 }
 
-void
+bool
 status_field::set_value(std::string value)
 {
+    if (value == this->sf_value.al_string) {
+        return false;
+    }
     auto& sa = this->sf_value.get_attrs();
 
     sa.clear();
 
     scrub_ansi_string(value, &sa);
     this->sf_value.with_string(value);
+    return true;
 }
 
 void
@@ -66,11 +70,11 @@ status_field::do_cylon()
         ? cycle_pos
         : (this->sf_width - (cycle_pos - this->sf_width) - 1);
     auto stop = std::min(start + 3, this->sf_width);
-    struct line_range lr(std::max<long>(start, 0L), stop);
+    line_range lr(std::max<long>(start, 0L), stop);
     const auto& vc = view_colors::singleton();
 
     auto attrs = vc.attrs_for_role(role_t::VCR_ACTIVE_STATUS);
-    attrs.ta_attrs |= A_REVERSE;
+    attrs |= text_attrs::style::reverse;
     sa.emplace_back(lr, VC_STYLE.value(attrs));
 
     this->sf_cylon_pos += 1;
@@ -80,7 +84,7 @@ void
 status_field::set_stitch_value(role_t left, role_t right)
 {
     auto& sa = this->sf_value.get_attrs();
-    struct line_range lr(0, 1);
+    line_range lr(0, 1);
 
     this->sf_value.get_string() = "::";
     sa.clear();
@@ -93,28 +97,34 @@ status_field::set_stitch_value(role_t left, role_t right)
 bool
 statusview_curses::do_update()
 {
-    int top, left = 0, right;
+    if (!this->vc_needs_update) {
+        return view_curses::do_update();
+    }
+
+    int left = 0;
     auto& vc = view_colors::singleton();
-    unsigned long width, height;
+    unsigned int width, height;
 
     this->sc_displayed_fields.clear();
     if (!this->vc_visible || this->sc_window == nullptr) {
         return false;
     }
 
-    getmaxyx(this->sc_window, height, width);
+    ncplane_dim_yx(this->sc_window, &height, &width);
     this->window_change();
 
-    top = this->vc_y < 0 ? height + this->vc_y : this->vc_y;
-    right = width;
-    auto attrs = vc.attrs_for_role(
-        this->sc_enabled ? this->sc_default_role : role_t::VCR_INACTIVE_STATUS);
+    int top = this->vc_y < 0 ? height + this->vc_y : this->vc_y;
+    int right = width;
+    const auto attrs = vc.attrs_for_role(
+        this->sc_enabled ? this->vc_default_role : role_t::VCR_INACTIVE_STATUS);
 
-    auto pair = vc.ensure_color_pair(attrs.ta_fg_color, attrs.ta_bg_color);
-    wattr_set(this->sc_window, attrs.ta_attrs, pair, nullptr);
-    wmove(this->sc_window, top, 0);
-    wclrtoeol(this->sc_window);
-    whline(this->sc_window, ' ', width);
+    nccell clear_cell;
+    nccell_init(&clear_cell);
+    nccell_prime(
+        this->sc_window, &clear_cell, " ", 0, view_colors::to_channels(attrs));
+    ncplane_cursor_move_yx(this->sc_window, top, 0);
+    ncplane_hline(this->sc_window, &clear_cell, width);
+    nccell_release(this->sc_window, &clear_cell);
 
     if (this->sc_source != nullptr) {
         auto field_count = this->sc_source->statusview_fields();
@@ -131,9 +141,9 @@ statusview_curses::do_update()
                 for (auto& sa : val.get_attrs()) {
                     if (sa.sa_type == &VC_STYLE) {
                         auto sa_attrs = sa.sa_value.get<text_attrs>();
-                        sa_attrs.ta_attrs &= ~(A_REVERSE | A_COLOR);
-                        sa_attrs.ta_fg_color = nonstd::nullopt;
-                        sa_attrs.ta_bg_color = nonstd::nullopt;
+                        sa_attrs.clear_style(text_attrs::style::reverse);
+                        sa_attrs.ta_fg_color = styling::color_unit::EMPTY;
+                        sa_attrs.ta_bg_color = styling::color_unit::EMPTY;
                         sa.sa_value = sa_attrs;
                     } else if (sa.sa_type == &VC_ROLE) {
                         if (sa.sa_value.get<role_t>()
@@ -162,7 +172,7 @@ statusview_curses::do_update()
             }
 
             if (val.length() > sf.get_width()) {
-                static const std::string ELLIPSIS = "\xE2\x8B\xAF";
+                static constexpr auto ELLIPSIS = "\xE2\x8B\xAF"_frag;
 
                 if (sf.get_width() > 11) {
                     size_t half_width = sf.get_width() / 2 - 1;
@@ -191,7 +201,7 @@ statusview_curses::do_update()
                 field);
         }
     }
-    wmove(this->sc_window, top + 1, 0);
+    this->vc_needs_update = false;
 
     return true;
 }
@@ -205,13 +215,10 @@ statusview_curses::window_change()
 
     int field_count = this->sc_source->statusview_fields();
     int total_shares = 0;
-    unsigned long width, height;
     double remaining = 0;
     std::vector<status_field*> resizable;
 
-    getmaxyx(this->sc_window, height, width);
-    // Silence the compiler. Remove this if height is used at a later stage.
-    (void) height;
+    auto width = ncplane_dim_x(this->sc_window);
     remaining = width - 2;
 
     for (int field = 0; field < field_count; field++) {
@@ -248,7 +255,10 @@ statusview_curses::window_change()
         remaining -= (actual_width - sf->get_min_width());
         total_shares -= sf->get_share();
 
-        sf->set_width(actual_width);
+        if (sf->get_width() != actual_width) {
+            sf->set_width(actual_width);
+            this->set_needs_update();
+        }
     }
 }
 
@@ -267,6 +277,10 @@ statusview_curses::handle_mouse(mouse_event& me)
             find_res.value()->df_field_index);
 
         sf.on_click(sf);
+    } else if (me.me_state == mouse_button_state_t::BUTTON_STATE_DRAGGED) {
+        if (this->sc_source->on_drag) {
+            this->sc_source->on_drag(me);
+        }
     }
 
     return true;
