@@ -31,6 +31,9 @@
 
 #include "tailer.looper.hh"
 
+#include <lnav.prompt.hh>
+
+#include "base/auto_pid.hh"
 #include "base/fs_util.hh"
 #include "base/humanize.network.hh"
 #include "base/lnav_log.hh"
@@ -77,7 +80,7 @@ read_err_pipe(const std::string& netloc,
                         = string_fragment(sbr.get_data(), 0, sbr.length())
                               .trim("\n");
                     if (eq.size() < 10) {
-                        eq.template emplace_back(line_str.to_string());
+                        eq.emplace_back(line_str.to_string());
                     }
 
                     auto level = line_str.startswith("error:")
@@ -154,10 +157,11 @@ tailer::looper::loop_body()
 
             if (create_res.isErr()) {
                 report_error(netloc, create_res.unwrapErr());
-                if (std::any_of(
-                        rpq.rpq_new_paths.begin(),
-                        rpq.rpq_new_paths.end(),
-                        [](const auto& pair) { return !pair.second.loo_tail; }))
+                if (std::any_of(rpq.rpq_new_paths.begin(),
+                                rpq.rpq_new_paths.end(),
+                                [](const auto& pair) {
+                                    return !pair.second.loo_follow;
+                                }))
                 {
                     rpq.send_synced_to_main(netloc);
                     to_erase.push_back(netloc);
@@ -231,6 +235,7 @@ tailer::looper::load_preview(int64_t id, const network::path& path)
                         .clear();
                     lnav_data.ld_preview_source[0].clear();
                     lnav_data.ld_bottom_source.grep_error(msg);
+                    lnav_data.ld_status[LNS_BOTTOM].set_needs_update();
                 });
             return;
         }
@@ -360,24 +365,39 @@ tailer::looper::host_tailer::for_host(const std::string& netloc)
         });
 
         log_debug("tailer(%s): writing to child", netloc.c_str());
-        auto sf = tailer_bin[0].to_string_fragment();
-        ssize_t total_bytes = 0;
         bool write_failed = false;
 
-        while (total_bytes < sf.length()) {
-            log_debug("attempting to write %d", sf.length() - total_bytes);
-            auto rc = write(
-                in_pipe.write_end(), sf.data(), sf.length() - total_bytes);
-
-            if (rc < 0) {
-                log_error("  tailer(%s): write failed -- %s",
-                          netloc.c_str(),
-                          strerror(errno));
-                write_failed = true;
+        auto sfp = tailer_bin[0].to_string_fragment_producer();
+        while (true) {
+            auto next_res = sfp->next();
+            if (next_res.is<string_fragment_producer::eof>()) {
                 break;
             }
-            log_debug("  wrote %d", rc);
-            total_bytes += rc;
+
+            if (next_res.is<string_fragment_producer::error>()) {
+                return Err(
+                    next_res.get<string_fragment_producer::error>().what);
+            }
+
+            auto sf = next_res.get<string_fragment>();
+
+            ssize_t total_bytes = 0;
+
+            while (total_bytes < sf.length()) {
+                log_debug("attempting to write %d", sf.length() - total_bytes);
+                auto rc = write(
+                    in_pipe.write_end(), sf.data(), sf.length() - total_bytes);
+
+                if (rc < 0) {
+                    log_error("  tailer(%s): write failed -- %s",
+                              netloc.c_str(),
+                              strerror(errno));
+                    write_failed = true;
+                    break;
+                }
+                log_debug("  wrote %d", rc);
+                total_bytes += rc;
+            }
         }
 
         in_pipe.write_end().reset();
@@ -448,18 +468,18 @@ tailer::looper::host_tailer::for_host(const std::string& netloc)
                                             std::move(err_pipe.read_end())));
 }
 
-static ghc::filesystem::path
+static std::filesystem::path
 remote_cache_path()
 {
     return lnav::paths::workdir() / "remotes";
 }
 
-ghc::filesystem::path
+std::filesystem::path
 tailer::looper::host_tailer::tmp_path()
 {
     auto local_path = remote_cache_path();
 
-    ghc::filesystem::create_directories(local_path);
+    std::filesystem::create_directories(local_path);
     auto_mem<char> resolved_path;
 
     resolved_path = realpath(local_path.c_str(), nullptr);
@@ -544,6 +564,7 @@ tailer::looper::host_tailer::load_preview(int64_t id, const std::string& path)
                     .get_description()
                     .set_cylon(false)
                     .set_value(msg);
+                lnav_data.ld_status[LNS_PREVIEW0].set_needs_update();
             });
         },
         [&](const synced& s) { require(false); });
@@ -578,9 +599,8 @@ tailer::looper::host_tailer::loop_body()
 
     this->ht_cycle_count += 1;
     if (this->ht_cycle_count % TOUCH_FREQ == 0) {
-        auto now
-            = ghc::filesystem::file_time_type{std::chrono::system_clock::now()};
-        ghc::filesystem::last_write_time(this->ht_local_path, now);
+        auto now = std::filesystem::file_time_type::clock::now();
+        std::filesystem::last_write_time(this->ht_local_path, now);
     }
 
     auto& conn = this->ht_state.get<connected>();
@@ -635,27 +655,27 @@ tailer::looper::host_tailer::loop_body()
                 auto desired_iter = conn.c_desired_paths.find(pe.pe_path);
                 if (desired_iter != conn.c_desired_paths.end()) {
                     report_error(this->get_display_path(pe.pe_path), pe.pe_msg);
-                    if (!desired_iter->second.loo_tail) {
+                    if (!desired_iter->second.loo_follow) {
                         conn.c_desired_paths.erase(desired_iter);
                     }
                 } else {
                     auto child_iter = conn.c_child_paths.find(pe.pe_path);
 
                     if (child_iter != conn.c_child_paths.end()
-                        && !child_iter->second.loo_tail)
+                        && !child_iter->second.loo_follow)
                     {
                         conn.c_child_paths.erase(child_iter);
                     }
                 }
 
-                auto remote_path = ghc::filesystem::absolute(
-                                       ghc::filesystem::path(pe.pe_path))
+                auto remote_path = std::filesystem::absolute(
+                                       std::filesystem::path(pe.pe_path))
                                        .relative_path();
                 auto local_path = this->ht_local_path / remote_path;
 
                 log_debug("removing %s", local_path.c_str());
                 this->ht_active_files.erase(local_path);
-                ghc::filesystem::remove_all(local_path);
+                std::filesystem::remove_all(local_path);
 
                 if (conn.c_desired_paths.empty() && conn.c_child_paths.empty())
                 {
@@ -706,8 +726,8 @@ tailer::looper::host_tailer::loop_body()
                 update_tailer_description(
                     this->ht_netloc, conn.c_desired_paths, this->ht_uname);
 
-                auto remote_path = ghc::filesystem::absolute(
-                                       ghc::filesystem::path(pob.pob_path))
+                auto remote_path = std::filesystem::absolute(
+                                       std::filesystem::path(pob.pob_path))
                                        .relative_path();
                 auto local_path = this->ht_local_path / remote_path;
                 auto open_res
@@ -746,7 +766,7 @@ tailer::looper::host_tailer::loop_body()
                             fc.fc_file_names[lpath_str]
                                 .with_filename(custom_name)
                                 .with_source(logfile_name_source::REMOTE)
-                                .with_tail(loo.loo_tail)
+                                .with_follow(loo.loo_follow)
                                 .with_non_utf_visibility(false)
                                 .with_visible_size_limit(256 * 1024);
                             update_active_files(fc);
@@ -769,7 +789,7 @@ tailer::looper::host_tailer::loop_body()
 
                 if (fstat(fd, &st) == -1 || !S_ISREG(st.st_mode)) {
                     log_debug("path changed, sending need block");
-                    ghc::filesystem::remove_all(local_path);
+                    std::filesystem::remove_all(local_path);
                     send_packet(conn.ht_to_child.get(),
                                 TPT_NEED_BLOCK,
                                 TPPT_STRING,
@@ -808,7 +828,7 @@ tailer::looper::host_tailer::loop_body()
                         log_debug(
                             "unable to read file, sending need block -- %s",
                             strerror(errno));
-                        ghc::filesystem::remove_all(local_path);
+                        std::filesystem::remove_all(local_path);
                         break;
                     }
                     if (bytes_read == 0) {
@@ -847,8 +867,8 @@ tailer::looper::host_tailer::loop_body()
                 return std::move(this->ht_state);
             },
             [&](const tailer::packet_tail_block& ptb) {
-                auto remote_path = ghc::filesystem::absolute(
-                                       ghc::filesystem::path(ptb.ptb_path))
+                auto remote_path = std::filesystem::absolute(
+                                       std::filesystem::path(ptb.ptb_path))
                                        .relative_path();
                 auto local_path = this->ht_local_path / remote_path;
 
@@ -856,7 +876,7 @@ tailer::looper::host_tailer::loop_body()
                           ptb.ptb_offset,
                           ptb.ptb_bits.size(),
                           local_path.c_str());
-                ghc::filesystem::create_directories(local_path.parent_path());
+                std::filesystem::create_directories(local_path.parent_path());
                 auto create_res = lnav::filesystem::create_file(
                     local_path, O_WRONLY | O_APPEND | O_CREAT, 0600);
 
@@ -869,10 +889,10 @@ tailer::looper::host_tailer::loop_body()
                            ptb.ptb_bits.data(),
                            ptb.ptb_bits.size(),
                            ptb.ptb_offset);
-                    auto mtime = ghc::filesystem::file_time_type{
+                    auto mtime = std::filesystem::file_time_type{
                         std::chrono::seconds{ptb.ptb_mtime}};
                     // XXX This isn't atomic with the write...
-                    ghc::filesystem::last_write_time(local_path, mtime);
+                    std::filesystem::last_write_time(local_path, mtime);
                 }
                 return std::move(this->ht_state);
             },
@@ -881,7 +901,7 @@ tailer::looper::host_tailer::loop_body()
                     auto iter = conn.c_desired_paths.find(ps.ps_path);
 
                     if (iter != conn.c_desired_paths.end()) {
-                        if (iter->second.loo_tail) {
+                        if (iter->second.loo_follow) {
                             conn.c_synced_desired_paths.insert(ps.ps_path);
                         } else {
                             log_info("synced desired path: %s",
@@ -893,7 +913,7 @@ tailer::looper::host_tailer::loop_body()
                     auto iter = conn.c_child_paths.find(ps.ps_path);
 
                     if (iter != conn.c_child_paths.end()) {
-                        if (iter->second.loo_tail) {
+                        if (iter->second.loo_follow) {
                             conn.c_synced_child_paths.insert(ps.ps_path);
                         } else {
                             log_info("synced child path: %s",
@@ -935,11 +955,11 @@ tailer::looper::host_tailer::loop_body()
                 return std::move(this->ht_state);
             },
             [&](const tailer::packet_link& pl) {
-                auto remote_path = ghc::filesystem::absolute(
-                                       ghc::filesystem::path(pl.pl_path))
+                auto remote_path = std::filesystem::absolute(
+                                       std::filesystem::path(pl.pl_path))
                                        .relative_path();
                 auto local_path = this->ht_local_path / remote_path;
-                auto remote_link_path = ghc::filesystem::path(pl.pl_link_value);
+                auto remote_link_path = std::filesystem::path(pl.pl_link_value);
                 std::string link_path;
 
                 if (remote_link_path.is_absolute()) {
@@ -954,8 +974,8 @@ tailer::looper::host_tailer::loop_body()
                 log_debug("symlinking %s -> %s",
                           local_path.c_str(),
                           link_path.c_str());
-                ghc::filesystem::create_directories(local_path.parent_path());
-                ghc::filesystem::remove_all(local_path);
+                std::filesystem::create_directories(local_path.parent_path());
+                std::filesystem::remove_all(local_path);
                 if (symlink(link_path.c_str(), local_path.c_str()) < 0) {
                     log_error("symlink failed: %s", strerror(errno));
                 }
@@ -964,7 +984,7 @@ tailer::looper::host_tailer::loop_body()
                     auto iter = conn.c_desired_paths.find(pl.pl_path);
 
                     if (iter != conn.c_desired_paths.end()) {
-                        if (iter->second.loo_tail) {
+                        if (iter->second.loo_follow) {
                             conn.c_synced_desired_paths.insert(pl.pl_path);
                         } else {
                             log_info("synced desired path: %s",
@@ -976,7 +996,7 @@ tailer::looper::host_tailer::loop_body()
                     auto iter = conn.c_child_paths.find(pl.pl_path);
 
                     if (iter != conn.c_child_paths.end()) {
-                        if (iter->second.loo_tail) {
+                        if (iter->second.loo_follow) {
                             conn.c_synced_child_paths.insert(pl.pl_path);
                         } else {
                             log_info("synced child path: %s",
@@ -1003,6 +1023,7 @@ tailer::looper::host_tailer::loop_body()
                             .clear();
                         lnav_data.ld_preview_source[0].clear();
                         lnav_data.ld_bottom_source.grep_error(ppe.ppe_msg);
+                        lnav_data.ld_status[LNS_BOTTOM].set_needs_update();
                     });
 
                 return std::move(this->ht_state);
@@ -1024,6 +1045,7 @@ tailer::looper::host_tailer::loop_body()
                             .set_value("For file: %s:%s",
                                        netloc.c_str(),
                                        ppd.ppd_path.c_str());
+                        lnav_data.ld_status[LNS_PREVIEW0].set_needs_update();
                         lnav_data.ld_preview_source[0]
                             .replace_with(str)
                             .set_text_format(detect_text_format(str));
@@ -1037,8 +1059,9 @@ tailer::looper::host_tailer::loop_body()
 
                 isc::to<main_looper&, services::main_t>().send(
                     [full_path](auto& mlooper) {
-                        lnav_data.ld_rl_view->add_possibility(
-                            ln_mode_t::COMMAND, "remote-path", full_path);
+                        static auto& prompt = lnav::prompt::get();
+
+                        prompt.p_remote_paths.insert(full_path);
                     });
                 return std::move(this->ht_state);
             });
@@ -1126,13 +1149,13 @@ tailer::looper::remote_path_queue::send_synced_to_main(
     std::set<std::string> synced_files;
 
     for (const auto& pair : this->rpq_new_paths) {
-        if (!pair.second.loo_tail) {
+        if (!pair.second.loo_follow) {
             synced_files.emplace(
                 fmt::format(FMT_STRING("{}{}"), netloc, pair.first));
         }
     }
     for (const auto& pair : this->rpq_existing_paths) {
-        if (!pair.second.loo_tail) {
+        if (!pair.second.loo_follow) {
             synced_files.emplace(
                 fmt::format(FMT_STRING("{}{}"), netloc, pair.first));
         }
@@ -1169,16 +1192,16 @@ void
 tailer::cleanup_cache()
 {
     (void) std::async(std::launch::async, []() {
-        auto now = std::chrono::system_clock::now();
+        auto now = std::filesystem::file_time_type::clock::now();
         auto cache_path = remote_cache_path();
         const auto& cfg = injector::get<const config&>();
-        std::vector<ghc::filesystem::path> to_remove;
+        std::vector<std::filesystem::path> to_remove;
 
         log_debug("cache-ttl %d", cfg.c_cache_ttl.count());
         for (const auto& entry :
-             ghc::filesystem::directory_iterator(cache_path))
+             std::filesystem::directory_iterator(cache_path))
         {
-            auto mtime = ghc::filesystem::last_write_time(entry.path());
+            auto mtime = std::filesystem::last_write_time(entry.path());
             auto exp_time = mtime + cfg.c_cache_ttl;
             if (now < exp_time) {
                 continue;
@@ -1189,7 +1212,7 @@ tailer::cleanup_cache()
 
         for (auto& entry : to_remove) {
             log_debug("removing cached remote: %s", entry.c_str());
-            ghc::filesystem::remove_all(entry);
+            std::filesystem::remove_all(entry);
         }
     });
 }
