@@ -33,23 +33,49 @@
 
 #include "text_format.hh"
 
+#include "base/from_trait.hh"
+#include "base/itertools.enumerate.hh"
+#include "base/itertools.hh"
 #include "base/lnav_log.hh"
 #include "config.h"
 #include "pcrepp/pcre2pp.hh"
 #include "yajl/api/yajl_parse.h"
 
+constexpr string_fragment TEXT_FORMAT_STRINGS[text_format_count] = {
+    "application/octet-stream"_frag,
+    "text/c"_frag,
+    "text/java"_frag,
+    "application/json"_frag,
+    "text/log"_frag,
+    "text/x-makefile"_frag,
+    "text/man"_frag,
+    "text/markdown"_frag,
+    "text/python"_frag,
+    "application/x-pcre"_frag,
+    "text/rust"_frag,
+    "application/sql"_frag,
+    "text/xml"_frag,
+    "application/yaml"_frag,
+    "application/toml"_frag,
+    "text/x-diff"_frag,
+    "text/x-shellscript"_frag,
+    "text/x-lnav-script"_frag,
+    "text/x-rst"_frag,
+    "text/plain"_frag,
+};
+
 text_format_t
 detect_text_format(string_fragment sf,
-                   nonstd::optional<ghc::filesystem::path> path)
+                   std::optional<std::filesystem::path> path)
 {
-    static const std::set<ghc::filesystem::path> FILTER_EXTS = {
+    static const std::set<std::filesystem::path> FILTER_EXTS = {
         ".bz2",
         ".gz",
         ".lzma",
         ".xz",
         ".zst",
     };
-    static const auto C_EXTS = std::set<ghc::filesystem::path>{
+    static const auto C_EXTS = std::set<std::filesystem::path>{
         ".h",
         ".hh",
         ".hpp",
@@ -58,17 +84,20 @@ detect_text_format(string_fragment sf,
         ".cpp",
         ".tpp",
     };
-    static const auto PY_EXT = ghc::filesystem::path(".py");
-    static const auto RS_EXT = ghc::filesystem::path(".rs");
-    static const auto JAVA_EXT = ghc::filesystem::path(".java");
-    static const auto TOML_EXT = ghc::filesystem::path(".toml");
-    static const auto XML_EXT = ghc::filesystem::path(".xml");
-    static const auto YAML_EXT = ghc::filesystem::path(".yaml");
-    static const auto YML_EXT = ghc::filesystem::path(".yml");
-    static const auto MAKEFILE_STEM = ghc::filesystem::path("Makefile");
-    static const auto MD_EXT = ghc::filesystem::path(".md");
-    static const auto MARKDOWN_EXT = ghc::filesystem::path(".markdown");
-    static const auto SH_EXT = ghc::filesystem::path(".sh");
+    static const auto PY_EXT = std::filesystem::path(".py");
+    static const auto RS_EXT = std::filesystem::path(".rs");
+    static const auto SQL_EXT = std::filesystem::path(".sql");
+    static const auto JAVA_EXT = std::filesystem::path(".java");
+    static const auto TOML_EXT = std::filesystem::path(".toml");
+    static const auto XML_EXT = std::filesystem::path(".xml");
+    static const auto YAML_EXT = std::filesystem::path(".yaml");
+    static const auto YML_EXT = std::filesystem::path(".yml");
+    static const auto MAKEFILE_STEM = std::filesystem::path("Makefile");
+    static const auto MD_EXT = std::filesystem::path(".md");
+    static const auto MARKDOWN_EXT = std::filesystem::path(".markdown");
+    static const auto SH_EXT = std::filesystem::path(".sh");
+    static const auto LNAV_EXT = std::filesystem::path(".lnav");
+    static const auto RST_EXT = std::filesystem::path(".rst");
 
     static const auto DIFF_MATCHERS = lnav::pcre2pp::code::from_const(
         R"(^--- .*\n\+\+\+ .*\n)", PCRE2_MULTILINE);
@@ -118,6 +147,7 @@ detect_text_format(string_fragment sf,
 
     static const auto SQL_MATCHERS = lnav::pcre2pp::code::from_const(
         "(?:"
+        "create\\s+table\\s+|"
         "select\\s+.+\\s+from\\s+|"
         "insert\\s+into\\s+.+\\s+"
         "values"
@@ -134,6 +164,13 @@ detect_text_format(string_fragment sf,
     static const auto SH_MATCHERS
         = lnav::pcre2pp::code::from_const("^#!.+sh\\b", PCRE2_MULTILINE);
 
+    static const auto LNAV_MATCHERS = lnav::pcre2pp::code::from_const(
+        "(?:"
+        "^;SELECT\\s+|"
+        "^:[a-z0-9\\-]+\\s+"
+        ")",
+        PCRE2_MULTILINE | PCRE2_CASELESS);
+
     if (path) {
         while (FILTER_EXTS.count(path->extension()) > 0) {
             path = path->stem();
@@ -157,6 +194,10 @@ detect_text_format(string_fragment sf,
             return text_format_t::TF_RUST;
         }
 
+        if (ext == SQL_EXT) {
+            return text_format_t::TF_SQL;
+        }
+
         if (ext == TOML_EXT) {
             return text_format_t::TF_TOML;
         }
@@ -177,9 +218,17 @@ detect_text_format(string_fragment sf,
             return text_format_t::TF_MAKEFILE;
         }
 
-        if (stem == SH_EXT) {
+        if (ext == SH_EXT) {
             return text_format_t::TF_SHELL_SCRIPT;
         }
+
+        if (ext == LNAV_EXT) {
+            return text_format_t::TF_LNAV_SCRIPT;
+        }
+
+        if (ext == RST_EXT) {
+            return text_format_t::TF_RESTRUCTURED_TEXT;
+        }
     }
 
     {
@@ -219,6 +268,10 @@ detect_text_format(string_fragment sf,
         return text_format_t::TF_C_LIKE;
     }
 
+    if (LNAV_MATCHERS.find_in(sf).ignore_error()) {
+        return text_format_t::TF_LNAV_SCRIPT;
+    }
+
     if (SQL_MATCHERS.find_in(sf).ignore_error()) {
         return text_format_t::TF_SQL;
     }
@@ -230,7 +283,7 @@ detect_text_format(string_fragment sf,
     return text_format_t::TF_UNKNOWN;
 }
 
-nonstd::optional<text_format_meta_t>
+std::optional<text_format_meta_t>
 extract_text_meta(string_fragment sf, text_format_t tf)
 {
     static const auto MAN_NAME = lnav::pcre2pp::code::from_const(
@@ -238,8 +291,7 @@ extract_text_meta(string_fragment sf, text_format_t tf)
 
     switch (tf) {
         case text_format_t::TF_MAN: {
-            static thread_local auto md
-                = lnav::pcre2pp::match_data::unitialized();
+            thread_local auto md = lnav::pcre2pp::match_data::unitialized();
 
             auto find_res
                 = MAN_NAME.capture_from(sf).into(md).matches().ignore_error();
@@ -255,5 +307,19 @@ extract_text_meta(string_fragment sf, text_format_t tf)
             break;
     }
 
-    return nonstd::nullopt;
+    return std::nullopt;
+}
+
+template<>
+Result<text_format_t, std::string>
+from(const string_fragment sf)
+{
+    for (const auto& [index, format_sf] :
+         lnav::itertools::enumerate(TEXT_FORMAT_STRINGS))
+    {
+        if (format_sf == sf) {
+            return Ok(static_cast<text_format_t>(index));
+        }
+    }
+    return Err(fmt::format(FMT_STRING("unrecognized text format: {}"), sf));
 }
