@@ -35,6 +35,7 @@
 #include "base/ansi_scrubber.hh"
 #include "base/humanize.time.hh"
 #include "base/injector.hh"
+#include "base/lnav_log.hh"
 #include "base/time_util.hh"
 #include "config.h"
 #include "data_scanner.hh"
@@ -43,10 +44,9 @@
 #include "log_format_fwd.hh"
 #include "logfile.hh"
 #include "shlex.hh"
-#include "text_overlay_menu.hh"
 #include "view_curses.hh"
 
-const auto REVERSE_SEARCH_OFFSET = 2000_vl;
+constexpr auto REVERSE_SEARCH_OFFSET = 2000_vl;
 
 void
 text_filter::revert_to_last(logfile_filter_state& lfs, size_t rollback_size)
@@ -78,20 +78,21 @@ text_filter::revert_to_last(logfile_filter_state& lfs, size_t rollback_size)
     }
 }
 
-void
+bool
 text_filter::add_line(logfile_filter_state& lfs,
                       logfile::const_iterator ll,
                       const shared_buffer_ref& line)
 {
-    bool match_state = this->matches(line_source{*lfs.tfs_logfile, ll}, line);
-
     if (ll->is_message()) {
         this->end_of_message(lfs);
     }
+    auto retval = this->matches(line_source{*lfs.tfs_logfile, ll}, line);
 
     lfs.tfs_message_matched[this->lf_index]
-        = lfs.tfs_message_matched[this->lf_index] || match_state;
+        = lfs.tfs_message_matched[this->lf_index] || retval;
     lfs.tfs_lines_for_message[this->lf_index] += 1;
+
+    return retval;
 }
 
 void
@@ -139,7 +140,9 @@ text_accel_source::get_line_accel_direction(vis_line_t vl)
             continue;
         }
 
-        if (!la.add_point(curr_line->get_time_in_millis())) {
+        if (!la.add_point(
+                curr_line->get_time<std::chrono::milliseconds>().count()))
+        {
             break;
         }
 
@@ -164,13 +167,13 @@ text_accel_source::get_time_offset_for_line(textview_curses& tc, vis_line_t vl)
         = tc.get_bookmarks()[&textview_curses::BM_USER_EXPR].next(vl);
     if (!prev_umark && !prev_emark && (next_umark || next_emark)) {
         auto next_line = this->text_accel_get_line(
-            std::max(next_umark.value_or(0), next_emark.value_or(0)));
+            std::max(next_umark.value_or(0_vl), next_emark.value_or(0_vl)));
 
         diff_tv = curr_tv - next_line->get_timeval();
     } else {
         auto prev_row
-            = std::max(prev_umark.value_or(0), prev_emark.value_or(0));
-        auto first_line = this->text_accel_get_line(prev_row);
+            = std::max(prev_umark.value_or(0_vl), prev_emark.value_or(0_vl));
+        auto* first_line = this->text_accel_get_line(prev_row);
         auto start_tv = first_line->get_timeval();
         diff_tv = curr_tv - start_tv;
     }
@@ -178,6 +181,8 @@ text_accel_source::get_time_offset_for_line(textview_curses& tc, vis_line_t vl)
     return humanize::time::duration::from_tv(diff_tv).to_string();
 }
 
+const bookmark_type_t textview_curses::BM_ERRORS("error");
+const bookmark_type_t textview_curses::BM_WARNINGS("warning");
 const bookmark_type_t textview_curses::BM_USER("user");
 const bookmark_type_t textview_curses::BM_USER_EXPR("user-expr");
 const bookmark_type_t textview_curses::BM_SEARCH("search");
@@ -199,6 +204,7 @@ void
 textview_curses::reload_config(error_reporter& reporter)
 {
     const static auto DEFAULT_THEME_NAME = std::string("default");
+    const auto& vc = view_colors::singleton();
 
     for (auto iter = this->tc_highlights.begin();
          iter != this->tc_highlights.end();)
@@ -211,15 +217,6 @@ textview_curses::reload_config(error_reporter& reporter)
         iter = this->tc_highlights.erase(iter);
     }
 
-    std::map<std::string, scoped_value_t> vars;
-    auto curr_theme_iter
-        = lnav_config.lc_ui_theme_defs.find(lnav_config.lc_ui_theme);
-    if (curr_theme_iter != lnav_config.lc_ui_theme_defs.end()) {
-        for (const auto& vpair : curr_theme_iter->second.lt_vars) {
-            vars[vpair.first] = vpair.second;
-        }
-    }
-
     for (const auto& theme_name : {DEFAULT_THEME_NAME, lnav_config.lc_ui_theme})
     {
         auto theme_iter = lnav_config.lc_ui_theme_defs.find(theme_name);
@@ -228,55 +225,57 @@ textview_curses::reload_config(error_reporter& reporter)
             continue;
         }
 
+        auto vars = &theme_iter->second.lt_vars;
         for (const auto& hl_pair : theme_iter->second.lt_highlights) {
             if (hl_pair.second.hc_regex.pp_value == nullptr) {
                 continue;
             }
 
             const auto& sc = hl_pair.second.hc_style;
-            std::string fg1, bg1, fg_color, bg_color, errmsg;
+            std::string fg_color, bg_color, errmsg;
             bool invalid = false;
             text_attrs attrs;
 
-            fg1 = sc.sc_color;
-            bg1 = sc.sc_background_color;
-            shlex(fg1).eval(fg_color, scoped_resolver{&vars});
-            shlex(bg1).eval(bg_color, scoped_resolver{&vars});
+            auto fg1 = sc.sc_color;
+            auto bg1 = sc.sc_background_color;
+            shlex(fg1).eval(fg_color, scoped_resolver{vars});
+            shlex(bg1).eval(bg_color, scoped_resolver{vars});
 
-            auto fg = styling::color_unit::from_str(fg_color).unwrapOrElse(
-                [&](const auto& msg) {
-                    reporter(&sc.sc_color,
-                             lnav::console::user_message::error(
-                                 attr_line_t("invalid color -- ")
-                                     .append_quoted(sc.sc_color))
-                                 .with_reason(msg));
-                    invalid = true;
-                    return styling::color_unit::make_empty();
-                });
-            auto bg = styling::color_unit::from_str(bg_color).unwrapOrElse(
-                [&](const auto& msg) {
-                    reporter(&sc.sc_background_color,
-                             lnav::console::user_message::error(
-                                 attr_line_t("invalid background color -- ")
-                                     .append_quoted(sc.sc_background_color))
-                                 .with_reason(msg));
-                    invalid = true;
-                    return styling::color_unit::make_empty();
-                });
+            attrs.ta_fg_color = vc.match_color(
+                styling::color_unit::from_str(fg_color).unwrapOrElse(
+                    [&](const auto& msg) {
+                        reporter(&sc.sc_color,
+                                 lnav::console::user_message::error(
+                                     attr_line_t("invalid color -- ")
+                                         .append_quoted(sc.sc_color))
+                                     .with_reason(msg));
+                        invalid = true;
+                        return styling::color_unit::EMPTY;
+                    }));
+            attrs.ta_bg_color = vc.match_color(
+                styling::color_unit::from_str(bg_color).unwrapOrElse(
+                    [&](const auto& msg) {
+                        reporter(&sc.sc_background_color,
+                                 lnav::console::user_message::error(
+                                     attr_line_t("invalid background color -- ")
+                                         .append_quoted(sc.sc_background_color))
+                                     .with_reason(msg));
+                        invalid = true;
+                        return styling::color_unit::EMPTY;
+                    }));
             if (invalid) {
                 continue;
             }
 
             if (sc.sc_bold) {
-                attrs.ta_attrs |= A_BOLD;
+                attrs |= text_attrs::style::bold;
             }
             if (sc.sc_underline) {
-                attrs.ta_attrs |= A_UNDERLINE;
+                attrs |= text_attrs::style::underline;
             }
             this->tc_highlights[{highlight_source_t::THEME, hl_pair.first}]
                 = highlighter(hl_pair.second.hc_regex.pp_value)
                       .with_attrs(attrs)
-                      .with_color(fg, bg)
                       .with_nestable(false);
         }
     }
@@ -289,7 +288,7 @@ textview_curses::reload_config(error_reporter& reporter)
 void
 textview_curses::invoke_scroll()
 {
-    this->tc_selected_text = nonstd::nullopt;
+    this->tc_selected_text = std::nullopt;
     if (this->tc_sub_source != nullptr) {
         this->tc_sub_source->scroll_invoked(this);
     }
@@ -300,18 +299,17 @@ textview_curses::invoke_scroll()
 void
 textview_curses::reload_data()
 {
-    this->tc_selected_text = nonstd::nullopt;
+    this->tc_selected_text = std::nullopt;
     if (this->tc_sub_source != nullptr) {
         this->tc_sub_source->text_update_marks(this->tc_bookmarks);
-    }
-    if (this->tc_sub_source != nullptr) {
+
         auto* ttt = dynamic_cast<text_time_translator*>(this->tc_sub_source);
 
         if (ttt != nullptr) {
             ttt->data_reloaded(this);
         }
+        listview_curses::reload_data();
     }
-    listview_curses::reload_data();
 }
 
 void
@@ -338,7 +336,16 @@ textview_curses::grep_begin(grep_proc<vis_line_t>& gp,
             }
         }
         if (pair.first != pair.second) {
-            search_bv.erase(pair.first, pair.second);
+            auto to_del = std::vector<vis_line_t>{};
+            for (auto file_iter = pair.first; file_iter != pair.second;
+                 ++file_iter)
+            {
+                to_del.emplace_back(*file_iter);
+            }
+
+            for (auto cl : to_del) {
+                search_bv.bv_tree.erase(cl);
+            }
         }
     }
 
@@ -351,7 +358,7 @@ textview_curses::grep_end_batch(grep_proc<vis_line_t>& gp)
     if (this->tc_follow_deadline.tv_sec
         && this->tc_follow_selection == this->get_selection())
     {
-        struct timeval now;
+        timeval now;
 
         gettimeofday(&now, nullptr);
         if (this->tc_follow_deadline < now) {
@@ -373,15 +380,22 @@ textview_curses::grep_end(grep_proc<vis_line_t>& gp)
 {
     this->tc_searching -= 1;
     this->grep_end_batch(gp);
+    if (this->tc_searching == 0 && this->tc_search_start_time) {
+        const auto now = std::chrono::steady_clock::now();
+        this->tc_search_duration
+            = std::chrono::duration_cast<std::chrono::milliseconds>(
+                now - this->tc_search_start_time.value());
+        this->tc_search_start_time = std::nullopt;
+        if (this->tc_state_event_handler) {
+            this->tc_state_event_handler(*this);
+        }
+    }
 
     ensure(this->tc_searching >= 0);
 }
 
 void
-textview_curses::grep_match(grep_proc<vis_line_t>& gp,
-                            vis_line_t line,
-                            int start,
-                            int end)
+textview_curses::grep_match(grep_proc<vis_line_t>& gp, vis_line_t line)
 {
     this->tc_bookmarks[&BM_SEARCH].insert_once(vis_line_t(line));
     if (this->tc_sub_source != nullptr) {
@@ -427,12 +441,12 @@ textview_curses::handle_mouse(mouse_event& me)
         && (me.me_button != mouse_button_t::BUTTON_LEFT
             || me.me_state != mouse_button_state_t::BUTTON_STATE_RELEASED))
     {
-        this->tc_selected_text = nonstd::nullopt;
+        this->tc_selected_text = std::nullopt;
         this->set_needs_update();
     }
 
-    nonstd::optional<int> overlay_content_min_y;
-    nonstd::optional<int> overlay_content_max_y;
+    std::optional<int> overlay_content_min_y;
+    std::optional<int> overlay_content_max_y;
     if (this->tc_press_line.is<overlay_content>()) {
         auto main_line
             = this->tc_press_line.get<overlay_content>().oc_main_line;
@@ -460,7 +474,6 @@ textview_curses::handle_mouse(mouse_event& me)
 
     switch (me.me_state) {
         case mouse_button_state_t::BUTTON_STATE_PRESSED: {
-            this->tc_text_selection_active = true;
             this->tc_press_line = mouse_line;
             this->tc_press_left = this->lv_left + me.me_press_x;
             if (!this->lv_selectable) {
@@ -468,11 +481,16 @@ textview_curses::handle_mouse(mouse_event& me)
             }
             mouse_line.match(
                 [this, &me, sub_delegate, &mouse_line](const main_content& mc) {
+                    this->tc_text_selection_active = true;
+                    this->tc_press_left = this->lv_left
+                        + mc.mc_line_range.lr_start + me.me_press_x;
                     if (this->vc_enabled) {
                         if (this->tc_supports_marks
                             && me.me_button == mouse_button_t::BUTTON_LEFT
-                            && me.is_modifier_pressed(
-                                mouse_event::modifier_t::shift))
+                            && (me.is_modifier_pressed(
+                                    mouse_event::modifier_t::shift)
+                                || me.is_modifier_pressed(
+                                    mouse_event::modifier_t::ctrl)))
                         {
                             this->tc_selection_start = mc.mc_line;
                         }
@@ -488,9 +506,7 @@ textview_curses::handle_mouse(mouse_event& me)
                     }
                 },
                 [](const overlay_menu& om) {},
-                [](const static_overlay_content& soc) {
-
-                },
+                [](const static_overlay_content& soc) {},
                 [this](const overlay_content& oc) {
                     this->set_overlay_selection(oc.oc_line);
                 },
@@ -514,8 +530,10 @@ textview_curses::handle_mouse(mouse_event& me)
                             auto line_sf
                                 = string_fragment::from_str(al.get_string());
                             auto cursor_sf = line_sf.sub_cell_range(
-                                this->lv_left + me.me_x,
-                                this->lv_left + me.me_x);
+                                this->lv_left + mc.mc_line_range.lr_start
+                                    + me.me_x,
+                                this->lv_left + mc.mc_line_range.lr_start
+                                    + me.me_x);
                             auto ds = data_scanner(line_sf);
                             auto tf = this->tc_sub_source->get_text_format();
                             while (true) {
@@ -553,6 +571,7 @@ textview_curses::handle_mouse(mouse_event& me)
                                             tok_sf.sf_begin,
                                             tok_sf.sf_end,
                                         },
+                                        al.al_attrs,
                                         tok_sf.to_string(),
                                     };
                                     this->set_needs_update();
@@ -570,15 +589,9 @@ textview_curses::handle_mouse(mouse_event& me)
                         sub_delegate->text_handle_mouse(*this, mouse_line, me);
                     }
                 },
-                [](const static_overlay_content& soc) {
-
-                },
-                [](const overlay_menu& om) {
-
-                },
-                [](const overlay_content& oc) {
-
-                },
+                [](const static_overlay_content& soc) {},
+                [](const overlay_menu& om) {},
+                [](const overlay_content& oc) {},
                 [](const empty_space& es) {});
             break;
         }
@@ -589,10 +602,14 @@ textview_curses::handle_mouse(mouse_event& me)
                 if (mouse_line.is<main_content>()) {
                     auto& mc = mouse_line.get<main_content>();
                     attr_line_t al;
-                    auto low_x = std::min(this->tc_press_left,
-                                          (int) this->lv_left + me.me_x);
-                    auto high_x = std::max(this->tc_press_left,
-                                           (int) this->lv_left + me.me_x);
+                    auto low_x
+                        = std::min(this->tc_press_left,
+                                   (int) this->lv_left
+                                       + mc.mc_line_range.lr_start + me.me_x);
+                    auto high_x
+                        = std::max(this->tc_press_left,
+                                   (int) this->lv_left
+                                       + mc.mc_line_range.lr_start + me.me_x);
 
                     this->set_selection_without_context(mc.mc_line);
                     if (this->tc_supports_marks
@@ -615,6 +632,7 @@ textview_curses::handle_mouse(mouse_event& me)
                                     cursor_sf.sf_begin,
                                     cursor_sf.sf_end,
                                 },
+                                al.al_attrs,
                                 cursor_sf.to_string(),
                             };
                         }
@@ -623,11 +641,9 @@ textview_curses::handle_mouse(mouse_event& me)
             } else {
                 if (this->tc_press_line.is<main_content>()) {
                     if (me.me_y < 0) {
-                        this->shift_selection(
-                            listview_curses::shift_amount_t::up_line);
+                        this->shift_selection(shift_amount_t::up_line);
                     } else if (me.me_y >= height) {
-                        this->shift_selection(
-                            listview_curses::shift_amount_t::down_line);
+                        this->shift_selection(shift_amount_t::down_line);
                     } else if (mouse_line.is<main_content>()) {
                         this->set_selection_without_context(
                             mouse_line.get<main_content>().mc_line);
@@ -653,22 +669,19 @@ textview_curses::handle_mouse(mouse_event& me)
         }
         case mouse_button_state_t::BUTTON_STATE_RELEASED: {
             auto* ov = this->get_overlay_source();
-            if (ov != nullptr && mouse_line.is<listview_curses::overlay_menu>()
+            if (ov != nullptr && mouse_line.is<overlay_menu>()
                 && this->tc_selected_text)
             {
-                auto* tom = dynamic_cast<text_overlay_menu*>(ov);
-                if (tom != nullptr) {
-                    auto& om = mouse_line.get<listview_curses::overlay_menu>();
-                    auto& sti = this->tc_selected_text.value();
+                auto& om = mouse_line.get<overlay_menu>();
+                auto& sti = this->tc_selected_text.value();
 
-                    for (const auto& mi : tom->tom_menu_items) {
-                        if (om.om_line == mi.mi_line
-                            && me.is_click_in(mouse_button_t::BUTTON_LEFT,
-                                              mi.mi_range))
-                        {
-                            mi.mi_action(sti.sti_value);
-                            break;
-                        }
+                for (const auto& mi : ov->los_menu_items) {
+                    if (om.om_line == mi.mi_line
+                        && me.is_click_in(mouse_button_t::BUTTON_LEFT,
+                                          mi.mi_range))
+                    {
+                        mi.mi_action(sti.sti_value);
+                        break;
                     }
                 }
             }
@@ -676,7 +689,7 @@ textview_curses::handle_mouse(mouse_event& me)
             if (me.is_click_in(mouse_button_t::BUTTON_RIGHT, 0, INT_MAX)) {
                 auto* lov = this->get_overlay_source();
                 if (lov != nullptr) {
-                    lov->set_show_details_in_overlay(
+                    this->set_show_details_in_overlay(
                         !lov->get_show_details_in_overlay());
                 }
             }
@@ -684,10 +697,83 @@ textview_curses::handle_mouse(mouse_event& me)
                 if (this->tc_selection_start) {
                     this->toggle_user_mark(&BM_USER,
                                            this->tc_selection_start.value(),
-                                           this->get_selection());
+                                           this->get_selection().value());
                     this->reload_data();
                 }
-                this->tc_selection_start = nonstd::nullopt;
+                this->tc_selection_start = std::nullopt;
+            }
+            if (mouse_line.is<main_content>()) {
+                const auto mc = mouse_line.get<main_content>();
+                attr_line_t al;
+
+                this->textview_value_for_row(mc.mc_line, al);
+                auto line_sf = string_fragment::from_str(al.get_string());
+                auto cursor_sf = line_sf.sub_cell_range(
+                    this->lv_left + me.me_x, this->lv_left + me.me_x);
+                auto link_iter = find_string_attr_containing(
+                    al.get_attrs(), &VC_HYPERLINK, cursor_sf.sf_begin);
+                if (link_iter != al.get_attrs().end()) {
+                    auto href = link_iter->sa_value.get<std::string>();
+                    auto* ta = dynamic_cast<text_anchors*>(this->tc_sub_source);
+
+                    if (me.me_button == mouse_button_t::BUTTON_LEFT
+                        && ta != nullptr && startswith(href, "#")
+                        && !startswith(href, "#/frontmatter"))
+                    {
+                        auto row_opt = ta->row_for_anchor(href);
+
+                        if (row_opt.has_value()) {
+                            this->set_selection(row_opt.value());
+                        }
+                    } else {
+                        this->tc_selected_text = selected_text_info{
+                            me.me_x,
+                            mc.mc_line,
+                            link_iter->sa_range,
+                            al.get_attrs(),
+                            al.to_string_fragment(link_iter).to_string(),
+                            href,
+                        };
+                    }
+                }
+                if (this->tc_on_click) {
+                    this->tc_on_click(*this, al, cursor_sf.sf_begin);
+                }
+            }
+            if (mouse_line.is<overlay_content>()) {
+                const auto& oc = mouse_line.get<overlay_content>();
+                std::vector<attr_line_t> ov_lines;
+
+                this->lv_overlay_source->list_value_for_overlay(
+                    *this, oc.oc_main_line, ov_lines);
+                const auto& al = ov_lines[oc.oc_line];
+                auto line_sf = string_fragment::from_str(al.get_string());
+                auto cursor_sf = line_sf.sub_cell_range(
+                    this->lv_left + me.me_x, this->lv_left + me.me_x);
+                auto link_iter = find_string_attr_containing(
+                    al.get_attrs(), &VC_HYPERLINK, cursor_sf.sf_begin);
+                if (link_iter != al.get_attrs().end()) {
+                    auto href = link_iter->sa_value.get<std::string>();
+                    auto* ta = dynamic_cast<text_anchors*>(this->tc_sub_source);
+
+                    if (me.me_button == mouse_button_t::BUTTON_LEFT
+                        && ta != nullptr && startswith(href, "#")
+                        && !startswith(href, "#/frontmatter"))
+                    {
+                        auto row_opt = ta->row_for_anchor(href);
+
+                        if (row_opt.has_value()) {
+                            this->tc_sub_source->get_location_history() |
+                                [&oc](auto lh) {
+                                    lh->loc_history_append(oc.oc_main_line);
+                                };
+                            this->set_selection(row_opt.value());
+                        }
+                    }
+                }
+                if (this->tc_on_click) {
+                    this->tc_on_click(*this, al, cursor_sf.sf_begin);
+                }
             }
             if (this->tc_delegate != nullptr) {
                 this->tc_delegate->text_handle_mouse(*this, mouse_line, me);
@@ -696,7 +782,7 @@ textview_curses::handle_mouse(mouse_event& me)
                 sub_delegate->text_handle_mouse(*this, mouse_line, me);
             }
             if (mouse_line.is<overlay_menu>()) {
-                this->tc_selected_text = nonstd::nullopt;
+                this->tc_selected_text = std::nullopt;
                 this->set_needs_update();
             }
             break;
@@ -707,72 +793,27 @@ textview_curses::handle_mouse(mouse_event& me)
 }
 
 void
-textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
+textview_curses::apply_highlights(attr_line_t& al,
+                                  const line_range& body,
+                                  const line_range& orig_line)
 {
-    auto& sa = value_out.get_attrs();
-    auto& str = value_out.get_string();
-    auto source_format = this->tc_sub_source->get_text_format();
     intern_string_t format_name;
 
-    this->tc_sub_source->text_value_for_line(*this, row, str);
-    this->tc_sub_source->text_attrs_for_line(*this, row, sa);
-
-    for (const auto& attr : sa) {
-        require_ge(attr.sa_range.lr_start, 0);
-    }
-
-    scrub_ansi_string(str, &sa);
-    struct line_range body, orig_line;
-
-    body = find_string_attr_range(sa, &SA_BODY);
-    if (body.lr_start == -1) {
-        body.lr_start = 0;
-        body.lr_end = str.size();
-    }
-
-    orig_line = find_string_attr_range(sa, &SA_ORIGINAL_LINE);
-    if (!orig_line.is_valid()) {
-        orig_line.lr_start = 0;
-        orig_line.lr_end = str.size();
-    }
-
-    auto format_attr_opt = get_string_attr(sa, SA_FORMAT);
-    if (format_attr_opt) {
+    auto format_attr_opt = get_string_attr(al.al_attrs, SA_FORMAT);
+    if (format_attr_opt.has_value()) {
         format_name = format_attr_opt.value().get();
     }
 
-    if (this->is_selectable() && this->tc_cursor_role
-        && this->tc_disabled_cursor_role)
-    {
-        vis_line_t sel_start, sel_end;
-
-        sel_start = sel_end = this->get_selection();
-        if (this->tc_selection_start) {
-            if (this->tc_selection_start.value() < sel_end) {
-                sel_start = this->tc_selection_start.value();
-            } else {
-                sel_end = this->tc_selection_start.value();
-            }
-        }
-
-        if (sel_start <= row && row <= sel_end) {
-            auto role = this->get_overlay_selection()
-                ? this->tc_disabled_cursor_role.value()
-                : this->tc_cursor_role.value();
-
-            sa.emplace_back(line_range{orig_line.lr_start, -1},
-                            VC_ROLE.value(role));
-        }
+    auto source_format = this->tc_sub_source->get_text_format();
+    if (source_format == text_format_t::TF_BINARY) {
+        return;
     }
-
-    for (auto& tc_highlight : this->tc_highlights) {
+    for (const auto& tc_highlight : this->tc_highlights) {
         bool internal_hl
             = tc_highlight.first.first == highlight_source_t::INTERNAL
             || tc_highlight.first.first == highlight_source_t::THEME;
 
-        if (!tc_highlight.second.h_text_formats.empty()
-            && tc_highlight.second.h_text_formats.count(source_format) == 0)
-        {
+        if (!tc_highlight.second.applies_to_format(source_format)) {
             continue;
         }
 
@@ -788,11 +829,66 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
 
         // Internal highlights should only apply to the log message body so
         // that we don't start highlighting other fields.  User-provided
-        // highlights should apply only to the line itself and not any of the
-        // surrounding decorations that are added (for example, the file lines
-        // that are inserted at the beginning of the log view).
+        // highlights should apply only to the line itself and not any of
+        // the surrounding decorations that are added (for example, the file
+        // lines that are inserted at the beginning of the log view).
         int start_pos = internal_hl ? body.lr_start : orig_line.lr_start;
-        tc_highlight.second.annotate(value_out, start_pos);
+        tc_highlight.second.annotate(al, start_pos);
+    }
+}
+
+void
+textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
+{
+    auto& sa = value_out.get_attrs();
+    auto& str = value_out.get_string();
+
+    this->tc_sub_source->text_value_for_line(*this, row, str);
+    this->tc_sub_source->text_attrs_for_line(*this, row, sa);
+
+    for (const auto& attr : sa) {
+        require_ge(attr.sa_range.lr_start, 0);
+    }
+
+    line_range body, orig_line;
+
+    body = find_string_attr_range(sa, &SA_BODY);
+    if (!body.is_valid()) {
+        body.lr_start = 0;
+        body.lr_end = str.size();
+    }
+
+    orig_line = find_string_attr_range(sa, &SA_ORIGINAL_LINE);
+    if (!orig_line.is_valid()) {
+        orig_line.lr_start = 0;
+        orig_line.lr_end = str.size();
+    }
+
+    if (this->is_selectable() && this->tc_cursor_role
+        && this->tc_disabled_cursor_role)
+    {
+        vis_line_t sel_start, sel_end;
+
+        sel_start = sel_end = this->get_selection().value_or(0_vl);
+        if (this->tc_selection_start) {
+            if (this->tc_selection_start.value() < sel_end) {
+                sel_start = this->tc_selection_start.value();
+            } else {
+                sel_end = this->tc_selection_start.value();
+            }
+        }
+
+        if (sel_start <= row && row <= sel_end) {
+            auto role = (this->get_overlay_selection() || !this->vc_enabled)
+                ? this->tc_disabled_cursor_role.value()
+                : this->tc_cursor_role.value();
+
+            sa.emplace_back(line_range{0, -1}, VC_ROLE.value(role));
+        }
+    }
+
+    if (!body.empty() || !orig_line.empty()) {
+        this->apply_highlights(value_out, body, orig_line);
     }
 
     if (this->tc_hide_fields) {
@@ -801,12 +897,9 @@ textview_curses::textview_value_for_row(vis_line_t row, attr_line_t& value_out)
 
     const auto& user_marks = this->tc_bookmarks[&BM_USER];
     const auto& user_expr_marks = this->tc_bookmarks[&BM_USER_EXPR];
-    if (std::binary_search(user_marks.begin(), user_marks.end(), row)
-        || std::binary_search(
-            user_expr_marks.begin(), user_expr_marks.end(), row))
-    {
+    if (user_marks.bv_tree.exists(row) || user_expr_marks.bv_tree.exists(row)) {
         sa.emplace_back(line_range{orig_line.lr_start, -1},
-                        VC_STYLE.value(text_attrs{A_REVERSE}));
+                        VC_STYLE.value(text_attrs::with_reverse()));
     }
 
     if (this->tc_selected_text) {
@@ -822,11 +915,11 @@ void
 textview_curses::execute_search(const std::string& regex_orig)
 {
     std::string regex = regex_orig;
-    std::shared_ptr<lnav::pcre2pp::code> code;
 
     if ((this->tc_search_child == nullptr)
         || (regex != this->tc_current_search))
     {
+        std::shared_ptr<lnav::pcre2pp::code> code;
         this->match_reset();
 
         this->tc_search_child.reset();
@@ -881,6 +974,8 @@ textview_curses::execute_search(const std::string& regex_orig)
             if (top > 0) {
                 gp->queue_request(0_vl, top);
             }
+            this->tc_search_start_time = std::chrono::steady_clock::now();
+            this->tc_search_duration = std::nullopt;
             gp->start();
 
             this->tc_search_child = std::make_shared<grep_highlighter>(
@@ -908,7 +1003,7 @@ textview_curses::execute_search(const std::string& regex_orig)
     }
 }
 
-nonstd::optional<std::pair<int, int>>
+std::optional<std::pair<int, int>>
 textview_curses::horiz_shift(vis_line_t start, vis_line_t end, int off_start)
 {
     auto hl_iter
@@ -916,7 +1011,7 @@ textview_curses::horiz_shift(vis_line_t start, vis_line_t end, int off_start)
     if (hl_iter == this->tc_highlights.end()
         || hl_iter->second.h_regex == nullptr)
     {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
     int prev_hit = -1, next_hit = INT_MAX;
 
@@ -937,7 +1032,7 @@ textview_curses::horiz_shift(vis_line_t start, vis_line_t end, int off_start)
     }
 
     if (prev_hit == -1 && next_hit == INT_MAX) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
     return std::make_pair(prev_hit, next_hit);
 }
@@ -947,16 +1042,12 @@ textview_curses::set_user_mark(const bookmark_type_t* bm,
                                vis_line_t vl,
                                bool marked)
 {
-    bookmark_vector<vis_line_t>& bv = this->tc_bookmarks[bm];
-    bookmark_vector<vis_line_t>::iterator iter;
+    auto& bv = this->tc_bookmarks[bm];
 
     if (marked) {
         bv.insert_once(vl);
     } else {
-        iter = std::lower_bound(bv.begin(), bv.end(), vl);
-        if (iter != bv.end() && *iter == vl) {
-            bv.erase(iter);
-        }
+        bv.bv_tree.erase(vl);
     }
     if (this->tc_sub_source) {
         this->tc_sub_source->text_mark(bm, vl, marked);
@@ -987,19 +1078,9 @@ textview_curses::toggle_user_mark(const bookmark_type_t* bm,
     if (end_line >= this->get_inner_height()) {
         end_line = vis_line_t(this->get_inner_height() - 1);
     }
-    for (vis_line_t curr_line = start_line; curr_line <= end_line; ++curr_line)
-    {
-        bookmark_vector<vis_line_t>& bv = this->tc_bookmarks[bm];
-        bookmark_vector<vis_line_t>::iterator iter;
-        bool added;
-
-        iter = bv.insert_once(curr_line);
-        if (iter == bv.end()) {
-            added = true;
-        } else {
-            bv.erase(iter);
-            added = false;
-        }
+    for (auto curr_line = start_line; curr_line <= end_line; ++curr_line) {
+        auto& bv = this->tc_bookmarks[bm];
+        auto [insert_iter, added] = bv.insert_once(curr_line);
         if (this->tc_sub_source) {
             this->tc_sub_source->text_mark(bm, curr_line, added);
         }
@@ -1059,21 +1140,28 @@ textview_curses::set_sub_source(text_sub_source* src)
     return *this;
 }
 
-bool
+std::optional<line_info>
 textview_curses::grep_value_for_line(vis_line_t line, std::string& value_out)
 {
-    bool retval = false;
-
+    // log_debug("grep line %d", line);
     if (this->tc_sub_source
         && line < (int) this->tc_sub_source->text_line_count())
     {
-        this->tc_sub_source->text_value_for_line(
+        auto retval = this->tc_sub_source->text_value_for_line(
             *this, line, value_out, text_sub_source::RF_RAW);
-        scrub_ansi_string(value_out, nullptr);
-        retval = true;
+        if (retval.li_utf8_scan_result.is_valid()
+            && retval.li_utf8_scan_result.usr_has_ansi)
+        {
+            // log_debug("has ansi %d",
+            // retval.li_utf8_scan_result.usr_has_ansi);
+            auto new_size = erase_ansi_escapes(value_out);
+            value_out.resize(new_size);
+        }
+        // log_debug("  line off %lld", retval.li_file_range.fr_offset);
+        return retval;
     }
 
-    return retval;
+    return std::nullopt;
 }
 
 void
@@ -1089,8 +1177,8 @@ text_sub_source::scroll_invoked(textview_curses* tc)
 void
 text_time_translator::ttt_scroll_invoked(textview_curses* tc)
 {
-    if (tc->get_inner_height() > 0 && tc->get_selection() >= 0_vl) {
-        this->time_for_row(tc->get_selection()) |
+    if (tc->get_inner_height() > 0 && tc->get_selection()) {
+        this->time_for_row(tc->get_selection().value()) |
             [this](auto new_top_ri) { this->ttt_top_row_info = new_top_ri; };
     }
 }
@@ -1099,7 +1187,7 @@ void
 text_time_translator::data_reloaded(textview_curses* tc)
 {
     if (tc->get_inner_height() == 0) {
-        this->ttt_top_row_info = nonstd::nullopt;
+        this->ttt_top_row_info = std::nullopt;
         return;
     }
     if (this->ttt_top_row_info) {
@@ -1111,7 +1199,7 @@ text_time_translator::data_reloaded(textview_curses* tc)
 template class bookmark_vector<vis_line_t>;
 
 bool
-empty_filter::matches(nonstd::optional<line_source> ls,
+empty_filter::matches(std::optional<line_source> ls,
                       const shared_buffer_ref& line)
 {
     return false;
@@ -1123,7 +1211,7 @@ empty_filter::to_command() const
     return "";
 }
 
-nonstd::optional<size_t>
+std::optional<size_t>
 filter_stack::next_index()
 {
     bool used[32];
@@ -1148,7 +1236,7 @@ filter_stack::next_index()
             return lpc;
         }
     }
-    return nonstd::nullopt;
+    return std::nullopt;
 }
 
 std::shared_ptr<text_filter>
@@ -1252,7 +1340,7 @@ vis_location_history::loc_history_append(vis_line_t top)
     this->vlh_history.push_back(top);
 }
 
-nonstd::optional<vis_line_t>
+std::optional<vis_line_t>
 vis_location_history::loc_history_back(vis_line_t current_top)
 {
     if (this->lh_history_position == 0) {
@@ -1263,7 +1351,7 @@ vis_location_history::loc_history_back(vis_line_t current_top)
     }
 
     if (this->lh_history_position + 1 >= this->vlh_history.size()) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     this->lh_history_position += 1;
@@ -1271,11 +1359,11 @@ vis_location_history::loc_history_back(vis_line_t current_top)
     return this->current_position();
 }
 
-nonstd::optional<vis_line_t>
+std::optional<vis_line_t>
 vis_location_history::loc_history_forward(vis_line_t current_top)
 {
     if (this->lh_history_position == 0) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     this->lh_history_position -= 1;
@@ -1367,21 +1455,27 @@ logfile_filter_state::resize(size_t newsize)
     }
 }
 
-nonstd::optional<size_t>
+void
+logfile_filter_state::reserve(size_t expected)
+{
+    this->tfs_mask.reserve(expected);
+}
+
+std::optional<size_t>
 logfile_filter_state::content_line_to_vis_line(uint32_t line)
 {
     if (this->tfs_index.empty()) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     auto iter = std::lower_bound(
         this->tfs_index.begin(), this->tfs_index.end(), line);
 
     if (iter == this->tfs_index.end() || *iter != line) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
-    return nonstd::make_optional(std::distance(this->tfs_index.begin(), iter));
+    return std::make_optional(std::distance(this->tfs_index.begin(), iter));
 }
 
 std::string
