@@ -29,14 +29,15 @@
  * @file yajlpp.cc
  */
 
+#include <filesystem>
 #include <regex>
 #include <utility>
 
 #include "yajlpp.hh"
 
+#include "base/math_util.hh"
 #include "config.h"
 #include "fmt/format.h"
-#include "ghc/filesystem.hpp"
 #include "yajl/api/yajl_parse.h"
 #include "yajlpp_def.hh"
 
@@ -211,7 +212,8 @@ json_path_handler_base::gen(yajlpp_gen_context& ygc, yajl_gen handle) const
 
     if (this->jph_children) {
         for (const auto& lpath : local_paths) {
-            std::string full_path = json_ptr::encode_str(lpath);
+            stack_buf allocator;
+            auto full_path = json_ptr::encode(lpath, allocator);
             int start_depth = ygc.ygc_depth;
 
             yajl_gen_string(handle, lpath);
@@ -219,12 +221,11 @@ json_path_handler_base::gen(yajlpp_gen_context& ygc, yajl_gen handle) const
             ygc.ygc_depth += 1;
 
             if (this->jph_obj_provider) {
-                static thread_local auto md
-                    = lnav::pcre2pp::match_data::unitialized();
+                thread_local auto md = lnav::pcre2pp::match_data::unitialized();
 
-                auto find_res = this->jph_regex->capture_from(full_path)
-                                    .into(md)
-                                    .matches();
+                auto find_res
+                    = this->jph_regex->capture_from(full_path).into(md).matches(
+                        PCRE2_NO_UTF_CHECK);
 
                 ygc.ygc_obj_stack.push(this->jph_obj_provider(
                     {&md, yajlpp_provider_context::nindex},
@@ -461,19 +462,24 @@ json_path_handler_base::walk(
         this->jph_path_provider(root, local_paths);
 
         for (const auto& lpath : local_paths) {
+            stack_buf allocator;
+            const void* field = nullptr;
+            if (this->jph_field_getter) {
+                field = this->jph_field_getter(root, lpath);
+            }
             cb(*this,
                fmt::format(FMT_STRING("{}{}{}"),
                            base,
-                           json_ptr::encode_str(lpath),
+                           json_ptr::encode(lpath, allocator),
                            this->jph_children ? "/" : ""),
-               nullptr);
+               field);
         }
         if (this->jph_obj_deleter) {
             local_paths.clear();
             this->jph_path_provider(root, local_paths);
         }
         if (this->jph_field_getter) {
-            const auto* field = this->jph_field_getter(root, nonstd::nullopt);
+            const auto* field = this->jph_field_getter(root, std::nullopt);
             if (field != nullptr) {
                 cb(*this, base, field);
             }
@@ -484,9 +490,13 @@ json_path_handler_base::walk(
         std::string full_path = base + this->jph_property;
         if (this->jph_children) {
             full_path += "/";
-        }
 
-        cb(*this, full_path, nullptr);
+            const void* field = nullptr;
+            if (this->jph_field_getter) {
+                field = this->jph_field_getter(root, this->jph_property);
+            }
+            cb(*this, full_path, field);
+        }
     }
 
     if (this->jph_children) {
@@ -495,7 +505,11 @@ json_path_handler_base::walk(
                 static const intern_string_t POSS_SRC
                     = intern_string::lookup("possibilities");
 
-                std::string full_path = base + json_ptr::encode_str(lpath);
+                stack_buf allocator;
+                auto full_path
+                    = fmt::format(FMT_STRING("{}{}"),
+                                  base,
+                                  json_ptr::encode(lpath, allocator));
                 if (this->jph_children) {
                     full_path += "/";
                 }
@@ -508,14 +522,16 @@ json_path_handler_base::walk(
 
                 ypc.set_path(full_path).with_obj(root).update_callbacks();
                 if (this->jph_obj_provider) {
-                    static thread_local auto md
+                    thread_local auto md
                         = lnav::pcre2pp::match_data::unitialized();
 
-                    const auto short_path = json_ptr::encode_str(lpath) + "/";
+                    stack_buf allocator2;
+                    const auto short_path = fmt::format(
+                        FMT_STRING("{}/"), json_ptr::encode(lpath, allocator2));
 
                     if (!this->jph_regex->capture_from(short_path)
                              .into(md)
-                             .matches()
+                             .matches(PCRE2_NO_UTF_CHECK)
                              .ignore_error())
                     {
                         log_error(
@@ -533,7 +549,7 @@ json_path_handler_base::walk(
             }
         }
     } else {
-        for (auto& lpath : local_paths) {
+        for (const auto& lpath : local_paths) {
             const void* field = nullptr;
 
             if (this->jph_field_getter) {
@@ -544,7 +560,7 @@ json_path_handler_base::walk(
     }
 }
 
-nonstd::optional<int>
+std::optional<int>
 json_path_handler_base::to_enum_value(const string_fragment& sf) const
 {
     for (int lpc = 0; this->jph_enum_values[lpc].first; lpc++) {
@@ -555,7 +571,7 @@ json_path_handler_base::to_enum_value(const string_fragment& sf) const
         }
     }
 
-    return nonstd::nullopt;
+    return std::nullopt;
 }
 
 const char*
@@ -600,9 +616,9 @@ json_path_handler_base::get_types() const
 
 yajlpp_parse_context::yajlpp_parse_context(
     intern_string_t source, const struct json_path_container* handlers)
-    : ypc_source(source), ypc_handlers(handlers)
+    : ypc_source(source), ypc_handlers(handlers),
+      ypc_path(auto_buffer::alloc(4096))
 {
-    this->ypc_path.reserve(4096);
     this->ypc_path.push_back('/');
     this->ypc_path.push_back('\0');
     this->ypc_callbacks = DEFAULT_CALLBACKS;
@@ -633,9 +649,12 @@ yajlpp_parse_context::map_start(void* ctx)
 }
 
 int
-yajlpp_parse_context::map_key(void* ctx, const unsigned char* key, size_t len)
+yajlpp_parse_context::map_key(void* ctx,
+                              const unsigned char* key,
+                              size_t len,
+                              yajl_string_props_t* props)
 {
-    yajlpp_parse_context* ypc = (yajlpp_parse_context*) ctx;
+    auto* ypc = (yajlpp_parse_context*) ctx;
     int retval = 1;
 
     require(ypc->ypc_path.size() >= 2);
@@ -644,29 +663,37 @@ yajlpp_parse_context::map_key(void* ctx, const unsigned char* key, size_t len)
     if (ypc->ypc_path.back() != '/') {
         ypc->ypc_path.push_back('/');
     }
-    for (size_t lpc = 0; lpc < len; lpc++) {
-        switch (key[lpc]) {
-            case '~':
-                ypc->ypc_path.push_back('~');
-                ypc->ypc_path.push_back('0');
-                break;
-            case '/':
-                ypc->ypc_path.push_back('~');
-                ypc->ypc_path.push_back('1');
-                break;
-            case '#':
-                ypc->ypc_path.push_back('~');
-                ypc->ypc_path.push_back('2');
-                break;
-            default:
-                ypc->ypc_path.push_back(key[lpc]);
-                break;
+    auto path_start = ypc->ypc_path.size();
+    ypc->ypc_path.expand_to(
+        roundup_size(ypc->ypc_path.size() + len + props->ptr_escapes, 4096));
+    ypc->ypc_path.resize_by(len + props->ptr_escapes);
+    if (props->ptr_escapes > 0) {
+        for (size_t lpc = 0; lpc < len; lpc++) {
+            switch (key[lpc]) {
+                case '~':
+                    ypc->ypc_path[path_start++] = '~';
+                    ypc->ypc_path[path_start++] = '0';
+                    break;
+                case '/':
+                    ypc->ypc_path[path_start++] = '~';
+                    ypc->ypc_path[path_start++] = '1';
+                    break;
+                case '#':
+                    ypc->ypc_path[path_start++] = '~';
+                    ypc->ypc_path[path_start++] = '2';
+                    break;
+                default:
+                    ypc->ypc_path[path_start++] = key[lpc];
+                    break;
+            }
         }
+    } else {
+        memcpy(&ypc->ypc_path[path_start], key, len);
     }
     ypc->ypc_path.push_back('\0');
 
     if (ypc->ypc_alt_callbacks.yajl_map_key != nullptr) {
-        retval = ypc->ypc_alt_callbacks.yajl_map_key(ctx, key, len);
+        retval = ypc->ypc_alt_callbacks.yajl_map_key(ctx, key, len, props);
     }
 
     if (ypc->ypc_handlers != nullptr) {
@@ -701,12 +728,15 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
 
     if (!this->ypc_active_paths.empty()) {
         std::string curr_path(&this->ypc_path[0], this->ypc_path.size() - 1);
+        auto path_iter = this->ypc_active_paths.find(curr_path);
 
-        if (this->ypc_active_paths.find(curr_path)
-            == this->ypc_active_paths.end())
-        {
+        if (path_iter == this->ypc_active_paths.end()) {
             return;
         }
+        path_iter->second += 1;
+        log_trace("%s: found active path: %s",
+                  this->ypc_source.c_str(),
+                  curr_path.c_str());
     }
 
     if (child_start == 0 && !this->ypc_obj_stack.empty()) {
@@ -718,11 +748,11 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
     auto path_frag = string_fragment::from_byte_range(
         this->ypc_path.data(), 1 + child_start, this->ypc_path.size() - 1);
     for (const auto& jph : handlers->jpc_children) {
-        static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+        thread_local auto md = lnav::pcre2pp::match_data::unitialized();
 
         if (jph.jph_regex->capture_from(path_frag)
                 .into(md)
-                .matches()
+                .matches(PCRE2_NO_UTF_CHECK)
                 .ignore_error()
             && (md.remaining().empty() || md.remaining().startswith("/")))
         {
@@ -736,7 +766,8 @@ yajlpp_parse_context::update_callbacks(const json_path_container* orig_handlers,
                     ? static_cast<size_t>(-1)
                     : this->ypc_array_index[this->ypc_array_handler_count - 1];
 
-                if ((cap.sf_end != (int) this->ypc_path.size() - 1)
+                if ((!jph.is_array()
+                     || cap.sf_end != (int) this->ypc_path.size() - 1)
                     && (!jph.is_array()
                         || index != yajlpp_provider_context::nindex))
                 {
@@ -856,7 +887,7 @@ yajlpp_parse_context::array_end(void* ctx)
 int
 yajlpp_parse_context::handle_unused(void* ctx)
 {
-    yajlpp_parse_context* ypc = (yajlpp_parse_context*) ctx;
+    auto* ypc = (yajlpp_parse_context*) ctx;
 
     if (ypc->ypc_ignore_unused) {
         return 1;
@@ -887,7 +918,8 @@ yajlpp_parse_context::handle_unused(void* ctx)
             expected_types.emplace_back("float");
         }
         if (ypc->ypc_callbacks.yajl_string
-            != (int (*)(void*, const unsigned char*, size_t))
+            != (int (*)(
+                void*, const unsigned char*, size_t, yajl_string_props_t*))
                 yajlpp_parse_context::handle_unused)
         {
             expected_types.emplace_back("string");
@@ -922,8 +954,9 @@ yajlpp_parse_context::handle_unused(void* ctx)
 
         attr_line_t help_text;
 
-        if (accepted_handlers->jpc_children.size() == 1
-            && accepted_handlers->jpc_children.front().jph_is_array)
+        if (accepted_handlers == nullptr) {
+        } else if (accepted_handlers->jpc_children.size() == 1
+                   && accepted_handlers->jpc_children.front().jph_is_array)
         {
             const auto& jph = accepted_handlers->jpc_children.front();
 
@@ -956,12 +989,12 @@ yajlpp_parse_context::handle_unused(void* ctx)
 int
 yajlpp_parse_context::handle_unused_or_delete(void* ctx)
 {
-    yajlpp_parse_context* ypc = (yajlpp_parse_context*) ctx;
+    auto* ypc = (yajlpp_parse_context*) ctx;
 
     if (!ypc->ypc_handler_stack.empty()
         && ypc->ypc_handler_stack.back()->jph_obj_deleter)
     {
-        static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+        thread_local auto md = lnav::pcre2pp::match_data::unitialized();
 
         auto key_start = ypc->ypc_path_index_stack.back();
         auto path_frag = string_fragment::from_byte_range(
@@ -970,7 +1003,7 @@ yajlpp_parse_context::handle_unused_or_delete(void* ctx)
         ypc->ypc_handler_stack.back()
             ->jph_regex->capture_from(path_frag)
             .into(md)
-            .matches();
+            .matches(PCRE2_NO_UTF_CHECK);
 
         ypc->ypc_handler_stack.back()->jph_obj_deleter(
             provider_ctx, ypc->ypc_obj_stack.top());
@@ -981,18 +1014,18 @@ yajlpp_parse_context::handle_unused_or_delete(void* ctx)
 }
 
 const yajl_callbacks yajlpp_parse_context::DEFAULT_CALLBACKS = {
-    yajlpp_parse_context::handle_unused_or_delete,
-    (int (*)(void*, int)) yajlpp_parse_context::handle_unused,
-    (int (*)(void*, long long)) yajlpp_parse_context::handle_unused,
-    (int (*)(void*, double)) yajlpp_parse_context::handle_unused,
+    handle_unused_or_delete,
+    (int (*)(void*, int)) handle_unused,
+    (int (*)(void*, long long)) handle_unused,
+    (int (*)(void*, double)) handle_unused,
     nullptr,
-    (int (*)(void*, const unsigned char*, size_t))
-        yajlpp_parse_context::handle_unused,
-    yajlpp_parse_context::map_start,
-    yajlpp_parse_context::map_key,
-    yajlpp_parse_context::map_end,
-    yajlpp_parse_context::array_start,
-    yajlpp_parse_context::array_end,
+    (int (*)(void*, const unsigned char*, size_t, yajl_string_props_t*))
+        handle_unused,
+    map_start,
+    map_key,
+    map_end,
+    array_start,
+    array_end,
 };
 
 yajl_status
@@ -1001,9 +1034,8 @@ yajlpp_parse_context::parse(const unsigned char* jsonText, size_t jsonTextLen)
     this->ypc_json_text = jsonText;
     this->ypc_json_text_len = jsonTextLen;
 
-    yajl_status retval = yajl_parse(this->ypc_handle, jsonText, jsonTextLen);
-
-    size_t consumed = yajl_get_bytes_consumed(this->ypc_handle);
+    const auto retval = yajl_parse(this->ypc_handle, jsonText, jsonTextLen);
+    const auto consumed = yajl_get_bytes_consumed(this->ypc_handle);
 
     this->ypc_line_number
         += std::count(&jsonText[0], &jsonText[consumed], '\n');
@@ -1027,10 +1059,11 @@ yajlpp_parse_context::parse(const unsigned char* jsonText, size_t jsonTextLen)
 yajl_status
 yajlpp_parse_context::complete_parse()
 {
-    yajl_status retval = yajl_complete_parse(this->ypc_handle);
+    const auto retval = yajl_complete_parse(this->ypc_handle);
 
     if (retval != yajl_status_ok && this->ypc_error_reporter) {
-        auto* msg = yajl_get_error(this->ypc_handle, 0, nullptr, 0);
+        auto* msg = yajl_get_error(
+            this->ypc_handle, 0, this->ypc_json_text, this->ypc_json_text_len);
 
         this->report_error(lnav::console::user_message::error("invalid JSON")
                                .with_reason((const char*) msg)
@@ -1041,15 +1074,14 @@ yajlpp_parse_context::complete_parse()
     return retval;
 }
 
-bool
-yajlpp_parse_context::parse_doc(const string_fragment& sf)
+yajl_status
+yajlpp_parse_context::parse_frag(string_fragment sf)
 {
-    bool retval = true;
-
-    this->ypc_json_text = (const unsigned char*) sf.data();
+    this->ypc_json_text = sf.udata();
     this->ypc_json_text_len = sf.length();
 
-    auto rc = yajl_parse(this->ypc_handle, this->ypc_json_text, sf.length());
+    const auto rc
+        = yajl_parse(this->ypc_handle, this->ypc_json_text, sf.length());
     size_t consumed = yajl_get_bytes_consumed(this->ypc_handle);
     this->ypc_total_consumed += consumed;
     this->ypc_line_number += std::count(
@@ -1068,16 +1100,58 @@ yajlpp_parse_context::parse_doc(const string_fragment& sf)
                     .with_snippet(this->get_snippet()));
             yajl_free_error(this->ypc_handle, msg);
         }
-        retval = false;
-    } else if (this->complete_parse() != yajl_status_ok) {
-        retval = false;
     }
 
     this->ypc_json_text = nullptr;
+    this->ypc_json_text_len = 0;
+
+    return rc;
+}
+
+bool
+yajlpp_parse_context::parse_doc(string_fragment_producer& sfp)
+{
+    auto retval = true;
+
+    while (true) {
+        auto next_res = sfp.next();
+        if (next_res.is<string_fragment_producer::eof>()) {
+            break;
+        }
+        if (next_res.is<string_fragment_producer::error>()) {
+            const auto err = next_res.get<string_fragment_producer::error>();
+            this->report_error(
+                lnav::console::user_message::error("unable to read file")
+                    .with_reason(err.what)
+                    .with_snippet(this->get_snippet()));
+            break;
+        }
+
+        auto sf = next_res.get<string_fragment>();
+
+        if (this->parse_frag(sf) != yajl_status_ok) {
+            log_error("parse frag failed %s", this->ypc_source.c_str());
+            retval = false;
+            break;
+        }
+    }
+    if (retval && this->complete_parse() != yajl_status_ok) {
+        retval = false;
+    }
 
     return retval;
 }
 
+string_fragment
+yajlpp_parse_context::get_path_as_string_fragment() const
+{
+    if (this->ypc_path.size() <= 1) {
+        return string_fragment();
+    }
+    return string_fragment::from_bytes(&this->ypc_path[1],
+                                       this->ypc_path.size() - 2);
+}
+
 const intern_string_t
 yajlpp_parse_context::get_path() const
 {
@@ -1165,12 +1239,10 @@ yajlpp_parse_context::set_path(const std::string& path)
     return *this;
 }
 
-const char*
-yajlpp_parse_context::get_path_fragment(int offset,
-                                        char* frag_in,
-                                        size_t& len_out) const
+string_fragment
+yajlpp_parse_context::get_path_fragment(int offset, stack_buf& allocator) const
 {
-    const char* retval;
+    string_fragment retval;
     size_t start, end;
 
     if (offset < 0) {
@@ -1182,18 +1254,52 @@ yajlpp_parse_context::get_path_fragment(int offset,
     } else {
         end = this->ypc_path.size() - 1;
     }
-    if (this->ypc_handlers) {
-        len_out
-            = json_ptr::decode(frag_in, &this->ypc_path[start], end - start);
-        retval = frag_in;
-    } else {
-        retval = &this->ypc_path[start];
-        len_out = end - start;
+    retval = string_fragment::from_bytes(&this->ypc_path[start], end - start);
+    if (this->ypc_handlers != nullptr) {
+        retval = json_ptr::decode(retval, allocator);
     }
 
     return retval;
 }
 
+yajl_status
+yajlpp_parse_context::parse(string_fragment_producer& sfp)
+{
+    yajl_status retval = yajl_status_ok;
+    while (retval == yajl_status_ok) {
+        auto next_res = sfp.next();
+        if (next_res.is<string_fragment_producer::eof>()) {
+            break;
+        }
+
+        if (next_res.is<string_fragment_producer::error>()) {
+            const auto err = next_res.get<string_fragment_producer::error>();
+            this->report_error(
+                lnav::console::user_message::error("unable to read file")
+                    .with_reason(err.what)
+                    .with_snippet(this->get_snippet()));
+            break;
+        }
+
+        auto sf = next_res.get<string_fragment>();
+        retval = this->parse(sf);
+        if (retval != yajl_status_ok) {
+            auto* msg
+                = yajl_get_error(this->ypc_handle, 1, sf.udata(), sf.length());
+            auto um = lnav::console::user_message::error("invalid JSON")
+                          .with_snippet(lnav::console::snippet::from(
+                              this->ypc_source, attr_line_t((const char*) msg)))
+                          .with_errno_reason();
+            this->report_error(um);
+            yajl_free_error(this->ypc_handle, msg);
+        }
+    }
+
+    retval = this->complete_parse();
+
+    return retval;
+}
+
 int
 yajlpp_parse_context::get_line_number() const
 {
@@ -1283,6 +1389,11 @@ yajlpp_gen_context::with_context(yajlpp_parse_context& ypc)
 json_path_handler&
 json_path_handler::with_children(const json_path_container& container)
 {
+    // XXX pattern with children cannot match everything, need to use [^/]+
+    require(!this->jph_is_pattern_property
+            || (this->jph_property.find(".*") == std::string::npos
+                && this->jph_property.find(".+") == std::string::npos));
+
     this->jph_children = &container;
     return *this;
 }
@@ -1343,7 +1454,7 @@ json_path_handler_base::validate_string(yajlpp_parse_context& ypc,
                              .with_reason("empty values are not allowed")
                              .with_snippet(ypc.get_snippet())
                              .with_help(this->get_help_text(&ypc)));
-    } else if (sf.length() < this->jph_min_length) {
+    } else if (sf.length() < (ssize_t) this->jph_min_length) {
         ypc.report_error(
             lnav::console::user_message::error(
                 attr_line_t()
@@ -1583,8 +1694,8 @@ dump_schema_to(const json_path_container& jpc, const char* internals_dir)
 {
     yajlpp_gen genner;
     yajlpp_gen_context ygc(genner, jpc);
-    auto internals_dir_path = ghc::filesystem::path(internals_dir);
-    auto schema_file_name = ghc::filesystem::path(jpc.jpc_schema_id).filename();
+    auto internals_dir_path = std::filesystem::path(internals_dir);
+    auto schema_file_name = std::filesystem::path(jpc.jpc_schema_id).filename();
     auto schema_path = internals_dir_path / schema_file_name;
     auto file = std::unique_ptr<FILE, decltype(&fclose)>(
         fopen(schema_path.c_str(), "w+"), fclose);
