@@ -27,10 +27,6 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifdef __CYGWIN__
-#    include <alloca.h>
-#endif
-
 #include <fstream>
 #include <iostream>
 
@@ -74,7 +70,7 @@ main(int argc, char* argv[])
     }
 
     {
-        std::vector<ghc::filesystem::path> paths;
+        std::vector<std::filesystem::path> paths;
         std::vector<lnav::console::user_message> errors;
 
         load_formats(paths, errors);
@@ -137,7 +133,6 @@ main(int argc, char* argv[])
                 retval = EXIT_FAILURE;
             } else {
                 std::shared_ptr<log_format> format;
-                char* log_line;
                 bool found = false;
                 char cmd[2048];
                 std::string line;
@@ -148,8 +143,6 @@ main(int argc, char* argv[])
                     line = "             " + line;
                 }
 
-                log_line = (char*) alloca(line.length());
-                strcpy(log_line, &line[13]);
                 auto sub_line = line.substr(13);
                 struct line_range body(0, sub_line.length());
                 shared_buffer share_manager;
@@ -194,7 +187,7 @@ main(int argc, char* argv[])
                 string_attrs_t sa;
 
                 if (format.get() != nullptr) {
-                    format->annotate(0, sa, ll_values, false);
+                    format->annotate(nullptr, 0, sa, ll_values, false);
                     body = find_string_attr_range(sa, &SA_BODY);
                 }
 
