@@ -13,6 +13,12 @@ printf '000\n000\n#Fields: 0\n0\n#Fields: 0\n0' | run_cap_test \
 printf '#Date:\t3/9/3/0\x85 2\n0\n' | run_cap_test \
     env TEST_COMMENT="invalid w3c timestamp" ${lnav_test} -n
 
+printf '[8.0000]0\n' | run_cap_test \
+    env TEST_COMMENT="zero timestamp" ${lnav_test} -n
+
+printf '#Fields: date time 0 date\n2000/2 00:00 0 2000/80' | run_cap_test \
+    env TEST_COMMENT="invalid w3c #1451" ${lnav_test} -n
+
 cat > rollover_in.0 <<EOF
 2600/2 0 00:00:00 0:
 00:2 0 00:00:00 0:
@@ -61,12 +67,10 @@ EOF
 2004-12-05T19:16:24.387 192.168.0.1 → 192.168.0.10 DHCP 342 DHCP ACK      - Transaction ID 0x3d1e
 EOF
 
-  run_test ${lnav_test} -n ${test_dir}/dhcp-trunc.pcapng
+  run_cap_test ${lnav_test} -n ${test_dir}/dhcp-trunc.pcapng
 
-  check_error_output "truncated pcap file is not recognized" <<EOF
-✘ error: unable to open file: {test_dir}/dhcp-trunc.pcapng
- reason: tshark: The file "{test_dir}/dhcp-trunc.pcapng" appears to have been cut short in the middle of a packet.
-EOF
+  gunzip -c ${test_dir}/capture.btsnoop.gz > capture.btsnoop
+  run_cap_test ${lnav_test} -n -c ':goto 790' capture.btsnoop
 fi
 
 
@@ -98,13 +102,16 @@ log_time
 EOF
 fi
 
+unset YES_COLOR
 if test x"${LIBARCHIVE_LIBS}" != x""; then
-    run_test env TMPDIR=tmp ${lnav_test} -n \
+    rm -rf logfile-tmp
+    mkdir logfile-tmp
+    run_test env TMPDIR=logfile-tmp ${lnav_test} -n \
       -c ':config /tuning/archive-manager/min-free-space -1' \
       ${srcdir}/logfile_syslog.0
 
     check_error_output "invalid min-free-space allowed?" <<EOF
-✘ error: “-1” is not a valid value for option “/tuning/archive-manager/min-free-space”
+  error: “-1” is not a valid value for option “/tuning/archive-manager/min-free-space”
  reason: value must be greater than or equal to 0
  --> input:1
  = help: Property Synopsis
@@ -113,16 +120,16 @@ if test x"${LIBARCHIVE_LIBS}" != x""; then
            The minimum free space, in bytes, to maintain when unpacking archives
 EOF
 
-    rm -rf tmp/lnav-*
+    rm -rf logfile-tmp/lnav-*
     if test x"${XZ_CMD}" != x""; then
         ${XZ_CMD} -z -c ${srcdir}/logfile_syslog.1 > logfile_syslog.1.xz
 
-        run_test env TMPDIR=tmp ${lnav_test} -n \
+        run_test env TMPDIR=logfile-tmp ${lnav_test} -n \
             -c ':config /tuning/archive-manager/min-free-space 1125899906842624' \
             -c ':config /tuning/archive-manager/cache-ttl 1d' \
             ${srcdir}/logfile_syslog.0
 
-        run_test env TMPDIR=tmp ${lnav_test} -d /tmp/lnav.err -n \
+        run_test env TMPDIR=logfile-tmp ${lnav_test} -d /tmp/lnav.err -n \
             logfile_syslog.1.xz
 
         sed -e "s|lnav-user-[0-9]*-work|lnav-user-NNN-work|g" \
@@ -132,15 +139,15 @@ EOF
             `test_err_filename` > test_logfile.big.out
         mv test_logfile.big.out `test_err_filename`
         check_error_output "decompression worked?" <<EOF
-✘ error: unable to open file: /logfile_syslog.1.xz
- reason: available space on disk (NNN) is below the minimum-free threshold (1.0PB).  Unable to unpack 'logfile_syslog.1.xz' to 'tmp/lnav-user-NNN-work/archives/arc-NNN-logfile_syslog.1.xz'
+  error: unable to open file: /logfile_syslog.1.xz
+ reason: available space on disk (NNN) is below the minimum-free threshold (1.0PB).  Unable to unpack 'logfile_syslog.1.xz' to 'logfile-tmp/lnav-user-NNN-work/archives/arc-NNN-logfile_syslog.1.xz'
 EOF
 
-        run_test env TMPDIR=tmp ${lnav_test} -n \
+        run_test env TMPDIR=logfile-tmp ${lnav_test} -n \
             -c ':config /tuning/archive-manager/min-free-space 33554432' \
             ${srcdir}/logfile_syslog.0
 
-        run_test env TMPDIR=tmp ${lnav_test} -n \
+        run_test env TMPDIR=logfile-tmp ${lnav_test} -n \
             logfile_syslog.1.xz
 
         check_output "decompression not working" <<EOF
@@ -155,8 +162,8 @@ EOF
 
     dd if=test-logs.tgz of=test-logs-trunc.tgz bs=4096 count=20
 
-    mkdir -p tmp
-    run_test env TMPDIR=tmp ${lnav_test} \
+    mkdir -p logfile-tmp
+    run_test env TMPDIR=logfile-tmp ${lnav_test} \
         -c ':config /tuning/archive-manager/cache-ttl 1d' \
         -n test-logs.tgz
 
@@ -167,26 +174,26 @@ EOF
 10.112.81.15 - - [15/Feb/2013:06:00:31 +0000] "-" 400 0 "-" "-"
 EOF
 
-    if ! test -f tmp/*/archives/*-test-logs.tgz/test/logfile_access_log.0; then
+    if ! test -f logfile-tmp/*/archives/*-test-logs.tgz/test/logfile_access_log.0; then
         echo "archived file not unpacked"
         exit 1
     fi
 
-    if test -w tmp/*/archives/*-test-logs.tgz/test/logfile_access_log.0; then
+    if test -w logfile-tmp/*/archives/*-test-logs.tgz/test/logfile_access_log.0; then
         echo "archived file is writable"
         exit 1
     fi
 
-    env TMPDIR=tmp ${lnav_test} -d /tmp/lnav.err \
+    env TMPDIR=logfile-tmp ${lnav_test} -d /tmp/lnav.err \
         -c ':config /tuning/archive-manager/cache-ttl 0d' \
         -n -q ${srcdir}/logfile_syslog.0
 
-    if test -f tmp/lnav*/archives/*-test-logs.tgz/test/logfile_access_log.0; then
+    if test -f logfile-tmp/lnav*/archives/*-test-logs.tgz/test/logfile_access_log.0; then
         echo "archive cache not deleted?"
         exit 1
     fi
 
-    run_test env TMPDIR=tmp ${lnav_test} -n\
+    run_test env TMPDIR=logfile-tmp ${lnav_test} -n\
         -c ';SELECT view_name, basename(filepath), visible FROM lnav_view_files' \
         test-logs.tgz
 
@@ -196,15 +203,18 @@ log       logfile_access_log.0       1
 log       logfile_access_log.1       1
 EOF
 
-    run_test env TMPDIR=tmp ${lnav_test} -n \
+    run_test env TMPDIR=logfile-tmp ${lnav_test} -n \
         test-logs-trunc.tgz
 
-    sed -e "s|${builddir}||g" `test_err_filename` | head -2 \
+    sed -e "s|${builddir}||g" \
+        -e 's/truncated gzip input/truncated/g' \
+        -e 's/Truncated tar archive detected while reading data/truncated/g' \
+        `test_err_filename` | head -2 \
         > test_logfile.trunc.out
     mv test_logfile.trunc.out `test_err_filename`
     check_error_output "truncated tgz not reported correctly" <<EOF
-✘ error: unable to open file: /test-logs-trunc.tgz
- reason: failed to extract 'src/lnav' from archive '/test-logs-trunc.tgz' -- truncated gzip input
+  error: unable to open file: /test-logs-trunc.tgz
+ reason: failed to extract 'src/lnav' from archive '/test-logs-trunc.tgz' -- truncated
 EOF
 
     mkdir -p rotmp
@@ -219,9 +229,12 @@ EOF
         > test_logfile.rotmp.out
     cp test_logfile.rotmp.out `test_err_filename`
     check_error_output "archive not unpacked" <<EOF
-✘ error: unable to open file: /test-logs.tgz
+  error: unable to open file: /test-logs.tgz
  reason: unable to create directory: rotmp/lnav-user-NNN-work/archives -- Permission denied
 EOF
+
+    run_cap_test env TMPDIR=logfile-tmp ${lnav_test} -n \
+        ${srcdir}/abs-archive.tar.gz
 fi
 
 touch unreadable.log
@@ -234,7 +247,7 @@ sed -e "s|/.*/unreadable.log|unreadable.log|g" `test_err_filename` | head -3 \
 
 mv test_logfile.unreadable.out `test_err_filename`
 check_error_output "able to read an unreadable log file?" <<EOF
-✘ error: file exists, but is not readable: unreadable.log
+  error: file exists, but is not readable: unreadable.log
  reason: Permission denied
 EOF
 
@@ -602,6 +615,8 @@ info 0x0
 error 0x0
 EOF
 
+run_cap_test ./drive_logfile -t -f logfmt_log ${srcdir}/logfile_grafana.0
+
 run_test ${lnav_test} -C ${test_dir}/logfile_bad_access_log.0
 
 sed -ibak -e "s|/.*/logfile_bad_access_log.0|logfile_bad_access_log.0|g" `test_err_filename`
@@ -736,3 +751,16 @@ run_cap_test ${lnav_test} -n \
 run_cap_test ${lnav_test} -n \
     -c ':set-file-timezone bad' \
     ${test_dir}/logfile_syslog.0
+
+run_cap_test ${lnav_test} -n \
+    -c ';SELECT log_time FROM all_logs' \
+    ${test_dir}/logfile_yday.0
+
+touch -t 202411030000 ${test_dir}/logfile_dst.0
+
+run_cap_test env TZ=America/Los_Angeles ${lnav_test} -n \
+    -c ':set-file-timezone America/Los_Angeles' \
+    ${test_dir}/logfile_dst.0
+
+cat ${test_dir}/logfile_generic.0 | run_cap_test ${lnav_test} -n \
+    -c ':test-comment generic before piper'
