@@ -9,6 +9,14 @@ unset XDG_CONFIG_HOME
 run_cap_test ${lnav_test} -nN \
     -c ";SELECT 1 = ?"
 
+# XXX The timestamp on the file is used to determine the year for syslog files.
+touch -t 200711030923 ${test_dir}/logfile_syslog.0
+run_cap_test ${lnav_test} -n \
+    -c ";.dump syslog_log.sql syslog_log" \
+    ${test_dir}/logfile_syslog.0
+
+run_cap_test cat syslog_log.sql
+
 run_cap_test ${lnav_test} -n \
     -c ";.read nonexistent-file" \
     ${test_dir}/logfile_empty.0
@@ -161,13 +169,13 @@ Wed May 19 12:00:04 +0000 2021 line 4
 EOF
 
 
-run_test ${lnav_test} -n \
+run_cap_test ${lnav_test} -n \
     -c ";SELECT * FROM access_log LIMIT 0" \
-    -c ':switch-to-view db' \
     ${test_dir}/logfile_access_log.0
 
-check_output "output generated for empty result set?" <<EOF
-EOF
+run_cap_test ${lnav_test} -n \
+    -c "|${test_dir}/empty-result.lnav" \
+    ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
     -c ":goto 2" \
@@ -377,8 +385,6 @@ run_cap_test ${lnav_test} -n \
     -c ':write-csv-to -' \
     ${test_dir}/logfile_access_log.0
 
-# XXX The timestamp on the file is used to determine the year for syslog files.
-touch -t 200711030923 ${test_dir}/logfile_syslog.0
 run_cap_test ${lnav_test} -n \
     -c ";select * from syslog_log" \
     -c ':write-csv-to -' \
@@ -390,6 +396,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_syslog.0
 
 check_output "log_time collation failed on null" <<EOF
+log_line,log_time,log_level,log_hostname,log_msgid,log_pid,log_pri,log_procname,log_struct,log_syslog_tag,syslog_version,log_part,log_idle_msecs,log_mark,log_comment,log_tags,log_annotations,log_filters
 EOF
 
 
@@ -465,7 +472,7 @@ EOF
 
 
 run_test ${lnav_test} -n \
-    -c ';SELECT name,value FROM environ WHERE name = "SQL_ENV_VALUE"' \
+    -c ";SELECT name,value FROM environ WHERE name = 'SQL_ENV_VALUE'" \
     -c ':write-csv-to -' \
     ${test_dir}/logfile_access_log.0
 
@@ -484,20 +491,20 @@ run_cap_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
-    -c ';INSERT INTO environ (name, value) VALUES ("", null)' \
+    -c ";INSERT INTO environ (name, value) VALUES ('', null)" \
     ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
-    -c ';INSERT INTO environ (name, value) VALUES ("foo=bar", "bar")' \
+    -c ";INSERT INTO environ (name, value) VALUES ('foo=bar', 'bar')" \
     ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
-    -c ';INSERT INTO environ (name, value) VALUES ("SQL_ENV_VALUE", "bar")' \
+    -c ";INSERT INTO environ (name, value) VALUES ('SQL_ENV_VALUE', 'bar')" \
     ${test_dir}/logfile_access_log.0
 
 run_test ${lnav_test} -n \
-    -c ';INSERT OR IGNORE INTO environ (name, value) VALUES ("SQL_ENV_VALUE", "bar")' \
-    -c ';SELECT * FROM environ WHERE name = "SQL_ENV_VALUE"' \
+    -c ";INSERT OR IGNORE INTO environ (name, value) VALUES ('SQL_ENV_VALUE', 'bar')" \
+    -c ";SELECT * FROM environ WHERE name = 'SQL_ENV_VALUE'" \
     -c ':write-csv-to -' \
     ${test_dir}/logfile_access_log.0
 
@@ -508,8 +515,8 @@ EOF
 
 
 run_test ${lnav_test} -n \
-    -c ';REPLACE INTO environ (name, value) VALUES ("SQL_ENV_VALUE", "bar")' \
-    -c ';SELECT * FROM environ WHERE name = "SQL_ENV_VALUE"' \
+    -c ";REPLACE INTO environ (name, value) VALUES ('SQL_ENV_VALUE', 'bar')" \
+    -c ";SELECT * FROM environ WHERE name = 'SQL_ENV_VALUE'" \
     -c ':write-csv-to -' \
     ${test_dir}/logfile_access_log.0
 
@@ -520,7 +527,7 @@ EOF
 
 
 run_test ${lnav_test} -n \
-    -c ';INSERT INTO environ (name, value) VALUES ("foo_env", "bar")' \
+    -c ";INSERT INTO environ (name, value) VALUES ('foo_env', 'bar')" \
     -c ';SELECT $foo_env as val' \
     -c ':write-csv-to -' \
     ${test_dir}/logfile_access_log.0
@@ -532,7 +539,7 @@ EOF
 
 
 run_test ${lnav_test} -n \
-    -c ';UPDATE environ SET name="NEW_ENV_VALUE" WHERE name="SQL_ENV_VALUE"' \
+    -c ";UPDATE environ SET name='NEW_ENV_VALUE' WHERE name='SQL_ENV_VALUE'" \
     -c ";SELECT * FROM environ WHERE name like '%ENV_VALUE'" \
     -c ':write-csv-to -' \
     ${test_dir}/logfile_access_log.0
@@ -544,12 +551,13 @@ EOF
 
 
 run_test ${lnav_test} -n \
-    -c ';DELETE FROM environ WHERE name="SQL_ENV_VALUE"' \
-    -c ';SELECT * FROM environ WHERE name like "%ENV_VALUE"' \
+    -c ";DELETE FROM environ WHERE name='SQL_ENV_VALUE'" \
+    -c ";SELECT * FROM environ WHERE name like '%ENV_VALUE'" \
     -c ':write-csv-to -' \
     ${test_dir}/logfile_access_log.0
 
 check_output "delete from environ table does not work" <<EOF
+name,value
 EOF
 
 
@@ -560,6 +568,7 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 check_output "delete environ table does not work" <<EOF
+name,value
 EOF
 
 
@@ -587,11 +596,11 @@ CREATE VIRTUAL TABLE regexp_capture USING regexp_capture_impl();
 CREATE VIRTUAL TABLE regexp_capture_into_json USING regexp_capture_into_json_impl();
 CREATE VIRTUAL TABLE xpath USING xpath_impl();
 CREATE VIRTUAL TABLE fstat USING fstat_impl();
-CREATE TABLE lnav_events (
+CREATE TABLE [1m[35mlnav_events[0m (
    ts TEXT NOT NULL DEFAULT(strftime('%Y-%m-%dT%H:%M:%f', 'now')),
    content TEXT
 );
-CREATE TABLE http_status_codes
+CREATE TABLE [1m[35mhttp_status_codes[0m
 EOF
 
 
@@ -623,6 +632,21 @@ log_line,log_part,log_time
 22,end,2015-04-07 07:32:56.000
 EOF
 
+run_cap_test ${lnav_test} -n \
+    -c ":goto 1" \
+    -c ":partition-name middle" \
+    -c ":goto 21" \
+    -c ":partition-name end" \
+    -c ":goto 0" \
+    -c ":next-section" \
+    -c ";SELECT log_top_line()" \
+    -c ":write-csv-to -" \
+    -c ":switch-to-view log" \
+    -c ":next-section" \
+    -c ";SELECT log_top_line()" \
+    -c ":write-csv-to -" \
+    ${test_dir}/logfile_pretty.0
+
 run_test ${lnav_test} -n \
     -c ":goto 1" \
     -c ":partition-name middle" \
@@ -778,9 +802,19 @@ run_test ${lnav_test} -n \
     ${test_dir}/logfile_multiline.0
 
 check_output "able to select a continued line?" <<EOF
+[
+
+]
 EOF
 
 
+run_cap_test ${lnav_test} -n \
+    -c ":create-search-table est_size estimated size (?<size>\d+\.\d+ \w+)" \
+    -c ";SELECT log_line, size FROM est_size ORDER BY size DESC" \
+    -c ":write-csv-to -" \
+    ${test_dir}/logfile_spark.0
+
+
 run_test ${lnav_test} -n \
     -c ":create-search-table search_test1 (\w+), world!" \
     -c ";select col_0 from search_test1" \
@@ -891,3 +925,66 @@ run_cap_test ${lnav_test} -Nn \
 run_cap_test ${lnav_test} -n \
      -c ";.msgformats" \
      ${test_dir}/logfile_for_join.0
+
+run_cap_test ${lnav_test} -Nn \
+    -c ";SELECT log_line, sc_bytes, json_object('columns', json_object('sc_bytes', json_object('color', '#f00'))) AS __lnav_style__ FROM access_log"  \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -Nn \
+    -c ";SELECT log_line, sc_bytes, json_object('columns', json_object('sc_ytes', json_object('color', '#f00'))) AS __lnav_style__ FROM access_log"  \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT bro_response_body_len AS bro_response_body_len1, humanize_file_size(bro_response_body_len) AS bro_response_body_len2 FROM bro_http_log ORDER BY bro_response_body_len DESC" \
+    ${test_dir}/logfile_bro_http.log.0
+
+run_cap_test ${lnav_test} -nN \
+    -c ";SELECT '1us' AS Duration UNION ALL SELECT '500ms' UNION ALL SELECT '1s'"
+
+run_cap_test env TEST_COMMENT="alignment demo" ${lnav_test} -nN -f- <<'EOF'
+;SELECT 'left' AS "Alignment Demo", '{"columns": {"Alignment Demo": {"text-align": "start"}}}' AS __lnav_style__
+ UNION ALL
+ SELECT 'center', '{"columns": {"Alignment Demo": {"text-align": "center"}}}' AS __lnav_style__
+ UNION ALL
+ SELECT 'right', '{"columns": {"Alignment Demo": {"text-align": "end"}}}' AS __lnav_style__
+:switch-to-view db
+EOF
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT * FROM access_log" \
+    -c ":hide-fields log_time" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT * FROM access_log" \
+    -c ":hide-fields bad" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT log_line FROM vmw_log WHERE log_opid = '7e1280cf'" \
+    ${test_dir}/logfile_vpxd.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT log_line_link FROM access_log WHERE log_line = 1" \
+    -c ';SELECT log_line FROM access_log WHERE log_line_link = $log_line_link' \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT log_line_link FROM access_log WHERE log_line = 1" \
+    -c ';SELECT log_line FROM access_log WHERE log_line_link > $log_line_link' \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT log_line_link FROM access_log WHERE log_line = 1" \
+    -c ':switch-to-view log' \
+    -c ':eval :goto $log_line_link' \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT * FROM lnav_focused_msg" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";UPDATE lnav_focused_msg SET log_mark = 1" \
+    -c ":switch-to-view log" \
+    ${test_dir}/logfile_access_log.0
